﻿using System;
using System.Collections.Generic;
using System.Linq;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Syncfusion.Drawing;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;
using Vinno.IUS.Common.Utilities;
using System.Globalization;
using System.Text.RegularExpressions;
using Syncfusion.Office;
using System.Net.Http;
using Vinno.IUS.Common.IO;
using System.IO;
using Newtonsoft.Json.Linq;

namespace ReportService
{
    public static class ParagraphGenerater
    {
        // private static readonly bool _isUbuntu = File.Exists("/etc/os-release") && File.ReadAllText("/etc/os-release").Contains("Ubuntu");
        private static string GetFontName()
        {
            return "Noto Sans SC";
        }
        public static void Generate(ReportInfo reportInfo, Paragraph paragraph, WTextBody wordTextBody)
        {
            var wordParagraph = wordTextBody.AddParagraph();
            SetParagraphFormat(paragraph, wordParagraph);
            SetParagraphSpacing(wordParagraph);
            foreach (var element in paragraph.Elements)
            {
                AddElement(element, wordParagraph, reportInfo, wordTextBody);
            }
        }

        private static void SetParagraphFormat(Paragraph paragraph, IWParagraph wordParagraph)
        {
            switch (paragraph.HorizontalAlignment)
            {
                case HorizontalLayout.Left:
                    wordParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Left;
                    break;
                case HorizontalLayout.Center:
                    wordParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Center;
                    break;
                case HorizontalLayout.Right:
                    wordParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Right;
                    break;
                case HorizontalLayout.Stretch:
                    wordParagraph.ParagraphFormat.HorizontalAlignment = HorizontalAlignment.Left;
                    break;
            }
        }

        public static IWTextRange AppendTextEx(this IWParagraph paragraph, string text)
        {
            var textRange = paragraph.AppendText(text);
            textRange.CharacterFormat.LocaleIdFarEast = (short)LocaleIDs.zh_CN;
            return textRange;
        }

        private static IEnumerable<string> Split(string staticText)
        {
            var splits = staticText.Split(' ');
            var last = " ";
            foreach (var split in splits)
            {
                if (split == string.Empty)
                    last = " ";
                else
                {
                    if (last != " ")
                        yield return " ";

                    last = split;
                }

                yield return last;
            }
        }

        private static void AddStaticText(IWParagraph wordparagraph, StaticText staticText, string text)
        {
            var cloneStaticText = (StaticText)staticText.Clone();
            cloneStaticText.LineLength = text.Length;
            cloneStaticText.LineWidth = 0;
            var wTextRange = wordparagraph.AppendTextEx(string.Empty);
            SetCharacterFormat(wTextRange, cloneStaticText, staticText.GetReportTemplateDocument().InvertColor);
            SetTextElementValue(cloneStaticText, wTextRange, text);
        }

        private static string QueryInputTextRange(ReportInfo reportInfo, ITextElement textElement)
        {
            var textElementValue = string.Empty;
            if (textElement.Tag != null && reportInfo.ElementValues.ContainsKey(textElement.Tag))
            {
                if (reportInfo.ElementValues[textElement.Tag] is ReportCommentsElementValue reportCommentsElementValue)
                {
                    var commentList = reportCommentsElementValue.Value.ToList();
                    for (var i = 0; i < commentList.Count; i++)
                    {
                        var comment = commentList[i];
                        textElementValue = comment.Value;
                    }
                    return textElementValue;
                }

                if (reportInfo.ElementValues[textElement.Tag] is StringListElementValue stringListElementValue)
                {
                    textElementValue = string.Join(";", stringListElementValue.Value.Select(v => v.Value));
                }
                else if (reportInfo.ElementValues[textElement.Tag].GetValue() != null)
                {
                    if (textElement.Tag.Name == "ResultPrompt" && !string.IsNullOrEmpty(textElement.Tag.ParentName))
                    {
                        var tempValue = reportInfo.ElementValues[textElement.Tag]?.GetValue().ToString();
                        if (!string.IsNullOrEmpty(tempValue))
                        {
                            JObject json = JObject.Parse(tempValue);
                            if (json != null && json.ContainsKey(textElement.Tag.ParentName))
                            {
                                textElementValue = json[textElement.Tag.ParentName].ToString();
                            }
                        }
                    }
                    else
                    {
                        textElementValue = reportInfo.ElementValues[textElement.Tag]?.GetValue().ToString();
                    }
                }
            }
            else if (textElement.MeasureTag != null && reportInfo.MeasureElementValues?.Count > 0)
            {
                var fetusTag = textElement.GetFetusTag();
                var dictionary = reportInfo.MeasureElementValues[fetusTag];
                var matchedKey = dictionary.Keys.FirstOrDefault(k => IsMeasureTagEquals(k, textElement.MeasureTag));
                if (matchedKey != null)
                {
                    var str = dictionary[matchedKey]?.GetValue()?.ToString() ?? "";
                    var measuresStart = str.IndexOf('[');
                    if (textElement.MeasureTag.MeasureResultType == MeasureResultType.Measures.ToString())
                    {
                        if (measuresStart > 0)
                        {
                            textElementValue = str.Substring(measuresStart);
                        }
                        else
                        {
                            textElementValue = String.Empty;
                        }
                    }
                    else
                    {
                        if (measuresStart > 0)
                        {
                            str = str.Substring(0, measuresStart);
                        }
                        textElementValue = str;
                    }
                }
            }
            return textElementValue;
        }

        public static bool QueryElement(IElement element, ReportInfo reportInfo, List<string> staticTextGroupFieldList, List<string> inputGroupFieldList, string tempGroupField)
        {
            if (element is StaticText staticText)
            {
                if (!staticTextGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    staticTextGroupFieldList.Add(tempGroupField.ToLower());
                }
                return false;
            }

            if (element is InputText textElement)
            {
                if (!inputGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    inputGroupFieldList.Add(tempGroupField.ToLower());
                }
                var textElementValue = QueryInputTextRange(reportInfo, textElement);
                if (!string.IsNullOrWhiteSpace(textElementValue))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            if (element is SingleSelected singleSelected)
            {
                if (!inputGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    inputGroupFieldList.Add(tempGroupField.ToLower());
                }
                if (singleSelected.DisplayStyle == SelectorDisplayStyle.OnlyShowSelectedItem)
                {
                    //AddInputTextRange(wordparagraph, reportInfo, singleSelected, textBody);
                    var textElementValue = QueryInputTextRange(reportInfo, singleSelected);
                    if (!string.IsNullOrWhiteSpace(textElementValue))
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    var selecteItems = new List<string>();
                    if (singleSelected.Tag == null)
                        return false;
                    if (reportInfo.ElementValues.ContainsKey(singleSelected.Tag) && reportInfo.ElementValues[singleSelected.Tag] is TextElementValue textElementValue)
                    {
                        if (!string.IsNullOrEmpty(textElementValue.Value))
                        {
                            selecteItems.Add(textElementValue.Value);
                        }
                    }
                    if (selecteItems?.Count > 0)
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
            }

            if (element is MultiSelected multiSelected)
            {
                if (!inputGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    inputGroupFieldList.Add(tempGroupField.ToLower());
                }
                if (multiSelected.DisplayStyle == SelectorDisplayStyle.OnlyShowSelectedItem)
                {
                    //AddInputTextRange(wordparagraph, reportInfo, multiSelected, textBody);
                    var textElementValue = QueryInputTextRange(reportInfo, multiSelected);
                    if (!string.IsNullOrWhiteSpace(textElementValue))
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    var selecteItems = new List<string>();
                    if (multiSelected.Tag == null)
                        return false;
                    if (reportInfo.ElementValues.ContainsKey(multiSelected.Tag) && reportInfo.ElementValues[multiSelected.Tag] is StringListElementValue stringListElementValue)
                    {
                        var selectItemValues = stringListElementValue.Value?.Where(d => !string.IsNullOrEmpty(d.Value))?.Select(v => v.Value)?.ToList() ?? new List<string>();
                        if (selectItemValues?.Count > 0)
                        {
                            selecteItems.AddRange(selectItemValues);
                        }
                    }
                    if (selecteItems?.Count > 0)
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
            }

            if (element is MultiSelectedVertical mltiSelectedVertical)
            {
                if (!inputGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    inputGroupFieldList.Add(tempGroupField.ToLower());
                }
                var selecteItems = new List<string>();
                if (mltiSelectedVertical.Tag == null)
                    return false;
                if (reportInfo.ElementValues.ContainsKey(mltiSelectedVertical.Tag) && reportInfo.ElementValues[mltiSelectedVertical.Tag] is StringListElementValue stringListElementValue)
                {
                    var selectItemValues = stringListElementValue.Value?.Where(d => !string.IsNullOrEmpty(d.Value))?.Select(v => v.Value)?.ToList() ?? new List<string>();
                    if (selectItemValues?.Count > 0)
                    {
                        selecteItems.AddRange(selectItemValues);
                    }
                }
                if (selecteItems?.Count > 0)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            if (element is DateTimeElement dateTimeElement)
            {
                if (!inputGroupFieldList.Contains(tempGroupField.ToLower()))
                {
                    inputGroupFieldList.Add(tempGroupField.ToLower());
                }
                var textElementValue = string.Empty;
                if (dateTimeElement.Tag != null && reportInfo.ElementValues.ContainsKey(dateTimeElement.Tag))
                {
                    var dateTimeElementValue = reportInfo.ElementValues[dateTimeElement.Tag] as DateTimeElementValue;
                    if (dateTimeElementValue != null && dateTimeElementValue.Value.HasValue)
                    {
                        if (TranslateHelper.CurrentLanguage == TranslateHelper.ChineseLanguageKey)
                        {
                            textElementValue = dateTimeElementValue.Value.Value.ToString(dateTimeElement.DateTimeFormat);
                        }
                        else
                        {
                            textElementValue = dateTimeElementValue.Value.Value.ToString(dateTimeElement.DateTimeFormat, CultureInfo.CreateSpecificCulture("en-GB"));
                        }
                    }
                }
                if (!string.IsNullOrWhiteSpace(textElementValue))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            return false;
        }

        private static void AddElement(IElement element, IWParagraph wordparagraph, ReportInfo reportInfo, WTextBody textBody)
        {
            if (element is StaticText staticText)
            {
                Split(staticText.Text).ToList()
                    .ForEach(x => AddStaticText(wordparagraph, staticText, x));
                return;
            }

            if (element is InputText inputText)
            {
                AddInputTextRange(wordparagraph, reportInfo, inputText, textBody);
                return;
            }

            if (element is SingleSelected singleSelected)
            {
                if (singleSelected.DisplayStyle == SelectorDisplayStyle.OnlyShowSelectedItem)
                {
                    AddInputTextRange(wordparagraph, reportInfo, singleSelected, textBody);
                }
                else
                {
                    var selecteItems = new List<string>();
                    if (singleSelected.Tag == null)
                        return;
                    if (reportInfo.ElementValues.ContainsKey(singleSelected.Tag) && reportInfo.ElementValues[singleSelected.Tag] is TextElementValue textElementValue)
                    {
                        selecteItems.Add(textElementValue.Value);
                    }
                    if (singleSelected.DisplayStyle == SelectorDisplayStyle.ShowAllItems)
                    {
                        AddCheckBox(wordparagraph, singleSelected, singleSelected.Items, selecteItems);
                    }
                }

                return;
            }

            //if (element is SingleSelectedVertical singleSelectedVertical)
            //{
            //    var selecteItems = new List<string>();
            //    if (reportInfo.ElementValues.ContainsKey(singleSelectedVertical.Tag) && reportInfo.ElementValues[singleSelectedVertical.Tag] is TextElementValue textElementValue)
            //    {
            //        selecteItems.Add(textElementValue.Value);
            //    }
            //    if (!(wordparagraph.Owner is WTextBody wordTextBody))
            //        return;
            //    wordTextBody.ChildEntities.Remove(wordparagraph);
            //    var wordTable = wordTextBody.AddTable();

            //    AddCheckBoxVertical(wordTable, singleSelectedVertical, singleSelectedVertical.Items, selecteItems);
            //}

            if (element is MultiSelected multiSelected)
            {
                if (multiSelected.DisplayStyle == SelectorDisplayStyle.OnlyShowSelectedItem)
                {
                    AddInputTextRange(wordparagraph, reportInfo, multiSelected, textBody);
                }
                else
                {
                    var selecteItems = new List<string>();
                    if (multiSelected.Tag == null)
                        return;
                    if (reportInfo.ElementValues.ContainsKey(multiSelected.Tag) && reportInfo.ElementValues[multiSelected.Tag] is StringListElementValue stringListElementValue)
                    {
                        selecteItems.AddRange(stringListElementValue.Value.Select(v => v.Value));
                    }
                    if (multiSelected.DisplayStyle == SelectorDisplayStyle.ShowAllItems)
                    {
                        AddCheckBox(wordparagraph, multiSelected, multiSelected.Items, selecteItems);
                    }
                    if (multiSelected.DisplayStyle == SelectorDisplayStyle.ShowAllItemsVerticalAlignment)
                    {
                        if (!(wordparagraph.Owner is WTextBody wordTextBody))
                            return;
                        wordTextBody.ChildEntities.Remove(wordparagraph);
                        var wordTable = wordTextBody.AddTable();
                        AddCheckBoxV(wordTable, multiSelected, multiSelected.Items, selecteItems);
                    }
                }

                return;
            }

            if (element is MultiSelectedVertical mltiSelectedVertical)
            {
                var selecteItems = new List<string>();
                if (mltiSelectedVertical.Tag == null)
                    return;
                if (reportInfo.ElementValues.ContainsKey(mltiSelectedVertical.Tag) && reportInfo.ElementValues[mltiSelectedVertical.Tag] is StringListElementValue stringListElementValue)
                {
                    selecteItems.AddRange(stringListElementValue.Value.Select(v => v.Value));
                }
                if (!(wordparagraph.Owner is WTextBody wordTextBody))
                    return;
                wordTextBody.ChildEntities.Remove(wordparagraph);
                var wordTable = wordTextBody.AddTable();
                AddCheckBoxVertical(wordTable, mltiSelectedVertical, mltiSelectedVertical.Items, selecteItems);
            }

            if (element is DateTimeElement dateTimeElement)
            {
                var wTextRange = wordparagraph.AppendText(string.Empty);
                var textElementValue = string.Empty;
                if (dateTimeElement.Tag != null && reportInfo.ElementValues.ContainsKey(dateTimeElement.Tag))
                {
                    var dateTimeElementValue = reportInfo.ElementValues[dateTimeElement.Tag] as DateTimeElementValue;
                    if (dateTimeElementValue != null && dateTimeElementValue.Value.HasValue)
                    {
                        if (TranslateHelper.CurrentLanguage == TranslateHelper.ChineseLanguageKey)
                        {
                            textElementValue = dateTimeElementValue.Value.Value.ToString(dateTimeElement.DateTimeFormat);
                        }
                        else
                        {
                            textElementValue = dateTimeElementValue.Value.Value.ToString(dateTimeElement.DateTimeFormat, CultureInfo.CreateSpecificCulture("en-GB"));
                        }
                    }
                    var textDateElementValue = reportInfo.ElementValues[dateTimeElement.Tag] as TextElementValue;
                    if (textDateElementValue!=null && !string.IsNullOrWhiteSpace(textDateElementValue.Value))
                    {
                        if (DateTime.TryParse(textDateElementValue.Value, out DateTime elementDate))
                        {
                            if (TranslateHelper.CurrentLanguage == TranslateHelper.ChineseLanguageKey)
                            {
                                textElementValue = elementDate.ToString(dateTimeElement.DateTimeFormat);
                            }
                            else
                            {
                                textElementValue = elementDate.ToString(dateTimeElement.DateTimeFormat, CultureInfo.CreateSpecificCulture("en-GB"));
                            }
                        }
                    }
                }

                SetCharacterFormat(wTextRange, dateTimeElement);
                wTextRange.Text = textElementValue;
                return;
            }

            if (element is Space space)
            {
                var emptyTextRange = wordparagraph.AppendText(string.Empty);
                //for (int i = 0; i < space.SpaceUnitCount; i++)
                //{
                //    emptyTextRange.Text += " ";
                //}

                var length = 0;
                if (space.SpaceWidth > 0)
                {
                    //length = (int)Math.Ceiling(space.SpaceWidth / 5.2);
                    length = (int)Math.Floor(space.SpaceWidth / 5.2);
                }

                var extendString = string.Empty;
                for (int i = 0; i < length; i++)
                {
                    emptyTextRange.Text += " ";
                }

                emptyTextRange.CharacterFormat.FontName = GetFontName();
                emptyTextRange.CharacterFormat.FontSize = 10.5f;
                return;
            }

            if (element is Line line)
            {
#if __MACOS__
                var lineShape = wordparagraph.AppendShape(AutoShapeType.Line, (float) line.Width, 0);
                lineShape.LineFormat.Color = line.GetReportTemplateDocument().InvertColor
                    ? Color.FromArgb(255 - line.Stroke.R, 255 - line.Stroke.G, 255 - line.Stroke.B)
                    : Color.FromArgb(line.Stroke.R, line.Stroke.G, line.Stroke.B);
                lineShape.LineFormat.Weight = (float)line.Thickness;
                SetLineDashStyle(line, lineShape);

                var paragraph = line.Parent as Paragraph;
                if (paragraph.Elements.All(a => a is Line))
                {
                    wordparagraph.ParagraphFormat.LineSpacing = 6 + (float)line.Thickness;
                    lineShape.VerticalPosition = wordparagraph.ParagraphFormat.LineSpacing / 2;
                }
#else
                TableGenerater.GenerateTableLine(textBody, line);
                var paragraph = line.Parent as Paragraph;
                if (paragraph.Elements.All(a => a is Line))
                {
                    textBody.ChildEntities.Remove(wordparagraph);
                }
#endif
                return;
            }

            if (element is PageNumberElement pageNumberElement)
            {
                var pageField = wordparagraph.AppendField("PG", FieldType.FieldPage);
                SetCharacterFormat(pageField, pageNumberElement);
                if (pageNumberElement.PageNumberStyle == Vinno.vCloud.Report.Interfaces.PageNumberStyle.NumberAndTotalNumber)
                {
                    var textRange = wordparagraph.AppendText("/");
                    SetCharacterFormat(textRange, pageNumberElement);
                    var pageNumbersField = wordparagraph.AppendField("NP", FieldType.FieldNumPages);
                    SetCharacterFormat(pageNumbersField, pageNumberElement);
                }

                return;
            }

            if (element is StaticImage staticImage && staticImage.Source != null)
            {
                wordparagraph.ParagraphFormat.BeforeSpacing = 2f;
                var staticPicture = wordparagraph.AppendPicture(staticImage.Source);
                // workaround for SyncFusion bug
                var bordersHeight = staticImage.GetLogicalAncestry()
                    .OfType<RTCell>()
                    .Sum(x => x.Borders.Top.Thickness + x.Borders.Bottom.Thickness);
                var document = element.GetReportTemplateDocument();
                var maxWidth = document.GetPageClientWidth();
                var maxHeight = document.GetPageClientHeight() - 10 - bordersHeight;
                staticPicture.Height = (float)Math.Min(staticImage.Height, maxHeight);
                staticPicture.Width = (float)Math.Min(staticImage.Width, maxWidth);
                SetPictureAlignment(staticImage, staticPicture);
                return;
            }

            if (element is InputImage inputImage)
            {
                if (inputImage.Tag != null && reportInfo.ElementValues.ContainsKey(inputImage.Tag))
                {
                    if (!(wordparagraph.Owner is WTextBody wordTextBody))
                        return;

                    wordTextBody.ChildEntities.Remove(wordparagraph);
                    SetImageTableAlignment(wordparagraph.ParagraphFormat.HorizontalAlignment, inputImage);
                    var table = TableGenerater.AddImageListTable(inputImage, wordTextBody, 1);
                    table.TableFormat.Paddings.Top = 2f;
                    var width = inputImage.Width;
                    var height = inputImage.Height;
                    TableGenerater.InflateImageListTable(inputImage, table, ref width, ref height);
                    var image = reportInfo.ElementValues[inputImage.Tag];
                    var values = new List<IImageElementValue>();
                    if (image is FileImageValue file)
                    {
                        values.Add(file);
                    }
                    else if (image is BufferImageValue buffer)
                    {
                        values.Add(buffer);
                    }
                    else if (image is ImageListElementValue imageList)
                    {
                        values.AddRange(imageList.Value);
                    }
                    else if (image.GetValue().GetType() == typeof(string))
                    {
                        var imagePath = (string)image.GetValue();
                        if (!string.IsNullOrWhiteSpace(imagePath))
                        {
                            Uri uri = new Uri(imagePath);
                            if (uri.IsFile)
                            {
                                var imageBytes = File.ReadAllBytes(imagePath);
                                var fileImage = new BufferImageValue(imageBytes);
                                values.Add(fileImage);
                            }
                            else
                            {
                                using (HttpClient client = new HttpClient())
                                {
                                    var imageBytes = client.GetByteArrayAsync(imagePath).Result;
                                    var fileImage = new BufferImageValue(imageBytes);
                                    values.Add(fileImage);
                                }
                            }
                        }
                    }
                    TableGenerater.AddImageListValues(table, values.ToArray(), width, height, inputImage.TransparentBackground, false, inputImage.Tag);
                }
            }
        }

        private static bool AddReportPhysician(IElement element, IWParagraph wordparagraph, ReportInfo reportInfo, WTextBody textBody)
        {
            if (element.Tag?.Name != "ReportPhysician")
            {
                return false;
            }
            var elementValue = reportInfo.ElementValues[element.Tag]?.GetValue()?.ToString();
            if (string.IsNullOrWhiteSpace(elementValue))
            {
                return false;
            }
            var base64Regex = "^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$";
            if (!Regex.IsMatch(elementValue, base64Regex))
            {
                if (element is InputText inputText)
                {
                    AddInputTextRange(wordparagraph, reportInfo, inputText, textBody);
                    return true;
                }
            }
            else
            {
                var eleValue = reportInfo.ElementValues[element.Tag];
                if (eleValue is TextElementValue textElementValue)
                {
                    var image = wordparagraph.AppendPicture(Convert.FromBase64String(textElementValue.Value));
                    var ratio = image.Width / Math.Max(1, image.Height);
                    if (ratio != 0)
                    {
                        image.Width = 130;
                        image.Height = image.Width / ratio;
                    }
                    return true;
                }
            }


            return false;
        }

        private static void AddCheckBoxVertical(IWTable wordTable, ITextElement textElement, List<string> items, List<string> selecteItems)
        {
            wordTable.TableFormat.Borders.BorderType = BorderStyle.None;
            wordTable.TableFormat.Paddings.All = 0;
            wordTable.TableFormat.Paddings.Top = 1.5f;
            var rowCount = items.Count;
            var columnCount = 1;
            wordTable.ResetCells(rowCount, columnCount);
            var rowIndex = 0;
            foreach (var item in items)
            {
                var wordparagraph = wordTable.Rows[rowIndex].Cells[0].AddParagraph();
                var checkBox = wordparagraph.AppendInlineContentControl(ContentControlType.CheckBox) as InlineContentControl;
                if (checkBox == null)
                {
                    return;
                }

                var textRange = wordparagraph.AppendText(item);
                SetCharacterFormat(textRange, textElement);
                checkBox.ParagraphItems.Add(textRange);
                SetCheckBoxFormart(checkBox);
                var fi = checkBox.ParagraphItems.FirstItem as WTextRange;
                if (fi != null)
                {
                    fi.CharacterFormat.FontSize = (float)textElement.FontSize;
                    fi.CharacterFormat.TextColor = textElement.GetReportTemplateDocument().InvertColor
             ? Syncfusion.Drawing.Color.FromArgb(255 - fi.CharacterFormat.TextColor.R, 255 - fi.CharacterFormat.TextColor.G,
                 255 - fi.CharacterFormat.TextColor.B)
             : Syncfusion.Drawing.Color.FromArgb(fi.CharacterFormat.TextColor.R, fi.CharacterFormat.TextColor.G, fi.CharacterFormat.TextColor.B);
                }
                if (selecteItems.Contains(item))
                {
                    checkBox.ContentControlProperties.IsChecked = true;
                }
                rowIndex++;
            }
        }

        private static void SetParagraphSpacing(IWParagraph wordparagraph)
        {
            wordparagraph.ParagraphFormat.LeftIndent = 2f;
        }

        private static void AddCheckBoxV(IWTable wordTable, ITextElement textElement, List<string> items, List<string> selectedItems)
        {
            wordTable.TableFormat.Borders.BorderType = BorderStyle.None;
            wordTable.TableFormat.Paddings.All = 0;
            wordTable.TableFormat.Paddings.Top = 1.5f;
            var rowCount = items.Count;
            var columnCount = 1;
            wordTable.ResetCells(rowCount, columnCount);
            var rowIndex = 0;
            foreach (var item in items)
            {
                var wordparagraph = wordTable.Rows[rowIndex].Cells[0].AddParagraph();
                var checkBox = wordparagraph.AppendInlineContentControl(ContentControlType.CheckBox) as InlineContentControl;
                if (checkBox == null)
                {
                    return;
                }

                var textRange = wordparagraph.AppendText(item);
                SetCharacterFormat(textRange, textElement);
                checkBox.ParagraphItems.Add(textRange);
                SetCheckBoxFormart(checkBox);
                var fi = checkBox.ParagraphItems.FirstItem as WTextRange;
                if (fi != null)
                {
                    fi.CharacterFormat.FontSize = (float)textElement.FontSize;
                    fi.CharacterFormat.TextColor = textElement.GetReportTemplateDocument().InvertColor
             ? Syncfusion.Drawing.Color.FromArgb(255 - fi.CharacterFormat.TextColor.R, 255 - fi.CharacterFormat.TextColor.G,
                 255 - fi.CharacterFormat.TextColor.B)
             : Syncfusion.Drawing.Color.FromArgb(fi.CharacterFormat.TextColor.R, fi.CharacterFormat.TextColor.G, fi.CharacterFormat.TextColor.B);
                }
                // if (selectedItems.Contains(item))
                // {
                //     checkBox.ContentControlProperties.IsChecked = true;
                // }
                if (selectedItems.Contains((rowIndex + 1).ToString()))
                {
                    checkBox.ContentControlProperties.IsChecked = true;
                }
                wordparagraph.AppendText(" ");
                rowIndex++;
            }
        }

        private static void AddCheckBox(IWParagraph wordparagraph, ITextElement textElement, List<string> items, List<string> selectedItems)
        {
            foreach (var item in items)
            {
                var checkBox = wordparagraph.AppendInlineContentControl(ContentControlType.CheckBox) as InlineContentControl;
                if (checkBox == null)
                {
                    return;
                }

                var textRange = wordparagraph.AppendText(item);
                SetCharacterFormat(textRange, textElement);
                checkBox.ParagraphItems.Add(textRange);
                SetCheckBoxFormart(checkBox);
                var fi = checkBox.ParagraphItems.FirstItem as WTextRange;
                if (fi != null)
                {
                    fi.CharacterFormat.FontSize = (float)textElement.FontSize;
                    fi.CharacterFormat.TextColor = textElement.GetReportTemplateDocument().InvertColor
             ? Syncfusion.Drawing.Color.FromArgb(255 - fi.CharacterFormat.TextColor.R, 255 - fi.CharacterFormat.TextColor.G,
                 255 - fi.CharacterFormat.TextColor.B)
             : Syncfusion.Drawing.Color.FromArgb(fi.CharacterFormat.TextColor.R, fi.CharacterFormat.TextColor.G, fi.CharacterFormat.TextColor.B);
                }
                if (selectedItems.Contains(item))
                {
                    checkBox.ContentControlProperties.IsChecked = true;
                }
                wordparagraph.AppendText(" ");
            }
        }

        private static void SetCheckBoxFormart(InlineContentControl checkBox)
        {
            var state = new CheckBoxState();
            state.Font = "Wingdings";
            state.Value = '\u00FE'.ToString();
            checkBox.ContentControlProperties.CheckedState = state;
        }

        private static void SetPictureAlignment(IImage image, IWPicture staticPicture)
        {
            switch (image.HorizontalAlignment)
            {
                case HorizontalLayout.Left:
                    staticPicture.HorizontalAlignment = ShapeHorizontalAlignment.Left;
                    break;
                case HorizontalLayout.Center:
                    staticPicture.HorizontalAlignment = ShapeHorizontalAlignment.Center;
                    break;
                case HorizontalLayout.Right:
                    staticPicture.HorizontalAlignment = ShapeHorizontalAlignment.Right;
                    break;
            }

            switch (image.VerticalAlignment)
            {
                case VerticalLayout.Top:
                    staticPicture.VerticalAlignment = ShapeVerticalAlignment.Top;
                    break;
                case VerticalLayout.Center:
                    staticPicture.VerticalAlignment = ShapeVerticalAlignment.Center;
                    break;
                case VerticalLayout.Bottom:
                    staticPicture.VerticalAlignment = ShapeVerticalAlignment.Bottom;
                    break;
            }
        }

        private static void SetImageTableAlignment(HorizontalAlignment horizontalAlignment, IElement element)
        {
            switch (horizontalAlignment)
            {
                case HorizontalAlignment.Left:
                case HorizontalAlignment.Justify:
                    element.HorizontalAlignment = HorizontalLayout.Left;
                    break;
                case HorizontalAlignment.Center:
                    element.HorizontalAlignment = HorizontalLayout.Center;
                    break;
                case HorizontalAlignment.Right:
                    element.HorizontalAlignment = HorizontalLayout.Right;
                    break;
            }
        }

        private static void SetLineDashStyle(Line line1, Syncfusion.DocIO.DLS.Shape lineShape)
        {
            switch (line1.BorderStyle)
            {
                case RTBorderStyle.Solid:
                    lineShape.LineFormat.DashStyle = LineDashing.Solid;
                    break;
                case RTBorderStyle.Dash:
                    lineShape.LineFormat.DashStyle = LineDashing.Dash;
                    break;
                case RTBorderStyle.Dot:
                    lineShape.LineFormat.DashStyle = LineDashing.Dot;
                    break;
                case RTBorderStyle.DashDot:
                    lineShape.LineFormat.DashStyle = LineDashing.DashDot;
                    break;
                case RTBorderStyle.DashDotDot:
                    lineShape.LineFormat.DashStyle = LineDashing.DashDotDot;
                    break;
            }
        }

        private static void AddInputTextRange(IWParagraph wordparagraph, ReportInfo reportInfo, ITextElement textElement, WTextBody textBody)
        {
            var wTextRange = wordparagraph.AppendText(string.Empty);
            var textElementValue = string.Empty;
            if (textElement.Tag != null && reportInfo.ElementValues.ContainsKey(textElement.Tag))
            {
                if (reportInfo.ElementValues[textElement.Tag] is ReportCommentsElementValue reportCommentsElementValue)
                {
                    var commentList = reportCommentsElementValue.Value.ToList();
                    for (var i = 0; i < commentList.Count; i++)
                    {
                        var comment = commentList[i];   
                        textElementValue = comment.Value;
                        if (i != 0)
                        {
                            wTextRange = wordparagraph.AppendText(string.Empty);
                        }
                        if (comment.IsTitie)
                        {
                            wTextRange.CharacterFormat.Bold = true;
                            wTextRange.CharacterFormat.UnderlineStyle = UnderlineStyle.Single;
                        }
                        else
                        {
                            if (comment.IsBold)
                            {
                                wTextRange.CharacterFormat.Bold = true;

                            }
                            if (comment.IsUnderline)
                            {
                                wTextRange.CharacterFormat.UnderlineStyle = UnderlineStyle.Single;
                            }
                        }

                        if (comment.IsItalic)
                        {
                            wTextRange.CharacterFormat.Italic = true;
                        }

                        //IsNoWrap代表不换行
                        if (!comment.IsNoWrap)
                        {
                            wordparagraph = textBody.AddParagraph();
                        }
                        SetCharacterFormat(wTextRange, textElement);
                        SetTextElementValue(textElement, wTextRange, textElementValue);
                    }
                    return;
                }

                if (reportInfo.ElementValues[textElement.Tag] is StringListElementValue stringListElementValue)
                {
                    textElementValue = string.Join(";", stringListElementValue.Value.Select(v => v.Value));
                }
                else if (reportInfo.ElementValues[textElement.Tag].GetValue() != null)
                {
                    if (textElement.Tag.Name == "ResultPrompt" && !string.IsNullOrEmpty(textElement.Tag.ParentName))
                    {
                        var tempValue = reportInfo.ElementValues[textElement.Tag]?.GetValue().ToString();
                        if (!string.IsNullOrEmpty(tempValue))
                        {
                            JObject json = JObject.Parse(tempValue);
                            if (json != null && json.ContainsKey(textElement.Tag.ParentName))
                            {
                                textElementValue = json[textElement.Tag.ParentName].ToString();
                            }
                        }
                    }
                    else {
                        textElementValue = reportInfo.ElementValues[textElement.Tag]?.GetValue().ToString();
                    }
                }
            }
            else if (textElement.MeasureTag != null && reportInfo.MeasureElementValues?.Count > 0)
            {
                var fetusTag = textElement.GetFetusTag();
                var dictionary = reportInfo.MeasureElementValues[fetusTag];
                var matchedKey = dictionary.Keys.FirstOrDefault(k => IsMeasureTagEquals(k, textElement.MeasureTag));
                if (matchedKey != null)
                {
                    var str = dictionary[matchedKey]?.GetValue()?.ToString() ?? "";
                    var measuresStart = str.IndexOf('[');
                    if (textElement.MeasureTag.MeasureResultType == MeasureResultType.Measures.ToString())
                    {
                        if (measuresStart > 0)
                        {
                            textElementValue = str.Substring(measuresStart);
                        }
                        else
                        {
                            textElementValue = String.Empty;
                        }
                    }
                    else
                    {
                        if (measuresStart > 0)
                        {
                            str = str.Substring(0, measuresStart);
                        }
                        textElementValue = str;
                    }
                }
            }

            if (ElementTag.HospitalName.Equals(textElement.Tag))
            {
                wTextRange.Text = textElementValue;
            }
            else
            {
                SetTextElementValue(textElement, wTextRange, textElementValue);
            }
            SetCharacterFormat(wTextRange, textElement);
        }

        private static bool IsMeasureTagEquals(MeasureTag measureTag1, MeasureTag measureTag2)
        {
            return new SortedSet<string>(measureTag1.AvailableMethods).SetEquals(new SortedSet<string>(measureTag2.AvailableMethods)) &&
                   (string.IsNullOrEmpty(measureTag1.BaseType) && string.IsNullOrEmpty(measureTag2.BaseType) || string.Equals(measureTag1.BaseType, measureTag2.BaseType)) &&
                   (string.IsNullOrEmpty(measureTag1.CalculationId) && string.IsNullOrEmpty(measureTag2.CalculationId) || string.Equals(measureTag1.CalculationId, measureTag2.CalculationId)) &&
                   (string.IsNullOrEmpty(measureTag1.MatchId) && string.IsNullOrEmpty(measureTag2.MatchId) || string.Equals(measureTag1.MatchId, measureTag2.MatchId)) &&
                   (string.IsNullOrEmpty(measureTag1.Method) && string.IsNullOrEmpty(measureTag2.Method) || string.Equals(measureTag1.Method, measureTag2.Method)) &&
                   (string.IsNullOrEmpty(measureTag1.Mode) && string.IsNullOrEmpty(measureTag2.Mode) || string.Equals(measureTag1.Mode, measureTag2.Mode)) &&
                   (string.IsNullOrEmpty(measureTag1.Output) && string.IsNullOrEmpty(measureTag2.Output) || string.Equals(measureTag1.Output, measureTag2.Output)) &&
                   (string.IsNullOrEmpty(measureTag1.OutputV2) && string.IsNullOrEmpty(measureTag2.OutputV2) || string.Equals(measureTag1.OutputV2, measureTag2.OutputV2)) &&
                   (string.IsNullOrEmpty(measureTag1.Unit) && string.IsNullOrEmpty(measureTag2.Unit) || string.Equals(measureTag1.Unit, measureTag2.Unit)) &&
                   (string.IsNullOrEmpty(measureTag1.UserId) && string.IsNullOrEmpty(measureTag2.UserId) || string.Equals(measureTag1.UserId, measureTag2.UserId)) &&
                   (string.IsNullOrEmpty(measureTag1.MeasureResultType) && string.IsNullOrEmpty(measureTag2.MeasureResultType) || string.Equals(measureTag1.MeasureResultType, measureTag2.MeasureResultType));
        }

        private static void SetCharacterFormat(IWTextRange wTextRange, ITextElement textElement, bool? forceInvertColor = null)
        {
            // if (ContainsChinese(wTextRange.Text))
            // {
            //     wTextRange.CharacterFormat.FontName = GetFontName();
            // }
            // else
            // {
            //     wTextRange.CharacterFormat.FontName = _isUbuntu ? GetFontName() : textElement.FontName;
            // }
            wTextRange.CharacterFormat.FontName = GetFontName();
            wTextRange.CharacterFormat.FontSize = (float)textElement.FontSize;
            SetFontStyles(wTextRange, textElement);
            wTextRange.CharacterFormat.TextColor = forceInvertColor ?? textElement.GetReportTemplateDocument().InvertColor
                ? Syncfusion.Drawing.Color.FromArgb(255 - textElement.FontColor.R, 255 - textElement.FontColor.G,
                    255 - textElement.FontColor.B)
                : Syncfusion.Drawing.Color.FromArgb(textElement.FontColor.R, textElement.FontColor.G, textElement.FontColor.B);
        }

        private static bool ContainsChinese(string str)
        {
            // 匹配中文字符的正则表达式
            Regex regex = new Regex("[\u4e00-\u9fa5]");
            // 使用正则表达式判断字符串中是否包含中文字符
            return regex.IsMatch(str);
        }

        private static void SetTextElementValue(ITextElement textElement, IWTextRange wtext, string textElementValue)
        {
            wtext.Text = textElementValue;
            var length = 0;
            if (textElement.LineWidth > 0)
            {
                length = (int)Math.Round(textElement.LineWidth / 5.2, 0, MidpointRounding.AwayFromZero);
            }
            else
            {
                length = textElement.LineLength;
            }

            if (length > textElementValue.Length)
            {
                var extendString = string.Empty;
                for (int i = 0; i < length - wtext.Text.Length; i++)
                {
                    extendString += " ";
                }
                extendString += " ";
                if (wtext.Owner is WParagraph wparagraph)
                {
                    var extendText = wparagraph.AppendText(extendString);
                    extendText.CharacterFormat.FontName = GetFontName();
                    extendText.CharacterFormat.FontSize = 10.5f;
                }
            }
        }

        private static void SetFontStyles(IWTextRange wTextRange, ITextElement textElement)
        {
            foreach (var fontStyle in textElement.FontStyles)
            {
                switch (fontStyle)
                {
                    case RTFontStyle.Bold:
                        wTextRange.CharacterFormat.Bold = true;
                        break;
                    case RTFontStyle.Italic:
                        wTextRange.CharacterFormat.Italic = true;
                        break;
                    case RTFontStyle.Underline:
                        wTextRange.CharacterFormat.UnderlineStyle = UnderlineStyle.Words;
                        break;
                    case RTFontStyle.Strikeout:
                        wTextRange.CharacterFormat.Strikeout = true;
                        break;
                }
            }
        }
    }
}