﻿using System.Collections.Generic;
using System.IO;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;
using PDFiumSharp;
using Syncfusion.DocIORenderer;
using Vinno.IUS.Common.IO;
using SkiaSharp;
using System;
using Syncfusion.Pdf.Graphics;

namespace ReportService
{
    public class ReportPreviewImagesGenerater : IReportPreviewImagesGenerater
    {
        /// <summary>
        /// 生成图片
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <returns></returns>
        public IEnumerable<ImageInfo> GenerateReportPreviewImages(ReportInfo reportInfo)
        {
            var imageInfos = new List<ImageInfo>();
            using (var wordDocument = ReportWordDocumentManager.GetGenerateWordDocument(reportInfo))
            {
                if (wordDocument != null)
                {
                        DocIORenderer renderer = new DocIORenderer();
                        using (var pdfDocument = renderer.ConvertToPDF((ReportWordDocument)wordDocument))
                        {
                            using (var pdfms = new MemoryStream())
                            {
                                pdfDocument.Save(pdfms);
                                using (var fiumPdf = new PdfDocument(pdfms.GetBuffer()))
                                {
                                    var firstPages = fiumPdf.Pages;
                                    foreach (var firstPage in firstPages)
                                    {
                                        var width = (int)(firstPage.Size.Width * 4);
                                        var height = (int)(firstPage.Size.Height * 4);
                                        using (var pageBitmap = new PDFiumBitmap(width, height, true))
                                        {
                                            firstPage.Render(pageBitmap, (0, 0, width, height));
                                            using (var bytes = pageBitmap.AsBmpStream())
                                            {
                                                if (bytes != null && bytes.Length > 0)
                                                {
                                                    using (SKBitmap sourceBitmap = SKBitmap.Decode(bytes))
                                                    {
                                                        // 创建目标 SKBitmap 对象
                                                        using (SKBitmap targetBitmap = new SKBitmap(width, height, SKColorType.Rgba8888, SKAlphaType.Premul))
                                                        {
                                                            // 创建 SKCanvas 对象
                                                            using (SKCanvas canvas = new SKCanvas(targetBitmap))
                                                            {
                                                                // 绘制白色矩形
                                                                canvas.Clear(SKColors.White);
                                                                // 将源 SKBitmap 对象绘制到目标 SKBitmap 对象上
                                                                canvas.DrawBitmap(sourceBitmap, SKRect.Create(width, height));
                                                            }
                                                            using (MemoryStream stream = new MemoryStream())
                                                            {
                                                                targetBitmap.Encode(stream, SKEncodedImageFormat.Jpeg, 100);
                                                                var imageInfo = new ImageInfo();
                                                                imageInfo.ImageBuffer = new ByteBuffer(stream.GetBuffer());
                                                                imageInfos.Add(imageInfo);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    
                }
            }
            return imageInfos;
        }

        /// <summary>
        /// 转换为pdf
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <param name="pdfSavePath"></param>
        public byte[] ConvertReportToPdf(ReportInfo reportInfo)
        {
            using (var wordDocument = ReportWordDocumentManager.GetGenerateWordDocument(reportInfo))
            {
                if (wordDocument != null)
                {
                    try
                    {
                        DocIORenderer renderer = new DocIORenderer();
                        using (var pdfDocument = renderer.ConvertToPDF((ReportWordDocument)wordDocument))
                        {
                             using (var pdfms = new MemoryStream())
                            {
                                pdfDocument.Save(pdfms);
                                return pdfms.GetBuffer();
                            }
                        }
                    }
                    catch(Exception ex)
                    {
                        Console.WriteLine($"ConvertReportToPdf Error: {ex}");
                    }
                    finally
                    {
                        ((ReportWordDocument)wordDocument).Close();
                    }
                }
                return null;
            }
        }

        /// <summary>
        /// 转换为image
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <param name="pdfSavePath"></param>
        public IEnumerable<ImageInfo> ConvertPDFReportToImage(string pdfPath)
        {
            List<ImageInfo> imageInfos = new List< ImageInfo >();
            using (var fiumPdf = new PdfDocument(pdfPath))
            {
                var firstPages = fiumPdf.Pages;
                foreach (var firstPage in firstPages)
                {
                    var width = (int)(firstPage.Size.Width * 4);
                    var height = (int)(firstPage.Size.Height * 4);
                    using (var pageBitmap = new PDFiumBitmap(width, height, true))
                    {
                        firstPage.Render(pageBitmap, (0, 0, width, height));
                        using (var bytes = pageBitmap.AsBmpStream())
                        {
                            if (bytes != null && bytes.Length > 0)
                            {
                                using (SKBitmap sourceBitmap = SKBitmap.Decode(bytes))
                                {
                                    // 创建目标 SKBitmap 对象
                                    using (SKBitmap targetBitmap = new SKBitmap(width, height, SKColorType.Rgba8888, SKAlphaType.Premul))
                                    {
                                        // 创建 SKCanvas 对象
                                        using (SKCanvas canvas = new SKCanvas(targetBitmap))
                                        {
                                            // 绘制白色矩形
                                            canvas.Clear(SKColors.White);
                                            // 将源 SKBitmap 对象绘制到目标 SKBitmap 对象上
                                            canvas.DrawBitmap(sourceBitmap, SKRect.Create(width, height));
                                        }
                                        using (MemoryStream stream = new MemoryStream())
                                        {
                                            targetBitmap.Encode(stream, SKEncodedImageFormat.Jpeg, 100);
                                            var imageInfo = new ImageInfo();
                                            imageInfo.ImageBuffer = new ByteBuffer(stream.GetBuffer());
                                            imageInfos.Add(imageInfo);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return imageInfos;
        }
    }
    

}