using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.DTO.Report;

namespace ReportService.Posters
{
    /// <summary>
    /// 报告内容反序列化
    /// </summary>
    public class ReportElementDTOConverter : JsonCreationConverter<ReportElementDTO>
    {
        protected override ReportElementDTO Create(Type objectType, JObject jObject)
        {
            if (FieldExists("Value", jObject, out object value))
            {
                if (value is JObject && jObject["TagId"] == null)
                {
                    return new ReportElementStringDTO();
                }
                else if (value is JArray)
                {
                    return new ReportElementArrayDTO();
                }
            }
            return new ReportElementStringDTO();
        }

        private bool FieldExists(string fieldName, JObject jObject, out object value)
        {
            value = jObject[fieldName];
            return jObject[fieldName] != null;
        }
    }

    public abstract class JsonCreationConverter<T> : JsonConverter
    {
        /// <summary>
        /// Create an instance of objectType, based properties in the JSON object
        /// </summary>
        /// <param name="objectType">type of object expected</param>
        /// <param name="jObject">
        /// contents of JSON object that will be deserialized
        /// </param>
        /// <returns></returns>
        protected abstract T Create(Type objectType, JObject jObject);

        public override bool CanConvert(Type objectType)
        {
            return typeof(T).IsAssignableFrom(objectType);
        }

        public override bool CanWrite
        {
            get { return false; }
        }

        public override object ReadJson(JsonReader reader,
                                        Type objectType,
                                         object existingValue,
                                         JsonSerializer serializer)
        {
            // Load JObject from stream
            JObject jObject = JObject.Load(reader);

            // Create target object based on JObject
            T target = Create(objectType, jObject);

            // Populate the object properties
            serializer.Populate(jObject.CreateReader(), target);

            return target;
        }
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            throw new NotImplementedException();
        }
    }
}