using System.Text;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using System.Net.Http.Headers;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using Newtonsoft.Json;
using System.Collections.Concurrent;
using System.Dynamic;
using System.Text.RegularExpressions;
using System.Runtime.Serialization.Json;
using System.Net;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Record;
using WingServerCommon.Interfaces.FileTransfer;
using WingInterfaceLibrary.Request.Storage;
using WingInterfaceLibrary.Interface;

namespace ReportService.Posters
{
    /// <summary>
    /// TODO do we need to post 
    /// </summary>

    public class ReportPoster : CacheReportPostersDTO, ICacheObject
    {
        private readonly string _defaultValueKey = "DefaultValue:";
        private readonly HttpClient _httpClient = new HttpClient();
        private FileTransferService _fileTransferService = new FileTransferService();

        /// <summary>
        /// The report poster unique id
        /// </summary>
        /// <value></value>
        public string ReportPosterCode { get; set; }

        private IReadOnlyList<TemplateItem> TemplateItems { get; set; } = new List<TemplateItem>();
        public string AIReportLanguage
        {
            get
            {
                var aiReportLanguage = "zh-CN";
                var languageItem = TemplateItems.FirstOrDefault(x => x.TemplateField.ToLower() == "aireportlanguage");
                if (languageItem != null)
                {
                    aiReportLanguage = languageItem.VinnoReportField.Replace(_defaultValueKey, "");
                }
                return aiReportLanguage;
            }
        }
        private bool InitialLowercase => !TemplateItems.Any(x => x.TemplateField == "InitialLowercase") || TemplateItems.Any(x => x.TemplateField == "InitialLowercase" && x.VinnoReportField == $"{_defaultValueKey}1");

        public ReportPoster(ReportPosterDTO poster)
        {
            Code = poster.ReportPosterCode;
            ReportPosterCode = poster.ReportPosterCode;
            ReceiverName = poster.ReceiverName;
            TargetUrl = poster.TargetUrl;
            Contact = poster.Contact;
            Phone = poster.Phone;
            PosterType = (int)poster.PosterType;
            ReportFormat = (int)poster.ReportFormat;
            PostTimeout = poster.PostTimeout;
            Template = poster.Template;
            DeviceIds = poster.DeviceIds;
            _httpClient.Timeout = TimeSpan.FromSeconds(poster.PostTimeout);
            TemplateItems = LoadConfigFile(poster.Template);
        }

        public Tuple<bool, string, string> Post(IStorageService _storageService, ReportDTO reportInfo, RecordInfoDTO recordInfo)
        {
            var reportId = reportInfo.ReportCode;
            string postContent = string.Empty;
            Tuple<bool, string> postResult = new Tuple<bool, string>(false, "");
            try
            {
                Logger.WriteLineInfo($"{ReceiverName} Begin to post report {reportId} with patientId {reportInfo.PatientId}({CryptionHelper.Decrypt(reportInfo.PatientName)}) to {TargetUrl}.");
                if (ReportFormat == (int)ReportFormatEnum.Json)
                {
                    postContent = GetJsonContent(_storageService, reportInfo, recordInfo);
                    postResult = DoPost(reportId, postContent, "application/json");
                }
                else if (ReportFormat == (int)ReportFormatEnum.Xml)
                {
                    postContent = GetXMLContent(_storageService, reportInfo, recordInfo);
                    postResult = DoPost(reportId, postContent, "application/xml");
                }
                else if (ReportFormat == (int)ReportFormatEnum.Text)
                {
                    postContent = GetXMLContent(_storageService, reportInfo, recordInfo);
                    postResult = DoPost(reportId, postContent, "text/xml");
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{ReceiverName} post report err, reportId: {reportId}, {ex}");
                postResult = new Tuple<bool, string>(false, ex.Message);
            }

            return new Tuple<bool, string, string>(postResult.Item1, postContent, postResult.Item2);
        }

        private Tuple<bool, string> DoPost(string reportId, string postContent, string mediaType)
        {
            try
            {
                HttpContent content = new ByteArrayContent(Encoding.UTF8.GetBytes(postContent));
                content.Headers.ContentType = new MediaTypeHeaderValue(mediaType);

                var postResult = _httpClient.PostAsync(TargetUrl, content);
                postResult.Wait();
                var response = postResult.Result;
                response.EnsureSuccessStatusCode();
                var resultContent = response.Content.ReadAsStringAsync();
                resultContent.Wait();
                var result = resultContent.Result;

                var args = new PostReportResultArgs();
                if (result == "\"success\"" || result == "success") //TODO remove this if we dont support old customer in future
                {
                    return new Tuple<bool, string>(true, result);
                }
                else
                {
                    using (var ms = new MemoryStream(Encoding.UTF8.GetBytes(result)))
                    {
                        try
                        {
                            var posterResult = (PosterResult)new DataContractJsonSerializer(typeof(PosterResult)).ReadObject(ms);
                            if (posterResult.success || posterResult.code == "0")
                            {
                                return new Tuple<bool, string>(true, result);
                            }
                        }
                        catch (Exception exception)
                        {
                            Logger.WriteLineWarn($"{ReceiverName} Convert result about poster report: {reportId} failed, PostResult:{resultContent.Result}, Exception:{exception}");
                        }
                    }
                    return new Tuple<bool, string>(false, result);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{ReceiverName} Post report {reportId} ex:{ex}");
                return new Tuple<bool, string>(false, ex.Message);
            }
        }

        private string GetJsonContent(IStorageService _storageService, ReportDTO reportInfo, RecordInfoDTO recordInfo)
        {
            var postDatas = GetPostDatas(_storageService, reportInfo, recordInfo);
            IDictionary<string, object> expandoObjects = new ExpandoObject();
            foreach (var d in postDatas)
            {
                if (d.Key == "images" && d.Value is List<string> images)
                {
                    var base64Images = new List<string>();
                    foreach (var img in images)
                    {
                        base64Images.Add(ToBase64Images(img));
                    }

                    expandoObjects.Add(d.Key, base64Images);
                }
                else
                {
                    expandoObjects.Add(d);
                }
            }
            var settings = new JsonSerializerSettings
            {
                DateTimeZoneHandling = DateTimeZoneHandling.Utc,
                DateFormatString = "yyyy-MM-ddTHH:mm:ssZ"
            };
            return JsonConvert.SerializeObject(expandoObjects as ExpandoObject, settings);
        }

        private string DictionaryToModelString(ConcurrentDictionary<string, object> dic)
        {
            IDictionary<string, object> expandoObjects = new ExpandoObject();
            foreach (var d in dic)
            {
                expandoObjects.Add(d);
            }
            var settings = new JsonSerializerSettings
            {
                DateTimeZoneHandling = DateTimeZoneHandling.Utc,
                DateFormatString = "yyyy-MM-ddTHH:mm:ssZ"
            };
            return JsonConvert.SerializeObject(expandoObjects as ExpandoObject);
        }

        protected virtual string GetXMLContent(IStorageService _storageService, ReportDTO reportInfo, RecordInfoDTO recordInfo)
        {
            var postDatas = GetPostDatas(_storageService, reportInfo, recordInfo);
            var xmlReportBuilder = new StringBuilder();
            xmlReportBuilder.Append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            xmlReportBuilder.Append("<ReportInfo>");
            foreach (var key in postDatas.Keys)
            {
                var ele = postDatas[key];
                if (ele is String value)
                {
                    Regex reg = new Regex("[<>&]");
                    if (reg.Match(value).Success)
                    {
                        value = FormatStringValue(value);
                    }
                    xmlReportBuilder.AppendFormat("<{0}>{1}</{0}>", key, value);
                }
                else if (key == "ReportImages" && ele is List<string> images && images?.Any() == true)
                {
                    xmlReportBuilder.Append("<Images>");
                    foreach (var imgUrl in images)
                    {
                        var imageVal = $"<Image Path=\"{imgUrl}\" Type =\"4\" >{ToBase64Images(imgUrl)}</Image>";
                        xmlReportBuilder.Append(imageVal);
                    }
                    xmlReportBuilder.Append("</Images>");
                }
            }
            xmlReportBuilder.Append("</ReportInfo>");
            return xmlReportBuilder.ToString();
        }

        protected ConcurrentDictionary<string, object> GetPostDatas(IStorageService _storageService, ReportDTO reportInfo, RecordInfoDTO recordInfo)
        {
            var postDatas = new ConcurrentDictionary<string, object>();
            var reportDictionary = ReportInfoToDictionary(_storageService, reportInfo, recordInfo);
            var excludeFields = new List<string>
                {
                    "PostJson",
                    "TimeFormatter",
                    "FormatterAge",
                    "AIReportLanguage",
                    "AddHours"
                };
            foreach (var item in TemplateItems)
            {
                if (excludeFields.Contains(item.TemplateField))
                {
                    continue;
                }
                else if (item.VinnoReportField.Contains(_defaultValueKey))
                {
                    var realKey = item.TemplateField;
                    if (InitialLowercase)
                    {
                        realKey = Regex.Replace(realKey, @"^\S", match => match.Value.ToLower());
                    }
                    var realValue = item.VinnoReportField.Replace(_defaultValueKey, "");
                    postDatas.AddOrUpdate(realKey, realValue, (k, v) => { return realValue; });
                }
                else if (reportDictionary.Keys.Contains(item.VinnoReportField))
                {
                    var realKey = item.TemplateField;
                    if (InitialLowercase)
                    {
                        realKey = Regex.Replace(realKey, @"^\S", match => match.Value.ToLower());
                    }
                    var ele = reportDictionary[item.VinnoReportField];
                    postDatas.AddOrUpdate(realKey, ele, (k, v) => { return ele; });
                }
                else
                {
                    var realKey = item.TemplateField;
                    if (InitialLowercase)
                    {
                        realKey = Regex.Replace(realKey, @"^\S", match => match.Value.ToLower());
                    }
                    postDatas.AddOrUpdate(realKey, "", (k, v) => { return ""; });
                }
            }
            return postDatas;
        }

        private ConcurrentDictionary<string, object> ReportInfoToDictionary(IStorageService _storageService, ReportDTO reportInfo, RecordInfoDTO recordInfo)
        {
            var reportDatas = new ConcurrentDictionary<string, object>();
            if (!string.IsNullOrWhiteSpace(reportInfo.ReportDatasJson))
            {
                var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportInfo.ReportDatasJson, new ReportElementDTOConverter());
                foreach (var r in reportElements)
                {
                    if (r is ReportElementStringDTO es)
                    {
                        reportDatas.AddOrUpdate(es.Name, es.Value, (k, v) => { return es.Value; });
                    }
                    if (r is ReportElementArrayDTO ea)
                    {
                        reportDatas.AddOrUpdate(ea.Name, ea.Value, (k, v) => { return ea.Value; });
                    }
                }
                var reportDatasJson = DictionaryToModelString(reportDatas);
                reportDatas.AddOrUpdate("ReportDatas", reportDatasJson, (k, v) => { return reportDatasJson; });
            }
            var _formatterAge = false;
            var formatterageItem = TemplateItems.FirstOrDefault(x => x.TemplateField.ToLower() == "formatterage");
            if (formatterageItem != null)
            {
                if (!string.IsNullOrEmpty(formatterageItem.VinnoReportField) && formatterageItem.VinnoReportField.ToLower() == "defaultvalue:1")
                {
                    _formatterAge = true;
                }
            }
            if (_formatterAge)
            {
                if (reportDatas.Keys.Contains("PatientAge"))
                {
                    var value = reportDatas["PatientAge"]?.ToString() ?? string.Empty;
                    var newAge = TransferAgeString(value);
                    reportDatas.AddOrUpdate("PatientAge", newAge, (k, v) => { return newAge; });
                }
            }

            var timeFormatterStr = "yyyy-MM-ddTHH:mm:ss";
            var formatterageTimeItem = TemplateItems.FirstOrDefault(x => x.TemplateField.ToLower() == "timeformatter");
            if (formatterageTimeItem != null)
            {
                if (!string.IsNullOrEmpty(formatterageTimeItem.VinnoReportField) && formatterageTimeItem.VinnoReportField.ToLower().Contains("defaultvalue"))
                {
                    var timeFormatList = formatterageTimeItem.VinnoReportField.Split(':');
                    var newList = Array.FindAll(timeFormatList, i => i.ToLower() != "defaultvalue");
                    timeFormatterStr = String.Join(":", newList);
                }
            }
            var addHours = 0;
            var addHoursItem = TemplateItems.FirstOrDefault(x => x.TemplateField.ToLower() == "addhours");
            if (addHoursItem != null)
            {
                if (!string.IsNullOrEmpty(addHoursItem.VinnoReportField) && addHoursItem.VinnoReportField.ToLower().Contains("defaultvalue"))
                {
                    var addHourList = addHoursItem.VinnoReportField.Split(':');
                    if (addHourList != null && addHourList.Length == 2 && !string.IsNullOrWhiteSpace(addHourList[1]))
                    {
                        if (int.TryParse(addHourList[1], out int intHours) && intHours != 0)
                        {
                            addHours = intHours;
                        }
                    }
                }
            }
            if (!reportDatas.Keys.Contains("DeviceExamId"))
            {
                reportDatas.AddOrUpdate("DeviceExamId", recordInfo.DeviceExamId, (k, v) => { return recordInfo.DeviceExamId; });
            }
            if (!reportDatas.Keys.Contains("RecordCode"))
            {
                reportDatas.AddOrUpdate("RecordCode", reportInfo.RecordCode, (k, v) => { return reportInfo.RecordCode; });
            }
            if (!reportDatas.Keys.Contains("PatientId"))
            {
                reportDatas.AddOrUpdate("PatientId", reportInfo.PatientId, (k, v) => { return reportInfo.PatientId; });
            }
            if (!reportDatas.Keys.Contains("PatientName"))
            {
                reportDatas.AddOrUpdate("PatientName", CryptionHelper.Decrypt(reportInfo.PatientName), (k, v) => { return CryptionHelper.Decrypt(reportInfo.PatientName); });
            }
            if (!reportDatas.Keys.Contains("ReportCode"))
            {
                reportDatas.AddOrUpdate("ReportCode", reportInfo.ReportCode, (k, v) => { return reportInfo.ReportCode; });
            }
            if (!reportDatas.Keys.Contains("DeviceCode"))
            {
                reportDatas.AddOrUpdate("DeviceCode", reportInfo.DeviceCode, (k, v) => { return reportInfo.DeviceCode; });
            }
            if (!reportDatas.Keys.Contains("DeviceName"))
            {
                reportDatas.AddOrUpdate("DeviceName", reportInfo.DeviceName, (k, v) => { return reportInfo.DeviceName; });
            }
            if (!reportDatas.Keys.Contains("ExamDate"))
            {
                reportDatas.AddOrUpdate("ExamDate", reportInfo.ExamDate.ToString(timeFormatterStr), (k, v) => { return reportInfo.ExamDate.ToString(timeFormatterStr); });
            }
            if (!reportDatas.Keys.Contains("ReportTemplateName"))
            {
                reportDatas.AddOrUpdate("ReportTemplateName", reportInfo.ReportTemplateName, (k, v) => { return reportInfo.ReportTemplateName; });
            }
            if (!reportDatas.Keys.Contains("Gender"))
            {
                var gender = string.Empty;
                if (reportDatas.Keys.Contains("PatientGender"))
                {
                    gender = reportDatas["PatientGender"]?.ToString() ?? string.Empty;
                }
                if (gender.Contains("男") || gender.Contains("雄"))
                {
                    gender = "Male";
                }
                else if (gender.Contains("女") || gender.Contains("雌"))
                {
                    gender = "Female";
                }
                else
                {
                    gender = "NotFilled";
                }
                reportDatas.AddOrUpdate("Gender", gender, (k, v) => { return gender; });
            }
            if (!reportDatas.Keys.Contains("AgeInt"))
            {
                var age = string.Empty;
                if (reportDatas.Keys.Contains("PatientAge"))
                {
                    age = reportDatas["PatientAge"]?.ToString() ?? string.Empty;
                }
                reportDatas.AddOrUpdate("AgeInt", TransferAgeString(age), (k, v) => { return TransferAgeString(age); });
            }
            if (!reportDatas.Keys.Contains("ReportPhysicians"))
            {
                var reportPhysicians = string.Empty;
                if (reportDatas.Keys.Contains("ReportPhysician"))
                {
                    if (reportDatas["ReportPhysician"] is List<string> values)
                    {
                        reportPhysicians = string.Join(";", values?.ToList() ?? new List<string>());
                    }
                    else
                    {
                        reportPhysicians = reportDatas["ReportPhysician"]?.ToString() ?? string.Empty;
                    }
                }
                reportDatas.AddOrUpdate("ReportPhysicians", reportPhysicians, (k, v) => { return reportPhysicians; });
            }
            if (reportDatas.Keys.Contains("DigitalSignature"))
            {
                var digitalSignatureUrl = reportDatas["DigitalSignature"]?.ToString();
                if (!string.IsNullOrWhiteSpace(digitalSignatureUrl))
                {
                    var digitalSignatureBase64 = ToBase64Images(digitalSignatureUrl);
                    reportDatas.AddOrUpdate("ReviewerSign", digitalSignatureBase64, (k, v) => { return digitalSignatureBase64; });
                }
            }
            if (!reportDatas.Keys.Contains("PatientInspectionSite"))
            {
                var sites = new List<string>();
                if (reportDatas.Keys.Contains("PatientInspectionSiteSingle"))
                {
                    var site = reportDatas["PatientInspectionSiteSingle"]?.ToString();
                    if (!string.IsNullOrWhiteSpace(site))
                    {
                        sites.Add(site);
                    }
                }
                reportDatas.AddOrUpdate("PatientInspectionSite", sites, (k, v) => { return sites; });
            }
            if (!reportDatas.Keys.Contains("ReportPreviewUrls"))
            {
                var reportUrls = new List<string>();
                string reportBase64 = string.Empty;
                string reportUrl = string.Empty;
                try
                {
                    var files = ReportPreviewHelper.GetReportPreviewPath(reportInfo, UploadFileTypeEnum.JPG);
                    if (files?.Any() ?? false)
                    {
                        foreach (var filePath in files)
                        {
                            var urls = _fileTransferService.FileTransferAsync(filePath).Result;
                            if (urls?.Any() ?? false)
                            {
                                reportUrls.Add(urls.FirstOrDefault());
                            }
                        }
                        if (files.Count == 1)
                        {
                            reportBase64 = LocalFileToBase64Images(files.FirstOrDefault());
                            reportUrl = reportUrls.FirstOrDefault() ?? string.Empty;
                        }
                        else
                        {
                            var reportFilePath = ReportPreviewHelper.GetReportPreview(files, reportInfo.RecordCode);
                            if (!string.IsNullOrWhiteSpace(reportFilePath))
                            {
                                reportBase64 = LocalFileToBase64Images(reportFilePath);
                            }
                            var urls = _fileTransferService.FileTransferAsync(reportFilePath).Result;
                            if (urls?.Any() ?? false)
                            {
                                reportUrl = urls.FirstOrDefault();
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                }
                reportDatas.AddOrUpdate("ReportPreviewUrls", string.Join(";", reportUrls), (k, v) => { return string.Join(";", reportUrls); });
                reportDatas.AddOrUpdate("ReportPreviewString", reportBase64, (k, v) => { return reportBase64; });
                reportDatas.AddOrUpdate("ReportPreviewUrl", reportUrl, (k, v) => { return reportUrl; });
            }
            //检查信息
            reportDatas.AddOrUpdate("EquipmentSN", recordInfo.EquipmentSN ?? string.Empty, (k, v) => { return recordInfo.EquipmentSN ?? string.Empty; });
            reportDatas.AddOrUpdate("RecordTime", recordInfo.CreateTime.ToString(timeFormatterStr), (k, v) => { return recordInfo.CreateTime.ToString(timeFormatterStr); });
            reportDatas.AddOrUpdate("ExamDate", recordInfo.CreateTime.ToString(timeFormatterStr), (k, v) => { return recordInfo.CreateTime.ToString(timeFormatterStr); });

            reportDatas.AddOrUpdate("CreateTime", reportInfo.CreateTime.ToString(timeFormatterStr), (k, v) => { return reportInfo.CreateTime.ToString(timeFormatterStr); });
            reportDatas.AddOrUpdate("UpdateTime", reportInfo.UpdateTime.ToString(timeFormatterStr), (k, v) => { return reportInfo.UpdateTime.ToString(timeFormatterStr); });
            reportDatas.AddOrUpdate("ReportTime", reportInfo.UpdateTime.ToString(timeFormatterStr), (k, v) => { return reportInfo.UpdateTime.ToString(timeFormatterStr); });

            reportDatas.AddOrUpdate("RecordTime2", recordInfo.CreateTime.AddHours(addHours).ToString(timeFormatterStr), (k, v) => { return recordInfo.CreateTime.AddHours(addHours).ToString(timeFormatterStr); });
            reportDatas.AddOrUpdate("ReportTime2", reportInfo.UpdateTime.AddHours(addHours).ToString(timeFormatterStr), (k, v) => { return reportInfo.UpdateTime.AddHours(addHours).ToString(timeFormatterStr); });

            // 兼容老版本关键字
            if (!reportDatas.Keys.Contains("ExamId") && reportDatas.Keys.Contains("RecordCode"))
            {
                var examId = reportDatas["RecordCode"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("ExamId", examId, (k, v) => { return examId; });
            }
            if (!reportDatas.Keys.Contains("TemplateName") && reportDatas.Keys.Contains("ReportTemplateName"))
            {
                var templateName = reportDatas["ReportTemplateName"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("TemplateName", templateName, (k, v) => { return templateName; });
            }
            if (!reportDatas.Keys.Contains("UserName") && reportDatas.Keys.Contains("PatientName"))
            {
                var userName = reportDatas["PatientName"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("UserName", userName, (k, v) => { return userName; });
            }
            if (!reportDatas.Keys.Contains("Age") && reportDatas.Keys.Contains("PatientAge"))
            {
                var age = reportDatas["PatientAge"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("Age", age, (k, v) => { return age; });
            }
            if (!reportDatas.Keys.Contains("Description") && reportDatas.Keys.Contains("ReportDescription"))
            {
                var description = reportDatas["ReportDescription"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("Description", description, (k, v) => { return description; });
            }
            if (!reportDatas.Keys.Contains("Summary") && reportDatas.Keys.Contains("ReportSummary"))
            {
                var summary = reportDatas["ReportSummary"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("Summary", summary, (k, v) => { return summary; });
            }
            if (!reportDatas.Keys.Contains("Images") && reportDatas.Keys.Contains("ReportImages"))
            {
                var reportImages = reportDatas["ReportImages"];
                reportDatas.AddOrUpdate("Images", reportImages, (k, v) => { return reportImages; });
            }
            if (!reportDatas.Keys.Contains("Scanner") && reportDatas.Keys.Contains("InspectionPhysician"))
            {
                var inspectionPhysician = reportDatas["InspectionPhysician"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("Scanner", inspectionPhysician, (k, v) => { return inspectionPhysician; });
            }
            if (!reportDatas.Keys.Contains("DescriptionDic") && reportDatas.Keys.Contains("ReportDatas"))
            {
                var descriptionDic = reportDatas["ReportDatas"];
                reportDatas.AddOrUpdate("DescriptionDic", descriptionDic, (k, v) => { return descriptionDic; });
            }
            if (!reportDatas.Keys.Contains("PartName") && reportDatas.Keys.Contains("PatientInspectionSite"))
            {
                var partName = reportDatas["PatientInspectionSite"];
                reportDatas.AddOrUpdate("PartName", partName, (k, v) => { return partName; });
            }
            if (!reportDatas.Keys.Contains("OrganName") && reportDatas.Keys.Contains("HospitalName"))
            {
                var organName = reportDatas["HospitalName"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("OrganName", organName, (k, v) => { return organName; });
            }
            if (!reportDatas.Keys.Contains("Reviewer") && reportDatas.Keys.Contains("ReportPhysicians"))
            {
                var reviewer = reportDatas["ReportPhysicians"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("Reviewer", reviewer, (k, v) => { return reviewer; });
            }
            if (!reportDatas.Keys.Contains("ReportPreview") && reportDatas.Keys.Contains("ReportPreviewString"))
            {
                var reportPreview = reportDatas["ReportPreviewString"]?.ToString() ?? string.Empty;
                reportDatas.AddOrUpdate("ReportPreview", reportPreview, (k, v) => { return reportPreview; });
            }

            return reportDatas;
        }

        /// <summary>
        /// 调用storage服务上传文件
        /// </summary>
        /// <returns></returns>
        private List<string> GetFileTransferByStorageAsync(IStorageService _storageService, string sourceFilePath, bool isRechristen = false, bool isNeedPartUpload = false, bool isCopy = false)
        {
            var request = new UploadFileTransferRequest()
            {
                SourceFilePath = sourceFilePath,
                IsRechristen = isRechristen,
                IsNeedPartUpload = isNeedPartUpload,
                IsCopy = isCopy,
            };
            var resList = _storageService.UploadFileByServerConfigAsync(request).Result;
            return resList;
        }

        /// <summary>
        /// filter age string
        /// </summary>
        /// <param name="ageStr">age string</param>        
        /// <returns>36</returns>
        private string TransferAgeString(string ageStr)
        {
            if (string.IsNullOrWhiteSpace(ageStr))
            {
                return "";
            }
            ageStr = ageStr.Replace(" ", "").Replace("　", "");
            int i = 0;
            for (i = 0; i < ageStr.Length; i++)
            {
                if (!Char.IsNumber(ageStr, i))
                {
                    break;
                }
            }
            var newAge = ageStr.Substring(0, i);
            var value = Regex.Replace(newAge, @"[^\d]", "");
            return value;
        }

        /// <summary>
        /// load config xml file
        /// </summary>
        /// <param name="configContent">config Content</param>        
        /// <returns></returns>
        private static IReadOnlyList<TemplateItem> LoadConfigFile(string configContent)
        {
            var list = new List<TemplateItem>();
            if (string.IsNullOrEmpty(configContent))
            {
                return list;
            }

            var configLoader = new ConfigLoader();
            var sections = configLoader.LoadConfig(configContent);
            if (sections.Count == 1)
            {
                foreach (var item in sections[0].Items)
                {
                    list.Add(new TemplateItem(item.Key, item.Value));
                }
            }

            return list;
        }

        protected string FormatStringValue(string content)
        {
            if (!string.IsNullOrEmpty(content))
            {
                return $"<![CDATA[{content}]]>";
            }
            return string.Empty;
        }

        protected string ToBase64Images(string imageUrl)
        {
            if (!string.IsNullOrWhiteSpace(imageUrl) && imageUrl.Contains("http"))
            {
                if (!imageUrl.StartsWith("http"))
                {
                    imageUrl = imageUrl.Substring(imageUrl.IndexOf("http"));
                }
                var bytes = DownloadAsync(imageUrl).Result;
                string result = Convert.ToBase64String(bytes);
                return result;
            }
            return string.Empty;
        }

        protected string LocalFileToBase64Images(string localFile)
        {
            try
            {
                if (File.Exists(localFile))
                {
                    var bytes = File.ReadAllBytes(localFile);
                    return Convert.ToBase64String(bytes);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportService ReportPoster LocalFileToBase64Images error, ex:{ex}");
            }
            return string.Empty;
        }

        /// <summary>下载文件</summary>
        /// <param name="fileUrl"></param>
        /// <returns></returns>
        private async Task<byte[]> DownloadAsync(string fileUrl)
        {
            try
            {
                if (string.IsNullOrEmpty(fileUrl))
                {
                    return new byte[0];
                }
                long fileSize = 0;
                using (var request = new HttpRequestMessage())
                {
                    request.RequestUri = new Uri(fileUrl);
                    request.Method = HttpMethod.Get;
                    var response = await _httpClient.SendAsync(request);

                    if (response != null && response.StatusCode == HttpStatusCode.OK)
                    {
                        var contentLength = response.Content.Headers.ContentLength;
                        fileSize = contentLength == null ? 0 : contentLength.Value;
                    }
                }
                if (fileSize <= 0)
                {
                    throw new NotSupportedException($"fileSize is {fileSize}");
                }
                byte[] bytes = await _httpClient.GetByteArrayAsync(fileUrl);
                return bytes;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportPoster download file err, url: {fileUrl}, {ex}");
            }
            finally
            {
                //Logger.WriteLineInfo($"download file:{fileUrl}");
            }
            return new byte[0];
        }
    }

    public class PostReportResultArgs : EventArgs
    {

    }
}