using System.Text;
using ReportService.Posters;
using ReportService.ReportConverter;
using WingInterfaceLibrary.DTO.Report;

namespace ReportService
{
    public class ReportJsonSerializer
    {
        public static string Serialize(string filePath, ReportDTO reportInfo)
        {
            var dic = new Dictionary<string,  ReportMeasureElementTag>();
            if (!string.IsNullOrEmpty(reportInfo.ReportMeasureDatasJson))
            {
                var reportDataInfo = new
                {
                    Id = reportInfo.ReportCode,
                    Template = Newtonsoft.Json.JsonConvert.DeserializeObject(reportInfo.ReportTemplateJson),
                    ElementValues = new List<dynamic>(),
                    MeasureElementValues = new List<dynamic>(),
                    PreviewImages = new List<dynamic>(),
                    CustomTagList = new List<dynamic>(),
                };
                var jsonData = Newtonsoft.Json.JsonConvert.SerializeObject(reportDataInfo);
                var reportObj = Vinno.vCloud.Report.Models.ReportJsonSerializer.Deserialize(jsonData);
                if (reportObj != null && reportObj.Template != null)
                {
                    var reportTemplate = reportObj.Template;
                    foreach (var blockElement in reportTemplate.Blocks)
                    {
                        if (blockElement is Vinno.vCloud.Report.Models.RTTable rtTable)
                        {
                            Generate(rtTable, dic);
                        }
                    }
                }
            }
            var reportJson = new
            {
                Id = reportInfo.ReportCode,
                Template = Newtonsoft.Json.JsonConvert.DeserializeObject(reportInfo.ReportTemplateJson),
                ElementValues = GetReportElements(reportInfo.ReportDatasJson),
                MeasureElementValues = GetReportMeasureElements(reportInfo.ReportMeasureDatasJson, dic),
                PreviewImages = new List<dynamic>(),
                CustomTagList = new List<dynamic>(),
            };
            var bytes = new UTF8Encoding().GetBytes(Newtonsoft.Json.JsonConvert.SerializeObject(reportJson));
            FileHelper.SaveFile(filePath, bytes);
            return filePath;
        }

        private static List<IReportElementValue> GetReportElements(string reportDatasJson)
        {
            var elements = new List<IReportElementValue>();
            if (!string.IsNullOrWhiteSpace(reportDatasJson))
            {
                var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportDatasJson, new ReportElementDTOConverter());
                foreach (var reportElement in reportElements)
                {
                    if (reportElement is ReportElementArrayDTO ea && ea.Value is List<string> images && ea.Name == "ReportImages")
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        elements.Add(new ReportElementValue
                        {
                            Value = images.Select(x => new ReportFileImageValue { Value = x }),
                            ValueType = "ImageList",
                        });
                    }
                    else if (reportElement is ReportElementStringDTO es1 && es1.Name == "ReportImages")
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        var value = es1?.Value?.ToString() ?? string.Empty;
                        List<ReportFileImageValue> tempValues = null;
                        if (!string.IsNullOrEmpty(value) && value.Split(';').Length > 1)
                        {
                            var imageList = value.Split(';').ToList();
                            tempValues = imageList.Select(x => new ReportFileImageValue { Value = x }).ToList();
                        }
                        else
                        {   
                            tempValues = new List<ReportFileImageValue> { new ReportFileImageValue { Value = value } };
                        }
                        elements.Add(new ReportElementValue
                        {
                            Value = tempValues,
                            ValueType = "ImageList",
                        });
                    }
                    else if (reportElement is ReportElementStringDTO es2 && es2.Name == "PatientECGImage")
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        var value = es2?.Value?.ToString() ?? string.Empty;
                        List<ReportFileImageValue> tempValues = null;
                        if (!string.IsNullOrEmpty(value) && value.Split(';').Length > 1)
                        {
                            var imageList = value.Split(';').ToList();
                            tempValues = imageList.Select(x => new ReportFileImageValue { Value = x }).ToList();
                        }
                        else
                        {   
                            tempValues = new List<ReportFileImageValue> { new ReportFileImageValue { Value = value } };
                        }
                        elements.Add(new ReportElementValue
                        {
                            Value = tempValues,
                            ValueType = "ImageList",
                        });
                    }
                    else if (reportElement is ReportElementArrayDTO ea3 && ea3.Value is List<string> images3 && ea3.Name == "PatientECGImage")
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        elements.Add(new ReportElementValue
                        {
                            Value = images3.Select(x => new ReportFileImageValue { Value = x }),
                            ValueType = "ImageList",
                        });
                    }
                    else if (reportElement is ReportElementStringDTO es3 && (es3.Name.EndsWith("Time") || es3.Name.EndsWith("Date")))
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        var value = es3?.Value?.ToString() ?? string.Empty;
                        var valueType = "String";
                        if (es3.Name.Equals("PatientCreatedDate"))
                        {
                            
                        }
                        else
                        {
                            if (DateTime.TryParse(value, out DateTime date))
                            {
                                valueType = "Datetime";
                                value = date.ToString("s");
                            }
                        }
                        elements.Add(new ReportElementValue
                        {
                            Value = value,
                            ValueType = valueType,
                        });
                    }
                    else if (reportElement is ReportElementStringDTO es)
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        var value = es?.Value?.ToString() ?? string.Empty;
                        var valueType = "String";
                        // if (DateTime.TryParse(value, out DateTime date))
                        // {
                        //     valueType = "Datetime";
                        //     value = date.ToString("s");
                        // }
                        elements.Add(new ReportElementValue
                        {
                            Value = value,
                            ValueType = valueType,
                        });
                    }
                    else if (reportElement is ReportElementArrayDTO ea2 && ea2.Value is List<string> stringList)
                    {
                        elements.Add(new ReportElementTag
                        {
                            Id = reportElement.TagId,
                            Name = reportElement.Name,
                        });
                        elements.Add(new ReportElementValue
                        {
                            Value = stringList.Select(x => new ReportStringValue { Value = x }),
                            ValueType = "StringList",
                        });
                    }
                }
            }
            return elements;
        }
    
        /// <summary>
        /// 数据转换-测量项转换
        /// </summary>
        /// <param name="reportDatasJson"></param>
        /// <param name="dic">测量项id与measure键值对</param>
        /// <returns>转换后给dll生成报告的标准数据</returns>
        private static List<MeasureElementValuesModel> GetReportMeasureElements(string reportDatasJson, Dictionary<string,  ReportMeasureElementTag> dic)
        {
            var elements = new List<MeasureElementValuesModel>();
            if (!string.IsNullOrWhiteSpace(reportDatasJson))
            {
                var fetusA = new MeasureElementValuesModel() 
                {
                    Key = new MeasureElementValueKey() {
                        Id = "4D916526-9947-4EDE-8630-337632A35627",
                        Name = "FetusA",
                        IsDefault = true,
                        ElementTagType = "General"
                    },
                    Value = new List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>>()
                };    
                var existList = new List<ReportMeasureElementTag>();
                var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportDatasJson, new ReportElementDTOConverter());
                foreach (var reportElement in reportElements)
                {
                    if (reportElement is ReportElementStringDTO es)
                    {
                        var id = es?.Key?.ToString() ?? string.Empty;
                        if (!string.IsNullOrWhiteSpace(id) && dic.Keys.Contains(id)) 
                        {
                            if (!CheckKeyIsExist(existList, dic[id]))
                            {
                                var value = es?.Value?.ToString() ?? string.Empty;
                                var valueType = "String";
                                var valuePair = new ReportMeasureElementValue
                                {
                                    Value = value,
                                    ValueType = valueType,
                                };
                                var addItem = new KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>(dic[id], valuePair);
                                fetusA.Value.Add(addItem);
                                existList.Add(dic[id]);
                            }
                        }
                    }
                }
                elements.Add(fetusA);

                var fetusB = new MeasureElementValuesModel() 
                {
                    Key = new MeasureElementValueKey() {
                        Id = "B5A2AAE5-4DDB-4945-9515-82DB367B04F2",
                        Name = "FetusB",
                        IsDefault = true,
                        ElementTagType = "General"
                    },
                    Value = new List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>>()
                };
                elements.Add(fetusB);

                var fetusC = new MeasureElementValuesModel() 
                {
                    Key = new MeasureElementValueKey() {
                        Id = "35C22E6B-D15E-43CD-88F9-99AA79C3575D",
                        Name = "FetusC",
                        IsDefault = true,
                        ElementTagType = "General"
                    },
                    Value = new List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>>()
                };
                elements.Add(fetusC);

                var fetusD = new MeasureElementValuesModel() 
                {
                    Key = new MeasureElementValueKey() {
                        Id = "24498307-BB99-458B-8EC0-AC8434DE491D",
                        Name = "FetusD",
                        IsDefault = true,
                        ElementTagType = "General"
                    },
                    Value = new List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>>()
                };
                elements.Add(fetusD);

                var fetusE = new MeasureElementValuesModel() 
                {
                    Key = new MeasureElementValueKey() {
                        Id = "AEC6FDB6-1CD4-4C43-896A-2E914B837545",
                        Name = "FetusE",
                        IsDefault = true,
                        ElementTagType = "General"
                    },
                    Value = new List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>>()
                };
                elements.Add(fetusE);
            }
            return elements;
        }

        private static bool CheckKeyIsExist(List<ReportMeasureElementTag> list, ReportMeasureElementTag model)
        {
            var isExist = false;
            var jsonAdd = Newtonsoft.Json.JsonConvert.SerializeObject(model);
            foreach (var item in list)
            {
                var jsonItem = Newtonsoft.Json.JsonConvert.SerializeObject(item);
                if (jsonItem.Equals(jsonAdd))
                { 
                    isExist = true;
                    break;
                }
            }
            return isExist;
        }
    
        /// <summary>
        /// 获取所有测量项id与测量内容的键值对
        /// </summary>
        /// <param name="table">模板表格数据</param>
        /// <param name="dic">测量项id与measure键值对</param>
        private static void Generate(Vinno.vCloud.Report.Models.RTTable table, Dictionary<string, ReportMeasureElementTag> dic)
        {
            if (table.RowDefinitions.Count > 0 && table.ColumnDefinitions.Count > 0)
            {
                foreach (var cellItem in table.Cells)
                {
                    foreach (var child in cellItem.Value.Blocks)
                    {
                        if (child is Vinno.vCloud.Report.Models.RTTable rtTable)
                        {
                            Generate(rtTable, dic);
                        }
                        else
                        {
                            if (child is Vinno.vCloud.Report.Models.Paragraph paragraph)
                            {
                                foreach (var element in paragraph.Elements)
                                {
                                    if (element is Vinno.vCloud.Report.Models.InputText inputText)
                                    {
                                        if (inputText.MeasureTag != null)
                                        {
                                            if (!dic.Keys.Contains(inputText.Id))
                                            {
                                                var measureTagContent = new ReportMeasureElementTag
                                                {
                                                    AvailableMethods = inputText.MeasureTag.AvailableMethods,
                                                    BaseType = inputText.MeasureTag.BaseType,
                                                    CalculationId = inputText.MeasureTag.CalculationId,
                                                    MatchId = inputText.MeasureTag.MatchId,
                                                    Method = string.IsNullOrEmpty(inputText.MeasureTag.Method) ? null : inputText.MeasureTag.Method,
                                                    Mode = inputText.MeasureTag.Mode,
                                                    Output = inputText.MeasureTag.Output,
                                                    OutputV2 = inputText.MeasureTag.OutputV2,
                                                    Unit = inputText.MeasureTag.Unit,
                                                    UserId = inputText.MeasureTag.UserId,
                                                    MeasureResultType = string.IsNullOrEmpty(inputText.MeasureTag.MeasureResultType) ? "Statistical" : inputText.MeasureTag.MeasureResultType,
                                                };
                                                dic.Add(inputText.Id, measureTagContent);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /// <summary>
    /// 测量项转换后的实体
    /// </summary>
    public class MeasureElementValuesModel
    {
        /// <summary>
        /// 键
        /// </summary>
        public MeasureElementValueKey Key { get; set; }

        /// <summary>
        /// 值
        /// </summary>
        public List<KeyValuePair<ReportMeasureElementTag, ReportMeasureElementValue>> Value { get; set; }
    }

    /// <summary>
    /// 测量项值对象中的键
    /// </summary>
    public class ReportMeasureElementTag
    {
        public string[] AvailableMethods  { get; set; }
        public string BaseType { get; set; }
        public string CalculationId { get; set; }
        public string MatchId { get; set; }
        public string Method { get; set; }
        public string Mode { get; set; }
        public string Output { get; set; }
        public string OutputV2 { get; set; }
        public string Unit { get; set; }
        public string UserId { get; set; }
        public string MeasureResultType { get; set; } = "Statistical";
    }

    /// <summary>
    /// 测量项值对象中的值
    /// </summary>
    public class ReportMeasureElementValue
    {
        public object Value { get; set; }
        public string ValueType { get; set; }
    }

    /// <summary>
    /// 测量项键实体
    /// </summary>
    public class MeasureElementValueKey
    {
        public string Id { get; set; }
        public string Name { get; set; }
        public bool IsDefault { get; set; } = true;
        public string ElementTagType { get; set; } = "General";
    }

    public interface IReportElementValue
    {

    }

    public class ReportElementTag : IReportElementValue
    {
        public string Id { get; set; }
        public string Name { get; set; }
        public bool IsDefault { get; set; } = true;
        public string ElementTagType { get; set; } = "General";
    }

    public class ReportElementValue : IReportElementValue
    {
        public object Value { get; set; }
        public string ValueType { get; set; }
    }

    public class ReportFileImageValue
    {
        public string ExamDataId { get; set; } = "";
        public string Value { get; set; }
        public string ValueType { get; set; } = "FileImage";
    }

    public class ReportStringValue
    {
        public string Value { get; set; }
        public string ValueType { get; set; } = "String";
    }
}