using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Request.Report;
using WingServerCommon.Service;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.Interface;
using WingServerCommon.Log;
using SkiaSharp;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.Enum.VitalEnum;
using Newtonsoft.Json;

namespace ReportService.Service
{
    public partial class ReportService : JsonRpcService, IUltrasoundReportService
    {
        /// <summary>
        /// 根据业务编码查询关联报告a
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>报告集合</returns>
        /// <value></value>
        /// <remarks>POST</remarks>
        public async Task<List<ReportDTO>> VitalFindReportsAsync(FindReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var isEncryptedShow = false;
            //当前登录人
            var result = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });

            var userInfos = new List<WingInterfaceLibrary.DTO.Vital.UserDTO>();
            if (result?.Any() ?? false)
            {
                try
                {
                    var reportUserCodes = result.Select(x => x.ReportUserCode).Distinct().ToList();
                    var vitalUserList = await _vitalUserDBService.GetUserListAsync(new WingInterfaceLibrary.Request.DBVitalRequest.GetUserListDBRequest
                    {
                        Codes = reportUserCodes,
                    });
                    if (vitalUserList?.Any() ?? false)
                    {
                        userInfos.AddRange(vitalUserList);
                    }
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"ReportService VitalFindReportsAsync failed, ex:{ex}");
                }
            }
            foreach (var item in result)
            {
                if (string.IsNullOrWhiteSpace(item.ReportUserName) && !string.IsNullOrWhiteSpace(item.ReportUserCode))
                {
                    var reportUser = userInfos.FirstOrDefault(x => x.Code == item.ReportUserCode);
                    if (reportUser != null)
                    {
                        item.ReportUserName = !string.IsNullOrWhiteSpace(reportUser.RealName) ? reportUser.RealName : reportUser.UserName;
                    }
                }
                if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                {
                    foreach (var preview in item.ReportPreviewList)
                    {
                        if (string.IsNullOrWhiteSpace(preview.ThumbnailToken))
                        {
                            preview.ThumbnailToken = preview.FileToken;
                        }
                    }
                }
                item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, isEncryptedShow);
                if (item.DiagnosisLabels.Contains("VAid"))
                {
                    var language = "en-US";
                    if (!string.IsNullOrWhiteSpace(request.LanguageCode))
                    {
                        language = request.LanguageCode;
                    }
                    var reportPreviewList = new List<ReportPreviewDTO>();
                    if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                    {
                        reportPreviewList = item.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                        if (!reportPreviewList.Any())
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                        if (!reportPreviewList.Any() && item.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                    }
                    item.ReportPreviewList = reportPreviewList;
                }
                item.CanEditReport = item.DiagnosisLabels.Contains("VAid") || item.ReportUserCode == userCode;
                if (item.ReportPreviewList == null || !item.ReportPreviewList.Any() || !item.ReportPreviewList.Any(x => x.FileToken.ToLower().Contains(".dat") || x.FileToken.ToLower().Contains(".jpg")))
                {
                    JoinQueue(item.ReportCode, 0, false);
                }
            }
            result = result.Where(v => v.ReportPreviewList != null && v.ReportPreviewList.Count > 0)?.ToList();
            return result;
        }

        /// <summary>
        /// 查询报告详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportDTO> VitalFindReportByCodeAsync(FindReportByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var isEncryptedShow = false;

            var reportInfo = await FindReportByCodeAsync(request.ReportCode, request.Language);
            reportInfo.PatientName = CryptionHelper.NameEncrypt(reportInfo.PatientName, isEncryptedShow);
            if (reportInfo.DiagnosisLabels.Contains("VAid"))
            {
                var language = "en-US";
                if (!string.IsNullOrWhiteSpace(request.Language))
                {
                    language = request.Language;
                }
                var reportPreviewList = new List<ReportPreviewDTO>();
                if (reportInfo.ReportPreviewList != null && reportInfo.ReportPreviewList.Any())
                {
                    reportPreviewList = reportInfo.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                    if (!reportPreviewList.Any())
                    {
                        reportPreviewList = reportInfo.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                    }
                    if (!reportPreviewList.Any() && reportInfo.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                    {
                        reportPreviewList = reportInfo.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                    }
                }
                reportInfo.ReportPreviewList = reportPreviewList;
            }
            return reportInfo;
        }

        /// <summary>
        /// 读取数据并生成报告所需的json
        /// </summary>
        private async Task<List<dynamic>> ReadDataFromExam(string examData, string userCode, WingInterfaceLibrary.DTO.Vital.ElectrocardiogramRecord examRecord)
        {

            var newDataJsonDynamicList = new List<dynamic>();
            try
            {

                var userInfo = await _vitalUserDBService.GetUserDetailAsync(new WingInterfaceLibrary.Request.DBVitalRequest.GetUserDBRequest
                {
                    Code = userCode
                });
                //当前用户的签名
                var doctorSign = userInfo?.Signature ?? string.Empty;
                var examDoctorInfo = await _vitalUserDBService.GetUserDetailAsync(new WingInterfaceLibrary.Request.DBVitalRequest.GetUserDBRequest
                {
                    Code = examRecord.ExamDoctor
                });
                var examDoctor = examDoctorInfo?.Signature ?? string.Empty;

                var orgName = examRecord?.OrganizationName ?? string.Empty;
                if (string.IsNullOrEmpty(orgName) && !string.IsNullOrEmpty(examRecord?.OrganizationCode))
                {
                    var orgDBReq = new WingInterfaceLibrary.Request.DBVitalRequest.GetOrganizationDBRequest()
                    {
                        Code = examRecord.OrganizationCode
                    };
                    var orgInfo = await _vitalOrganizationDBService.GetOrganizationDetailAsync(orgDBReq);
                    if (!string.IsNullOrEmpty(orgInfo?.OrganizationName))
                    {
                        orgName = orgInfo.OrganizationName;
                    }
                }
                var examTime = UTCToBeijing(examRecord.ExamTime).ToString("yyyy-MM-dd HH:mm:ss.fff");

                var patientName = examRecord?.PatientName ?? string.Empty;
                var physicalExamNumber = examRecord?.PhysicalExamNumber ?? string.Empty;
                var cardNo = examRecord?.CardNo ?? string.Empty;
                var sex = "";
                if (examRecord.PatientGender == GenderEnum.Female)
                {
                    sex = "女";
                }
                else if (examRecord.PatientGender == GenderEnum.Male)
                {
                    sex = "男";
                }
                else if (examRecord.PatientGender == GenderEnum.Unspecified)
                {
                    sex = "未说明的性别";
                }
                else
                {
                    sex = "未知的性别";
                }
                var age = "";
                if (examRecord?.Birthday > Convert.ToDateTime("1900-1-1"))
                {
                    age = CalculateAge(examRecord.Birthday).ToString() ?? string.Empty;
                }
                // var birthday = "";
                // if (examRecord?.Birthday > Convert.ToDateTime("1900-1-1"))
                // {
                //     birthday = examRecord?.Birthday.ToUniversalTime().ToString("yyyy-MM-dd") ?? "";
                // }
                // var phone = examRecord?.Phone ?? string.Empty;
                // var adress = examRecord?.PatientAddress ?? string.Empty;


                var index = "";
                var HR = "";
                var PR = "";
                var PDur = "";
                var QRSDur = "";
                var TDur = "";
                var QTDur = "";
                var QTCDur = "";
                var QT_QTC_Dur = "";
                var PAxis = "";
                var QRSAxis = "";
                var TAxis = "";
                var P_QRS_T_Axis = "";
                var Rv5_Sv1_1 = "";
                var Rv5_Sv1_2 = "";
                var ASSESS12 = "";
                var ecgImage = "";
                var HEART12 = "";
                if (!string.IsNullOrEmpty(examData))
                {
                    JObject json = JObject.Parse(examData);
                    if (json == null)
                    {
                        return newDataJsonDynamicList;
                    }

                    var Analyse12 = "";
                    if (json.ContainsKey("Analyse12"))
                    {
                        Analyse12 = json["Analyse12"].ToString();
                    }
                    if (json.ContainsKey("ASSESS12"))
                    {
                        ASSESS12 = json["ASSESS12"].ToString();
                    }
                    if (json.ContainsKey("HEART12"))
                    {
                        HEART12 = json["HEART12"].ToString();
                    }
                    var ECG12 = "";
                    if (json.ContainsKey("ECG12"))
                    {
                        ECG12 = json["ECG12"].ToString();
                    }

                    var baselineFiltering = "";
                    var eMGFiltering = "";
                    var ac = "";
                    var powerLineFiltering = "";
                    var paperSpeed = "";
                    var gain = "";
                    if (json.ContainsKey("BaselineFiltering"))
                    {
                        baselineFiltering = json["BaselineFiltering"].ToString();
                    }
                    if (json.ContainsKey("EMGFiltering"))
                    {
                        eMGFiltering = json["EMGFiltering"].ToString();
                    }
                    if (json.ContainsKey("AC"))
                    {
                        ac = json["AC"].ToString();
                    }
                    if (json.ContainsKey("PowerLineFiltering"))
                    {
                        powerLineFiltering = json["PowerLineFiltering"].ToString();
                    }
                    if (json.ContainsKey("PaperSpeed"))
                    {
                        paperSpeed = json["PaperSpeed"].ToString();
                    }
                    if (json.ContainsKey("Gain"))
                    {
                        gain = json["Gain"].ToString();
                    }
                    var text = baselineFiltering + " " + eMGFiltering + " " + ac + " " + powerLineFiltering + " " + paperSpeed + " " + gain;
                    ecgImage = await GetImageAddTextAsync(ECG12, text);
                    if (!string.IsNullOrEmpty(Analyse12))
                    {
                        JObject jsonAnalyse12 = JObject.Parse(Analyse12);
                        if (jsonAnalyse12 != null)
                        {
                            if (string.IsNullOrWhiteSpace(doctorSign))
                            {
                                if (jsonAnalyse12.ContainsKey("DiagnosticEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["DiagnosticEleSignature"].ToString()))
                                {
                                    doctorSign = jsonAnalyse12["DiagnosticEleSignature"].ToString();
                                }
                            }
                            if (string.IsNullOrWhiteSpace(examDoctor))
                            {
                                if (jsonAnalyse12.ContainsKey("ReviewEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["ReviewEleSignature"].ToString()))
                                {
                                    examDoctor = jsonAnalyse12["ReviewEleSignature"].ToString();
                                }
                            }
                            if (jsonAnalyse12.ContainsKey("Rv5_Sv1_1"))
                            {
                                Rv5_Sv1_1 = jsonAnalyse12["Rv5_Sv1_1"].ToString() + " mv";
                            }
                            if (jsonAnalyse12.ContainsKey("Rv5_Sv1_2"))
                            {
                                Rv5_Sv1_2 = jsonAnalyse12["Rv5_Sv1_2"].ToString().Trim() + " mv";
                            }
                            if (jsonAnalyse12.ContainsKey("QRSAxis"))
                            {
                                QRSAxis = jsonAnalyse12["QRSAxis"].ToString() + " deg";
                            }
                            if (jsonAnalyse12.ContainsKey("TAxis"))
                            {
                                TAxis = jsonAnalyse12["TAxis"].ToString() + " deg";
                            }
                            if (jsonAnalyse12.ContainsKey("P_QRS_T_Axis"))
                            {
                                P_QRS_T_Axis = jsonAnalyse12["P_QRS_T_Axis"].ToString() + " deg";
                            }
                            if (jsonAnalyse12.ContainsKey("QTDur"))
                            {
                                QTDur = jsonAnalyse12["QTDur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("QTCDur"))
                            {
                                QTCDur = jsonAnalyse12["QTCDur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("QT_QTC_Dur"))
                            {
                                QT_QTC_Dur = jsonAnalyse12["QT_QTC_Dur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("PAxis"))
                            {
                                PAxis = jsonAnalyse12["PAxis"].ToString() + " deg";
                            }
                            if (jsonAnalyse12.ContainsKey("QRSDur"))
                            {
                                QRSDur = jsonAnalyse12["QRSDur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("TDur"))
                            {
                                TDur = jsonAnalyse12["TDur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("HR"))
                            {
                                HR = jsonAnalyse12["HR"].ToString() + " bpm";
                            }
                            if (jsonAnalyse12.ContainsKey("PR"))
                            {
                                PR = jsonAnalyse12["PR"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("PDur"))
                            {
                                PDur = jsonAnalyse12["PDur"].ToString() + " ms";
                            }
                            if (jsonAnalyse12.ContainsKey("index"))
                            {
                                index = jsonAnalyse12["index"].ToString();
                            }
                            if (jsonAnalyse12.ContainsKey("advice"))
                            {
                                ASSESS12 = jsonAnalyse12["advice"].ToString();
                            }
                        }
                    }
                }

                #region 页脚
                dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "415df52f-3fef-46c5-a39f-c3e3090e79f2";
                dynamicObj.Value = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff");
                dynamicObj.Name = "ReportTime";
                dynamicObj.TagId = "4abbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "779493a0-8900-4503-b77b-b65913842b8c1";
                dynamicObj.Value = examDoctor;
                dynamicObj.Name = "ReportPhysician";
                dynamicObj.TagId = "5abbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "779493a0-8900-4503-b77b-b65913842b8c";
                dynamicObj.Value = doctorSign;
                dynamicObj.Name = "DigitalSignature";
                dynamicObj.TagId = "53545af1-59b7-41a8-9bd5-bdba1aaf9891";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion

                #region 抬头
                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "1790e450-19a6-4000-8895-3dda6a43aef9";
                dynamicObj.Value = orgName;
                dynamicObj.Name = "HospitalName";
                dynamicObj.TagId = "7abbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion

                #region 病人信息
                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "fced54ee-f02b-49e7-b443-c217e37287ab";
                dynamicObj.Value = patientName;
                dynamicObj.Name = "PatientName";
                dynamicObj.TagId = "91bbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "81d2aa8b-569f-40ce-8e9c-8e5f3c18eaeb";
                dynamicObj.Value = sex;
                dynamicObj.Name = "PatientGender";
                dynamicObj.TagId = "93bbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "2a112884-5ea2-478a-a81b-01c20391c75c";
                dynamicObj.Value = age;
                dynamicObj.Name = "PatientAge";
                dynamicObj.TagId = "92bbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "10d37b65-d37d-47b1-83bb-e320080d503d";
                dynamicObj.Value = physicalExamNumber;
                dynamicObj.Name = "PhysicalExamNumber";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362w";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "683d5247-8e06-4227-aded-10032b754c4b";
                dynamicObj.Value = examTime;
                dynamicObj.Name = "PatientExamDate";
                dynamicObj.TagId = "95bbaa61-a91d-4399-a3f4-df29117d2d59";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion

                #region 图像处理
                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "be12187d-c1d1-48dd-b56d-13c7cf897542";
                dynamicObj.Value = new List<string>() { ecgImage };
                dynamicObj.Name = "PatientECGImage";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A361z";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion

                #region 心电数据
                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "a3c110ca-8081-41df-a0aa-9604070ce7dc";
                dynamicObj.Value = HR;
                dynamicObj.Name = "HR";
                dynamicObj.TagId = "33D46E3F-A089-4B0A-8D1D-F4C7556BD62D";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "4a52b32f-876a-4365-b590-00d88d9cc36d";
                dynamicObj.Value = TAxis;
                dynamicObj.Name = "PatientTAxis";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362h";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "25c1d36b-1550-4fee-9c40-3b134feb26fb";
                dynamicObj.Value = PAxis;
                dynamicObj.Name = "PatientPAxis";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362f";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "a871072f-810e-4d57-ad9d-244fa21e47e2";
                dynamicObj.Value = QRSAxis;
                dynamicObj.Name = "PatientQRSAxis";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362g";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "9616f7d6-4c89-4b8a-8ee3-0675b829344e";
                dynamicObj.Value = PR;
                dynamicObj.Name = "PatientPRInterval";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362j";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "13f30eaf-e7b1-440c-a03d-0454b01120b1";
                dynamicObj.Value = QT_QTC_Dur;
                dynamicObj.Name = "PatientQTQTCInterval";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362l";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "75f69125-41eb-4067-bd7c-8942478c691f";
                dynamicObj.Value = Rv5_Sv1_1;
                dynamicObj.Name = "PatientRV5SV";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362m";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "996beabe-401c-44a9-8bd1-69b2905be9a2";
                dynamicObj.Value = QRSDur;
                dynamicObj.Name = "PatientQRSDuration";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362k";
                newDataJsonDynamicList.Add(dynamicObj);

                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "A1843C2E-B60A-483F-A5F0-B26F475A362n";
                dynamicObj.Value = Rv5_Sv1_2;
                dynamicObj.Name = "PatientRV5SV1";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362n";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion

                #region 诊断结果
                dynamicObj = new System.Dynamic.ExpandoObject();
                dynamicObj.Key = "e935734b-cdf9-4ff1-8cb6-18fb6e1dcc05";
                dynamicObj.Value = ASSESS12;
                dynamicObj.Name = "PatientEcgDiagnosticResult";
                dynamicObj.TagId = "A1843C2E-B60A-483F-A5F0-B26F475A362e";
                newDataJsonDynamicList.Add(dynamicObj);
                #endregion
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"ElectrocardiogramService ReadDataFromExam error, examRecord: {Newtonsoft.Json.JsonConvert.SerializeObject(examRecord)}, ex:{ex}");
            }
            return newDataJsonDynamicList;
        }

        /// <summary>
        /// 图像添加文字
        /// </summary>
        private async Task<string> GetImageAddTextAsync(string fileUrl, string text)
        {
            var imgUrl = "";
            if (!string.IsNullOrEmpty(fileUrl))
            {
                using (HttpClient client = new HttpClient())
                {
                    using (HttpResponseMessage response = client.GetAsync(fileUrl).Result)
                    {
                        var bytes = response.Content.ReadAsByteArrayAsync().Result;

                        using (var bitmap = SkiaSharp.SKBitmap.Decode(bytes))
                        {
                            using (SKCanvas canvas = new SKCanvas(bitmap))
                            {
                                // 创建一个SKPaint实例，设置文本属性
                                using (SKPaint paint = new SKPaint())
                                {
                                    paint.Color = SKColors.Black; // 设置文本颜色
                                    paint.TextSize = 16; // 设置文本大小

                                    var textBounds = new SKRect();
                                    paint.MeasureText(text, ref textBounds);
                                    float x = bitmap.Width - textBounds.Width - 20;

                                    // 绘制文本到位于（50, 50）的位置
                                    canvas.DrawText(text, x, 20, paint);
                                }
                            }
                            var _reportPreviewFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ReportPreviewTemp");
                            if (!Directory.Exists(_reportPreviewFolder))
                            {
                                Directory.CreateDirectory(_reportPreviewFolder);
                            }
                            else
                            {
                                //删除过时文件
                                var time = DateTime.Now.AddDays(-1);
                                DirectoryInfo theFolder = new DirectoryInfo(_reportPreviewFolder);
                                DirectoryInfo[] dics = theFolder.GetDirectories();
                                foreach (var item in dics)
                                {
                                    if (item.CreationTime < time)
                                    {
                                        Directory.Delete(item.FullName, true);
                                    }
                                }
                                FileInfo[] fileInfo = theFolder.GetFiles();
                                foreach (FileInfo File in fileInfo)  //遍历文件
                                {
                                    if (File.CreationTime < time)
                                    {
                                        File.Delete();
                                    }
                                }
                            }
                            DirectoryInfo reportFolder = new DirectoryInfo(_reportPreviewFolder);
                            string fileName = $"{Guid.NewGuid().ToString("N")}";
                            string imagePath = Path.Combine(_reportPreviewFolder, fileName) + ".jpg";
                            using (var fileStream = new FileStream(imagePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                            {
                                bitmap.Encode(fileStream, SkiaSharp.SKEncodedImageFormat.Jpeg, 100);
                            }
                            if (File.Exists(imagePath))
                            {
                                var _fileTransfer = new WingServerCommon.Interfaces.FileTransfer.FileTransferService();
                                var dataList = await _fileTransfer.FileTransferAsync(imagePath, false, false);
                                if (dataList == null || dataList.Count <= 0)
                                {
                                    Logger.WriteLineError($"Upload Files Way-2 {imagePath} Fail");
                                    return fileUrl;
                                }
                                var fileToken = dataList.FirstOrDefault();
                                if (!string.IsNullOrWhiteSpace(fileToken))
                                {
                                    imgUrl = fileToken;
                                }
                                File.Delete(imagePath);
                            }
                        }
                    }
                }
            }
            return imgUrl;
        }

        /// <summary>
        /// 创建远程心电报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> VitalAddEcgReportAsync(AddEcgReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var reqReportTempDB = new WingInterfaceLibrary.DB.Request.GetReportTemplateDBRequest()
            {
                ReportTemplateCode = request.ReportTemplateCode,
                LanguageCode = "zh-CN"
            };
            var reportTemplateInfo = await _reportDBService.GetReportTemplateAsync(reqReportTempDB) ?? new WingInterfaceLibrary.DTO.ReportTemplate.ReportTemplateDTO();
            var deviceCode = "";
            var patientCode = "";
            var record = await _vitalElectrocardiogramDBService.GetExamElectrocardiogramDBAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.Code))
            {
                ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
            }
            deviceCode = record.DeviceCode;
            patientCode = record.CardNo;
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            var reportDataList = await ReadDataFromExam(request.ReportDatasJson, userCode, record);
            var patientId = GetPatientId(request.ReportDatasJson);
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = reportTemplateInfo.ReportTemplateJson ?? "{}",
                ReportDatasJson = JsonConvert.SerializeObject(reportDataList),
                PatientId = patientId,
                PatientName = request.PatientName,
                ReportLabels = new List<string>(),
                PlatformLabels = new List<string>(),
                DiagnosisLabels = new List<string>(),
                ReportOrgan = DiagnosisOrganEnum.Null,
                ReferralRecordCode = "",
                ReportType = ReportTypeEnum.RemoteECG,
                IsReferral = false,
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = "",
                ReportExamDataJson = request.ReportDatasJson,
                HealthDiagnosis = request.HealthDiagnosis,
            });
            if (string.IsNullOrEmpty(reportId))
            {
                ThrowCustomerException(CustomerRpcCode.AddReportFailed, "Add report failed");
            }
            await CreateJingQiReportPostHistoryAsync(request.Token, reportId);
            var isUpdate = false;
            if (record.ExamState != WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Reported)
            {
                record.ExamState = WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Reported;
                isUpdate = true;
            }
            if (isUpdate)
            {
                await _vitalElectrocardiogramDBService.UpdateExamElectrocardiogramDBAsync(record);
            }
            await CreateVitalExamAsync(record, request.Token, reportId);
            return reportId;
        }

        private async Task<bool> CreateJingQiReportPostHistoryAsync(string token, string reportCode)
        {
            try
            {
                return await _vitalFacturyPostHistoryService.CreateJingQiReportPostHistoryAsync(new WingInterfaceLibrary.Request.Vital.CreateJingQiReportPostHistoryRequest
                {
                    Token = token,
                    ReportCode = reportCode,
                });
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportService CreateJingQiReportPostHistoryAsync error, ex:{ex}");
            }
            return false;
        }

        /// <summary>
        /// 创建报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> VitalAddReportAsync(AddReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);

            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            Logger.WriteLineWarn($"VitalAddReportAsync true, request: {Newtonsoft.Json.JsonConvert.SerializeObject(request)}");
            var deviceCode = "";
            var patientCode = "";
            var record = new RecordInfoDTO();
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !request.DiagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            var patientId = GetPatientId(request.ReportDatasJson);
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson ?? "{}",
                ReportDatasJson = request.ReportDatasJson,
                PatientId = patientId,
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = request.DiagnosisLabels ?? new List<string>(),
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
                ReportType = request.ReportType,
                IsReferral = request.IsReferral,
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = request.ReportMeasureDatasJson
            });
            if (string.IsNullOrEmpty(reportId))
            {
                ThrowCustomerException(CustomerRpcCode.AddReportFailed, "Add report failed");
            }
            await CreateJingQiReportPostHistoryAsync(request.Token, reportId);
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var isUpdate = false;
                    if (record.RecordStatus != RecordStatusEnum.Completed)
                    {
                        record.RecordStatus = RecordStatusEnum.Completed;
                        isUpdate = true;
                    }
                    var diagnosisLabels = request.DiagnosisLabels ?? new List<string>();
                    if (!record.IsUserReport && !diagnosisLabels.Contains("VAid"))
                    {
                        record.IsUserReport = true;
                        isUpdate = true;
                    }
                    if (isUpdate)
                    {
                        await _recordsInfoDBService.UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
                    }
                    break;
            }
            JoinQueue(reportId);
            return reportId;
        }

        /// <summary>
        /// 修改远程心电报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> VitalModifyEcgReportAsync(ModifyEcgReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            Logger.WriteLineWarn($"VitalModifyEcgReportAsync true, request: {Newtonsoft.Json.JsonConvert.SerializeObject(request)}");

            var reqReportTempDB = new WingInterfaceLibrary.DB.Request.GetReportTemplateDBRequest()
            {
                ReportTemplateCode = request.ReportTemplateCode,
                LanguageCode = "zh-CN"
            };
            var reportTemplateInfo = await _reportDBService.GetReportTemplateAsync(reqReportTempDB) ?? new WingInterfaceLibrary.DTO.ReportTemplate.ReportTemplateDTO();
            var deviceCode = "";
            var patientCode = "";
            var record = await _vitalElectrocardiogramDBService.GetExamElectrocardiogramDBAsync(request.RecordCode);
            if (record == null || string.IsNullOrWhiteSpace(record.Code))
            {
                ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
            }
            deviceCode = record.DeviceCode;
            patientCode = record.CardNo;
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                var examReportList = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
                var userOrganReport = examReportList.FirstOrDefault(x => x.ReportUserCode == userCode);
                request.ReportCode = userOrganReport?.ReportCode ?? string.Empty;
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            var reportDataList = await ReadDataFromExam(request.ReportDatasJson, userCode, record);
            var saveRequest = new SaveReportDBRequest
            {
                ReportCode = request.ReportCode,
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = reportTemplateInfo.ReportTemplateJson ?? "{}",
                ReportDatasJson = JsonConvert.SerializeObject(reportDataList),
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = new List<string>(),
                PlatformLabels = new List<string>(),
                DiagnosisLabels = new List<string>(),
                ReportOrgan = DiagnosisOrganEnum.Null,
                ReferralRecordCode = "",
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = "",
                ReportExamDataJson = request.ReportDatasJson,
                HealthDiagnosis = request.HealthDiagnosis,
            };
            var reportId = await _reportDBService.SaveReportAsync(saveRequest);
            if (!string.IsNullOrEmpty(reportId))
            {
                var isUpdate = false;
                if (record.ExamState != WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Reported)
                {
                    record.ExamState = WingInterfaceLibrary.Enum.VitalEnum.ExamStateEnum.Reported;
                    isUpdate = true;
                }
                if (isUpdate)
                {
                    await _vitalElectrocardiogramDBService.UpdateExamElectrocardiogramDBAsync(record);
                }
                await CreateJingQiReportPostHistoryAsync(request.Token, reportId);
                await CreateVitalExamAsync(record, request.Token, reportId);
                return true;
            }
            return false;
        }

        /// <summary>
        /// 创建vital检查
        /// </summary>
        private async Task<bool> CreateVitalExamAsync(WingInterfaceLibrary.DTO.Vital.ElectrocardiogramRecord record, string token, string reportId)
        {
            //获取vital机构信息，
            var res = false;
            var examRequest = new WingInterfaceLibrary.Request.Vital.CreateExamRequest();
            examRequest.Key = "HEIECG";
            examRequest.ExamData = record.ExamData;
            examRequest.PatientCode = record.CardNo;
            examRequest.PhysicalExamNumber = record.PhysicalExamNumber;
            examRequest.Token = token;
            examRequest.IsRemoteEcgExam = true;
            var result = await _vitalExamService.CreateExamAsync(examRequest);
            if (!string.IsNullOrEmpty(result))
            {
                res = true;
            }
            JoinEcgQueue(reportId);
            return res;
        }

        /// <summary>
        /// 修改报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> VitalModifyReportAsync(ModifyReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var deviceCode = "";
            var patientCode = "";
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
            }
            var diagnosisLabels = request.DiagnosisLabels ?? new List<string>();
            if (request.ReportOrgan != DiagnosisOrganEnum.Null && !string.IsNullOrWhiteSpace(userCode))
            {
                diagnosisLabels.Remove("VAid");
                var examReportList = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
                var userOrganReport = examReportList.FirstOrDefault(x => x.ReportUserCode == userCode && x.ReportOrgan == request.ReportOrgan);
                request.ReportCode = userOrganReport?.ReportCode ?? string.Empty;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !diagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            var saveRequest = new SaveReportDBRequest
            {
                ReportCode = request.ReportCode,
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson ?? "{}",
                ReportDatasJson = request.ReportDatasJson,
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = diagnosisLabels,
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = request.ReportMeasureDatasJson
            };
            var reportId = await _reportDBService.SaveReportAsync(saveRequest);
            if (!string.IsNullOrEmpty(reportId))
            {
                await CreateJingQiReportPostHistoryAsync(request.Token, reportId);
                DoReportConverter(new ReportConverterDetail()
                {
                    ReportCode = reportId
                });//直接生成报告预览图像
                return true;
            }

            return false;
        }

        /// <summary>
        /// 获取用户默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusAllDTO> VitalGetDefaultThesaurusContentAsync(GetDefaultThesaurusContentRequest request)
        {
            return await GetDefaultThesaurusContentAsync(request);
        }

        /// <summary>
        /// 获取词条项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> VitalGetThesaurusItemsAsync(GetThesaurusItemsRequest request)
        {
            return await GetThesaurusItemsAsync(request);
        }

        /// <summary>
        /// 获取词条库（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> VitalGetThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            return await GetThesaurusPageAsync(request);
        }

        /// <summary>
        /// 获取报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> VitalGetReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            return await GetReportTemplatePageAsync(request);
        }

        /// <summary>
        /// 获取用户默认报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> VitalGetDefaultReportTemplateContentAsync(GetDefaultReportTemplateContentRequest request)
        {
            return await GetDefaultReportTemplateContentAsync(request);
        }

        /// <summary>
        /// 查询报告模板详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> VitalFindReportTemplateByCodeAsync(FindReportTemplateByCodeRequest request)
        {
            return await FindReportTemplateByCodeAsync(request);
        }
    }
}