﻿using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.ReportTemplate;
using WingInterfaceLibrary.Result;
using System.Text;
using WingServerCommon.Config;
using WingInterfaceLibrary.DTO.Share;
using WingServerCommon.Config.Parameters;
using ReportService.Posters;
using System.Collections.Concurrent;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Log;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Request.DBRequest;
using WingInterfaceLibrary.DTO.Record;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.Internal.Interface;
using WingInterfaceLibrary.Request.DBCopy;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.Result.Management;
using System.Collections.Generic;
using System.Runtime.Intrinsics.X86;
using Newtonsoft.Json.Linq;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.Interface.DBVitalInterface;
using WingInterfaceLibrary.Request.Vital;
using ReportService.ReportConverter;
using Newtonsoft.Json;
using System.Globalization;
using WingInterfaceLibrary.Interface.VitalInterface;

namespace ReportService.Service
{
    /// <summary>
    /// 报告服务
    /// </summary>
    public partial class ReportService : JsonRpcService, IReportService
    {
        private IUserDBService _userService;
        private IDeviceInfoDBService _deviceInfoDBService;
        private IAuthenticationService _authenticationService;
        private IOrganizationDBService _organizationDBService;
        private INotificationService _notificationService;
        private WingInterfaceLibrary.Interface.VitalInterface.IVitalExamService _vitalExamService;
        private IRecordsInfoDBService _recordsInfoDBService;
        private IPatientInfoDBService _patientInfoDBService;
        private IReportDBService _reportDBService;
        private IAdminDBService _adminDBService;
        private IRemedicalDBService _remedicalDBService;
        private IEmailService _emailService;
        private ISMSService _smsService;
        private IShareDBService _shareDBService;
        private IRemedicalService _remedicalService;
        private IReportPosterDBService _reportPosterDBService;
        private IStorageService _storageService;
        private IConsultationRecordDBService _consultationRecordDBService;
        private IRegisterInfoDBService _registerInfoDBService;
        private IVitalHealthExamBookingDBService _vitalHealthExamBookingDBService;
        private IVitalExamDBService _vitalExamDBService;
        private IVitalReportDBService _vitalReportDBService;
        private IVitalOrganizationDBService _vitalOrganizationDBService;
        private IVitalUserDBService _vitalUserDBService;
        private IVitalTemplateDBService _vitalTemplateDBService;
        private IVitalDictionaryDBService _dictionaryDBService;
        private IVitalHealthStatisticDBService _vitalHealthStatisticDBService;
        private IVitalPrescriptionDBService _prescriptionDBService;
        private IVitalElectrocardiogramDBService _vitalElectrocardiogramDBService { get; set; }
        private IVitalFacturyPostHistoryService _vitalFacturyPostHistoryService { get; set; }
        private string _messageUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Report", "MessageUrl").Value;
        private string _reportUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Report", "ReportUrl").Value;
        private string _reportMessageTemplateId => EnvironmentConfigManager.GetParammeter<StringParameter>("Report", "ReportMessageTemplateId").Value;
        private string _curCDNHost => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "CDNServerUrl").Value;
        private IVitalPatientDBService _vitalPatientDBService;
        private IVitalPatientExtensionDBService _vitalPatientExtensionDBService;
        private string _httpService => EnvironmentConfigManager.GetParammeter<StringParameter>("Services", "JsonRpcHttp").Value;
        private readonly ConcurrentDictionary<string, string> _aiReportTemplateDictionary = new ConcurrentDictionary<string, string>();
        private readonly ReportPostersObserver _reportPosterObserver;
        private readonly HttpClient _httpClient = new HttpClient();
        private string _cloudServerUrl => EnvironmentConfigManager.GetParammeter<StringParameter>("Vital", "CloudServerUrl").Value;
        private bool _isNeedImageReport => EnvironmentConfigManager.GetParammeter<BoolParameter>("Vital", "IsNeedImageReport").Value;
        public ReportService()
        {
            CacheMaintenance.Instance.Get<IReportPostersManager>().RegisterDBFunction(FindReportPosterByCode, FindReportPostersWithCodes);
            _reportPosterObserver = new ReportPostersObserver(FindReporByCode, UpdateReportPushRecord, FindRecordByCode);
            _httpClient.Timeout = TimeSpan.FromMinutes(10);
        }

        /// <summary>
        /// Init service
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _userService = GetProxy<IUserDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _deviceInfoDBService = GetProxy<IDeviceInfoDBService>();
            _organizationDBService = GetProxy<IOrganizationDBService>();
            _notificationService = GetProxy<INotificationService>();
            _recordsInfoDBService = GetProxy<IRecordsInfoDBService>();
            _patientInfoDBService = GetProxy<IPatientInfoDBService>();
            _reportDBService = GetProxy<IReportDBService>();
            _adminDBService = GetProxy<IAdminDBService>();
            _remedicalDBService = GetProxy<IRemedicalDBService>();
            _emailService = GetProxy<IEmailService>();
            _smsService = GetProxy<ISMSService>();
            _shareDBService = GetProxy<IShareDBService>();
            _remedicalService = GetProxy<IRemedicalService>();
            _reportPosterDBService = GetProxy<IReportPosterDBService>();
            _storageService = GetProxy<IStorageService>();
            _consultationRecordDBService = GetProxy<IConsultationRecordDBService>();
            _registerInfoDBService = GetProxy<IRegisterInfoDBService>();
            _vitalHealthStatisticDBService = GetProxy<IVitalHealthStatisticDBService>();
            _vitalElectrocardiogramDBService = GetProxy<IVitalElectrocardiogramDBService>();
            _vitalFacturyPostHistoryService = GetProxy<IVitalFacturyPostHistoryService>();
            if (_httpService.Contains("VitalMixtureService"))
            {
                _vitalHealthExamBookingDBService = GetProxy<IVitalHealthExamBookingDBService>();
                _vitalExamDBService = GetProxy<IVitalExamDBService>();
                _vitalReportDBService = GetProxy<IVitalReportDBService>();
                _vitalOrganizationDBService = GetProxy<IVitalOrganizationDBService>();
                _vitalUserDBService = GetProxy<IVitalUserDBService>();
                _vitalTemplateDBService = GetProxy<IVitalTemplateDBService>();
                _vitalPatientDBService = GetProxy<IVitalPatientDBService>();
                _vitalPatientExtensionDBService = GetProxy<IVitalPatientExtensionDBService>();
                _dictionaryDBService = GetProxy<IVitalDictionaryDBService>();
                _prescriptionDBService = GetProxy<IVitalPrescriptionDBService>();
                _vitalExamService = GetProxy<WingInterfaceLibrary.Interface.VitalInterface.IVitalExamService>();
            }
            LoadReportPosterFromDb();
        }

        /// <summary>
        /// 通用异常处理方法
        /// </summary>
        /// <param name="customerRpcCodeEnum">异常标识码</param>
        /// <param name="msg">异常信息</param>
        /// <param name="internalMessage">内部异常信息，非必填</param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg, string internalMessage = null)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg, internalMessage);
        }

        /// <summary>
        /// 查询报告模板详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> FindReportTemplateByCodeAsync(FindReportTemplateByCodeRequest request)
        {
            Logger.WriteLineInfo($"ReportService FindReportTemplateByCodeAsync Init: request: {Newtonsoft.Json.JsonConvert.SerializeObject(request)}");

            var userCode = await ValidateTokenAsync(request.Token);
            var detailInfo = await _reportDBService.GetReportTemplateAsync(new GetReportTemplateDBRequest { ReportTemplateCode = request.ReportTemplateCode });
            if (detailInfo != null)
            {
                var language = "en-US";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }

                if (detailInfo.TemplatePreviewList?.Count > 0)
                {
                    detailInfo.TemplatePreviewList.ForEach(x =>
                    {
                        if (x.Language == language)
                        {
                            if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                            {
                                detailInfo.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                                var newDataList = new List<string>();
                                detailInfo.TemplatePreviewUrlList.ForEach(dx =>
                                {
                                    var fileName = Path.GetFileName(dx);
                                    var newUrl = _curCDNHost + "/" + fileName;
                                    newDataList.Add(newUrl);
                                });
                                detailInfo.TemplatePreviewUrlList = newDataList;
                            }
                        }
                    });
                    detailInfo.TemplatePreviewList = null;
                }
                Logger.WriteLineInfo($"ReportService FindReportTemplateByCodeAsync Completed: requestCode: {request.ReportTemplateCode} ,response: {Newtonsoft.Json.JsonConvert.SerializeObject(detailInfo.ReportTemplateName)}");
            }
            else
            {
                Logger.WriteLineInfo($"ReportService FindReportTemplateByCodeAsync Completed: requestCode: {request.ReportTemplateCode} : response is null");
            }
            return detailInfo;
        }

        /// <summary>
        /// 查询报告详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportDTO> FindReportByCodeAsync(FindReportByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var reportInfo = await FindReportByCodeAsync(request.ReportCode, request.Language);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode }) ?? new OrganizationDTO() { IsEncryptedShow = false };
            if (rootOrg != null)
            {
                reportInfo.PatientName = CryptionHelper.NameEncrypt(reportInfo.PatientName, rootOrg.IsEncryptedShow);
            }
            if (reportInfo.DiagnosisLabels.Contains("VAid"))
            {
                var language = "en-US";
                if (!string.IsNullOrWhiteSpace(request.Language))
                {
                    language = request.Language;
                }
                var reportPreviewList = new List<ReportPreviewDTO>();
                if (reportInfo.ReportPreviewList != null && reportInfo.ReportPreviewList.Any())
                {
                    reportPreviewList = reportInfo.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                    if (!reportPreviewList.Any())
                    {
                        reportPreviewList = reportInfo.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                    }
                    if (!reportPreviewList.Any() && reportInfo.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                    {
                        reportPreviewList = reportInfo.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                    }
                }
                reportInfo.ReportPreviewList = reportPreviewList;
            }
            return reportInfo;
        }

        private async Task<ReportDTO> FindReportByCodeAsync(string reportCode, string language)
        {
            var reportInfo = await _reportDBService.GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = reportCode });
            if (reportInfo.DiagnosisLabels.Contains("VAid") && !reportInfo.ReportPreviewList.Any(x => x.FileToken.Contains(".dat")))
            {
                if (string.IsNullOrWhiteSpace(language))
                {
                    language = "en-US";
                }
                var newReportDatasJson = await _remedicalService.GetReportElementByLanguageAsync(new GetReportElementByLanguageRequest
                {
                    ReportDatasJosn = reportInfo.ReportDatasJson,
                    Language = language,
                });
                reportInfo.ReportDatasJson = newReportDatasJson;
                reportInfo.ReportTemplateJson = GetAIReportTemplate(language, reportInfo.ReportOrgan);
                if (!string.IsNullOrWhiteSpace(reportInfo.ReportTemplateJson) && string.IsNullOrEmpty(reportInfo.ReportTemplateName))
                {
                    try
                    {
                        dynamic templateInfo = JObject.Parse(reportInfo.ReportTemplateJson);
                        reportInfo.ReportTemplateName = templateInfo?.Name;
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn($"Parse ReportTemplateJson error {ex}");
                    }
                }
            }
            return reportInfo;
        }

        /// <summary>
        /// 查询报告分享地址
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> FindReportShareUrlAsync(FindReportShareUrlRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var reportCode = request.ReportCode;
            return string.Format(_reportUrl, reportCode, !string.IsNullOrWhiteSpace(request.LanguageCode) ? request.LanguageCode : "en-US");
        }

        private async Task CheckReportConverter(string reportCode)
        {
            var reportInfo = await FindReportByCodeAsync(reportCode, "");
            var reportPreviewList = reportInfo.ReportPreviewList?.Where(x => x.FileType == UploadFileTypeEnum.JPG);
            if (reportPreviewList == null || !reportPreviewList.Any())
            {
                try
                {
                    DoReportConverter(reportCode, UploadFileTypeEnum.JPG);
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"check report convert, create report jpg failed, report code:{reportCode}");
                }
            }
        }

        /// <summary>
        /// 查询检查关联报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportDTO>> FindExamReportsAsync(FindReportsRequest request)
        {
            var result = await FindReportsAsync(request);
            return result;
        }

        /// <summary>
        /// 根据业务编码查询关联报告
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>报告集合</returns>
        /// <value></value>
        /// <remarks>POST</remarks>
        public async Task<List<ReportDTO>> FindReportsAsync(FindReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode }) ?? new OrganizationDTO() { IsEncryptedShow = false };
            var result = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
            foreach (var item in result)
            {
                if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                {
                    foreach (var preview in item.ReportPreviewList)
                    {
                        if (string.IsNullOrWhiteSpace(preview.ThumbnailToken))
                        {
                            preview.ThumbnailToken = preview.FileToken;
                        }
                    }
                }
                if (rootOrg != null)
                {
                    item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, rootOrg.IsEncryptedShow);
                }
                if (item.DiagnosisLabels.Contains("VAid"))
                {
                    var language = "en-US";
                    if (!string.IsNullOrWhiteSpace(request.LanguageCode))
                    {
                        language = request.LanguageCode;
                    }
                    var reportPreviewList = new List<ReportPreviewDTO>();
                    if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                    {
                        reportPreviewList = item.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                        if (!reportPreviewList.Any())
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                        if (!reportPreviewList.Any() && item.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                    }
                    item.ReportPreviewList = reportPreviewList;
                }
                item.CanEditReport = item.DiagnosisLabels.Contains("VAid") || item.ReportUserCode == userCode;
                if (item.ReportPreviewList == null || !item.ReportPreviewList.Any() || !item.ReportPreviewList.Any(x => x.FileToken.ToLower().Contains(".dat") || x.FileToken.ToLower().Contains(".jpg")))
                {
                    JoinQueue(item.ReportCode, 0, false);
                }
            }
            //过滤查询未生成预览图列表的报告  废弃 修复：0013377:
            result = result.Where(v => v.ReportPreviewList != null && v.ReportPreviewList.Count > 0)?.ToList();
            return result;
        }

        public async Task<List<ReportDTO>> FindPatientReportsAsync(FindPatientReportsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            //当前登录人
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var result = await _reportDBService.GetReportByPatientCodeAsync(request.PatientCode);
            var rootOrg = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode }) ?? new OrganizationDTO() { IsEncryptedShow = false };
            foreach (var item in result)
            {
                if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                {
                    foreach (var preview in item.ReportPreviewList)
                    {
                        if (string.IsNullOrWhiteSpace(preview.ThumbnailToken))
                        {
                            preview.ThumbnailToken = preview.FileToken;
                        }
                    }
                }
                if (item.DiagnosisLabels.Contains("VAid"))
                {
                    var language = "en-US";
                    if (!string.IsNullOrWhiteSpace(request.LanguageCode))
                    {
                        language = request.LanguageCode;
                    }
                    var reportPreviewList = new List<ReportPreviewDTO>();
                    if (item.ReportPreviewList != null && item.ReportPreviewList.Any())
                    {
                        reportPreviewList = item.ReportPreviewList.Where(v => v.Language == language)?.ToList() ?? new List<ReportPreviewDTO>();
                        if (!reportPreviewList.Any())
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => v.Language == "en-US")?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                        if (!reportPreviewList.Any() && item.ReportPreviewList.Any(rp => rp.FileToken.Contains(".dat")))
                        {
                            reportPreviewList = item.ReportPreviewList.Where(v => string.IsNullOrWhiteSpace(v.Language))?.ToList() ?? new List<ReportPreviewDTO>();
                        }
                    }
                    item.ReportPreviewList = reportPreviewList;
                }
                item.CanEditReport = item.DiagnosisLabels.Contains("VAid") || item.ReportUserCode == userCode;
                if (rootOrg != null)
                {
                    item.PatientName = CryptionHelper.NameEncrypt(item.PatientName, rootOrg.IsEncryptedShow);
                }
            }
            //过滤查询未生成预览图列表的报告 废弃 修复：0013377:
            result = result.Where(v => v.ReportPreviewList != null && v.ReportPreviewList.Count > 0)?.ToList();
            return result;
        }

        /// <summary>
        /// 查询报告标签配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ReportLabelDTO>> FindReportLabelsAsync(FindReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var usageType = ReportLabelUsageTypeEnum.People;
            var organzation = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (organzation != null)
            {
                usageType = (ReportLabelUsageTypeEnum)organzation.PatientType;
            }
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    break;
                case ReportTypeEnum.LiveConsultation:
                    var dbRequest = new ConsultationRecordCodeDBRequest { Code = request.RecordCode };
                    var consultationRecord = await _consultationRecordDBService.FindConsultationRecordByCodeAsync(dbRequest);
                    if (consultationRecord == null || string.IsNullOrWhiteSpace(consultationRecord.ConsultationCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    break;
            }
            return await _reportDBService.GetReportLabelsAsync(new GetReportLabelsDBRequest { LanguageCode = request.LanguageCode, UsageType = usageType });
        }

        /// <summary>
        /// 根据分享短码查询分享内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ShareDTO> FindShareContentAsync(FindShareContentRequest request)
        {
            return await _shareDBService.GetShareInfoAsync(new GetShareInfoDBRequest { ShortCode = request.ShortCode });
        }

        /// <summary>
        /// 超声机创建并上传报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SaveUltrasonicReportAsync(SaveUltrasonicReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.DeviceExamId))
            {
                ThrowCustomerException(CustomerRpcCode.ExamNotExistError, "Exam not exist error");
            }
            var deviceCode = "";
            var patientCode = "";
            var record = await _recordsInfoDBService.FindRecordInfoByDeviceExamIdAsync(request.DeviceExamId);
            if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
            }
            deviceCode = record.DeviceCode;
            patientCode = record.PatientCode;
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            var resourceJsonObj = new
            {
                ReportMeasureDatasJson = request.ReportMeasureDatasJson,
                ReportDatasJson = request.ReportDatasJson
            };
            var resourceJson = JsonConvert.SerializeObject(resourceJsonObj);
            bool isNeedUpdate = false;
            try
            {
                Logger.WriteLineWarn($"Save Vinno Report,deviceCode: {record.DeviceCode}, patientCode: {record.PatientCode}, ReportDatasJson: {request.ReportDatasJson}");
                var newDataJsonDynamicList = new List<dynamic>();
                var reportDataJsonList = JsonConvert.DeserializeObject<List<UltrasonicReportDataJson>>(request.ReportDatasJson);
                var registerInfo = new WingInterfaceLibrary.DTO.Vital.RegisterInfoDTO();
                string ultrasonicSignature = "";
                string orgName = "";
                if (reportDataJsonList?.Count > 0)
                {
                    var patientInfo = reportDataJsonList.FirstOrDefault(c => c.Name == "PatientId") ?? new UltrasonicReportDataJson();
                    if (!string.IsNullOrEmpty(patientInfo?.Value))
                    {
                        var childVal = JObject.Parse(patientInfo.Value);
                        if (childVal != null)
                        {
                            if (childVal["ValueType"].ToString() == "String")
                            {
                                var patientCodeStr = childVal["Value"].ToString();
                                //获取登记信息
                                if (!string.IsNullOrEmpty(patientCodeStr))
                                {
                                    var regReq = new GetRegisterPersonRequest()
                                    {
                                        PhysicalExamNumber = patientCodeStr
                                    };
                                    registerInfo = await _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regReq);
                                }
                            }
                        }
                    }
                    
                    var orgInfo = new WingInterfaceLibrary.DTO.Vital.OrganizationDTO();
                    if (!string.IsNullOrEmpty(registerInfo.OrganizationCode))
                    {
                        Logger.WriteLineWarn($"Save Vinno Report,registerInfo: {JsonConvert.SerializeObject(registerInfo) }");
                        // 确认后的处理规则 当超声机只有绑定一个账号时，以此账号的电子签名为准；
                        var userDBReq = new WingInterfaceLibrary.Request.DBVitalRequest.GetUserListByKeyDBRequest()
                        {
                            Key = "UltrasoundCode",
                            Value = deviceCode
                        };
                        var userLists = await _vitalUserDBService.GetUserListByKeyAsync(userDBReq);
                        if (userLists?.Count > 0)
                        {
                            Logger.WriteLineWarn($"Save Vinno Report,userLists【DeviceCode】: {JsonConvert.SerializeObject(userLists) }");
                            var findEntity = userLists.Where(c => !string.IsNullOrEmpty(c.Signature) && c.OrganizationCode == registerInfo.OrganizationCode && c.Signature != "-1").OrderByDescending(c => c.UpdateTime).FirstOrDefault() ?? new WingInterfaceLibrary.DTO.Vital.UserDTO();
                            if (!string.IsNullOrEmpty(findEntity?.Signature))
                            {
                                ultrasonicSignature = findEntity.Signature;
                            }
                        }
                        // 当超声机未绑定账号，且账号只有一个时，以此电子签名为准；
                        // 当有多个账号时，以最后修改电子签名为准；
                        if (string.IsNullOrEmpty(ultrasonicSignature))
                        {
                            userDBReq = new WingInterfaceLibrary.Request.DBVitalRequest.GetUserListByKeyDBRequest()
                            {
                                Key = "RoleCode",
                                Value = "Role_Sonographer"
                            };
                            userLists = await _vitalUserDBService.GetUserListByKeyAsync(userDBReq);
                            if (userLists?.Count > 0)
                            {
                                Logger.WriteLineWarn($"Save Vinno Report,userLists【RoleName】: {JsonConvert.SerializeObject(userLists) }");
                                var findEntity = userLists.Where(c => !string.IsNullOrEmpty(c.Signature) && c.OrganizationCode == registerInfo.OrganizationCode && c.Signature != "-1").OrderByDescending(c => c.UpdateTime).FirstOrDefault() ?? new WingInterfaceLibrary.DTO.Vital.UserDTO();
                                if (!string.IsNullOrEmpty(findEntity?.Signature))
                                {
                                    ultrasonicSignature = findEntity.Signature;
                                }
                            }
                        }
                        Logger.WriteLineWarn($"Save Vinno Report,ultrasonicSignature One: {ultrasonicSignature}");
                        // 以上都没有的时候，去找账号所在的机构上的电子签名；
                        var orgInfoReq = new WingInterfaceLibrary.Request.DBVitalRequest.GetOrganizationDBRequest()
                        {
                            Code = registerInfo.OrganizationCode
                        };
                        orgInfo = await _vitalOrganizationDBService.GetOrganizationDetailAsync(orgInfoReq) ?? new WingInterfaceLibrary.DTO.Vital.OrganizationDTO();
                        if (string.IsNullOrEmpty(ultrasonicSignature))
                        {
                            ultrasonicSignature = orgInfo.UrinalysisSignature;
                        }
                        orgName = orgInfo.OrganizationName;
                    }
                    Logger.WriteLineWarn($"Save Vinno Report,ultrasonicSignature Two: {ultrasonicSignature}");

                    foreach (var item in reportDataJsonList)
                    {
                        if (!string.IsNullOrEmpty(item.Value))
                        {
                            var childVal = JObject.Parse(item.Value);
                            if (childVal != null)
                            {
                                if (childVal["ValueType"].ToString() == "String")
                                {
                                    var newVal = childVal["Value"].ToString();
                                    if (!string.IsNullOrEmpty(registerInfo?.Code))
                                    {
                                        if (item.Name == "PatientName")
                                        {
                                            newVal = registerInfo.Name;
                                        }
                                        else if (item.Name == "PatientGender")
                                        {
                                            newVal = registerInfo.Sex;
                                        }
                                        else if (item.Name == "PatientApplicationDepartment")
                                        {
                                            newVal = "体检科";
                                        }
                                        else if (item.Name == "PatientAge")
                                        {
                                            newVal = registerInfo.Age;
                                        }
                                    }
                                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                                    dynamicObj.Key = item.Key;
                                    dynamicObj.Value = newVal;
                                    dynamicObj.Name = item.Name;
                                    dynamicObj.TagId = item.TagId;
                                    newDataJsonDynamicList.Add(dynamicObj);
                                }
                                else if (childVal["ValueType"].ToString() == "ImageList")
                                {
                                    var imgDataList = new List<string>();
                                    var imgObjList = (JArray)childVal["Value"];
                                    if (imgObjList?.Count > 0)
                                    {
                                        foreach (var imgItem in imgObjList)
                                        {
                                            var tempImg = imgItem["Value"].ToString() ?? string.Empty;
                                            var imgUrl = await TransferBase64ToImg(tempImg);
                                            imgDataList.Add(imgUrl);
                                        }
                                    }

                                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                                    dynamicObj.Key = item.Key;
                                    dynamicObj.Value = imgDataList;
                                    dynamicObj.Name = item.Name;
                                    dynamicObj.TagId = item.TagId;
                                    newDataJsonDynamicList.Add(dynamicObj);
                                }
                                else if (childVal["ValueType"].ToString() == "CommentStringList")
                                {
                                    var descStr = string.Empty;
                                    var comStrList = (JArray)childVal["Value"];
                                    if (comStrList?.Count > 0)
                                    {
                                        foreach (var cItem in comStrList)
                                        {
                                            var tempDesc = cItem["Value"].ToString() ?? string.Empty;
                                            descStr += tempDesc + "\n";
                                        }
                                    }

                                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                                    dynamicObj.Key = item.Key;
                                    dynamicObj.Value = descStr;
                                    dynamicObj.Name = item.Name;
                                    dynamicObj.TagId = item.TagId;
                                    newDataJsonDynamicList.Add(dynamicObj);
                                }
                            }
                        }
                    }
                }

                if (!newDataJsonDynamicList.Exists(c => c.Name == "PatientName"))
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "97ec1395-3e8d-4ca9-a36a-0baace5646da";
                    dynamicObj.Value = registerInfo.Name;
                    dynamicObj.Name = "PatientName";
                    dynamicObj.TagId = "91bbaa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }
                if (!newDataJsonDynamicList.Exists(c => c.Name == "PatientGender"))
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "f2e65e15-1a55-47ca-9e6c-42f9e8659daf";
                    dynamicObj.Value = registerInfo.Sex;
                    dynamicObj.Name = "PatientGender";
                    dynamicObj.TagId = "93bbaa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }
                if (!newDataJsonDynamicList.Exists(c => c.Name == "PatientApplicationDepartment"))
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "135ec3fe-1a2b-46dd-98d7-cfbee06e2d66";
                    dynamicObj.Value = "体检科";
                    dynamicObj.Name = "PatientApplicationDepartment";
                    dynamicObj.TagId = "9a3baa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }
                if (!newDataJsonDynamicList.Exists(c => c.Name == "PatientAge"))
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "92bbaa61-a91d-4399-a3f4-df29117d2d59";
                    dynamicObj.Value = registerInfo.Age;
                    dynamicObj.Name = "PatientAge";
                    dynamicObj.TagId = "92bbaa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }
                if (newDataJsonDynamicList.Exists(c => c.Name == "HospitalName"))
                {
                    foreach (var item in newDataJsonDynamicList)
                    {
                        if (item.Name == "HospitalName" && !string.IsNullOrEmpty(orgName))
                        {
                            item.Value = orgName;
                            break;
                        }
                    }
                }
                else
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "1790e450-19a6-4000-8895-3dda6a43aef9";
                    dynamicObj.Value = orgName;
                    dynamicObj.Name = "HospitalName";
                    dynamicObj.TagId = "7abbaa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }

                if (newDataJsonDynamicList.Exists(c => c.Name == "ReportTime"))
                {
                    foreach (var item in newDataJsonDynamicList)
                    {
                        if (item.Name == "ReportTime")
                        {
                            item.Value = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                            break;
                        }
                    }
                }
                else
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "415df52f-3fef-46c5-a39f-c3e3090e79f2";
                    dynamicObj.Value = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    dynamicObj.Name = "ReportTime";
                    dynamicObj.TagId = "4abbaa61-a91d-4399-a3f4-df29117d2d59";

                    newDataJsonDynamicList.Add(dynamicObj);
                }

                // if (string.IsNullOrEmpty(ultrasonicSignature))
                // {
                //     var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "UltrasonicSign.png"); ;
                //     var fileToken = UploadFileAsync(path, Path.GetFileName(path)).Result;
                //     if (!string.IsNullOrWhiteSpace(fileToken))
                //     {
                //         ultrasonicSignature = fileToken;
                //     }
                // }
                if (newDataJsonDynamicList.Exists(c => c.Name == "DigitalSignature"))
                {
                    foreach (var item in newDataJsonDynamicList)
                    {
                        if (item.Name == "DigitalSignature")
                        {
                            item.Value = ultrasonicSignature;
                            break;
                        }
                    }
                }
                else
                {
                    dynamic dynamicObj = new System.Dynamic.ExpandoObject();
                    dynamicObj.Key = "779493a0-8900-4503-b77b-b65913842b8c";
                    dynamicObj.Value = ultrasonicSignature;
                    dynamicObj.Name = "DigitalSignature";
                    dynamicObj.TagId = "53545af1-59b7-41a8-9bd5-bdba1aaf9891";

                    newDataJsonDynamicList.Add(dynamicObj);
                }
                request.ReportDatasJson = JsonConvert.SerializeObject(newDataJsonDynamicList);
                request.ReportMeasureDatasJson = "[]";
                isNeedUpdate = true;

            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"SaveUltrasonicReportAsync Fill Data Error, RecordCode:{record.RecordCode}, error:{ex}");
            }
            var patientId = GetPatientId(request.ReportDatasJson);
            var patientName = "";
            var patientVal = record.PatientInfo?.FirstOrDefault(c => c.Key == "Name")?.Value ?? string.Empty;
            if (!string.IsNullOrEmpty(patientVal))
            {
                patientName = CryptionHelper.NameEncrypt(patientVal, true);
            }
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = record.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson ?? "{}",
                ReportDatasJson = request.ReportDatasJson,
                PatientId = patientId,
                PatientName = patientName,
                ReportLabels = new List<string>(),
                PlatformLabels = new List<string>(),
                DiagnosisLabels = new List<string>(),
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = "",
                ReportType = request.ReportType,
                IsReferral = false,
                ReportTemplateCode = resourceJson,
                ReportMeasureDatasJson = request.ReportMeasureDatasJson,
                DeviceReportId = request.DeviceReportId,
                ReportPDFUrl = request.ReportPDFUrl.ToUrlToken(),
                LanguageCode = "",
            });
            if (string.IsNullOrEmpty(reportId))
            {
                ThrowCustomerException(CustomerRpcCode.AddReportFailed, "Add report failed");
            }
            //添加成功，是否合并报告
            if (!string.IsNullOrWhiteSpace(record.DevicePatientID) && record.DevicePatientID.Length == 14)
            {
                var req = new CreateHEIUltrasonicExamAndReportRequest()
                {
                    UltrasonicCode = record.RecordCode,
                    Type = 1
                };
                var insertNew = await CreateHEIUltrasonicExamAndReport(req);
            }

            var reportPosters = _reportPosterObserver.GetDevicePosters(deviceCode);
            if (reportPosters != null && reportPosters.Count > 0)
            {
                Logger.WriteLineInfo($"ReportService AddReportAsync GetDevicePosters: patientName: {CryptionHelper.Decrypt(patientName)}, posters: {string.Join(",", reportPosters.Select(x => x.ReceiverName))}");
                foreach (var item in reportPosters)
                {
                    var addReportPushRecordDBRequest = new AddReportPushRecordDBRequest()
                    {
                        Data = new ReportPushRecordDTO()
                        {
                            PatientId = patientId,
                            PatientName = patientName,
                            PoseterConfigId = item.Code,
                            State = PushDataStateEnum.UnDo,
                            PoseterConfigName = item.ReceiverName,
                            ReportId = reportId,
                            PatientRecordId = record.RecordCode,
                            PushResultToken = string.Empty,
                            PushResponse = string.Empty,
                        }

                    };
                    var reportPushRecord = await _reportPosterDBService.AddReportPushRecordAsync(addReportPushRecordDBRequest);
                    if (reportPushRecord != null)
                    {
                        JoinQueue(reportPushRecord);
                    }
                }
            }
            if (isNeedUpdate)
            {
                JoinQueue(reportId, 1);
            }
            return true;
        }

        /// <summary>
        /// 创建报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddReportAsync(AddReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var deviceCode = "";
            var patientCode = "";
            var record = new RecordInfoDTO();
            var consultationRecord = new ConsultationDetailDTO();
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
                case ReportTypeEnum.LiveConsultation:
                    var dbRequest = new ConsultationRecordCodeDBRequest { Code = request.RecordCode };
                    consultationRecord = await _consultationRecordDBService.FindConsultationRecordByCodeAsync(dbRequest);
                    if (consultationRecord == null || string.IsNullOrWhiteSpace(consultationRecord.ConsultationCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = consultationRecord.DeviceCode;
                    patientCode = consultationRecord.PatientCode;
                    break;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !request.DiagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            if (!string.IsNullOrWhiteSpace(request.ReferralRecordCode))
            {
                var referralRecordInfo = await _remedicalDBService.GetReferralRecordInfoAsync(new GetReferralRecordByCodeDBRequest
                {
                    Code = request.ReferralRecordCode
                });
                if (referralRecordInfo == null || string.IsNullOrWhiteSpace(referralRecordInfo.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNotExisted, "Referral Record Not Existed");
                }
                if (referralRecordInfo.ReferralStatus != ReferralStatusEnum.Accepted)
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNeverAccepted, "Referral Record Never Accepted");
                }
            }
            // var reports = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
            // var hasReport = reports.Any(w => w.RecordCode == request.RecordCode && w.ReportUserCode == userCode);
            // if (hasReport)
            // {
            //     ThrowCustomerException(CustomerRpcCode.ReportSubmitted, "ReportSubmitted");
            // }
            var patientId = GetPatientId(request.ReportDatasJson);
            var reportId = await _reportDBService.SaveReportAsync(new SaveReportDBRequest
            {
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson ?? "{}",
                ReportDatasJson = request.ReportDatasJson,
                PatientId = patientId,
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = request.DiagnosisLabels ?? new List<string>(),
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
                ReportType = request.ReportType,
                IsReferral = request.IsReferral,
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = request.ReportMeasureDatasJson
            });
            if (string.IsNullOrEmpty(reportId))
            {
                ThrowCustomerException(CustomerRpcCode.AddReportFailed, "Add report failed");
            }
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var isUpdate = false;
                    if (record.RecordStatus != RecordStatusEnum.Completed)
                    {
                        record.RecordStatus = RecordStatusEnum.Completed;
                        isUpdate = true;
                    }
                    var diagnosisLabels = request.DiagnosisLabels ?? new List<string>();
                    if (!record.IsUserReport && !diagnosisLabels.Contains("VAid"))
                    {
                        record.IsUserReport = true;
                        isUpdate = true;
                    }
                    if (isUpdate)
                    {
                        await _recordsInfoDBService.UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
                    }
                    //如果是转诊报告
                    if (record.IsReferral && !string.IsNullOrWhiteSpace(userCode))
                    {
                        var userInfo = await _userService.FindUserByCodeAsync(userCode);
                        var referralDatas = await _recordsInfoDBService.FindReferralDatasByRecordCodeDBAsync(record.RecordCode);
                        foreach (var referralData in referralDatas)
                        {
                            if (referralData.ReferralInOrganizationCode == userInfo.RootOrganizationCode && referralData.ReferralStatus == RecordReferralStatusEnum.ReferralIn)
                            {
                                referralData.ReferralStatus = RecordReferralStatusEnum.Processed;
                                _recordsInfoDBService.UpdateReferralDataAsync(referralData);
                            }
                        }
                    }
                    break;
                case ReportTypeEnum.LiveConsultation:
                    if (consultationRecord.ConsultationStatus == TransactionStatusEnum.PendingReport)
                    {
                        consultationRecord.ConsultationStatus = TransactionStatusEnum.End;
                        await _consultationRecordDBService.UpdateConsultationRecordAsync(consultationRecord);
                        //修改日程状态
                        var updateScheduleStateRequest = new UpdateScheduleStateDBRequest
                        {
                            RelevanceCode = consultationRecord.ConsultationCode,
                            Status = TransactionStatusEnum.End,
                        };
                        await _userService.UpdateScheduleStateDBAsync(updateScheduleStateRequest);
                    }
                    break;
            }
            var reportPosters = _reportPosterObserver.GetDevicePosters(deviceCode);
            if (reportPosters != null && reportPosters.Count > 0)
            {
                Logger.WriteLineInfo($"ReportService AddReportAsync GetDevicePosters: patientName: {CryptionHelper.Decrypt(request.PatientName)}, posters: {string.Join(",", reportPosters.Select(x => x.ReceiverName))}");
                foreach (var item in reportPosters)
                {
                    var addReportPushRecordDBRequest = new AddReportPushRecordDBRequest()
                    {
                        Data = new ReportPushRecordDTO()
                        {
                            PatientId = patientId,
                            PatientName = request.PatientName,
                            PoseterConfigId = item.Code,
                            State = PushDataStateEnum.UnDo,
                            PoseterConfigName = item.ReceiverName,
                            ReportId = reportId,
                            PatientRecordId = request.RecordCode,
                            PushResultToken = string.Empty,
                            PushResponse = string.Empty,
                        }

                    };
                    var reportPushRecord = await _reportPosterDBService.AddReportPushRecordAsync(addReportPushRecordDBRequest);
                    if (reportPushRecord != null)
                    {
                        JoinQueue(reportPushRecord);
                    }
                }
            }
            JoinQueue(reportId);
            return reportId;
        }

        /// <summary>
        /// 修改报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportAsync(ModifyReportRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.RecordCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }
            var deviceCode = "";
            var patientCode = "";
            switch (request.ReportType)
            {
                case ReportTypeEnum.RemoteDiagnosis:
                    var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (record == null || string.IsNullOrWhiteSpace(record.RecordCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = record.DeviceCode;
                    patientCode = record.PatientCode;
                    break;
                case ReportTypeEnum.LiveConsultation:
                    var dbRequest = new ConsultationRecordCodeDBRequest { Code = request.RecordCode };
                    var consultationRecord = await _consultationRecordDBService.FindConsultationRecordByCodeAsync(dbRequest);
                    if (consultationRecord == null || string.IsNullOrWhiteSpace(consultationRecord.ConsultationCode))
                    {
                        ThrowCustomerException(CustomerRpcCode.RecordNotExistError, "RecordInfo not exist error");
                    }
                    deviceCode = consultationRecord.DeviceCode;
                    patientCode = consultationRecord.PatientCode;
                    break;
            }
            var diagnosisLabels = request.DiagnosisLabels ?? new List<string>();
            if (request.ReportOrgan != DiagnosisOrganEnum.Null && !string.IsNullOrWhiteSpace(userCode))
            {
                diagnosisLabels.Remove("VAid");
                var examReportList = await _reportDBService.GetReportByRecordCodeAsync(new GetReportByRecordCodeDBRequest { RecordCode = request.RecordCode });
                var userOrganReport = examReportList.FirstOrDefault(x => x.ReportUserCode == userCode && x.ReportOrgan == request.ReportOrgan);
                request.ReportCode = userOrganReport?.ReportCode ?? string.Empty;
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateJson) && !diagnosisLabels.Contains("VAid"))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateJson empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ReportDatasJson))
            {
                ThrowCustomerException(CustomerRpcCode.ReportDatasJsonEmpty, "ReportDatasJson empty error");
            }
            if (!string.IsNullOrWhiteSpace(request.ReferralRecordCode))
            {
                var referralRecordInfo = await _remedicalDBService.GetReferralRecordInfoAsync(new GetReferralRecordByCodeDBRequest
                {
                    Code = request.ReferralRecordCode
                });
                if (referralRecordInfo == null || string.IsNullOrWhiteSpace(referralRecordInfo.Code))
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNotExisted, "Referral Record Not Existed");
                }
                if (referralRecordInfo.ReferralStatus != ReferralStatusEnum.Accepted)
                {
                    ThrowCustomerException(CustomerRpcCode.ReferralRecordNeverAccepted, "Referral Record Never Accepted");
                }
            }
            var saveRequest = new SaveReportDBRequest
            {
                ReportCode = request.ReportCode,
                RecordCode = request.RecordCode,
                ReportUserCode = userCode,
                DeviceCode = deviceCode,
                PatientCode = patientCode,
                ReportTemplateJson = request.ReportTemplateJson ?? "{}",
                ReportDatasJson = request.ReportDatasJson,
                PatientId = GetPatientId(request.ReportDatasJson),
                PatientName = request.PatientName,
                ReportLabels = request.ReportLabels ?? new List<string>(),
                PlatformLabels = request.PlatformLabels ?? new List<string>(),
                DiagnosisLabels = diagnosisLabels,
                ReportOrgan = request.ReportOrgan,
                ReferralRecordCode = request.ReferralRecordCode,
                ReportTemplateCode = request.ReportTemplateCode,
                ReportMeasureDatasJson = request.ReportMeasureDatasJson
            };
            var reportId = await _reportDBService.SaveReportAsync(saveRequest);

            if (!string.IsNullOrEmpty(reportId))
            {
                if (request.ReportType == ReportTypeEnum.RemoteDiagnosis)
                {
                    var record = await _recordsInfoDBService.FindRecordInfoByCodeAsync(request.RecordCode);
                    if (!record.IsUserReport && !diagnosisLabels.Contains("VAid"))
                    {
                        record.IsUserReport = true;
                        await _recordsInfoDBService.UpdateRecordInfoByCodeAsync(record.RecordCode, record, null);
                    }
                    //如果是转诊报告
                    if (record.IsReferral && !string.IsNullOrWhiteSpace(userCode))
                    {
                        var userInfo = await _userService.FindUserByCodeAsync(userCode);
                        var referralDatas = await _recordsInfoDBService.FindReferralDatasByRecordCodeDBAsync(record.RecordCode);
                        foreach (var referralData in referralDatas)
                        {
                            if (referralData.ReferralInOrganizationCode == userInfo.RootOrganizationCode && referralData.ReferralStatus == RecordReferralStatusEnum.ReferralIn)
                            {
                                referralData.ReferralStatus = RecordReferralStatusEnum.Processed;
                                _recordsInfoDBService.UpdateReferralDataAsync(referralData);
                            }
                        }
                    }
                }
                var reportPosters = _reportPosterObserver.GetDevicePosters(deviceCode);
                if (reportPosters != null && reportPosters.Count > 0)
                {
                    foreach (var item in reportPosters)
                    {
                        var reportPushRecord = await _reportPosterDBService.
                        UpdatePushRecordStateByReportIdAsync(reportId, item.Code, PushDataStateEnum.UnDo);
                        if (reportPushRecord == null)
                        {
                            var addReportPushRecordDBRequest = new AddReportPushRecordDBRequest()
                            {
                                Data = new ReportPushRecordDTO()
                                {
                                    PatientId = saveRequest.PatientId,
                                    PatientName = saveRequest.PatientName,
                                    PoseterConfigId = item.Code,
                                    State = PushDataStateEnum.UnDo,
                                    PoseterConfigName = item.ReceiverName,
                                    ReportId = reportId,
                                    PatientRecordId = request.RecordCode,
                                    PushResultToken = string.Empty,
                                    PushResponse = string.Empty,
                                }

                            };
                            reportPushRecord = await _reportPosterDBService.AddReportPushRecordAsync(addReportPushRecordDBRequest);
                        }
                        JoinQueue(reportPushRecord);
                    }
                }
                DoReportConverter(new ReportConverterDetail()
                {
                    ReportCode = reportId
                });//直接生成报告预览图像
                // JoinQueue(reportId);
                return true;
            }

            return false;
        }

        /// <summary>
        /// 修改报告标签
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ModifyReportLabelsAsync(ModifyReportLabelsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.RecordCodeEmpty, "RecordCode empty error");
            }

            var result = await _reportDBService.SaveReportLabelsAsync(new SaveReportLabelsDBRequest
            {
                ReportCode = request.ReportCode,
                ReportLabels = request.ReportLabels ?? new List<string>()
            });
            return result;
        }

        /// <summary>
        /// 分享报告-短信
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportBySmsAsync(ShareReportBySmsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportCodeEmpty, "ReportCode empty error");
            }
            if (request.MobileList?.Any() != true)
            {
                ThrowCustomerException(CustomerRpcCode.MobileListEmpty, "MobileList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.MessageUrlEmpty, "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode, request.LanguageCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                ThrowCustomerException(CustomerRpcCode.ReportPageNeverConfig, "Report Url empty error");
            }
            if (string.IsNullOrWhiteSpace(_reportMessageTemplateId))
            {
                ThrowCustomerException(CustomerRpcCode.ReportSMSTempalteNeverConfig, "SMS Template Id empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                ThrowCustomerException(CustomerRpcCode.CreateShareCodeError, "ShareCode empty error");
            }

            return await _smsService.GeneralMessageAsync(request.MobileList, _reportMessageTemplateId, new List<string> { shortCode });
        }

        /// <summary>
        /// 分享报告-邮件
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ShareReportByEmailAsync(ShareReportByEmailRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportCodeEmpty, "ReportCode empty error");
            }
            if (request.EmailList?.Any() != true)
            {
                ThrowCustomerException(CustomerRpcCode.EmailListEmpty, "EmailList empty error");
            }
            if (string.IsNullOrWhiteSpace(_messageUrl))
            {
                ThrowCustomerException(CustomerRpcCode.MessageUrlEmpty, "Message Url empty error");
            }
            var reportUrl = string.Format(_reportUrl, request.ReportCode, request.LanguageCode);
            if (string.IsNullOrWhiteSpace(reportUrl))
            {
                ThrowCustomerException(CustomerRpcCode.ReportPageNeverConfig, "Report Url empty error");
            }
            var shortCode = await _shareDBService.AddShareInfoAsync(new AddShareInfoDBRequest { Data = new ShareDTO { ShareContent = reportUrl } });
            if (string.IsNullOrWhiteSpace(shortCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportSMSTempalteNeverConfig, "SMS Template Id empty error");
            }

            return await _emailService.SendEmailAsync(request.LanguageCode, request.EmailList, "ShareReport", _messageUrl, shortCode);
        }

        /// <summary>
        /// 获取病人id
        /// </summary>
        /// <param name="reportDatasJson"></param>
        /// <returns></returns>
        private string GetPatientId(string reportDatasJson)
        {
            try
            {
                if (!string.IsNullOrWhiteSpace(reportDatasJson))
                {
                    var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportDatasJson, new ReportElementDTOConverter());
                    foreach (var r in reportElements)
                    {
                        if (r is ReportElementStringDTO es && es.Name.ToLower() == "patientid")
                        {
                            return es.Value?.ToString() ?? string.Empty;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"get patient id err, {ex}");
            }
            return string.Empty;
        }

        /// <summary>
        /// 获取邮箱Base64字符串
        /// </summary>
        /// <param name="content"></param>
        /// <returns></returns>
        private string GetBase64EmailString(string content)
        {
            content = Convert.ToBase64String(Encoding.UTF8.GetBytes(content));
            var str = $"=?UTF8?B?{content}?=";
            return str;
        }

        /// <summary>
        /// 客户端用户验证
        /// </summary>
        /// <param name="request"></param>
        /// <returns>User Code</returns>
        private async Task<string> ValidateTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }

        /// <summary>
        /// 拷贝成个人报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<string> CopyPersonalReportTemplateAsync(CopyReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, userCode, string.Empty, true);
        }

        /// <summary>
        /// 拷贝成机构报告模板
        /// </summary>
        /// <returns></returns>
        public async Task<string> CopyOrgReportTemplateAsync(CopyOrgReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            return await _reportDBService.CopyReportTemplateAsync(request.ReportTemplateCode, string.Empty, userInfo.RootOrganizationCode, true);
        }

        /// <summary>
        /// 获取报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            var defaultReportTemplateCode = await _userService.GetUserDefaultReportTemplateCodeAsync(userCode, request.LanguageCode);
            dBRequest.PersonalCode = userCode;
            dBRequest.PageSize = request.PageSize;
            dBRequest.CurrentPage = request.PageIndex;
            dBRequest.ReportTemplateName = request.ReportTemplateName;
            dBRequest.IsDefault = request.IsDefault;
            dBRequest.LanguageCode = request.LanguageCode;
            dBRequest.BusinessType = request.BusinessType;
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo != null && !string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                dBRequest.OrganizationCode = userInfo.OrganizationCode;
                // var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
                // dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            }
            // else
            // {
            //     dBRequest.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
            // }
            var result = await _reportDBService.GetReportTemplatePageAsync(dBRequest);
            if (result?.PageData?.Count > 0)
            {
                var language = "en-US";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }
                result.PageData.ForEach(item =>
                {
                    if (item.TemplatePreviewList?.Count > 0)
                    {
                        item.TemplatePreviewList.ForEach(x =>
                        {
                            if (x.Language == language)
                            {
                                if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                                {
                                    item.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                                }
                            }
                        });
                        item.TemplatePreviewList = null;
                    }
                    if (item.ReportTemplateName == "HEIECG(Vertical)")
                    {
                        item.ReportTemplateName = "远程心电(纵向)";
                    }
                    else if (item.ReportTemplateName == "HEIECG(Horizontal)")
                    {
                        item.ReportTemplateName = "远程心电(横向)";
                    }
                });
                //处理个人默认模板异常数据处理
                if (!string.IsNullOrWhiteSpace(defaultReportTemplateCode))
                {
                    bool isNeedClearDefaultTemplateCode = true;
                    result.PageData.ForEach(item =>
                    {
                        if (item.ReportTemplateCode == defaultReportTemplateCode)
                        {
                            item.IsUserDefault = true;
                            isNeedClearDefaultTemplateCode = false;
                        }
                    });
                    if (isNeedClearDefaultTemplateCode)
                    {
                        var dbRequest = new RemoveTemplateInfoDBRequest();
                        dbRequest.Code = defaultReportTemplateCode;
                        dbRequest.UserCode = userCode;
                        dbRequest.Type = TemplateQueryTypeEnum.ReportTemplate;
                        await _reportDBService.ClearDefaultTemplateInfoAsync(dbRequest);
                        defaultReportTemplateCode = await _userService.GetUserDefaultReportTemplateCodeAsync(userCode, request.LanguageCode);
                        if (!string.IsNullOrWhiteSpace(defaultReportTemplateCode))
                        {
                            result.PageData.ForEach(item =>
                            {
                                if (item.ReportTemplateCode == defaultReportTemplateCode)
                                {
                                    item.IsUserDefault = true;
                                }
                            });
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 获取机构报告模板分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ReportTemplateDTO>> GetOrgReportTemplatePageAsync(FindReportTemplatePagesRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector || string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            FindReportTemplatePagesDBRequest dBRequest = new FindReportTemplatePagesDBRequest();
            dBRequest.PageSize = request.PageSize;
            dBRequest.CurrentPage = request.PageIndex;
            dBRequest.ReportTemplateName = request.ReportTemplateName;
            dBRequest.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            dBRequest.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dBRequest.IsDefault = request.IsDefault;
            dBRequest.LanguageCode = request.LanguageCode;
            var result = await _reportDBService.GetOrgReportTemplatePageAsync(dBRequest);
            if (result?.PageData?.Count > 0)
            {
                var language = "en-US";
                if (!string.IsNullOrEmpty(request.LanguageCode))
                {
                    language = request.LanguageCode;
                }
                result.PageData.ForEach(item =>
                {
                    if (item.TemplatePreviewList?.Count > 0)
                    {
                        item.TemplatePreviewList.ForEach(x =>
                        {
                            if (x.Language == language)
                            {
                                if (!string.IsNullOrEmpty(x.FileToken) && x.FileToken.Split(';').Length > 0)
                                {
                                    item.TemplatePreviewUrlList = x.FileToken.Split(';').ToList();
                                }
                            }
                        });
                        item.TemplatePreviewList = null;
                    }
                });
            }
            return result;
        }

        /// <summary>
        /// 添加报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            try
            {
                var json = JsonHelper.DeserializerJson<object>(request.ReportTemplateJson);
            }
            catch (Exception ex)
            {
                ThrowCustomerException(CustomerRpcCode.JsonError, "Json format error");
            }
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = true;
            dbRequest.Data.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson ?? "{}";
            dbRequest.Data.ReportDatasJson = request.ReportDatasJson ?? "{}";
            dbRequest.Data.ReportTemplateType = request.ReportTemplateType;
            dbRequest.Data.LanguageCode = request.LanguageCode;
            if (dbRequest.Data.ReportTemplateType == ReportTemplateTypeEnum.Personal)
            {
                dbRequest.Data.PersonalCode = userCode;
                if (!string.IsNullOrEmpty(userInfo.OrganizationCode))
                {
                    dbRequest.Data.OrganizationCode = userInfo.OrganizationCode;
                }
            }
            else if (dbRequest.Data.ReportTemplateType == ReportTemplateTypeEnum.Organization)
            {
                if (!userInfo.IsDirector)
                {
                    ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
                }
                dbRequest.Data.OrganizationCode = userInfo.OrganizationCode;
            }
            // var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            // if (orgInfo == null)
            // {
            //     ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            // }
            // dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;

            if (!string.IsNullOrEmpty(request.ReportTemplateJson))
            {
                JObject objJson = JObject.Parse(request.ReportTemplateJson);
                if (objJson["Applicable"].ToString() == "Person")
                {
                    dbRequest.Data.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.People;
                }
                else
                {
                    dbRequest.Data.ReportTemplateUsageType = ReportTemplateUsageTypeEnum.Animal;
                }
            }

            var resp = await _reportDBService.AddReportTemplateAsync(dbRequest);
            //添加生成队列，生成报告模板图片
            if (!string.IsNullOrWhiteSpace(resp?.ReportTemplateCode))
            {
                await CreateReportTemplatePreviewAsync(resp);
                return resp.ReportTemplateCode;
            }
            return string.Empty;
        }

        /// <summary>
        /// 验证报告模板名称是否重复
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CheckReportTemplateName(UpdateReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson;
            //验证报告模板名称是否重复
            var res = await _reportDBService.CheckReportTemplateName(dbRequest);
            return res;
        }

        /// <summary>
        /// 更新报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateReportTemplateAsync(UpdateReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson ?? "{}";
            dbRequest.Data.ReportDatasJson = request.ReportDatasJson;
            dbRequest.Data.IsDefault = true;
            var resp = await _reportDBService.UpdateReportTemplateAsync(dbRequest);
            //添加生成队列，生成报告模板图片
            var res = false;
            if (!string.IsNullOrEmpty(resp?.ReportTemplateCode))
            {
                res = true;
                await CreateReportTemplatePreviewAsync(resp);
            }
            return res;
        }

        /// <summary>
        /// 删除报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateCodeEmpty, "ReportTemplate code empty error");
            }
            // var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            // if (ReportTemplateInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            dbRequest.UserCode = userCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddOrgReportTemplateAsync(AddReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddReportTemplateDBRequest();
            dbRequest.Data = new ReportTemplateDTO();
            dbRequest.Data.IsDefault = true;
            dbRequest.Data.ReportTemplateStateType = ReportTemplateStatusTypeEnum.Published;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ReportTemplateName = request.ReportTemplateName;
            dbRequest.Data.ReportTemplateJson = request.ReportTemplateJson ?? "{}";
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ReportTemplateUsageType = (ReportTemplateUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ReportTemplateType = ReportTemplateTypeEnum.Organization;
            var resp = await _reportDBService.AddReportTemplateAsync(dbRequest);
            if (!string.IsNullOrEmpty(resp?.ReportTemplateCode))
            {
                await CreateReportTemplatePreviewAsync(resp);
                return resp.ReportTemplateCode;
            }
            return string.Empty;
        }

        /// <summary>
        /// 删除机构报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgReportTemplateAsync(RemoveReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateCodeEmpty, "ReportTemplate code empty error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var ReportTemplateInfo = await _reportDBService.GetReportTemplateByCodeAsync(request.ReportTemplateCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotDirector, "User not director");
            }
            // if (ReportTemplateInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            if (ReportTemplateInfo.OrganizationCode != userInfo.OrganizationCode && ReportTemplateInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveReportTemplateDBRequest();
            dbRequest.ReportTemplateCode = request.ReportTemplateCode;
            return await _reportDBService.RemoveReportTemplateAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusAsync(AddThesaurusRequest request)
        {
            var thesaurusBusinessType = request.ThesaurusBusinessType;
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = true;
            dbRequest.Data.PersonalCode = userCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            dbRequest.Data.LanguageCode = request.LanguageCode;
            dbRequest.Data.ThesaurusBusinessType = thesaurusBusinessType;
            var orgCode = "";
            if (thesaurusBusinessType == ThesaurusBusinessTypeEnum.Electrocardiogram)
            {
                var userInfo = await _vitalUserDBService.GetUserDetailAsync(new WingInterfaceLibrary.Request.DBVitalRequest.GetUserDBRequest { Code = userCode });
                if (userInfo == null)
                {
                    ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
                }
                var orgInfo = await _vitalOrganizationDBService.GetOrganizationDetailAsync(new WingInterfaceLibrary.Request.DBVitalRequest.GetOrganizationDBRequest { Code = userInfo.OrganizationCode });
                if (orgInfo != null)
                {
                    request.PatientType = OrganizationPatientTypeEnum.Person;
                    orgCode = orgInfo.Code;
                }
            }
            else
            {
                var userInfo = await _userService.FindUserByCodeAsync(userCode);
                if (userInfo == null)
                {
                    ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
                }
                var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
                if (orgInfo != null)
                {
                    request.PatientType = orgInfo.PatientType;
                    orgCode = orgInfo.RootCode;
                }
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)request.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Personal;
            dbRequest.Data.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
            dbRequest.Data.OrganizationCode = orgCode;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddOrgThesaurusAsync(AddThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo == null || !userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new AddThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.IsDefault = true;
            dbRequest.Data.OrganizationCode = userInfo.RootOrganizationCode;
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            dbRequest.Data.LanguageCode = request.LanguageCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            if (orgInfo == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "The user is not assigned an organization");
            }
            dbRequest.Data.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            dbRequest.Data.ThesaurusType = ThesaurusTypeEnum.Organization;
            dbRequest.Data.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
            return await _reportDBService.AddThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 更新词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusAsync(UpdateThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var dbRequest = new UpdateThesaurusDBRequest();
            dbRequest.Data = new ThesaurusDTO();
            dbRequest.Data.ThesaurusName = request.ThesaurusName;
            dbRequest.Data.ThesaurusCode = request.ThesaurusCode;
            return await _reportDBService.UpdateThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusClassifyAsync(AddThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                ThrowCustomerException(CustomerRpcCode.EditThesaurusItemError, "Thesaurus item name empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ThesaurusItemName = request.ThesaurusItemName,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddThesaurusContentAsync(AddThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new AddThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.AddThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusClassifyAsync(UpdateThesaurusClassifyRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusItemName))
            {
                ThrowCustomerException(CustomerRpcCode.EditThesaurusItemError, "Thesaurus item name empty error");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemName = request.ThesaurusItemName,
                ThesaurusItemType = ThesaurusItemTypeEnum.Classification,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 添加词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThesaurusContentAsync(UpdateThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new UpdateThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItem = new ThesaurusItemDTO
            {
                ThesaurusItemCode = request.ThesaurusItemCode,
                ThesaurusItemConclusion = request.ThesaurusItemConclusion ?? string.Empty,
                ThesaurusItemDescription = request.ThesaurusItemDescription ?? string.Empty,
                ThesaurusItemType = ThesaurusItemTypeEnum.Contents,
                ParentItemCode = request.ParentItemCode ?? string.Empty
            };
            return await _reportDBService.UpdateThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new RemoveTemplateInfoDBRequest();
            dbRequest.Code = request.ThesaurusCode;
            dbRequest.UserCode = userCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 删除词条项
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveThesaurusItemAsync(RemoveThesaurusItemRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }
            // var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            var dbRequest = new RemoveThesaurusItemDBRequest();
            dbRequest.ThesaurusCode = request.ThesaurusCode;
            dbRequest.ThesaurusItemCode = request.ThesaurusItemCode;
            return await _reportDBService.RemoveThesaurusItemAsync(dbRequest);
        }

        /// <summary>
        /// 获取词条库（分页）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            request.PersonalCode = userCode;
            var defaultThesaurusCode = await _userService.GetUserDefaultThesaurusCodeAsync(userCode, request.LanguageCode);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (userInfo != null && !string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                request.OrganizationCode = userInfo.OrganizationCode;
                // var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
                // request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            }
            // else
            // {
            //     request.ThesaurusUsageType = ThesaurusUsageTypeEnum.People;
            // }
            var result = await _reportDBService.GetThesaurusPageAsync(request);
            if (!string.IsNullOrWhiteSpace(defaultThesaurusCode))
            {
                bool isNeedClearDefaultThesaurusCode = true;
                result.PageData.ForEach(item =>
                {
                    if (item.ThesaurusCode == defaultThesaurusCode)
                    {
                        item.IsUserDefault = true;
                        isNeedClearDefaultThesaurusCode = false;
                    }
                });
                if (isNeedClearDefaultThesaurusCode)
                {
                    var dbRequest = new RemoveTemplateInfoDBRequest();
                    dbRequest.Code = defaultThesaurusCode;
                    dbRequest.UserCode = userCode;
                    dbRequest.Type = TemplateQueryTypeEnum.Thesaurus;
                    await _reportDBService.ClearDefaultTemplateInfoAsync(dbRequest);
                    defaultThesaurusCode = await _userService.GetUserDefaultThesaurusCodeAsync(userCode, request.LanguageCode);
                    if (!string.IsNullOrWhiteSpace(defaultThesaurusCode))
                    {
                        result.PageData.ForEach(item =>
                        {
                            if (item.ThesaurusCode == defaultThesaurusCode)
                            {
                                item.IsUserDefault = true;
                            }
                        });
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// 获取词条项列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusItemsAsync(GetThesaurusItemsRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsAsync(request);
            return list.OrderByDescending(s => s.Sort).ToList();
        }

        /// <summary>
        /// 获取词条项详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusItemDTO> GetThesaurusItemByCodeAsync(GetThesaurusItemByCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.GetThesaurusItemByCodeAsync(request);
        }


        /// <summary>
        /// 根据父节点编码获取词条分类
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusClassifyByParentCodeAsync(GetThesaurusClassifyByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Classification);
            return list.OrderByDescending(s => s.Sort).ToList();
        }

        /// <summary>
        /// 根据父节点编码获取词条内容
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<ThesaurusItemDTO>> GetThesaurusContentByParentCodeAsync(GetThesaurusContentByParentCodeRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var list = await _reportDBService.GetThesaurusItemsDetailListAsync(request.ThesaurusCode, request.ParentItemCode, request.Keyword, ThesaurusItemTypeEnum.Contents);
            return list.OrderByDescending(s => s.Sort).ToList();
        }

        /// <summary>
        /// 删除机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> RemoveOrgThesaurusAsync(RemoveThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code empty error");
            }

            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            var thesaurusInfo = await _reportDBService.GetThesaurusByCodeAsync(request.ThesaurusCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotDirector, "User not director");
            }
            // if (thesaurusInfo.IsDefault)
            // {
            //     ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            // }
            if (thesaurusInfo.OrganizationCode != userInfo.OrganizationCode && thesaurusInfo.OrganizationCode != userInfo.RootOrganizationCode)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var dbRequest = new RemoveTemplateInfoDBRequest();
            dbRequest.Code = request.ThesaurusCode;
            return await _reportDBService.RemoveThesaurusAsync(dbRequest);
        }

        /// <summary>
        /// 获取机构词条库分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ThesaurusDTO>> GetOrgThesaurusPageAsync(GetThesaurusPageRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector || string.IsNullOrWhiteSpace(userInfo.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            request.OrganizationCode = userInfo.OrganizationCode;
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userInfo.RootOrganizationCode });
            request.ThesaurusUsageType = (ThesaurusUsageTypeEnum)(int)orgInfo.PatientType;
            return await _reportDBService.GetOrgThesaurusPageAsync(request);
        }

        /// <summary>
        /// 拷贝成个人词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyPersonalThesaurusAsync(CopyThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, userCode, string.Empty);
        }

        /// <summary>
        /// 拷贝成机构词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CopyOrgThesaurusAsync(CopyOrgThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (!userInfo.IsDirector)
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            return await _reportDBService.CopyThesaurusAsync(request.ThesaurusCode, string.Empty, userInfo.RootOrganizationCode);
        }

        public string GetAIReportTemplate(string language, DiagnosisOrganEnum diagnosisOrgan)
        {
            var key = $"{language}_{diagnosisOrgan.ToString()}";
            if (_aiReportTemplateDictionary.TryGetValue(key, out var reportTemplate))
            {
                return reportTemplate ?? string.Empty;
            }
            string filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Diagnosis", "ReportTemplate", $"{language}.json");
            if (diagnosisOrgan == DiagnosisOrganEnum.CarotidArtery)
            {
                var carotidTemplateFile = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Diagnosis", "ReportTemplate", $"Carotid_AI_zh-CN_Flyinsono_20230912.json");
                if (File.Exists(carotidTemplateFile))
                {
                    filePath = carotidTemplateFile;
                }
            }
            if (File.Exists(filePath))
            {
                var content = File.ReadAllText(filePath) ?? string.Empty;
                _aiReportTemplateDictionary.AddOrUpdate(key, content, (d, h) => content);
                return content;
            }
            return string.Empty;
        }



        /// <summary>
        /// 获取用户默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ThesaurusAllDTO> GetDefaultThesaurusContentAsync(GetDefaultThesaurusContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var defaultThesaurus = await _userService.GetUserDefaultThesaurusContentAsync(userCode, request.LanguageCode);
            if (defaultThesaurus == null)
            {
                var pageRequest = new GetThesaurusPageRequest();
                pageRequest.Token = request.Token;
                await GetThesaurusPageAsync(pageRequest);
                defaultThesaurus = await _userService.GetUserDefaultThesaurusContentAsync(userCode, request.LanguageCode);
            }
            return defaultThesaurus;
        }

        /// <summary>
        /// 设置用户默认词条库
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultThesaurusAsync(SetUserDefaultThesaurusRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (string.IsNullOrWhiteSpace(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "ThesaurusCode is empty");
            }
            return await _userService.SetUserDefaultThesaurusCodeAsync(userCode, request.ThesaurusCode, request.LanguageCode);
        }

        /// <summary>
        /// 获取用户默认报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ReportTemplateDTO> GetDefaultReportTemplateContentAsync(GetDefaultReportTemplateContentRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            var defaultReportTemplate = await _userService.GetUserDefaultReportTemplateContentAsync(userCode, request.LanguageCode);
            if (defaultReportTemplate == null)
            {
                var pageRequest = new FindReportTemplatePagesRequest();
                pageRequest.Token = request.Token;
                pageRequest.IsDefault = true;
                var pages = await GetReportTemplatePageAsync(pageRequest);
                defaultReportTemplate = pages?.PageData?.FirstOrDefault(w => w.ReportTemplateStateType == ReportTemplateStatusTypeEnum.Published);
                if (defaultReportTemplate == null)
                {
                    ThrowCustomerException(CustomerRpcCode.NotSetDefaultReportTemplate, "NotSetDefaultReportTemplate");
                }
                else
                {
                    await _userService.SetUserDefaultReportTemplateCodeAsync(userCode, defaultReportTemplate.ReportTemplateCode);
                }
            }
            return defaultReportTemplate;
        }

        /// <summary>
        /// 设置用户默认报告模板
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetUserDefaultReportTemplateAsync(SetUserDefaultReportTemplateRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (string.IsNullOrWhiteSpace(request.ReportTemplateCode))
            {
                ThrowCustomerException(CustomerRpcCode.ReportTemplateJsonEmpty, "ReportTemplateCode is empty");
            }
            return await _userService.SetUserDefaultReportTemplateCodeAsync(userCode, request.ReportTemplateCode, request.LanguageCode);
        }

        #region 报告模板 预览 服务端生成 图片功能 
        /// <summary>
        /// 创建报告模板预览
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateReportTemplatePreviewAsync(GetReportTemplateDBRequest request)
        {
            var result = false;
            JoinTemplateQueue(request);
            return result;
        }
        #endregion

        public async Task<bool> SyncBatchReportInfoAsync(SyncBatchReportInfoRequest request)
        {
            await _reportDBService.SyncBatchReportInfoDBAsync(request);
            return true;
        }

        public async Task<bool> SaveReportPreviewAsync(FindReportByCodeRequest request)
        {
            var report = await _reportDBService.GetReportByCodeAsync(new GetReportByCodeDBRequest { ReportCode = request.ReportCode });
            JoinQueue(request.ReportCode, 1, false);
            return true;
        }

        /// <summary>
        /// 导入词条信息
        /// </summary>
        /// <param name="request">导入请求实体</param>
        /// <returns></returns>
        public async Task<bool> ImportThesaurusAsync(ImportThesaurusInfoRequest request)
        {
            var userCode = await ValidateTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (string.IsNullOrEmpty(request.ThesaurusName))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusNameEmpty, "Thesaurus Name is empty");
            }
            if (string.IsNullOrEmpty(request.LanguageCode))
            {
                ThrowCustomerException(CustomerRpcCode.LanguageCodeIsEmpty, "Language code is empty");
            }
            if (((int)request.ThesaurusType) <= 0)
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusTypeEmpty, "Thesaurus type is empty");
            }
            if (string.IsNullOrEmpty(request.ThesaurusName))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusNameEmpty, "Thesaurus Name is empty");
            }
            if (request.ThesaurusTemplateInfo == null || request.ThesaurusTemplateInfo.ThesaurusList == null || request.ThesaurusTemplateInfo.ThesaurusList.Count <= 0)
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusContentEmpty, "Thesaurus content is empty");
            }
            var userInfo = await _userService.FindUserByCodeAsync(userCode);
            if (string.IsNullOrWhiteSpace(userInfo?.UserCode))
            {
                ThrowCustomerException(CustomerRpcCode.AuthenticationError, "No permission");
            }
            if (!string.IsNullOrEmpty(userInfo?.OrganizationCode))
            {
                var orgInfo = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfo.OrganizationCode);
                if (!string.IsNullOrEmpty(orgInfo?.Code))
                {
                    var patientType = orgInfo.PatientType;
                    if (patientType == (int)OrganizationPatientTypeEnum.Animals)
                    {
                        request.ThesaurusUsageType = ThesaurusUsageTypeEnum.Animal;
                    }
                    else
                    {
                        request.ThesaurusUsageType = ThesaurusUsageTypeEnum.People;
                    }
                }
            }
            else
            {
                //病人类型，就用请求数据
            }
            if (request.ThesaurusType == ThesaurusTypeEnum.Organization)
            {
                request.OrganizationCode = userInfo.OrganizationCode;
            }
            else
            {
                request.OrganizationCode = userInfo.OrganizationCode;
                request.PersonalCode = userCode;
            }
            request.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
            var result = await _reportDBService.ExistThesaurusInfoDBAsync(request);
            if (result)
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusNameDuplication, "Thesaurus name duplication");
            }
            request.ThesaurusStateType = ThesaurusStatusTypeEnum.Published;
            var res = await _reportDBService.ImportThesaurusDBAsync(request);
            return res;
        }

        /// <summary>
        /// 导出词条信息
        /// </summary>
        /// <param name="request">导出请求实体</param>
        /// <returns></returns>
        public async Task<ExportThesaurusTemplateResult> ExportThesaurusAsync(FindThesaurusByCodeRequest request)
        {
            if (string.IsNullOrEmpty(request.ThesaurusCode))
            {
                ThrowCustomerException(CustomerRpcCode.ThesaurusCodeEmpty, "Thesaurus code is empty");
            }
            var res = await _reportDBService.ExportThesaurusDBAsync(request);
            return res;
        }




        /// <summary>
        /// 根据体检号查询报告
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<WingInterfaceLibrary.DTO.Vital.ReportDTO>> GetVitalReportInfoAsync(GetVitalReportInfoByPhysicalExamNumberRequest request)
        {
            var reportReq = new GetReportConditionRequest()
            {
                PhysicalExamNumber = request.PhysicalExamNumber
            };
            var reportList = await _vitalReportDBService.GetVitalReportListDBAsync(reportReq);
            var result = reportList.ToList();
            return result;
        }


    }

    internal class UltrasonicReportDataJson
    {
        public string Key { get; set; }
        public string Value { get; set; }
        public string Name { get; set; }
        public string TagId { get; set; }
    }
}

