﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The class define the grid cell' row index, column index and merge information.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class CellPostion : IEquatable<CellPostion>, IComparable<CellPostion>
    {
        /// <summary>
        /// Gets the row index in grid.
        /// </summary>
        [DataMember]
        public int Row { get; set; }

        /// <summary>
        /// Gets the column index in grid.
        /// </summary>
        [DataMember]
        public int Column { get; set; }

        /// <summary>
        /// Gets ther row span of the cell in grid.
        /// </summary>
        [DataMember]
        public int RowSpan { get; set; }

        /// <summary>
        /// Gets ther cloumn span of the cell in grid.
        /// </summary>
        [DataMember]
        public int ColumnSpan { get; set; }

        /// <summary>
        /// Initialize <see cref="CellPostion"/>
        /// </summary>
        /// <param name="row">The row index.</param>
        /// <param name="column">The column index.</param>
        /// <param name="rowSpan">The row span.</param>
        /// <param name="columnSpan">The column span.</param>
        public CellPostion(int row, int column, int rowSpan, int columnSpan)
        {
            Row = row;
            Column = column;
            RowSpan = rowSpan;
            ColumnSpan = columnSpan;
        }

        public int CompareTo(CellPostion other)
        {
            if (ReferenceEquals(this, other))
                return 0;

            if (other is null)
                return 1;

            var rowComparison = Row.CompareTo(other.Row);
            if (rowComparison != 0)
                return rowComparison;

            var columnComparison = Column.CompareTo(other.Column);
            if (columnComparison != 0)
                return columnComparison;

            var rowSpanComparison = RowSpan.CompareTo(other.RowSpan);
            return rowSpanComparison == 0 ? ColumnSpan.CompareTo(other.ColumnSpan) : rowSpanComparison;
        }

        public CellPostion Clone()
        {
            var cellPosition = new CellPostion(Row, Column, RowSpan, ColumnSpan);
            return cellPosition;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="other"></param>
        /// <returns></returns>
        public bool Equals(CellPostion other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return Row == other.Row && Column == other.Column && RowSpan == other.RowSpan && ColumnSpan == other.ColumnSpan;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((CellPostion)obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = Row;
                hashCode = (hashCode * 397) ^ Column;
                hashCode = (hashCode * 397) ^ RowSpan;
                hashCode = (hashCode * 397) ^ ColumnSpan;
                return hashCode;
            }
        }
    }
}
