﻿using System;
using System.Collections.Generic;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The interface for report element.
    /// </summary>
    public interface IElement : IReportElement
    {
        /// <summary>
        /// Gets or sets the element index.
        /// </summary>
        int Index { get; set; }

        /// <summary>
        /// Gets the parent element.
        /// </summary>
        IReportElement Parent { get; set; }

        /// <summary>
        /// Gets or sets the input tag to indicates what value should be show in this element.
        /// </summary>
        ElementTag Tag { get; set; }

        /// <summary>
        /// Gets or sets the input tag to indicates what value should be show in this element.
        /// </summary>
        MeasureTag MeasureTag { get; set; }

        /// <summary>
        /// Gets or sets the horizontal alignment of the element.
        /// </summary>
        HorizontalLayout HorizontalAlignment { get; set; }

        /// <summary>
        /// Gets or sets the vertical alignment of the element.
        /// </summary>
        VerticalLayout VerticalAlignment { get; set; }

        /// <summary>
        /// Gets the block element margin.
        /// </summary>
        RTThickness Margin { get; set; }

        /// <summary>
        /// Gets the element type.
        /// </summary>
        ElementType ElementType { get; }

        /// <summary>
        /// Gets the report template document.
        /// </summary>
        /// <returns></returns>
        ReportTemplateDocument GetReportTemplateDocument();

        IElement Clone();

        ElementTag GetFetusTag();

        IEnumerable<IElement> GetLogicalAncestry();
        bool IsVisibleIn(ReportInfo reportInfo);
        bool IsFoldVisibleIn(ReportInfo reportInfo);
        void Delete();
        void UnDelete(bool? isHeader,int originalIndex);

        void UpdateElementProperty(IElement element);

        event EventHandler OnElementPropertyChanged;
    }
}