﻿using System;
using System.Collections.Generic;
using Vinno.IUS.Common.Args;

namespace Vinno.vCloud.Report.Interfaces
{
    public interface IReportTemplateBody
    {
        /// <summary>
        /// The event occurs when blocks collection changes.
        /// </summary>
        event EventHandler<CollectionChangedArgs<IBlockElement>> BlocksCollectionChanged;

        /// <summary>
        /// Gets all the report template blocks.
        /// </summary>
        IReadOnlyList<IBlockElement> Blocks { get; }

        /// <summary>
        /// Add a block.
        /// </summary>
        /// <param name="block">The new added block.</param>
        void AddBlock(IBlockElement block);

        /// <summary>
        /// Remove a block.
        /// </summary>
        /// <param name="block">The removed block.</param>
        void RemoveBlock(IBlockElement block);

        /// <summary>
        /// Insert block element in specific index.
        /// </summary>
        /// <param name="block">The new block element.</param>
        /// <param name="index">The index.</param>
        void InsertBlock(IBlockElement block, int index);

        /// <summary>
        /// Gets all elements except table, cell, paragraph
        /// </summary>
        /// <returns></returns>
        IEnumerable<IElement> GetAllElements();
    }
}
