﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;

namespace Vinno.vCloud.Report.Interfaces
{
    /// <summary>
    /// The text element interface.
    /// </summary>
    public interface ITextElement : IInLineElemnt
    {
        event EventHandler OnFontStylesChanged;
        event EventHandler OnFontColorChanged;

        /// <summary>
        /// Gets or sets the font name.
        /// </summary>
        string FontName { get; set; }

        /// <summary>
        /// Gets or sets the font size. Unit is pt.
        /// </summary>
        double FontSize { get; set; }

        /// <summary>
        /// Gets or sets the font styles.
        /// </summary>
        List<RTFontStyle> FontStyles { get; set; }

        /// <summary>
        /// Gets or sets the font color.
        /// </summary>
        RTColor FontColor { get; set; }

        /// <summary>
        /// Gets or sets the background.
        /// </summary>
        RTColor Background { get; set; }

        /// <summary>
        /// Gets or sets the lenght of the characters
        /// </summary>
        int LineLength { get; set; }

        double LineWidth { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the text can wrap.
        /// </summary>
        bool TextWrap { get; set; }
    }

    /// <summary>
    /// Define the font style.
    /// </summary>
    [DataContract(Namespace = "")]
    public enum RTFontStyle
    {
        [EnumMember]
        Regular,
        [EnumMember]
        Bold,
        [EnumMember]
        Italic,
        [EnumMember]
        Underline,
        [EnumMember]
        Strikeout
    }
}