﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.JsonConverters
{
    public class CellJsonConverter : AbstractElementConverter
    {
        /// <summary>Writes the JSON representation of the object.</summary>
        /// <param name="writer">The <see cref="T:Newtonsoft.Json.JsonWriter" /> to write to.</param>
        /// <param name="value">The value.</param>
        /// <param name="serializer">The calling serializer.</param>
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            throw new NotImplementedException();
        }

        /// <summary>Reads the JSON representation of the object.</summary>
        /// <param name="reader">The <see cref="T:Newtonsoft.Json.JsonReader" /> to read from.</param>
        /// <param name="objectType">Type of the object.</param>
        /// <param name="existingValue">The existing value of object being read.</param>
        /// <param name="serializer">The calling serializer.</param>
        /// <returns>The object value.</returns>
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var jToken = JToken.ReadFrom(reader);
            var elementType = jToken["ElementType"]["Name"].ToString();
            if (elementType == "RTCell")
            {
                var rtCell = new RTCell(null);
                InitializeElement(rtCell, jToken);

                var colorSerializerSetting = new JsonSerializerSettings
                {
                    Converters = new List<JsonConverter> {new RTColorJsonConverter()}
                };
                rtCell.Background = JsonConvert.DeserializeObject<RTColor>(jToken["Background"].ToString(), colorSerializerSetting);
                rtCell.GroupField = jToken["GroupField"]?.ToString() ?? "";
                var borderSerializerSettings = new JsonSerializerSettings
                {
                    Converters =
                        new List<JsonConverter>
                        {
                            new StringEnumConverter(),
                            new RTColorJsonConverter()
                        }
                };
                rtCell.Borders = JsonConvert.DeserializeObject<Borders>(jToken["Borders"].ToString(), borderSerializerSettings);
                
                var settings = new JsonSerializerSettings
                {
                    ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                    Converters =
                        new List<JsonConverter>
                        {
                            new BlockJsonConverter()
                        }
                };
                var blocks =
                    JsonConvert.DeserializeObject<List<IBlockElement>>(jToken["Blocks"].ToString(), settings);
                foreach (var element in blocks)
                {
                    element.Parent = rtCell;
                    rtCell.AddBlock(element);
                }
                return rtCell;
            }

            return null;
        }

        /// <summary>
        /// Determines whether this instance can convert the ICell type.
        /// </summary>
        /// <param name="objectType">Type of the object.</param>
        /// <returns>
        /// 	<c>true</c> if this instance can convert the specified object type; otherwise, <c>false</c>.
        /// </returns>
        public override bool CanConvert(Type objectType)
        {
            return typeof(ICell).IsAssignableFrom(objectType);
        }
    }
}