﻿using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.JsonConverters
{
    public class DictionaryJsonConverter<TKey, TValue> : JsonConverter
    {
        public override bool CanConvert(Type objectType)
        {
            return objectType == typeof(Dictionary<TKey, TValue>);
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue,
            JsonSerializer serializer)
        {
            return serializer.Deserialize<KeyValuePair<TKey, TValue>[]>(reader)
                .ToDictionary(x => x.Key, x => x.Value);
        }

        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            var array = ((Dictionary<TKey, TValue>) value).ToArray();
            serializer.Serialize(writer, array);
        }
    }
}