﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.Models;

namespace Vinno.vCloud.Report.JsonConverters
{
    /// <summary>
    /// "Keep it only for compatible, use <see cref="DictionaryJsonConverter{ElementTag, IElementValue}"/> instead
    /// </summary>
    [Obsolete]
    public class ElementValueDictionaryJsonConverter: JsonConverter
    {
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            if (value is Dictionary<ElementTag, IElementValue> dictionary)
            {
                //Loop over the Dictionary array and write each one
                writer.WriteStartArray();
                foreach (var entry in dictionary)
                {
                    //Write Key (Vector) 
                    serializer.Serialize(writer, entry.Key);
                    //Write Value (Collection)
                    serializer.Serialize(writer, entry.Value);
                }
                writer.WriteEndArray();
                return;
            }
            writer.WriteStartObject();
            writer.WriteEndObject();
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            //Where to re-create the json data into 
            var dict = new Dictionary<ElementTag, IElementValue>();
            if (reader.TokenType == JsonToken.StartArray)
            {
                var jArray = JArray.Load(reader);
                if (jArray != null && jArray.Count >= 2)
                {
                    //Do the loop faster with +=2
                    for (var i = 0; i < jArray.Count; i += 2)
                    {
                        //first item = key
                        var firstData = jArray[i + 0].ToString();
                        //second item = value
                        var secondData = jArray[i + 1].ToString();

                        //Create Vector2Int key data 
                        var elementTag = JsonConvert.DeserializeObject<ElementTag>(firstData);

                        //Create Collection value data
                        var settings = new JsonSerializerSettings
                        {
                            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                            Converters = new List<JsonConverter>
                            {
                                new StringEnumConverter(),
                                new ElementValueJsonConverter()
                            }
                        };
                        var elementValue = JsonConvert.DeserializeObject<IElementValue>(secondData, settings);

                        //Add both Key and Value to the Dictionary if key doesnt exit yet
                        if (!dict.ContainsKey(elementTag))
                            dict.Add(elementTag, elementValue);
                    }
                }
            }

            return dict;
        }

        public override bool CanConvert(Type objectType)
        {
            return typeof(Dictionary<ElementTag, IElementValue>).IsAssignableFrom(objectType);
        }
    }
}