﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using Vinno.vCloud.Report.Models;
using ValueType = Vinno.vCloud.Report.Models.ValueType;

namespace Vinno.vCloud.Report.JsonConverters
{
    public class ElementValueJsonConverter: JsonConverter
    {
        public override bool CanWrite => false;

        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            var jToken = JToken.ReadFrom(reader);
            var value = jToken.ToString();
            if (Enum.TryParse(jToken["ValueType"].ToString(), out ValueType valueType))
            {
                switch (valueType)
                {
                    case ValueType.String:
                        return JsonConvert.DeserializeObject<TextElementValue>(value);
                    case ValueType.Integer:
                        return JsonConvert.DeserializeObject<IntegerElementValue>(value);
                    case ValueType.Float:
                        return JsonConvert.DeserializeObject<FloatElementValue>(value);
                    case ValueType.Datetime:
                        return JsonConvert.DeserializeObject<DateTimeElementValue>(value);
                    case ValueType.BufferImage:
                        return JsonConvert.DeserializeObject<BufferImageValue>(value);
                    case ValueType.FileImage:
                        return JsonConvert.DeserializeObject<FileImageValue>(value);
                    case ValueType.StringList:
                        var stringListElementValueJsonSetting = new JsonSerializerSettings
                        {
                            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                            Converters =
                                new List<JsonConverter>
                                {
                                    new StringEnumConverter(),
                                    new ElementValueJsonConverter()
                                }
                        };
                        var stringValues =JsonConvert.DeserializeObject<List<TextElementValue>>(jToken["Value"].ToString(), stringListElementValueJsonSetting);
                        return new StringListElementValue(stringValues);
                    case ValueType.ImageList:
                        var listElementValueJsonSetting = new JsonSerializerSettings
                        {
                            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                            Converters =
                                new List<JsonConverter>
                                {
                                    new StringEnumConverter(),
                                    new ElementValueJsonConverter()
                                }
                        };
                        var values= JsonConvert.DeserializeObject<IEnumerable<IImageElementValue>>(jToken["Value"].ToString(), listElementValueJsonSetting);
                        return new ImageListElementValue(values);
                    case ValueType.CommentString:
                        return JsonConvert.DeserializeObject<CommentTextElementValue>(value);
                    case ValueType.CommentStringList:
                        var reportCommentElementValueJsonSetting = new JsonSerializerSettings
                        {
                            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                            Converters =
                                new List<JsonConverter>
                                {
                                    new StringEnumConverter(),
                                    new ElementValueJsonConverter()
                                }
                        };
                        var commentValues = JsonConvert.DeserializeObject<List<CommentTextElementValue>>(jToken["Value"].ToString(), reportCommentElementValueJsonSetting);
                        return new ReportCommentsElementValue(commentValues);
                }
            }

            return null;
        }

        public override bool CanConvert(Type objectType)
        {
            return typeof(IElementValue).IsAssignableFrom(objectType);
        }
    }
}