﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class AtlasElement : Element, IBlockElement, IInputImageListElement
    {
        public IReadOnlyList<IElement> Elements { get; private set; }
       
        [JsonIgnore]
        [DataMember]
        private List<IElement> DataContractElements
        {
            get => new List<IElement>(Elements);
            set => Elements = value;
        }

        /// <summary>Column Count</summary>
        [DataMember]
        public int Column { get; set; }

        /// <summary>Row Count</summary>
        [DataMember]
        public int Row { get; set; }

        /// <summary>Border Visibility</summary>
        [DataMember]
        public bool HasImageBorder { get; set; }

        /// <summary>ReadOnly</summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>Transparent Background</summary>
        [DataMember]
        public bool TransparentBackground { get; set; }

        /// <summary>CellSpacing</summary>
        [DataMember]
        public bool HasGap { get; set; }

        public AtlasElement(IReportElement parent) : base(parent)
        {
            Elements = new List<IElement>();
            ElementType = ElementType.AtlasElement;
            HorizontalAlignment = HorizontalLayout.Center;
            VerticalAlignment = VerticalLayout.Top;
            Column = 2;
            Row = 4;
            HasGap = true;
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var atlasElement = new AtlasElement(null);
            InitializeCloneElement(atlasElement);
            atlasElement.Row = Row;
            atlasElement.Column = Column;
            atlasElement.HasImageBorder = HasImageBorder;
            atlasElement.TransparentBackground = TransparentBackground;
            atlasElement.HasGap = HasGap;
            return atlasElement;
        }

        /// <summary>
        /// Get DefaultValue
        /// </summary>
        public IElementValue GetDefaultValue()
        {
            return new ImageListElementValue(new List<IImageElementValue>());
        }

        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is AtlasElement atlasElement)
            {
                Row = atlasElement.Row;
                Column = atlasElement.Column;
                HasImageBorder = atlasElement.HasImageBorder;
                TransparentBackground = atlasElement.TransparentBackground;
                HasGap = atlasElement.HasGap;
                base.UpdateElementProperty(atlasElement);
            }
        }
    }
}
