﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using Newtonsoft.Json;

namespace Vinno.vCloud.Report.Models
{
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class CustomizeCalculation
    {
        public CustomizeCalculation()
        {
            Unit = "None";
            ReferencedItems = new List<ReferenceItem>();
        }

        [JsonIgnore] public string Description { get; set; }

        [DataMember] public string Formula { get; set; }

        [DataMember] public string Id { get; set; }

        [JsonIgnore] public string ParsedFormula { get; private set; }

        [JsonIgnore] public List<ReferenceItem> ReferencedItems { get; }

        [DataMember] public string Unit { get; set; }

        protected bool Equals(CustomizeCalculation other)
        {
            return Formula == other.Formula && Id == other.Id && Unit == other.Unit;
        }

        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;

            if (ReferenceEquals(this, obj))
                return true;

            return obj.GetType() == GetType() && Equals((CustomizeCalculation) obj);
        }

        [SuppressMessage("ReSharper", "NonReadonlyMemberInGetHashCode")]
        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = Formula != null ? Formula.GetHashCode() : 0;
                hashCode = (hashCode * 397) ^ (Id != null ? Id.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Unit != null ? Unit.GetHashCode() : 0);
                return hashCode;
            }
        }

        public void ParseFormula()
        {
            ReferencedItems.Clear();
            ParsedFormula = string.Empty;
            var formula = Formula;
            if (string.IsNullOrEmpty(formula) || !formula.Contains('['))
                return;

            var refItems = new List<ReferenceItem>();
            var bracketNum = -1;
            var leftPos = -1;
            var target = new StringBuilder(70);
            var markStack = new Stack<char>();
            for (var i = 0; i < formula.Length; i++)
                switch (formula[i])
                {
                    case '[':
                        leftPos = i;
                        markStack.Push(formula[i]);
                        break;
                    case ']' when leftPos + 1 == i:
                    case ']' when markStack.Count == 0:
                        return;
                    case ']':
                        markStack.Pop();
                        bracketNum++;
                        target.Append('{').Append(bracketNum).Append('}');
                        var item = formula.Substring(leftPos + 1, i - leftPos - 1);

                        string category;
                        string unit;
                        var beginPos = item.IndexOf("\"", StringComparison.Ordinal);
                        var endPos = item.LastIndexOf("\"", StringComparison.Ordinal);
                        if (beginPos == -1 || endPos == -1 || endPos <= beginPos)
                            return;

                        var unitAndCategory = item.Substring(beginPos + 1, endPos - beginPos - 1);
                        var parts = unitAndCategory.Split(';');
                        if (parts.Length < 2) return;

                        category = parts[1];
                        if (string.IsNullOrEmpty(category) ||
                            !string.IsNullOrEmpty(parts[0]) && !MeasureXmlData.UnitExs.ContainsKey(parts[0]))
                            return;

                        unit = parts[0];
                        item = item.Substring(0, beginPos);
                        var referenceItem = new ReferenceItem(item, unit, category, this);
                        refItems.Add(referenceItem);
                        leftPos = -1;
                        break;
                    default:
                        if (leftPos < 0)
                            target.Append(formula[i]);

                        break;
                }

            if (markStack.Count == 0 && refItems.Count > 0)
            {
                ParsedFormula = target.ToString();
                ReferencedItems.AddRange(refItems);
            }
        }
    }
}