﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The date time element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class DateTimeElement: TextElement, IDateTimeElement
    {
        private bool _isReadOnly;
        private string _dateTimeFormat;
        private bool _isRealTime;
        private bool _showEnabled;
        /// <summary>
        /// Ges or sets the value to indicate whether the elment is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// Gets or sets the date time format.
        /// </summary>
        [DataMember]
        public string DateTimeFormat { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the element is current time.
        /// </summary>
        [DataMember]
        public bool IsRealTime { get; set; }

        [DataMember]
        public bool ShowEnabled { get; set; }

        /// <summary>
        /// Initialize a <see cref="DateTimeElement"/>
        /// </summary>
        /// <param name="parent"></param>
        public DateTimeElement(IElement parent) : base(parent)
        {
            ElementType = ElementType.DateTime;
            DateTimeFormat = "yyyy-MM-dd";
            ShowEnabled = true;
        }
        /// <summary>
        /// Get DefaultValue
        /// </summary>
        public IElementValue GetDefaultValue()
        {
            if (IsRealTime)
            {
                return new DateTimeElementValue(DateTime.Now);
            }
            else
            {
                return new DateTimeElementValue(null);
            }
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var dateTimeElement = new DateTimeElement(null);
            InitializeCloneTextElement(dateTimeElement);
            dateTimeElement.IsReadOnly = IsReadOnly;
            dateTimeElement.DateTimeFormat = DateTimeFormat;
            dateTimeElement.IsRealTime = IsRealTime;
            dateTimeElement.ShowEnabled = ShowEnabled;
            return dateTimeElement;
        }

        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is DateTimeElement dateTimeElement)
            {
                IsReadOnly = dateTimeElement.IsReadOnly;
                DateTimeFormat = dateTimeElement.DateTimeFormat;
                IsRealTime = dateTimeElement.IsRealTime;
                ShowEnabled = dateTimeElement.ShowEnabled;
                base.UpdateElementProperty(dateTimeElement);
            }
        }
    }
}