﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The input image element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class InputImage: RTImage, IInputImage, IInputImageListElement
    {
        public static readonly RTColor ImageBorderColor = RTColor.DimGray;
        public const int ImageBorderThickness = 1;
        public const int ImageBorderPadding = 3;
        public const int ImageBorderIncreament = ImageBorderThickness + ImageBorderPadding * 2;

        /// <summary>
        /// Gets or sets the input image comment.
        /// </summary>
        [DataMember]
        public string Comment { get; set; }

        /// <summary>
        /// Ges or sets the value to indicate whether the elment is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// Gets or sets the value to indicate whether the image has border or not.
        /// </summary>
        [DataMember]
        public bool HasBorder { get; set; }

        [DataMember]
        public bool TransparentBackground { get; set; }

        public int Column
        {
            get => 1;
            set { }
        }

        public bool HasGap
        {
            get => false;
            set { }
        }

        public bool HasImageBorder
        {
            get => HasBorder;
            set { }
        }

        /// <summary>
        /// Initialzie a <see cref="InputImage"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public InputImage(IElement parent) : base(parent)
        {
            ElementType = ElementType.InputImage;
            HorizontalAlignment = HorizontalLayout.Center;
            VerticalAlignment = VerticalLayout.Center;
        }

        public IElementValue GetDefaultValue()
        {
            return new FileImageValue(string.Empty);
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var inputImage= new InputImage(null);
            InitilizeCloneImageElement(inputImage);
            inputImage.Comment = Comment;
            inputImage.IsReadOnly = IsReadOnly;
            inputImage.HasBorder = HasBorder;
            inputImage.TransparentBackground = TransparentBackground;
            return inputImage;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is InputImage inputImage)
            {
                Comment = inputImage.Comment;
                IsReadOnly = inputImage.IsReadOnly;
                HasBorder = inputImage.HasBorder;
                TransparentBackground = inputImage.TransparentBackground;
                base.UpdateElementProperty(inputImage);
            }
        }
    }
}