﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The input text elment class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class InputText : TextElement, IInputText
    {
        /// <summary>
        /// Ges or sets the value to indicate whether the elment is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// Initialize a <see cref="InputText"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public InputText(IElement parent) : base(parent)
        {
            ElementType = ElementType.InputText;
        }
        /// <summary>
        /// Get DefaultValue
        /// </summary>
        public IElementValue GetDefaultValue()
        {
            return new TextElementValue(string.Empty);
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var inputText = new InputText(null);
            InitializeCloneTextElement(inputText);
            inputText.IsReadOnly = IsReadOnly;
            return inputText;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is InputText inputText)
            {
                IsReadOnly = inputText.IsReadOnly;
                base.UpdateElementProperty(inputText);
            }
        }
    }
}