﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The line element.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class Line: Shape, ILine
    {
        /// <summary>
        /// Gets or sets the line width.unit is pt.
        /// </summary>
        [DataMember]
        public double Width { get; set; }

        /// <summary>
        /// Gets or sets the dash style.
        /// </summary>
        [DataMember]
        public RTBorderStyle BorderStyle { get; set; }

        /// <summary>
        /// Initialize a <see cref="Line"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public Line(IElement parent) : base(parent)
        {
            ElementType = ElementType.Line;
            BorderStyle = RTBorderStyle.Solid;
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var line = new Line(null);
            InitializeCloneShape(line);
            line.Width = Width;
            line.BorderStyle = BorderStyle;
            return line;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is Line line)
            {
                Width = line.Width;
                BorderStyle = line.BorderStyle;
                SetShapeProperty(line);
            }
        }
    }
}