﻿using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Vinno.IUS.Common.Log;
using Vinno.IUS.Common.Utilities;
using Vinno.vCloud.Report.JsonConverters;

namespace Vinno.vCloud.Report.Models
{
    public class MeasureDataSerializer
    {
        public static string Serialize(MeasureDataInfo measureDataInfo, bool indented = true)
        {
            var settings = new JsonSerializerSettings
            {
                ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                Converters =
                    new List<JsonConverter>
                    {
                        new StringEnumConverter(),                        
                        new DictionaryJsonConverter<MeasureTagMatchInfo, string[]>(),
                    }
            };
            return JsonConvert.SerializeObject(measureDataInfo, indented ? Formatting.Indented : Formatting.None, settings);
        }

        /// <summary>
        /// Serialize MeasureDataInfo.
        /// </summary>
        /// <param name="filePath">The measure data file path.</param>
        /// <param name="measureDataInfo">The measure data info to be serialized.</param>
        public static void Serialize(string filePath, MeasureDataInfo measureDataInfo)
        {
            var data = new UTF8Encoding().GetBytes(Serialize(measureDataInfo));
            FileHelper.SaveFile(filePath, data);
        }

        /// <summary>
        /// Deserialize MeasureDataInfo.
        /// </summary>
        /// <param name="jsonStr">The json string.</param>
        /// <returns>The <see cref="MeasureDataInfo"/></returns>
        public static MeasureDataInfo Deserialize(string jsonStr)
        {
            try
            {
                var settings = new JsonSerializerSettings
                {
                    ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                    Converters =
                    new List<JsonConverter>
                    {
                        new StringEnumConverter(),
                        new DictionaryJsonConverter<MeasureTagMatchInfo, string[]>(),
                    }
                };
                return JsonConvert.DeserializeObject<MeasureDataInfo>(jsonStr, settings);
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize measure data info error {exception}");
                return null;
            }
        }

        /// <summary>
        /// Deserialize MeasureDataInfo.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The <see cref="MeasureDataInfo"/></returns>
        public static MeasureDataInfo DeserializeFromFile(string filePath)
        {
            try
            {
                if (File.Exists(filePath))
                {
                    return Deserialize(File.ReadAllText(filePath));
                }                
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize measure data info error {exception}");                
            }

            return null;
        }
    }
}
