﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The multi selected element.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class MultiSelectedVertical : TextElement, IMultiSelected
    {
        //private SelectorDisplayStyle _displayStyle;
        private List<string> _items;

        /// <summary>
        /// Ges or sets the value to indicate whether the element is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// Gets the data items source.
        /// </summary>
        [DataMember]
        public List<string> Items
        {
            get => _items;
            set
            {
                _items = value;
            }
        }

        [DataMember]
        public SelectorDisplayStyle DisplayStyle { get; set; }

        /// <summary>
        /// Initialize a <see cref="MultiSelected"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public MultiSelectedVertical(IElement parent) : base(parent)
        {
            //DisplayStyle = SelectorDisplayStyle.OnlyShowSelectedItem;
            ElementType = ElementType.MultiSelectedVertical;
            Items = new List<string>();
        }
        /// <summary>
        /// Get DefaultValue
        /// </summary>
        public IElementValue GetDefaultValue()
        {
            return new StringListElementValue(new List<TextElementValue>());
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var multiSelected = new MultiSelectedVertical(null);
            InitializeCloneTextElement(multiSelected);
            multiSelected.IsReadOnly = IsReadOnly;
            multiSelected.Items = Items.ToList();
            //multiSelected.DisplayStyle = DisplayStyle;
            return multiSelected;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is MultiSelectedVertical multiSelected)
            {
                IsReadOnly = multiSelected.IsReadOnly;
                Items = multiSelected.Items;
                //DisplayStyle = multiSelected.DisplayStyle;
                base.UpdateElementProperty(multiSelected);
            }
        }
    }
}