﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The element to show page number.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class PageNumberElement: TextElement, IPageNumberElement
    {
        /// <summary>
        /// Gets or sets the page number style.
        /// </summary>
        [DataMember]
        public PageNumberStyle PageNumberStyle { get; set; }

        /// <summary>
        /// Initialize a <see cref="PageNumberElement"/>
        /// </summary>
        /// <param name="parent">The parent.</param>
        public PageNumberElement(IElement parent) : base(parent)
        {
            ElementType = ElementType.PageNumber;
        }
        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var pageNumberElement = new PageNumberElement(null);
            InitializeCloneTextElement(pageNumberElement);
            pageNumberElement.PageNumberStyle = PageNumberStyle;
            return pageNumberElement;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is PageNumberElement pageNumberElement)
            {
                PageNumberStyle = pageNumberElement.PageNumberStyle;
                base.UpdateElementProperty(pageNumberElement);
            }
        }
    }
}