﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// Define the boder information.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTBorder:IEquatable<RTBorder>
    {
        private RTBorderStyle _borderStyle;
        public event EventHandler BorderStyleChanged;
        private RTColor _color;
        public event EventHandler BorderColorChanged;

        /// <summary>
        /// Gets or sets the border style.
        /// </summary>
        [DataMember]
        public RTBorderStyle BorderStyle
        {
            get { return _borderStyle; }
            set
            {
                if (_borderStyle != value)
                {
                    _borderStyle = value;
                    BorderStyleChanged?.Invoke(this, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets or sets the border color.
        /// </summary>
        [DataMember]
        public RTColor Color
        {
            get { return _color; }
            set
            {
                if (!_color.Equals(value))
                {
                    _color = value;
                    BorderColorChanged?.Invoke(this, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets or sets the border thickness. Unit is pt.
        /// </summary>
        [DataMember]
        public float Thickness { get; set; }

        public RTBorder()
        {
            BorderStyle = RTBorderStyle.Solid;
            Color = RTColor.Black;
            Thickness = 1;
        }

        /// <summary>
        /// Update self
        /// </summary>
        public void Update(RTBorder border)
        {
            BorderStyle = border.BorderStyle;
            Color = border.Color;
            Thickness = border.Thickness;
        }

        /// <summary>
        /// Compare whether the border contents are the same
        /// </summary>
        public bool Equals(RTBorder other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return BorderStyle == other.BorderStyle && Color.Equals(other.Color) && Thickness.Equals(other.Thickness);
        }

        /// <summary>
        /// Compare whether the two borders are the same
        /// </summary>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((RTBorder) obj);
        }

        /// <summary>
        /// Get HashCode
        /// </summary>
        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = (int) BorderStyle;
                hashCode = (hashCode * 397) ^ Color.GetHashCode();
                hashCode = (hashCode * 397) ^ Thickness.GetHashCode();
                return hashCode;
            }
        }
    }

    [DataContract(Namespace = "")]
    public enum RTBorderStyle
    {
        [EnumMember]
        None,
        [EnumMember]
        Solid,
        [EnumMember]
        Dash,
        [EnumMember]
        Dot,
        [EnumMember]
        DashDot,
        [EnumMember]
        DashDotDot,
        [EnumMember]
        Double
    }
}