﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The image element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public abstract class RTImage:InlineElement, IImage
    {
        /// <summary>
        /// Initialize a <see cref="RTImage"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        protected RTImage(IElement parent) : base(parent)
        {
            Stretch = ImageStretch.Uniform;
            Width = 200;
            Height = 200;
        }

        /// <summary>
        /// Gets r sets the image stretch.
        /// </summary>
        [DataMember]
        public ImageStretch Stretch { get; set; }

        /// <summary>
        /// Gets or sets the image width.
        /// </summary>
        [DataMember]
        public double Width { get; set; }

        /// <summary>
        /// Gets or sets the image height.
        /// </summary>
        [DataMember]
        public double Height { get; set; }

        protected void InitilizeCloneImageElement(IImage image)
        {
            InitializeCloneElement(image);
            image.Stretch = Stretch;
            image.Width = Width;
            image.Height = Height;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is RTImage image)
            {
                Stretch = image.Stretch;
                Width = image.Width;
                Height = image.Height;               
                base.UpdateElementProperty(image);
            }
        }
    }
}