﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The table element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class RTTable: RTGrid, IRTTable
    {
        /// <summary>
        /// When ElementValue is null, if AutoHide is true, then the element will hide, otherwisw visible.
        /// </summary>
        [DataMember]
        public bool AutoHide { get; set; }
        [DataMember]
        public bool IsAverageColumnWidth { get; set; }  

        [DataMember]
        public bool IsFoldOccupy { get; set; }
        [DataMember]
        public bool AllowBreakAcrossPages { get; set; }        

        /// <summary>
        /// Initialize a <see cref="RTTable"/>
        /// </summary>
        /// <param name="parent">The parent element.</param>
        public RTTable(IReportElement parent) : base(parent)
        {
            ElementType = ElementType.RTTable;
            AllowBreakAcrossPages = true;
        }

        /// <summary>
        /// Element clone
        /// </summary>
        public override IElement Clone()
        {
            var rtTable = new RTTable(null);
            InitializeCloneElement(rtTable);
            var cellDictionary = new Dictionary<CellPostion, ICell>();
            foreach (var cell in Cells)
            {
                var cloneCell = cell.Value.Clone() as RTCell;
                if (cloneCell != null)
                {
                    cloneCell.Parent = rtTable;
                    cellDictionary.Add(cell.Key.Clone(), cloneCell);
                }
            }
            rtTable.Cells = cellDictionary;
            rtTable.SetRowDefinitions(RowDefinitions.Select(x => new RTRowDefinition() {Height = x.Height, MinHeight = x.MinHeight,Type=x.Type }).ToList());
            rtTable.SetColumnDefinitions(ColumnDefinitions.Select(x => new RTColumnDefinition() {Width = x.Width}).ToList());
            rtTable.AutoHide = AutoHide;
            rtTable.IsAverageColumnWidth = IsAverageColumnWidth;
            rtTable.IsFoldOccupy = IsFoldOccupy;
            rtTable.AllowBreakAcrossPages = AllowBreakAcrossPages;
            return rtTable;
        }

        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is RTTable rtTable)
            {
                AutoHide = rtTable.AutoHide;
                IsAverageColumnWidth = rtTable.IsAverageColumnWidth;
                IsFoldOccupy = rtTable.IsFoldOccupy;
                AllowBreakAcrossPages = rtTable.AllowBreakAcrossPages;
                base.UpdateElementProperty(rtTable);
            }
        }

        protected override RTCell CreateCell()
        {
            var cell = new RTCell(this);
            return cell;
        }

        /// <summary>
        /// get isvisible
        /// </summary>
        public override bool IsVisibleIn(ReportInfo reportInfo)
        {
            if (reportInfo == null)
                return base.IsVisibleIn(null);

            return !AutoHide || this.GetLogicalDescendants<Element>().Any(x => x.HasTagIn(reportInfo));
        }

        /// <summary>
        /// get isvisible
        /// </summary>
        public override bool IsFoldVisibleIn(ReportInfo reportInfo)
        {
            if (reportInfo == null)
                return base.IsFoldVisibleIn(null);

            return !IsFoldOccupy || this.GetLogicalDescendants<Element>().Any(x => x.HasTagIn(reportInfo));
        }

        /// <summary>
        /// Judge whether the tag exists 
        /// </summary>
        public override bool HasTagIn(ReportInfo reportInfo)
        {
            return false;
        }
    }
}