﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Vinno.IUS.Common.Utilities;
using Vinno.IUS.Common.Log;
using Vinno.vCloud.Report.Interfaces;
using Vinno.vCloud.Report.JsonConverters;

namespace Vinno.vCloud.Report.Models
{
    public class ReportJsonSerializer
    {
        public static string Serialize(ReportInfo reportInfo, bool needLine = true)
        {
            var settings = new JsonSerializerSettings
            {
                ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                Converters =
                    new List<JsonConverter>
                    {
                        new StringEnumConverter(),
                        new ReadOnlyCellDictionaryJsonConverter(),
#pragma warning disable 612
                        new ElementValueDictionaryJsonConverter(),
#pragma warning restore 612
                        new ElementValueJsonConverter(),
                        new DictionaryJsonConverter<MeasureTag, TextElementValue>(),
                        new DictionaryJsonConverter<ElementTag, Dictionary<MeasureTag, TextElementValue>>(),
                    }
            };
            return JsonConvert.SerializeObject(reportInfo, needLine ? Formatting.Indented : Formatting.None, settings);
        }

        /// <summary>
        /// Serialize report template document.
        /// </summary>
        /// <param name="filePath">The report template file path.</param>
        /// <param name="reportInfo">The report info to be serialized.</param>
        public static void Serialize(string filePath, ReportInfo reportInfo)
        {
            var data = new UTF8Encoding().GetBytes(Serialize(reportInfo));
            FileHelper.SaveFile(filePath, data);
        }

        /// <summary>
        /// Deserialize report template document.
        /// </summary>
        /// <param name="jsonStr">The json string.</param>
        /// <returns>The <see cref="ReportInfo"/></returns>
        public static ReportInfo Deserialize(string jsonStr)
        {
            try
            {
                var settings = new JsonSerializerSettings
                {
                    ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
                    Converters = new List<JsonConverter> { new ReportJsonConverter() }
                };
                return JsonConvert.DeserializeObject<ReportInfo>(jsonStr, settings);
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize report info error {exception}");
                return null;
            }
        }

        /// <summary>
        /// Deserialize report template document.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The <see cref="ReportInfo"/></returns>
        public static ReportInfo DeSerialize(string filePath)
        {
            try
            {
                return Deserialize(File.ReadAllText(filePath));
            }
            catch (Exception exception)
            {
                Logger.WriteLineError($"Deserialize report info error {exception}");
                return null;
            }
        }
    }
}