﻿using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Xml;
using Vinno.IUS.Common.Log;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    public class ReportTemplateDataContractSerializer : IReportTemplateSerializer
    {
        private static readonly DataContractSerializer Serializer;
        private static readonly XmlWriterSettings WriterSettings;

        static ReportTemplateDataContractSerializer()
        {
            var knownTypes = Assembly.GetExecutingAssembly().GetExportedTypes()
                .Where(x => !x.IsInterface && !x.IsGenericTypeDefinition)
                .Where(x => typeof(IReportElement).IsAssignableFrom(x));
            Serializer = new DataContractSerializer(typeof(ReportTemplateDocument), "ReportTemplateDocument", "",
                knownTypes);
            WriterSettings = new XmlWriterSettings
            {
                Indent = true,
                OmitXmlDeclaration = true
            };
        }

        public IReportTemplateDocument DeSerialize(string filePath)
        {
            using (var stream = File.OpenRead(filePath))
            {
                return DeSerialize(stream);
            }
        }

        public string GetSerializeJson(IReportTemplateDocument reportTemplateDocument)
        {
            return "";
        }

        public void Serialize(string filePath, IReportTemplateDocument reportTemplateDocument)
        {
            reportTemplateDocument.Version = ReportTemplateVersion.GetReportTemplateVersion();
            using (var stream = File.Create(filePath))
                Serialize(stream, reportTemplateDocument);
        }

        public void Serialize(FileStream stream, IReportTemplateDocument reportTemplateDocument)
        {
            using (var writer = XmlWriter.Create(stream, WriterSettings))
            {
                Serializer.WriteStartObject(writer, reportTemplateDocument);
                writer.WriteAttributeString("xmlns", "arrays", null,
                    "http://schemas.microsoft.com/2003/10/Serialization/Arrays");
                Serializer.WriteObjectContent(writer, reportTemplateDocument);
                Serializer.WriteEndObject(writer);
            }
        }

        public IReportTemplateDocument DeSerialize(Stream stream)
        {
            try
            {
                return (IReportTemplateDocument) Serializer.ReadObject(stream);
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"Deserialize report template document error {ex}");
                return null;
            }
        }
    }
}