﻿using System.Collections.Generic;
using System.Linq;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    public static class ReportTemplateDocumentHelper
    {
        public static IEnumerable<T> GetLogicalDescendants<T>(this IReportTemplateDocument template)
        {
            return template.Header
                .Concat(template.Blocks)
                .Concat(template.Footer)
                .ToArray()
                .GetLogicalDescendants<T>();
        }

        public static IEnumerable<T> GetLogicalDescendants<T>(this IReportTemplateBody body)
        {
            return body.Blocks.ToArray().GetLogicalDescendants<T>();
        }

        public static IEnumerable<T> GetLogicalDescendants<T>(this IReadOnlyCollection<IBlockElement> blocks)
        {
            var firstLevel = blocks.OfType<T>();
            var deeperLevel = blocks.SelectMany(x => x.GetLogicalDescendants<T>());
            return firstLevel.Concat(deeperLevel);
        }

        public static IEnumerable<T> GetLogicalDescendants<T>(this IBlockElement element)
        {
            switch (element)
            {
                case IGrid grid:
                    var pairs = grid.Cells.OrderBy(x => x.Key)
                        .ToList();
                    var descendants = pairs.SelectMany(x => x.Value.GetLogicalDescendants<T>());
                    return pairs.Select(x => x.Value)
                        .OfType<T>()
                        .Concat(descendants);
                case InputImageList _:
                    return Enumerable.Empty<T>(); //workaround
                default:
                    var firstLevel = element.Elements.OfType<T>();
                    var deeperLevel = element.Elements.OfType<IBlockElement>()
                        .SelectMany(x => x.GetLogicalDescendants<T>());
                    return firstLevel.Concat(deeperLevel);
            }
        }

        public static List<InputText> GetTemplateMeasureElements(this IReportTemplateDocument template)
        {
            return template.GetLogicalDescendants<InputText>()
                .Where(x => x.MeasureTag != null)
                .ToList();
        }
    }
}