﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The shap element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public abstract class Shape:InlineElement,IShape
    {
        /// <summary>
        /// Gets or sets the stroke.
        /// </summary>
        [DataMember]
        public RTColor Stroke { get; set; }

        /// <summary>
        /// Gets the thickness.Unit is pt.
        /// </summary>
        [DataMember]
        public double Thickness { get; set; }

        /// <summary>
        /// Initialize a <see cref="Shape"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        protected Shape(IElement parent) : base(parent)
        {
            Stroke = RTColor.Black;
            Thickness = 1;
        }

        protected void InitializeCloneShape(IShape shape)
        {
            InitializeCloneElement(shape);
            shape.Stroke = Stroke;
            shape.Thickness = Thickness;
        }

        /// <summary>
        /// Set Property
        /// </summary>
        public void SetShapeProperty(IShape shape)
        {
            Stroke = shape.Stroke;
            Thickness = shape.Thickness;
            base.UpdateElementProperty(shape);
        }
    }
}