﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The single selected element.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class SingleSelected : TextElement, ISingleSelected
    {
        private SelectorDisplayStyle _displayStyle;
        private List<string> _items;
        /// <summary>
        /// Ges or sets the value to indicate whether the elment is readonly or not.
        /// </summary>
        [DataMember]
        public bool IsReadOnly { get; set; }

        /// <summary>
        /// Gets the data items source.
        /// </summary>
        [DataMember]
        public List<string> Items
        {
            get => _items;
            set
            {
                _items = value;
            }
        }

        [DataMember]
        public SelectorDisplayStyle DisplayStyle { get; set; }

        /// <summary>
        /// Initialize a <see cref="SingleSelected"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public SingleSelected(IElement parent) : base(parent)
        {
            DisplayStyle = SelectorDisplayStyle.OnlyShowSelectedItem;
            ElementType = ElementType.SingleSelected;
            Items = new List<string>();
        }

        /// <summary>
        /// Get Default elementvalue
        /// </summary>
        public IElementValue GetDefaultValue()
        {
            return new TextElementValue(string.Empty);
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var singleSelected = new SingleSelected(null);
            InitializeCloneTextElement(singleSelected);
            singleSelected.IsReadOnly = IsReadOnly;
            singleSelected.Items = Items.ToList();
            singleSelected.DisplayStyle = DisplayStyle;
            return singleSelected;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is SingleSelected singleSelected)
            {
                IsReadOnly = singleSelected.IsReadOnly;
                Items = singleSelected.Items;
                DisplayStyle = singleSelected.DisplayStyle;
                base.UpdateElementProperty(singleSelected);
            }
        }
    }
}