﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The space element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class Space: InlineElement, ISpace
    {
        /// <summary>
        /// Unit is pt
        /// </summary>
        private const double SpaceUnit =10.5;

        /// <summary>
        /// Gets or sets the space unit count.
        /// </summary>
        [DataMember]
        public int SpaceUnitCount { get; set; }

        [DataMember]
        public double SpaceWidth { get; set; }

        /// <summary>
        /// Gets the space width.
        /// </summary>
        public double Width => SpaceWidth;

        /// <summary>
        /// Initialize a <see cref="Space"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public Space(IElement parent) : base(parent)
        {
            ElementType = ElementType.Space;
            SpaceUnitCount = 0;
            SpaceWidth = 28.41;
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var space = new Space(null);
            InitializeCloneElement(space);
            space.SpaceUnitCount = SpaceUnitCount;
            space.SpaceWidth = SpaceWidth;
            return space;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is Space space)
            {
                SpaceUnitCount = space.SpaceUnitCount;
                SpaceWidth = space.SpaceWidth;
                base.UpdateElementProperty(space);
            }
        }
    }
}