﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The static image class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class StaticImage: RTImage, IStaticImage
    {
        public event EventHandler OnWidthChanged;
        /// <summary>
        /// Gets or sets the source.
        /// </summary>
        [DataMember]
        public byte[] Source { get; set; }

        /// <summary>
        /// Initialize a <see cref="StaticImage"/>
        /// </summary>
        /// <param name="parent">The parent element.</param>
        public StaticImage(IElement parent) : base(parent)
        {
            ElementType = ElementType.StaticImage;
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var staticImage = new StaticImage(null);
            InitilizeCloneImageElement(staticImage);
            staticImage.Source = Source;
            return staticImage;
        }
        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is StaticImage staticImage)
            {
                Source = staticImage.Source;
                base.UpdateElementProperty(staticImage);
                OnWidthChanged?.Invoke(staticImage, EventArgs.Empty);
            }
        }
    }
}