﻿using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Vinno.IUS.Common.Utilities;
using Vinno.vCloud.Report.Interfaces;

namespace Vinno.vCloud.Report.Models
{
    /// <summary>
    /// The static text element class.
    /// </summary>
    [JsonObject]
    [DataContract(IsReference = true, Namespace = "")]
    [Serializable]
    public class StaticText:TextElement, IStaticText
    {
        private string _text;
        public event EventHandler OnTextChanged;
        [JsonIgnore]
        public bool IgnoreUndo { get; set; }

        /// <summary>
        /// Gets or sets the text value.
        /// </summary>
        [DataMember]
        public string Text
        {
            get => _text;
            set
            {
                if (_text != value)
                {
                    _text = value;
                    LineLength = string.IsNullOrEmpty(_text) ? 0 : _text.Length;
                    //OnTextChanged?.Invoke(_text, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Initialize a <see cref="StaticText"/>
        /// </summary>
        /// <param name="parent">The element parent.</param>
        public StaticText(IElement parent) : base(parent)
        {
            IgnoreUndo = true;
               //TextWrap = true;
               ElementType = ElementType.StaticText;
            Text = TranslateHelper.Translate("DefaultStaticTextValue");
        }

        /// <summary>
        /// Element Clone
        /// </summary>
        public override IElement Clone()
        {
            var staticText= new StaticText(null);
            InitializeCloneTextElement(staticText);
            staticText.Text = Text;
            return staticText;
        }

        /// <summary>
        /// Update own properties
        /// </summary>
        /// <param name="element">NewElement</param>
        public override void UpdateElementProperty(IElement element)
        {
            if (element is StaticText staticText)
            {
                IgnoreUndo = false;
                Text = staticText.Text;
                base.UpdateElementProperty(staticText);
                IgnoreUndo = true;
            }
        }
    }
}