
using System.Collections.Concurrent;
using System.IO;
using System.Linq;
using System.Net.Http.Headers;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Syncfusion.Pdf;
using Vinno.IUS.Common.Utilities.Executors;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.DTO.Vital;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.VitalEnum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.DBVitalRequest;
using WingInterfaceLibrary.Request.Report;
using WingInterfaceLibrary.Request.Vital;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Interfaces.FileTransfer;
using WingServerCommon.Log;
using WingServerCommon.Service;

namespace ReportService.Service
{
    public partial class ReportService : JsonRpcService, IReportService
    {
        //体检报告生成队列
        private readonly SequenceExecutor<VitalReportConvertDetailRequest> _vitalReportConverterExecutor = new SequenceExecutor<VitalReportConvertDetailRequest>("VitalReportConverter");
        // 个人和健康体检队列
        private readonly SequenceExecutor<VitalReportConvertDetailRequest> _doPersonAndHealthExamReport = new SequenceExecutor<VitalReportConvertDetailRequest>("DoPersonAndHealthExamReport");
        //部分合并报告队列
        private readonly SequenceExecutor<MergePDFFileInfo> _vitalPartReportMergeExecutor = new SequenceExecutor<MergePDFFileInfo>("VitalPartReportMerge");
        private ConcurrentDictionary<string, string> _mergeReportSequence = new ConcurrentDictionary<string, string>();

        private ConcurrentDictionary<string, VitalReportConfigInfo> _reportConfigDic = new ConcurrentDictionary<string, VitalReportConfigInfo>();
        private ConcurrentDictionary<string, string> _exceptionDescConfigDic = new ConcurrentDictionary<string, string>();
        private ConcurrentDictionary<string, string> _zhongYiTiZhiDescConfigDic = new ConcurrentDictionary<string, string>();
        private static readonly bool _isUbuntu = File.Exists("/etc/os-release") && File.ReadAllText("/etc/os-release").Contains("Ubuntu");
        private readonly int _rateSeconds = 60;

        /// <summary>
        /// 加载体检检查列表
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public override void LoadAfterRegister()
        {
            LoadGeneralVitalReport();
        }

        /// <summary>
        /// 加载报告生成
        /// </summary>
        private void LoadGeneralVitalReport()
        {
            if (_httpService.Contains("VitalMixtureService"))
            {
                ReloadVitalReportConfig();

                // var re = new VitalReportConvertDetailRequest()
                // {

                //    PhysicalExamNumber = "01820240328006",
                //    IDCardNo = "321282198904132442",
                //    ExamType = 2,
                //    ExamCode = "59F817C805DD49678DA9528EE58B9692",
                //    ReportCode = "",
                //    Type = 1,
                //    IsCover = true
                // };
                // var d = JoinVitalReportQueueAsync(re).Result;

                // var re = new VitalReportConvertDetailRequest()
                // {

                //    PhysicalExamNumber = "01820240328006",
                //    IDCardNo  = "321282198904132442",
                //    ExamType  = 97,
                //    ExamCode  = "",
                //    ReportCode  = "",
                //    Type = 1,
                //    IsCover  = true
                // };
                // var d = DoPersonAndHealthExamReport(re);

                //JoinQueue("376699D8D55745EDBB398F7C3F902FA7", 0);
                try
                {
                    Task.Run(async () =>
                    {
                        while (true)
                        {
                            GetInspectedRegisterInfoList();
                            await Task.Delay(1 * 1000 * _rateSeconds);
                        }
                    });
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"ReportService LoadAfterRegister err, {ex}");
                }
            }
        }

        /// <summary>
        /// 加载报告配置
        /// </summary>
        private void ReloadVitalReportConfig()
        {
            var jsonContent = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "ReportConfig.json"));
            JObject json = JObject.Parse(jsonContent);
            var tempList = new List<string>() { "HEIUrinalysis", "HEIBloodRoutine", "HEIBiochemical" };
            foreach (var item in tempList)
            {
                JArray array = (JArray)json[item];
                foreach (JToken jObject in array)
                {
                    //赋值属性
                    var key = item + "_" + jObject["ElementTagKey"].ToString();//获取字符串中id值
                    var value = new VitalReportConfigInfo()
                    {
                        DictionaryKey = jObject["DictionaryKey"].ToString(),
                        ElementTagKey = jObject["ElementTagKey"].ToString(),
                        Description = jObject["Description"].ToString(),
                        Unit = jObject["Unit"].ToString(),
                        MinValue = jObject["MinValue"].ToString(),
                        MaxValue = jObject["MaxValue"].ToString(),
                    };
                    if (!_reportConfigDic.Keys.Contains(key))
                    {
                        _reportConfigDic.TryAdd(key, value);
                    }
                    else
                    {
                        _reportConfigDic[key] = value;
                    }
                }
            }

            jsonContent = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Exam", "ExamDescription.json"));
            JObject jsonExDesc = JObject.Parse(jsonContent);
            JArray arrayExDesc = (JArray)jsonExDesc["ExceptionDesc"];
            foreach (JToken jObject in arrayExDesc)
            {
                //赋值属性
                var key = jObject["Key"].ToString();
                var value = jObject["Value1"].ToString();
                if (!_exceptionDescConfigDic.Keys.Contains(key))
                {
                    _exceptionDescConfigDic.TryAdd(key, value);
                }
                else
                {
                    _exceptionDescConfigDic[key] = value;
                }
            }

            jsonContent = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Exam", "TCMDescription.json"));
            JObject tcmDescriptionJson = JObject.Parse(jsonContent);
            JArray tcmDescArray = (JArray)tcmDescriptionJson["TCMDesc"];
            foreach (JToken jObject in tcmDescArray)
            {
                //赋值属性
                var key = jObject["Key"].ToString();
                var value = ConvertToChinese(jObject["Value"].ToString());
                if (!_zhongYiTiZhiDescConfigDic.Keys.Contains(key))
                {
                    _zhongYiTiZhiDescConfigDic.TryAdd(key, value);
                }
                else
                {
                    _zhongYiTiZhiDescConfigDic[key] = value;
                }
            }

            CheckAndInsertDefaultReportTemplate();
        }

        /// <summary>
        /// 插入报告配置到db
        /// </summary>
        private void CheckAndInsertDefaultReportTemplate()
        {

            var reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f30";
            reportTemplate.TemplateName = "基础检查";
            var templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIBasic.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIBasic";
            var existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f31";
            reportTemplate.TemplateName = "生化检查";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIBiochemical.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIBiochemical";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f32";
            reportTemplate.TemplateName = "血常规检查";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIBloodRoutine.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIBloodRoutine";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f33";
            reportTemplate.TemplateName = "结论";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIConclusion.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIConclusion";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f34";
            reportTemplate.TemplateName = "封面";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEICover.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEICover";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f35";
            reportTemplate.TemplateName = "心电检查";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIECG.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIECG";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f36";
            reportTemplate.TemplateName = "中医体质";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEITCMC.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEITCMC";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f37";
            reportTemplate.TemplateName = "尿常规检查";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "Vital", "HEIUrinalysis.json");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIUrinalysis";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }


            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f38";
            reportTemplate.TemplateName = "个人基本信息表";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "personalInfoBasicTable.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIPersonInfo";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f39";
            reportTemplate.TemplateName = "健康体检表1";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable1.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIHealthExamination1";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3a";
            reportTemplate.TemplateName = "健康体检表2";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable2.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIHealthExamination2";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3b";
            reportTemplate.TemplateName = "健康体检表3";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable3.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIHealthExamination3";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3c";
            reportTemplate.TemplateName = "健康体检表4";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable4.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEIHealthExamination4";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3d";
            reportTemplate.TemplateName = "老年人生活自理能力评估表";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "oldPersonAssess.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEILNRZLNLPG";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3e";
            reportTemplate.TemplateName = "老年人中医药健康管理服务记录表1";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "traditionalChineseMedicineHealthTable1.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEITCMCDetailRecord1";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3f";
            reportTemplate.TemplateName = "老年人中医药健康管理服务记录表2";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "traditionalChineseMedicineHealthTable2.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEITCMCDetailRecord2";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            reportTemplate = new CreateTemplateDBRequest();
            reportTemplate.Code = "2e1441e904c2e2cbea7ba7d692e5e98f3f";
            reportTemplate.TemplateName = "老年人中医药健康管理服务记录表2";
            templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "traditionalChineseMedicineHealthTable2.html");
            reportTemplate.TemplateContent = File.ReadAllText(templatePath);
            reportTemplate.TemplateType = TemplateTypeEnum.Report;
            reportTemplate.Key = "HEITCMCDetailRecord2";
            existEntity = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync(reportTemplate.Code, "", TemplateTypeEnum.Report).Result;
            if (string.IsNullOrEmpty(existEntity?.Code))
            {
                var res = _vitalTemplateDBService.CreateTemplateAsync(reportTemplate).Result;
            }
            else
            {
                var updateReq = new UpdateTemplateDBRequest()
                {
                    Code = existEntity.Code,
                    TemplateName = existEntity.TemplateName,
                    TemplateType = existEntity.TemplateType,
                    Key = existEntity.Key,
                    TemplateContent = reportTemplate.TemplateContent
                };
                var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            }

            // var directPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "Prescription");
            // if (Directory.Exists(directPath))
            // {
            //     DirectoryInfo di = new DirectoryInfo(directPath);
            //     FileInfo[] files = di.GetFiles();

            //     foreach (FileInfo file in files)
            //     {
            //         var fileName = Path.GetFileNameWithoutExtension(file.Name);
            //         var firstTemplate = _vitalTemplateDBService.GetTemplateDetailByKeyAsync(new GetTemplateByKeyDBRequest { Key = "Key", Value = fileName }).Result;
            //         if (!string.IsNullOrEmpty(firstTemplate?.Code))
            //         {
            //             //更新
            //             var updateReq = new UpdateTemplateDBRequest()
            //             {
            //                 Code = firstTemplate.Code,
            //                 TemplateContent = File.ReadAllText(file.FullName)
            //             };
            //             var res = _vitalTemplateDBService.UpdateTemplateAsync(updateReq).Result;
            //         }
            //     }
            // }
        }

        /// <summary>
        /// 对外开放的reload方法
        /// </summary>
        public async Task<bool> ReloadVitalReportConfigAsync(ReloadReportConfigRequest request)
        {
            //var tempList = new List<string>() {"01820240328006"};
            //var registerInfoList = _registerInfoDBService.GetInspectedRegisterInfoListDBAsync(tempList, null, true).Result;
            //if (registerInfoList?.Count > 0)
            //{
            //    foreach (var registerInfo in registerInfoList)
            //    {
            //        //保存报告内容到报告表
            //        var reportQueryReq = new GetReportConditionRequest()
            //        {
            //            PhysicalExamNumber = registerInfo.PhysicalExamNumber
            //        };
            //        var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
            //        if (reportInfos?.Count > 0)
            //        {
            //            var reportInfoList = reportInfos.ToList();
            //            var ability_Code = "";
            //            //验证 个人健康体检是否存在，或者是否需要更新
            //            CheckIsNeedUpdateHealthExamReport(registerInfo, reportInfoList, ref ability_Code);
            //            //验证老年人生活自理能力评估表
            //            CheckIsNeedUpdateOldPersonSelfCareReport(registerInfo, reportInfoList, ability_Code);
            //        }
            //    }
            //}

            //{
            //    foreach (var registerInfo in registerInfoList)
            //    {
            //        //保存报告内容到报告表
            //        var reportQueryReq = new GetReportConditionRequest()
            //        {
            //            PhysicalExamNumber = registerInfo.PhysicalExamNumber
            //        };
            //        var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
            //        if (reportInfos?.Count > 0)
            //        {
            //            var reportInfoList = reportInfos.ToList();
            //            var ability_Code = "";
            //            //验证 个人健康体检是否存在，或者是否需要更新
            //            CheckIsNeedUpdateHealthExamReport(registerInfo, reportInfoList, ref ability_Code);
            //            //验证老年人生活自理能力评估表
            //            CheckIsNeedUpdateOldPersonSelfCareReport(registerInfo, reportInfoList, ability_Code);
            //        }
            //    }
            //}

            // var re = new VitalReportConvertDetailRequest()
            // {

            //     PhysicalExamNumber = "01820240328006",
            //     IDCardNo  = "321282198904132442",
            //     ExamType  = 6,
            //     ExamCode  = "20FE19F659F640849E7BAC63ED0AD7E0",
            //     ReportCode  = "",
            //     Type = 1,
            //     IsCover  = true
            // };
            // var d = JoinVitalReportQueueAsync(re);

            // var _reportPreviewImagesGenerater = new ReportPreviewImagesGenerater();
            // var path = @"D:\vCloudStorage\FlyinsonoStorage\20240419\3FFBDF5A5341469285A4F3FD4FE15F35.pdf";
            // var imgList = _reportPreviewImagesGenerater.ConvertPDFReportToImage(path);
            // if (imgList != null && imgList.Count() > 0)
            //     {
            //         int i = 0;
            //         var filePath = @"D:\vCloudCache";
            //         foreach (var image in imgList)
            //         {
            //             using (var bitmap = SkiaSharp.SKBitmap.Decode(image.ImageBuffer.GetBytes()))
            //             {
            //                 var imagePath = $"{filePath}_{i}.jpg";
            //                 using (var fileStream = new FileStream(imagePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
            //                 {
            //                     bitmap.Encode(fileStream, SkiaSharp.SKEncodedImageFormat.Jpeg, 100);
            //                     i++;
            //                 }
            //             }
            //         }
            //     }
            var res = true;
            if (request.Type == 0)
            {
                ReloadVitalReportConfig();
            }
            return res;
        }

        /// <summary>
        /// 个人信息和健康体检表接口
        /// </summary>
        public async Task<bool> SavePersonOrHealthExamReportAsync(VitalReportConvertDetailRequest request)
        {
            var res = true;
            try
            {
                if (_isUbuntu)
                {
                    Logger.WriteLineWarn($"SavePersonOrHealthExamReportAsync is _isUbuntu, Not join Queue, request:{JsonConvert.SerializeObject(request)}");
                }
                else
                {
                    _doPersonAndHealthExamReport.Add(DoPersonAndHealthExamReport, request);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"SavePersonOrHealthExamReportAsync Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                return true;
            }
            return res;
        }

        /// <summary>
        /// 个人和健康体检队列
        /// </summary>
        private bool DoPersonAndHealthExamReport(VitalReportConvertDetailRequest request)
        {
            var res = true;
            try
            {
                if (request.ExamType == 96)
                {
                    //调用生成个人基本信息表
                    res = DoPersonBasicInfoReport(request);
                }
                else if (request.ExamType == 97)
                {
                    res = DoHealthExamReport(request);
                }
                else if (request.ExamType == 95)
                {
                    res = DoOldPersonSelfCareReport(request);
                }
                else if (request.ExamType == 94)
                {
                    res = DoOldPersonTCMCDetailRecordReport(request);
                }
                if (res)
                {
                    //生成合并报告
                    var mergeInfo = new MergePDFFileInfo();
                    //单独生成部分合并报告
                    var regDBReq = new GetRegisterPersonRequest()
                    {
                        PhysicalExamNumber = request.PhysicalExamNumber
                    };
                    var registerInfo = _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq).Result;
                    if (string.IsNullOrEmpty(registerInfo?.Code))
                    {
                        Logger.WriteLineWarn($"HtmlToPdf GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                        return true;
                    }
                    mergeInfo.RegisterInfo = registerInfo;
                    var orgReq = new GetOrganizationDBRequest()
                    {
                        Code = registerInfo.OrganizationCode
                    };
                    var orgInfo = _vitalOrganizationDBService.GetOrganizationDetailAsync(orgReq).Result;
                    if (string.IsNullOrEmpty(orgInfo?.Code))
                    {
                        Logger.WriteLineWarn($"HtmlToPdf GetOrganizationDetailAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                        return true;
                    }
                    mergeInfo.OrganizationInfo = orgInfo;
                    mergeInfo.ExamType = request.ExamType;

                    var key = mergeInfo.RegisterInfo.PhysicalExamNumber + "_" + mergeInfo.ExamType;
                    if (!_mergeReportSequence.Keys.Contains(key))
                    {
                        _mergeReportSequence.TryAdd(key, mergeInfo.RegisterInfo.PhysicalExamNumber);
                    }
                    else
                    {
                        Logger.WriteLineWarn($"DealWithVitalPartMergePDFReport is exist queue, PhysicalExamNumber:{mergeInfo.RegisterInfo.PhysicalExamNumber}");
                        return true;
                    }
                    _vitalPartReportMergeExecutor.Add(DealWithVitalPartMergePDFReport, mergeInfo);
                }
                res = true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoPersonAndHealthExamReport Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                return true;
            }
            return res;
        }

        /// <summary>
        /// 个人健康检测队列
        /// </summary>
        private bool DoHealthExamReport(VitalReportConvertDetailRequest request)
        {
            var res = true;
            try
            {
                var keys = new List<string>() { "HEIHealthExamination1", "HEIHealthExamination2", "HEIHealthExamination3", "HEIHealthExamination4" };

                var filePathList = new List<string>();
                var streamList = new List<Stream>();
                var contractedDoctor = "";
                var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", request.IDCardNo);
                if (!Directory.Exists(_tempExmaFolder))
                {
                    Directory.CreateDirectory(_tempExmaFolder);
                }
                //获取
                var dbReq = new GetPatientByCodeDBRequest()
                {
                    Code = request.IDCardNo
                };
                var patientInfo = _vitalPatientDBService.GetPatientByCodeAsync(dbReq).Result;
                if (string.IsNullOrEmpty(patientInfo?.Code))
                {
                    Logger.WriteLineWarn($"GetPatientByCodeAsync patientInfo is null, request:{JsonConvert.SerializeObject(request)}");
                    return res;
                }

                var examReq = new GetExamRecordListDBRequest()
                {
                    PatientCode = request.IDCardNo,
                    Keys = new List<string>() { "ZZYBZK", "SHFS", "ZQCT", "FZJC", "XCZYWT", "ZYYYFMYGHYFJZS", "JKPJJZD" },
                    IsValidOperationDoctor = false,
                    PhysicalExamNumber = request.PhysicalExamNumber
                };
                var examRecords = _vitalExamDBService.GetExamRecordListAsync(examReq).Result;
                if (examRecords == null || examRecords?.Count <= 0)
                {
                    Logger.WriteLineWarn($"GetExamRecordListAsync examRecords is null, request:{JsonConvert.SerializeObject(request)}");
                    return true;
                }
                foreach (var key in keys)
                {
                    var tuple = GetVitalReportTemplateJson(key);
                    if (tuple != null)
                    {
                        var templateContent = tuple.Item2;
                        try
                        {
                            var htmlContent = TransferAndBindHtmlTemplate(key, templateContent, patientInfo, null, examRecords);
                            var converter = new SelectPdf.HtmlToPdf();
                            converter.Options.PdfPageSize = SelectPdf.PdfPageSize.A4;//设置页面大小，30种页面大小可以选
                            converter.Options.MarginTop = 40;   //上下左右边距设置  
                            converter.Options.MarginBottom = 40;
                            converter.Options.MarginLeft = 10;
                            converter.Options.MarginRight = 10;
                            var doc = new SelectPdf.PdfDocument();

                            var docStr = converter.ConvertHtmlString(htmlContent);
                            doc.Append(docStr);
                            var fileName = request.IDCardNo + "_" + key + "_" + Guid.NewGuid().ToString("N") + ".pdf";
                            var filePath = Path.Combine(_tempExmaFolder, fileName);
                            if (File.Exists(filePath))
                            {
                                File.Delete(filePath);
                            }
                            doc.Save(filePath);
                            doc.Close();

                            var firstStream = new FileStream(filePath, FileMode.Open, FileAccess.Read);
                            streamList.Add(firstStream);
                            filePathList.Add(filePath);

                        }
                        catch (Exception e)
                        {
                            Logger.WriteLineWarn($"HtmlToPdf Error, key:{key}, ex:{e}");
                        }
                    }
                }
                //多个文档合并健康档案
                var healthExamPDFUrl = "";
                if (streamList?.Count > 0)
                {
                    //Merge PDF documents 
                    using (var finalDocument = new PdfDocument())
                    {
                        try
                        {
                            Stream[] streams = streamList.ToArray();
                            PdfDocumentBase.Merge(finalDocument, streams);
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteLineWarn($"HtmlToPdf PdfDocumentBase.Merge Error, PhysicalExamNumber:{request.IDCardNo}, ex: {ex}");
                        }
                        //Save the document into a stream 
                        using (MemoryStream outputStream = new MemoryStream())
                        {
                            //确认保存路径
                            finalDocument.Save(outputStream);
                            var bytes = outputStream.GetBuffer();
                            if (bytes != null)
                            {
                                var pdfFile = ReportPreviewHelper.GetSaveFilePath() + ".pdf";
                                File.WriteAllBytes(pdfFile, bytes);
                                if (File.Exists(pdfFile))
                                {
                                    var fileToken = UploadFileAsync(pdfFile, Path.GetFileName(pdfFile)).Result;
                                    if (!string.IsNullOrWhiteSpace(fileToken))
                                    {
                                        healthExamPDFUrl = fileToken;
                                    }
                                    File.Delete(pdfFile);
                                }

                            }
                        }
                    }
                    foreach (var streamItem in streamList)
                    {
                        streamItem.Close();
                        streamItem.Dispose();
                    }
                    foreach (var filePathItem in filePathList)
                    {
                        if (File.Exists(filePathItem))
                        {
                            File.Delete(filePathItem);
                        }
                    }
                }
                else
                {
                    Logger.WriteLineWarn($"HtmlToPdf Stream List Error, PhysicalExamNumber:{request.IDCardNo}, streamList Count : 0");
                    return res;
                }
                var reportExamQueryReq = new GetReportConditionRequest()
                {
                    PhysicalExamNumber = request.PhysicalExamNumber
                };
                var reportInfoList = _vitalReportDBService.GetVitalReportListDBAsync(reportExamQueryReq).Result;
                if (reportInfoList?.Count > 0 && reportInfoList.Any(c => c.Key == "HEIHealthExamination"))
                {
                    var reportInfo = reportInfoList.FirstOrDefault(c => c.Key == "HEIHealthExamination") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                    //找到
                    var updateReq = new UpdateReportDBRequest()
                    {
                        TemplateCode = "",//模板code
                        Code = reportInfo.Code,
                        ReportDataJson = "",
                        ReportTemplateJson = "",
                        ReportState = ReportStateEnum.Reported,
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = healthExamPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        }
                    };
                    var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                }
                else
                {
                    //找到并生成其他的检验报告；
                    var dbRequest = new CreateReportDBRequest
                    {
                        Key = "HEIHealthExamination",
                        ContractedDoctor = "",
                        ReportState = ReportStateEnum.Reported,
                        TemplateCode = "",
                        PatientCode = request.IDCardNo,
                        ReportDataJson = "",
                        ReportTemplateJson = "",
                        PhysicalExamNumber = request.PhysicalExamNumber,
                        ExamCode = "",
                        OperationDoctor = contractedDoctor,
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = healthExamPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        }
                    };
                    var reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
                }
                res = true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoHealthExamReport Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                return true;
            }

            return res;
        }

        /// <summary>
        /// 老年人中医药健康管理服务记录表
        /// </summary>
        private bool DoOldPersonTCMCDetailRecordReport(VitalReportConvertDetailRequest request)
        {
            var res = true;
            try
            {
                var keys = new List<string>() { "HEITCMCDetailRecord1", "HEITCMCDetailRecord2" };
                var key = "HEITCMCDetailRecord";

                var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", request.IDCardNo);
                if (!Directory.Exists(_tempExmaFolder))
                {
                    Directory.CreateDirectory(_tempExmaFolder);
                }

                //获取
                var dbReq = new GetPatientByCodeDBRequest()
                {
                    Code = request.IDCardNo
                };
                var patientInfo = _vitalPatientDBService.GetPatientByCodeAsync(dbReq).Result;
                if (string.IsNullOrEmpty(patientInfo?.Code))
                {
                    Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport patientInfo is null, request:{JsonConvert.SerializeObject(request)}");
                    return res;
                }

                var examReq = new GetExamRecordListDBRequest()
                {
                    PatientCode = request.IDCardNo,
                    Keys = new List<string>() { "HEITCMC" },
                    IsValidOperationDoctor = false,
                    PhysicalExamNumber = request.PhysicalExamNumber
                };
                var examRecords = _vitalExamDBService.GetExamRecordListAsync(examReq).Result;
                if (examRecords == null || examRecords.Count <= 0)
                {
                    Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport HEITCMC is null, request:{JsonConvert.SerializeObject(request)}");
                    return res;
                }
                var filePathList = new List<string>();
                var streamList = new List<Stream>();
                var contractedDoctor = "";
                foreach (var keyItem in keys)
                {
                    var tuple = GetVitalReportTemplateJson(keyItem);
                    if (tuple != null)
                    {
                        var templateContent = tuple.Item2;
                        var htmlContent = TransferAndBindHtmlTemplate(keyItem, templateContent, patientInfo, null, examRecords, request.PhysicalExamNumber);
                        try
                        {
                            var converter = new SelectPdf.HtmlToPdf();
                            converter.Options.PdfPageSize = SelectPdf.PdfPageSize.A4;//设置页面大小，30种页面大小可以选
                            converter.Options.MarginTop = 40;   //上下左右边距设置  
                            converter.Options.MarginBottom = 40;
                            converter.Options.MarginLeft = 10;
                            converter.Options.MarginRight = 10;
                            // 设置导出PDF的相关配置
                            // converter.Options.PdfPageOrientation = SelectPdf.PdfPageOrientation.Landscape; // 设置横向打印
                            // converter.Options.CustomWkHtmlArgs = "--header-right '' --header-left '' --header-line --footer-right '' --footer-left '' --footer-center ''";

                            var doc = new SelectPdf.PdfDocument();

                            var docStr = converter.ConvertHtmlString(htmlContent);
                            doc.Append(docStr);
                            var fileName = request.IDCardNo + "_" + key + "_" + Guid.NewGuid().ToString("N") + ".pdf";
                            var filePath = Path.Combine(_tempExmaFolder, fileName);
                            if (File.Exists(filePath))
                            {
                                File.Delete(filePath);
                            }
                            doc.Save(filePath);
                            doc.Close();


                            var firstStream = new FileStream(filePath, FileMode.Open, FileAccess.Read);
                            streamList.Add(firstStream);
                            filePathList.Add(filePath);

                        }
                        catch (Exception e)
                        {
                            Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport HtmlToPdf Error, html:{JsonConvert.SerializeObject(htmlContent)}, ex:{e}");
                        }
                    }
                }
                //多个文档合并健康档案
                var tcmcPDFUrl = "";
                if (streamList?.Count > 0)
                {
                    //Merge PDF documents 
                    using (var finalDocument = new PdfDocument())
                    {
                        try
                        {
                            Stream[] streams = streamList.ToArray();
                            PdfDocumentBase.Merge(finalDocument, streams);
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport HtmlToPdf PdfDocumentBase.Merge Error, PhysicalExamNumber:{request.IDCardNo}, ex: {ex}");
                            return true;
                        }
                        //Save the document into a stream 
                        using (MemoryStream outputStream = new MemoryStream())
                        {
                            //确认保存路径
                            finalDocument.Save(outputStream);
                            var bytes = outputStream.GetBuffer();
                            if (bytes != null)
                            {
                                var pdfFile = ReportPreviewHelper.GetSaveFilePath() + ".pdf";
                                File.WriteAllBytes(pdfFile, bytes);
                                if (File.Exists(pdfFile))
                                {
                                    var fileToken = UploadFileAsync(pdfFile, Path.GetFileName(pdfFile)).Result;
                                    if (!string.IsNullOrWhiteSpace(fileToken))
                                    {
                                        tcmcPDFUrl = fileToken;
                                    }
                                    File.Delete(pdfFile);
                                }

                            }
                        }
                    }
                    foreach (var streamItem in streamList)
                    {
                        streamItem.Close();
                        streamItem.Dispose();
                    }
                    foreach (var filePathItem in filePathList)
                    {
                        if (File.Exists(filePathItem))
                        {
                            File.Delete(filePathItem);
                        }
                    }
                }
                else
                {
                    Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport HtmlToPdf Stream List Error, PhysicalExamNumber:{request.IDCardNo}, streamList Count : 0");
                    return res;
                }
                var reportExamQueryReq = new GetReportConditionRequest()
                {
                    PhysicalExamNumber = request.PhysicalExamNumber
                };
                var reportInfoList = _vitalReportDBService.GetVitalReportListDBAsync(reportExamQueryReq).Result;
                if (reportInfoList?.Count > 0 && reportInfoList.Any(c => c.Key == key))
                {
                    var reportInfo = reportInfoList.FirstOrDefault(c => c.Key == key) ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                    //找到
                    var updateReq = new UpdateReportDBRequest()
                    {
                        TemplateCode = "",//模板code
                        Code = reportInfo.Code,
                        ReportDataJson = "",
                        ReportTemplateJson = "",
                        ReportState = ReportStateEnum.Reported,
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = tcmcPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        }
                    };
                    var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                }
                else
                {
                    //找到并生成其他的检验报告；
                    var dbRequest = new CreateReportDBRequest
                    {
                        Key = key,
                        ContractedDoctor = "",
                        ReportState = ReportStateEnum.Reported,
                        TemplateCode = "",
                        PatientCode = request.IDCardNo,
                        ReportDataJson = "",
                        ReportTemplateJson = "",
                        PhysicalExamNumber = request.PhysicalExamNumber,
                        ExamCode = "",
                        OperationDoctor = contractedDoctor,
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = tcmcPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        }
                    };
                    var reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
                }
                res = true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"DoOldPersonTCMCDetailRecordReport Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                return true;
            }

            return res;
        }

        /// <summary>
        /// 老年人生活自理能力评估队列
        /// </summary>
        private bool DoOldPersonSelfCareReport(VitalReportConvertDetailRequest request)
        {
            var res = true;
            var key = "HEILNRZLNLPG";
            var personPDFUrl = "";
            var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", request.IDCardNo);
            if (!Directory.Exists(_tempExmaFolder))
            {
                Directory.CreateDirectory(_tempExmaFolder);
            }

            var tuple = GetVitalReportTemplateJson(key);
            if (tuple != null)
            {
                var templateContent = tuple.Item2;
                //获取
                var dbReq = new GetPatientByCodeDBRequest()
                {
                    Code = request.IDCardNo
                };
                var patientInfo = _vitalPatientDBService.GetPatientByCodeAsync(dbReq).Result;
                if (string.IsNullOrEmpty(patientInfo?.Code))
                {
                    Logger.WriteLineWarn($"DoOldPersonSelfCareReport patientInfo is null, request:{JsonConvert.SerializeObject(request)}");
                    return true;
                }

                var examReq = new GetPatientLastExamDBRequest()
                {
                    PatientCode = request.IDCardNo,
                    IsValidOperationDoctor = false,
                    Key = "LNRZLNLPG"
                };
                var examRecord = _vitalExamDBService.GetPatientLastExamAsync(examReq).Result;
                if (string.IsNullOrEmpty(examRecord?.Code))
                {
                    Logger.WriteLineWarn($"DoOldPersonSelfCareReport LNRZLNLPG is null, request:{JsonConvert.SerializeObject(request)}");
                    return true;
                }

                var htmlContent = TransferAndBindHtmlTemplate(key, templateContent, patientInfo, null, null, examData: examRecord);
                try
                {
                    var converter = new SelectPdf.HtmlToPdf();
                    converter.Options.PdfPageSize = SelectPdf.PdfPageSize.A4;//设置页面大小，30种页面大小可以选
                    converter.Options.MarginTop = 40;   //上下左右边距设置  
                    converter.Options.MarginBottom = 40;
                    converter.Options.MarginLeft = 10;
                    converter.Options.MarginRight = 10;
                    var doc = new SelectPdf.PdfDocument();

                    var docStr = converter.ConvertHtmlString(htmlContent);
                    doc.Append(docStr);
                    var fileName = request.IDCardNo + "_" + key + "_" + Guid.NewGuid().ToString("N") + ".pdf";
                    var filePath = Path.Combine(_tempExmaFolder, fileName);
                    if (File.Exists(filePath))
                    {
                        File.Delete(filePath);
                    }
                    doc.Save(filePath);
                    doc.Close();

                    //转换完成，上传文件
                    personPDFUrl = UploadFileAsync(filePath, Path.GetFileName(filePath)).Result;
                    //删除文件；
                    //保存报告内容到报告表
                    var reportQueryReq = new GetReportConditionRequest()
                    {
                        PhysicalExamNumber = request.PhysicalExamNumber
                    };
                    var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
                    if (reportInfos?.Count > 0 && reportInfos.Any(c => c.Key == key))
                    {
                        var reportInfo = reportInfos.FirstOrDefault(c => c.Key == key) ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                        //找到
                        var updateReq = new UpdateReportDBRequest()
                        {
                            TemplateCode = "",//模板code
                            Code = reportInfo.Code,
                            ReportDataJson = htmlContent,
                            ReportTemplateJson = templateContent,
                            ReportState = ReportStateEnum.Reported,
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = personPDFUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            }
                        };
                        var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                    }
                    else
                    {
                        //找到并生成其他的检验报告；
                        var dbRequest = new CreateReportDBRequest
                        {
                            Key = key,
                            ContractedDoctor = "",
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = "",
                            PatientCode = request.IDCardNo,
                            ReportDataJson = htmlContent,
                            ReportTemplateJson = templateContent,
                            PhysicalExamNumber = request.PhysicalExamNumber,
                            ExamCode = "",
                            OperationDoctor = patientInfo.ContractedDoctor,
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = personPDFUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            }
                        };
                        var reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
                    }
                    if (File.Exists(filePath))
                    {
                        File.Delete(filePath);
                    }
                    res = true;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DoOldPersonSelfCareReport Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                    return true;
                }
            }
            return res;
        }

        /// <summary>
        /// 个人基础信息队列
        /// </summary>
        private bool DoPersonBasicInfoReport(VitalReportConvertDetailRequest request)
        {
            var res = true;
            var key = "HEIPersonInfo";
            var personPDFUrl = "";
            var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", request.IDCardNo);
            if (!Directory.Exists(_tempExmaFolder))
            {
                Directory.CreateDirectory(_tempExmaFolder);
            }

            var tuple = GetVitalReportTemplateJson(key);
            if (tuple != null)
            {
                var templateContent = tuple.Item2;
                //获取
                var dbReq = new GetPatientByCodeDBRequest()
                {
                    Code = request.IDCardNo
                };
                var patientInfo = _vitalPatientDBService.GetPatientByCodeAsync(dbReq).Result;
                if (string.IsNullOrEmpty(patientInfo?.Code))
                {
                    Logger.WriteLineWarn($"GetPatientByCodeAsync patientInfo is null, request:{JsonConvert.SerializeObject(request)}");
                    return true;
                }
                var extensionDetailReq = new GetPatientExtensionByCodeAndKeyDBRequest()
                {
                    PatientCode = request.IDCardNo,
                    Key = "PatientHealthInfo",
                    IsValidOperationDoctor = false
                };
                var patientExtensionInfo = _vitalPatientExtensionDBService.GetPatientExtensionDetailByCodeAndKeyAsync(extensionDetailReq).Result;
                if (string.IsNullOrEmpty(patientExtensionInfo?.ExtensionData))
                {
                    Logger.WriteLineWarn($"GetPatientExtensionDetailByCodeAndKeyAsync patientExtensionInfo is null, request:{JsonConvert.SerializeObject(request)}");
                    patientExtensionInfo = new PatientExtensionDTO();
                }
                var htmlContent = TransferAndBindHtmlTemplate(key, templateContent, patientInfo, patientExtensionInfo, null);
                try
                {
                    var converter = new SelectPdf.HtmlToPdf();
                    converter.Options.PdfPageSize = SelectPdf.PdfPageSize.A4;//设置页面大小，30种页面大小可以选
                    converter.Options.MarginTop = 40;   //上下左右边距设置  
                    converter.Options.MarginBottom = 40;
                    converter.Options.MarginLeft = 10;
                    converter.Options.MarginRight = 10;
                    var doc = new SelectPdf.PdfDocument();

                    var docStr = converter.ConvertHtmlString(htmlContent);
                    doc.Append(docStr);
                    var fileName = request.IDCardNo + "_" + key + "_" + Guid.NewGuid().ToString("N") + ".pdf";
                    var filePath = Path.Combine(_tempExmaFolder, fileName);
                    if (File.Exists(filePath))
                    {
                        File.Delete(filePath);
                    }
                    doc.Save(filePath);
                    doc.Close();

                    //转换完成，上传文件
                    personPDFUrl = UploadFileAsync(filePath, Path.GetFileName(filePath)).Result;
                    //删除文件；
                    //保存报告内容到报告表
                    var reportQueryReq = new GetReportConditionRequest()
                    {
                        PhysicalExamNumber = request.PhysicalExamNumber
                    };
                    var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
                    if (reportInfos?.Count > 0 && reportInfos.Any(c => c.Key == key))
                    {
                        var reportInfo = reportInfos.FirstOrDefault(c => c.Key == key) ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                        //找到
                        var updateReq = new UpdateReportDBRequest()
                        {
                            TemplateCode = "",//模板code
                            Code = reportInfo.Code,
                            ReportDataJson = htmlContent,
                            ReportTemplateJson = templateContent,
                            ReportState = ReportStateEnum.Reported,
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = personPDFUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            }
                        };
                        var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                    }
                    else
                    {
                        //找到并生成其他的检验报告；
                        var dbRequest = new CreateReportDBRequest
                        {
                            Key = key,
                            ContractedDoctor = "",
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = "",
                            PatientCode = patientExtensionInfo.PatientCode,
                            ReportDataJson = htmlContent,
                            ReportTemplateJson = templateContent,
                            PhysicalExamNumber = request.PhysicalExamNumber,
                            ExamCode = "",
                            OperationDoctor = patientInfo.ContractedDoctor,
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = personPDFUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            }
                        };
                        var reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
                    }
                    if (File.Exists(filePath))
                    {
                        File.Delete(filePath);
                    }
                    res = true;
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"DoPersonBasicInfoReport Error, request:{JsonConvert.SerializeObject(request)}, ex:{ex}");
                    return true;
                }
            }
            return res;
        }


        /// <summary>
        /// 转换和绑定html模板
        /// </summary>
        private string TransferAndBindHtmlTemplate(string key, string htmlContent, PatientDTO patientInfo, PatientExtensionDTO patientExtensionInfo, List<ExamRecordDTO> examRecords, string physicalExamNumber = "",
        ExamDTO examData = null)
        {
            var result = "";
            switch (key)
            {
                case "HEIPersonInfo":
                    result = BindPersonInfoHtml(htmlContent, patientInfo, patientExtensionInfo);
                    break;
                case "HEIHealthExamination1":
                    result = BindHealthExam1InfoHtml(htmlContent, patientInfo, examRecords);
                    break;
                case "HEIHealthExamination2":
                    result = BindHealthExam2InfoHtml(htmlContent, patientInfo, examRecords);
                    break;
                case "HEIHealthExamination3":
                    result = BindHealthExam3InfoHtml(htmlContent, patientInfo, examRecords);
                    break;
                case "HEIHealthExamination4":
                    result = BindHealthExam4InfoHtml(htmlContent, patientInfo, examRecords);
                    break;
                case "HEILNRZLNLPG":
                    result = BindHEILNRZLNLPGInfoHtml(htmlContent, patientInfo, examData);
                    break;
                case "HEITCMCDetailRecord1":
                    result = BindHEITCMCDetailRecordInfoHtml(htmlContent, patientInfo, examRecords, 1, "");
                    break;
                case "HEITCMCDetailRecord2":
                    result = BindHEITCMCDetailRecordInfoHtml(htmlContent, patientInfo, examRecords, 2, physicalExamNumber);
                    break;
                default:
                    break;
            }
            return result;
        }

        /// <summary>
        /// 绑定老年人中医药健康服务管理记录表
        /// </summary>
        private string BindHEITCMCDetailRecordInfoHtml(string htmlContent, PatientDTO patientInfo, List<ExamRecordDTO> examRecords, int type, string physicalExamNumberStr)
        {
            var curExamRecord = examRecords?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();
            var PatientName = patientInfo.PatientName;
            var PhysicalExamNumber = patientInfo.RecordNo;
            var PhysicalExamNumberHtml = "";
            var PhysicalExamNumberTitle = "";
            if (!string.IsNullOrEmpty(PhysicalExamNumber))
            {
                char[] chars = PhysicalExamNumber.ToCharArray();
                foreach (char c in chars)
                {
                    PhysicalExamNumberHtml += "<div class='box'><span>" + c + "</span></div>";
                }
                PhysicalExamNumberTitle = "<span style='font-weight: bold;'>编号：</span>";
            }

            var result = htmlContent;
            //获取中医体质信息和结论信息
            var templateReq = new GetTemplateByKeyDBRequest()
            {
                IsValidOperationDoctor = false,
                Key = "Key",
                Value = "LNRZYYJKGLFWJL"
            };
            var templateInfo = _vitalTemplateDBService.GetTemplateDetailByKeyAsync(templateReq).Result;
            if (string.IsNullOrEmpty(templateInfo?.Code) || string.IsNullOrEmpty(templateInfo?.TemplateContent))
            {
                Logger.WriteLineWarn($"BindHEITCMCDetailRecordInfoHtml is empty, request:{JsonConvert.SerializeObject(templateReq)}");
                return result;
            }
            var questionList = new List<string>();
            var scoreDic = new Dictionary<string, DataItemDTO>();
            var physicalConclusion = "";
            var healthGuidanceList = new List<string>();
            var curExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "HEITCMC") ?? new ExamRecordDataDTO();
            if (string.IsNullOrEmpty(curExamRecordData?.ExamData))
            {
                Logger.WriteLineWarn($"curExamRecordData is empty, patientInfo:{JsonConvert.SerializeObject(patientInfo)}");
                return result;
            }
            else
            {
                var tcmcObj = JObject.Parse(curExamRecordData.ExamData);
                if (tcmcObj != null)
                {
                    if (tcmcObj.ContainsKey("qusition1"))
                    {
                        questionList.Add((tcmcObj["qusition1"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition2"))
                    {
                        questionList.Add((tcmcObj["qusition2"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition3"))
                    {
                        questionList.Add((tcmcObj["qusition3"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition4"))
                    {
                        questionList.Add((tcmcObj["qusition4"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition5"))
                    {
                        questionList.Add((tcmcObj["qusition5"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition6"))
                    {
                        questionList.Add((tcmcObj["qusition6"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition7"))
                    {
                        questionList.Add((tcmcObj["qusition7"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition8"))
                    {
                        questionList.Add((tcmcObj["qusition8"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition9"))
                    {
                        questionList.Add((tcmcObj["qusition8"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition10"))
                    {
                        questionList.Add((tcmcObj["qusition10"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition11"))
                    {
                        questionList.Add((tcmcObj["qusition11"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition12"))
                    {
                        questionList.Add((tcmcObj["qusition12"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition13"))
                    {
                        questionList.Add((tcmcObj["qusition13"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition14"))
                    {
                        questionList.Add((tcmcObj["qusition14"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition15"))
                    {
                        questionList.Add((tcmcObj["qusition15"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition16"))
                    {
                        questionList.Add((tcmcObj["qusition16"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition17"))
                    {
                        questionList.Add((tcmcObj["qusition17"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition18"))
                    {
                        questionList.Add((tcmcObj["qusition18"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition19"))
                    {
                        questionList.Add((tcmcObj["qusition19"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition20"))
                    {
                        questionList.Add((tcmcObj["qusition20"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition21"))
                    {
                        questionList.Add((tcmcObj["qusition21"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition22"))
                    {
                        questionList.Add((tcmcObj["qusition22"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition23"))
                    {
                        questionList.Add((tcmcObj["qusition23"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition24"))
                    {
                        questionList.Add((tcmcObj["qusition24"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition25"))
                    {
                        questionList.Add((tcmcObj["qusition25"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition26"))
                    {
                        questionList.Add((tcmcObj["qusition26"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition27"))
                    {
                        questionList.Add((tcmcObj["qusition27"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition28"))
                    {
                        questionList.Add((tcmcObj["qusition28"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition29"))
                    {
                        questionList.Add((tcmcObj["qusition29"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition30"))
                    {
                        questionList.Add((tcmcObj["qusition30"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition31"))
                    {
                        questionList.Add((tcmcObj["qusition31"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition32"))
                    {
                        questionList.Add((tcmcObj["qusition32"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("qusition33"))
                    {
                        questionList.Add((tcmcObj["qusition33"]?.ToString() ?? string.Empty));
                    }
                    if (tcmcObj.ContainsKey("PhysicalConclusion"))
                    {
                        physicalConclusion = tcmcObj["PhysicalConclusion"]?.ToString() ?? string.Empty;
                    }
                    if (tcmcObj.ContainsKey("HealthGuidance"))
                    {
                        var healthGuidances = (JArray)tcmcObj["HealthGuidance"];
                        if (healthGuidances?.Count > 0)
                        {
                            foreach (var healthGuidanceItem in healthGuidances)
                            {
                                healthGuidanceList.Add((healthGuidanceItem?.ToString() ?? string.Empty));
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Ping_Score"))
                    {
                        var tScore = tcmcObj["Ping_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Ping_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Qi_Score"))
                    {
                        var tScore = tcmcObj["Qi_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Qi_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Qiyu_Score"))
                    {
                        var tScore = tcmcObj["Qiyu_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Qiyu_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Tan_Score"))
                    {
                        var tScore = tcmcObj["Tan_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Tan_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Yin_Score"))
                    {
                        var tScore = tcmcObj["Yin_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Yin_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Yang_Score"))
                    {
                        var tScore = tcmcObj["Yang_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Yang_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Te_Score"))
                    {
                        var tScore = tcmcObj["Te_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Te_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Xue_Score"))
                    {
                        var tScore = tcmcObj["Xue_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Xue_Score", dataItem);
                            }
                        }
                    }
                    if (tcmcObj.ContainsKey("Shi_Score"))
                    {
                        var tScore = tcmcObj["Shi_Score"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(tScore))
                        {
                            var tScoreObj = JObject.Parse(tScore);
                            if (tScoreObj != null)
                            {
                                var dataItem = new DataItemDTO();
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Key = tScoreObj["Result"]?.ToString() ?? string.Empty;
                                }
                                if (tScoreObj.ContainsKey("Result"))
                                {
                                    dataItem.Value = tScoreObj["Value"]?.ToString() ?? string.Empty;
                                }
                                scoreDic.Add("Shi_Score", dataItem);
                            }
                        }
                    }
                }
            }
            var bindValue = "✓";
            var OldPersonTCMCRecord = "";
            //读取模板
            var templateContentJson = "{\"TemplateContent\":" + templateInfo.TemplateContent + "}";
            var tcmcTemplateJsonObj = JObject.Parse(templateContentJson);
            if (tcmcTemplateJsonObj != null)
            {
                if (tcmcTemplateJsonObj.ContainsKey("TemplateContent"))
                {
                    var templateContentArray = (JArray)tcmcTemplateJsonObj["TemplateContent"];
                    if (templateContentArray?.Count > 0)
                    {
                        foreach (var templateItem in templateContentArray)
                        {
                            var tempKey = templateItem["key"]?.ToString() ?? string.Empty;
                            if (tempKey == "constitution")
                            {
                                var childrenList = (JArray)templateItem["children"];
                                if (childrenList?.Count > 0)
                                {
                                    for (var i = 0; i < childrenList.Count; i++)
                                    {
                                        if (type == 1)
                                        {
                                            if (i < 25)
                                            { }
                                            else
                                            {
                                                continue;
                                            }
                                        }
                                        if (type == 2)
                                        {
                                            if (i >= 25)
                                            { }
                                            else
                                            {
                                                continue;
                                            }
                                        }
                                        var childItem = childrenList[i];
                                        var lable = childItem["label"]?.ToString() ?? string.Empty;
                                        OldPersonTCMCRecord += "<tr>";
                                        OldPersonTCMCRecord += $"<td colspan='5'>{lable}</td>";
                                        var options = (JArray)childItem["options"];
                                        if (options?.Count > 0)
                                        {
                                            foreach (var optionItem in options)
                                            {
                                                var value = optionItem["value"]?.ToString() ?? string.Empty;
                                                var optionLabel = optionItem["label"]?.ToString() ?? string.Empty;
                                                if (value == questionList[i])
                                                {
                                                    OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}{value}<br/><br/>{optionLabel}</td>";
                                                }
                                                else
                                                {
                                                    OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{value}<br/><br/>{optionLabel}</td>";
                                                }
                                            }
                                        }
                                        else
                                        {
                                            if (questionList[i] == "1")
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}1</td>";
                                            }
                                            else
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>1</td>";
                                            }
                                            if (questionList[i] == "2")
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}2</td>";
                                            }
                                            else
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>2</td>";
                                            }
                                            if (questionList[i] == "3")
                                            {

                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}3</td>";
                                            }
                                            else
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>3</td>";
                                            }
                                            if (questionList[i] == "4")
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}4</td>";
                                            }
                                            else
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>4</td>";
                                            }
                                            if (questionList[i] == "5")
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>{bindValue}5</td>";
                                            }
                                            else
                                            {
                                                OldPersonTCMCRecord += $"<td style='width:90px' class='textAlign'>5</td>";
                                            }
                                        }
                                        OldPersonTCMCRecord += "</tr>";
                                    }
                                }
                            }
                            else if (tempKey == "healthGuidance")
                            {
                                if (type == 1)
                                {
                                    continue;
                                }
                                var childrenList = (JArray)templateItem["children"];
                                var optionList = (JArray)templateItem["options"];
                                if (childrenList?.Count > 0)
                                {
                                    var tcmcScoreList = new List<TCMCScoreInfo>();
                                    OldPersonTCMCRecord += "<tr>";
                                    OldPersonTCMCRecord += $"<td style='width:60px' class='textAlign'>体质类型</td>";
                                    for (var i = 0; i < childrenList.Count; i++)
                                    {
                                        var childItem = childrenList[i];
                                        var lable = childItem["label"]?.ToString() ?? string.Empty;
                                        OldPersonTCMCRecord += $"<td class='textAlign'>{lable}</td>";
                                        var groupKeys = (JArray)childItem["groupKeys"];
                                        if (groupKeys?.Count > 0)
                                        {
                                            var curKey = groupKeys[0]?.ToString() ?? string.Empty;
                                            if (scoreDic.Keys.Contains(curKey))
                                            {
                                                var findEntity = scoreDic[curKey];
                                                var tcmcResult = new TCMCScoreInfo();
                                                tcmcResult.Key = curKey;
                                                tcmcResult.Score = findEntity.Value;
                                                if (findEntity.Key == "是")
                                                {
                                                    tcmcResult.Result1 = bindValue;
                                                }
                                                else if (findEntity.Key == "倾向是")
                                                {
                                                    tcmcResult.Result2 = bindValue;
                                                }
                                                if (optionList?.Count > 0)
                                                {
                                                    foreach (var optionItem in optionList)
                                                    {
                                                        var optionLabel = optionItem["label"]?.ToString() ?? string.Empty;
                                                        var optionValue = optionItem["value"]?.ToString() ?? string.Empty;
                                                        var dicKey = optionValue + ".&nbsp;" + optionLabel;
                                                        tcmcResult.HealthGuidance.Add(dicKey, "");
                                                        if (curKey == physicalConclusion)
                                                        {
                                                            if (healthGuidanceList.Contains(optionValue))
                                                            {
                                                                tcmcResult.HealthGuidance[dicKey] = bindValue;
                                                            }
                                                        }
                                                    }
                                                }
                                                tcmcScoreList.Add(tcmcResult);
                                            }
                                        }
                                    }
                                    OldPersonTCMCRecord += "</tr>";
                                    var tzbs1 = "";
                                    var tzbs2 = "";
                                    var tzbs3 = "";
                                    var jkzdList = new List<string>();

                                    if (tcmcScoreList?.Count > 0)
                                    {
                                        var rows = 0;
                                        foreach (var item in tcmcScoreList)
                                        {
                                            tzbs1 += $"<td>1.&nbsp;得分&nbsp;{item.Score}</td>";
                                            tzbs2 += $"<td>{item.Result1}2.&nbsp;是&nbsp;</td>";
                                            tzbs3 += $"<td>{item.Result2}3.&nbsp;倾向是&nbsp;</td>";
                                            var j = 0;
                                            rows = item.HealthGuidance.Count;
                                            foreach (var hKey in item.HealthGuidance.Keys)
                                            {
                                                if (j >= 0 && j < jkzdList.Count)
                                                {
                                                    //存在
                                                    //取值，修改
                                                    var temp = jkzdList[j];
                                                    temp += $"<td>{item.HealthGuidance[hKey] + hKey}</td>";
                                                    jkzdList[j] = temp;
                                                }
                                                else
                                                {
                                                    //不存在，
                                                    // 新增
                                                    var temp = $"<td>{item.HealthGuidance[hKey] + hKey}</td>";
                                                    jkzdList.Add(temp);
                                                }
                                                j++;
                                            }
                                        }

                                        OldPersonTCMCRecord += "<tr>";
                                        OldPersonTCMCRecord += $"<td rowspan='3' class='textAlign'>体质辨识</td>";
                                        OldPersonTCMCRecord += tzbs1;
                                        OldPersonTCMCRecord += "</tr>";

                                        OldPersonTCMCRecord += "<tr>";
                                        OldPersonTCMCRecord += tzbs2;
                                        OldPersonTCMCRecord += "</tr>";

                                        OldPersonTCMCRecord += "<tr>";
                                        OldPersonTCMCRecord += tzbs3;
                                        OldPersonTCMCRecord += "</tr>";

                                        if (jkzdList?.Count > 0)
                                        {
                                            var jIndex = 0;
                                            foreach (var jkzdItem in jkzdList)
                                            {
                                                if (jIndex == 0)
                                                {
                                                    OldPersonTCMCRecord += "<tr>";
                                                    OldPersonTCMCRecord += $"<td rowspan='{rows}' class='textAlign'>中医药保健指导</td>";
                                                    OldPersonTCMCRecord += jkzdItem;
                                                    OldPersonTCMCRecord += "</tr>";
                                                }
                                                else
                                                {
                                                    OldPersonTCMCRecord += "<tr>";
                                                    OldPersonTCMCRecord += jkzdItem;
                                                    OldPersonTCMCRecord += "</tr>";
                                                }
                                                jIndex++;
                                            }
                                        }
                                    }


                                }
                            }
                        }
                        if (type == 2)
                        {
                            OldPersonTCMCRecord += "<tr>";
                            OldPersonTCMCRecord += $"<td colspan='2' class='textAlign'>填表日期</td>";
                            OldPersonTCMCRecord += $"<td colspan='3' class='textAlign'>{curExamRecord.UpdateTime.ToString("yyyy-MM-dd")}</td>";
                            OldPersonTCMCRecord += $"<td colspan='2' class='textAlign'>医师签名</td>";

                            var reportExamQueryReq = new GetReportConditionRequest()
                            {
                                PhysicalExamNumber = physicalExamNumberStr,
                                Key = "HEITCMC"
                            };
                            var signBase64 = "";
                            var doctorName = "";
                            var reportInfoList = _vitalReportDBService.GetVitalReportListDBAsync(reportExamQueryReq).Result;
                            if (reportInfoList?.Count > 0 && reportInfoList.Any(c => c.Key == "HEITCMC"))
                            {
                                var tcmcReportInfo = reportInfoList.FirstOrDefault(c => c.Key == "HEITCMC");
                                signBase64 = tcmcReportInfo.DoctorSign;
                                doctorName = tcmcReportInfo.DoctorName;
                            }
                            if (!string.IsNullOrEmpty(signBase64))
                            {
                                OldPersonTCMCRecord += $"<td colspan='3' class='textAlign'><img style='width:227px;height:42px;' src='{signBase64}' /></td>";
                            }
                            else
                            {
                                Logger.WriteLineWarn($"TCMCDetailRecordReport sign is empty, reportInfoList:{JsonConvert.SerializeObject(reportInfoList)}");
                                OldPersonTCMCRecord += $"<td colspan='3' class='textAlign'>{doctorName}</td>";
                            }
                            OldPersonTCMCRecord += "</tr>";
                        }
                    }
                }
            }

            result = result.Replace("{{PatientName}}", PatientName);
            result = result.Replace("{{PhysicalExamNumberTitle}}", PhysicalExamNumberTitle);
            result = result.Replace("{{PhysicalExamNumber}}", PhysicalExamNumberHtml);
            result = result.Replace("{{OldPersonTCMCRecord}}", OldPersonTCMCRecord);
            return result;
        }

        /// <summary>
        /// 绑定老年人自理能力评估表
        /// </summary>
        private string BindHEILNRZLNLPGInfoHtml(string htmlContent, PatientDTO patientInfo, ExamDTO curExamRecordData)
        {
            // var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            // curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();
            var PatientName = patientInfo.PatientName;
            var PhysicalExamNumber = patientInfo.RecordNo;
            var PhysicalExamNumberHtml = "";
            var PhysicalExamNumberTitle = "";
            if (!string.IsNullOrEmpty(PhysicalExamNumber))
            {
                char[] chars = PhysicalExamNumber.ToCharArray();
                foreach (char c in chars)
                {
                    PhysicalExamNumberHtml += "<div class='box'><span>" + c + "</span></div>";
                }
                PhysicalExamNumberTitle = "<span style='font-weight: bold;'>编号：</span>";
            }

            var AllJudgingRatings = "";

            var JudgingRatings1 = "";

            var JudgingRatings2 = "";

            var JudgingRatings3 = "";

            var JudgingRatings4 = "";

            var JudgingRatings5 = "";

            // var curExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "LNRZLNLPG") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(curExamRecordData?.ExamData))
            {
                var ZZYBZKObject = JObject.Parse(curExamRecordData.ExamData);
                if (ZZYBZKObject != null)
                {
                    if (ZZYBZKObject.ContainsKey("qusition1"))
                    {
                        JudgingRatings1 = ZZYBZKObject["qusition1"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("qusition2"))
                    {
                        JudgingRatings2 = ZZYBZKObject["qusition2"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("qusition3"))
                    {
                        JudgingRatings3 = ZZYBZKObject["qusition3"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("qusition4"))
                    {
                        JudgingRatings4 = ZZYBZKObject["qusition4"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("qusition5"))
                    {
                        JudgingRatings5 = ZZYBZKObject["qusition5"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("SelfCareScore"))
                    {
                        AllJudgingRatings = ZZYBZKObject["SelfCareScore"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var result = htmlContent;
            result = result.Replace("{{PatientName}}", PatientName);
            result = result.Replace("{{PhysicalExamNumberTitle}}", PhysicalExamNumberTitle);
            result = result.Replace("{{PhysicalExamNumber}}", PhysicalExamNumberHtml);
            result = result.Replace("{{AllJudgingRatings}}", AllJudgingRatings);

            result = result.Replace("{{JudgingRatings1}}", JudgingRatings1);

            result = result.Replace("{{JudgingRatings2}}", JudgingRatings2);

            result = result.Replace("{{JudgingRatings3}}", JudgingRatings3);

            result = result.Replace("{{JudgingRatings4}}", JudgingRatings4);

            result = result.Replace("{{JudgingRatings5}}", JudgingRatings5);
            return result;
        }

        /// <summary>
        /// 绑定健康体检html模板1
        /// </summary>
        private string BindHealthExam1InfoHtml(string htmlContent, PatientDTO patientInfo, List<ExamRecordDTO> examRecords)
        {
            var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();
            var PatientName = patientInfo.PatientName;
            var PhysicalExamNumber = patientInfo.RecordNo;
            var PhysicalExamNumberHtml = "";
            var PhysicalExamNumberTitle = "";
            if (!string.IsNullOrEmpty(PhysicalExamNumber))
            {
                char[] chars = PhysicalExamNumber.ToCharArray();
                foreach (char c in chars)
                {
                    PhysicalExamNumberHtml += "<div class='box'><span>" + c + "</span></div>";
                }
                PhysicalExamNumberTitle = "<span style='font-weight: bold;'>编号：</span>";
            }
            var ExamTime = curExamRecord.ExamTime.ToString("yyyy-MM-dd");
            var PrincipalName = patientInfo.CreatedDoctorName ?? string.Empty;

            var Symptom1 = "";
            var Symptom2 = "";
            var Symptom3 = "";
            var Symptom4 = "";
            var Symptom5 = "";
            var Symptom6 = "";
            var Symptom7 = "";
            var Symptom8 = "";
            var Symptom9 = "";
            var Symptom10 = "";
            var Temperature = "";
            var Pulse_Frequency = "";
            var Breathe = "";
            var Sbp_Left = "";
            var Dbp_Left = "";
            var Sbp_Right = "";
            var Dbp_Right = "";
            var Height = "";
            var Weight = "";
            var Waist = "";
            var Bmi = "";
            var Older_Health_State = "";
            var Older_Living_Ability = "";
            var Older_Intelligence = "";
            var Older_Cognitive = "";
            var Older_Depression = "";
            var Older_Emotional = "";
            var EatingHabits1 = "";
            var EatingHabits2 = "";
            var EatingHabits3 = "";
            var Exercise_Frequency = "";
            var Exercise_Daily_Time = "";
            var Exercise_Time = "";
            var Exercise_Way = "";
            var Smoking_State = "";
            var Smoking_Daily = "";
            var Smoking_Start_Age = "";
            var Smoking_Quit_Age = "";
            var Drinking_Frequency = "";
            var Drinking_Daily = "";
            var Drinking_Quit_Age = "";
            var Drinking_Is_Quit = "";
            var Drinking_Start_Age = "";
            var Drinking_Is_Drunk = "";
            var Drinking_Type1 = "";
            var Drinking_Type2 = "";
            var Drinking_Type3 = "";
            var Drinking_Type4 = "";
            var Risk_Working_Type = "";
            var Risk_Working_Time = "";
            var OccupationalDiseaseExposureHistory = "";
            var Risk_Dust = "";
            var Risk_Dust_Protect = "";
            var Risk_Dust_Is_Quit = "";
            var Risk_Radiate = "";
            var Risk_Radiate_Protect = "";
            var Risk_Radiate_Is_Quit = "";
            var Risk_Physics = "";
            var Risk_Physics_Protect = "";
            var Risk_Physics_Is_Quit = "";
            var Risk_Chemical = "";
            var Risk_Chemical_Protect = "";
            var Risk_Chemical_Is_Quit = "";
            var Risk_Other = "";
            var Risk_Other_Protect = "";
            var Risk_Other_Is_Quit = "";

            var curExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZZYBZK") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(curExamRecordData?.ExamData))
            {
                var ZZYBZKObject = JObject.Parse(curExamRecordData.ExamData);
                if (ZZYBZKObject != null)
                {
                    if (ZZYBZKObject.ContainsKey("Temperature"))
                    {
                        Temperature = ZZYBZKObject["Temperature"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Pulse_Frequency"))
                    {
                        Pulse_Frequency = ZZYBZKObject["Pulse_Frequency"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Breathe"))
                    {
                        Breathe = ZZYBZKObject["Breathe"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Symptom"))
                    {
                        var symptom = (JArray)ZZYBZKObject["Symptom"];
                        if (symptom?.Count > 0)
                        {
                            var symptomList = symptom.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= symptomList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    Symptom1 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    Symptom2 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    Symptom3 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    Symptom4 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    Symptom5 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 6)
                                {
                                    Symptom6 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 7)
                                {
                                    Symptom7 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 8)
                                {
                                    Symptom8 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 9)
                                {
                                    Symptom9 = symptomList[(i - 1)].ToString();
                                }
                                else if (i == 10)
                                {
                                    Symptom10 = symptomList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (ZZYBZKObject.ContainsKey("BloodPressure"))
                    {
                        var bloodPressure = ZZYBZKObject["BloodPressure"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(bloodPressure))
                        {
                            var bloodPressureObj = JObject.Parse(bloodPressure);
                            if (bloodPressureObj != null)
                            {
                                if (bloodPressureObj.ContainsKey("left") && bloodPressureObj["left"] != null)
                                {
                                    var leftStr = bloodPressureObj["left"]?.ToString() ?? string.Empty;
                                    if (!string.IsNullOrEmpty(leftStr))
                                    {
                                        leftStr = leftStr.Replace("[", "").Replace("]", "").Replace("\"", "");
                                    }
                                    var leftList = leftStr.Split(",");
                                    if (leftList?.Length > 0)
                                    {
                                        for (var i = 1; i <= leftList.Length; i++)
                                        {
                                            if (i == 1)
                                            {
                                                Sbp_Left = leftList[(i - 1)].ToString();
                                            }
                                            else if (i == 2)
                                            {
                                                Dbp_Left = leftList[(i - 1)].ToString();
                                            }
                                        }
                                    }
                                }
                                if (bloodPressureObj.ContainsKey("right") && bloodPressureObj["right"] != null)
                                {
                                    var rightStr = bloodPressureObj["right"]?.ToString() ?? string.Empty;
                                    if (!string.IsNullOrEmpty(rightStr))
                                    {
                                        rightStr = rightStr.Replace("[", "").Replace("]", "").Replace("\"", "");
                                    }
                                    var rightList = rightStr.Split(",");
                                    if (rightList?.Length > 0)
                                    {
                                        for (var i = 1; i <= rightList.Length; i++)
                                        {
                                            if (i == 1)
                                            {
                                                Sbp_Right = rightList[(i - 1)].ToString();
                                            }
                                            else if (i == 2)
                                            {
                                                Dbp_Right = rightList[(i - 1)].ToString();
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                    if (ZZYBZKObject.ContainsKey("Height"))
                    {
                        Height = ZZYBZKObject["Height"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Bmi"))
                    {
                        Bmi = ZZYBZKObject["Bmi"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Weight"))
                    {
                        Weight = ZZYBZKObject["Weight"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("WAISTLINE"))
                    {
                        Waist = ZZYBZKObject["WAISTLINE"]?.ToString() ?? string.Empty;
                    }

                    if (ZZYBZKObject.ContainsKey("Older_Health_State"))
                    {
                        Older_Health_State = ZZYBZKObject["Older_Health_State"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Older_Living_Ability"))
                    {
                        Older_Living_Ability = ZZYBZKObject["Older_Living_Ability"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Older_Cognitive"))
                    {
                        Older_Cognitive = ZZYBZKObject["Older_Cognitive"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Older_Intelligence"))
                    {
                        Older_Intelligence = ZZYBZKObject["Older_Intelligence"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Older_Depression"))
                    {
                        Older_Depression = ZZYBZKObject["Older_Depression"]?.ToString() ?? string.Empty;
                    }
                    if (ZZYBZKObject.ContainsKey("Older_Emotional"))
                    {
                        Older_Emotional = ZZYBZKObject["Older_Emotional"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var SHFSExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "SHFS") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(SHFSExamRecordData?.ExamData))
            {
                var SHFSObject = JObject.Parse(SHFSExamRecordData.ExamData);
                if (SHFSObject != null)
                {
                    if (SHFSObject.ContainsKey("Exercise_Frequency"))
                    {
                        Exercise_Frequency = SHFSObject["Exercise_Frequency"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Exercise_Daily_Time"))
                    {
                        Exercise_Daily_Time = SHFSObject["Exercise_Daily_Time"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Exercise_Time"))
                    {
                        Exercise_Time = SHFSObject["Exercise_Time"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Exercise_Way"))
                    {
                        Exercise_Way = SHFSObject["Exercise_Way"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Diet"))
                    {
                        var diet = (JArray)SHFSObject["Diet"];
                        if (diet?.Count > 0)
                        {
                            var dietList = diet.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= dietList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    EatingHabits1 = dietList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    EatingHabits2 = dietList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    EatingHabits3 = dietList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (SHFSObject.ContainsKey("Smoking_State"))
                    {
                        Smoking_State = SHFSObject["Smoking_State"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Smoking_Daily"))
                    {
                        Smoking_Daily = SHFSObject["Smoking_Daily"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Smoking_Start_Age"))
                    {
                        Smoking_Start_Age = SHFSObject["Smoking_Start_Age"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Smoking_Quit_Age"))
                    {
                        Smoking_Quit_Age = SHFSObject["Smoking_Quit_Age"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Frequency"))
                    {
                        Drinking_Frequency = SHFSObject["Drinking_Frequency"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Daily"))
                    {
                        Drinking_Daily = SHFSObject["Drinking_Daily"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Quit_Age"))
                    {
                        Drinking_Quit_Age = SHFSObject["Drinking_Quit_Age"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Is_Quit"))
                    {
                        Drinking_Is_Quit = SHFSObject["Drinking_Is_Quit"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Start_Age"))
                    {
                        Drinking_Start_Age = SHFSObject["Drinking_Start_Age"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Is_Drunk"))
                    {
                        Drinking_Is_Drunk = SHFSObject["Drinking_Is_Drunk"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Drinking_Type"))
                    {
                        var drinking_Type = (JArray)SHFSObject["Drinking_Type"];
                        if (drinking_Type?.Count > 0)
                        {
                            var drinking_TypeList = drinking_Type.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= drinking_TypeList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    Drinking_Type1 = drinking_TypeList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    Drinking_Type2 = drinking_TypeList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    Drinking_Type3 = drinking_TypeList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    Drinking_Type4 = drinking_TypeList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (SHFSObject.ContainsKey("Risk_Working_Type"))
                    {
                        Risk_Working_Type = SHFSObject["Risk_Working_Type"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Working_Time"))
                    {
                        Risk_Working_Time = SHFSObject["Risk_Working_Time"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Working_State"))
                    {
                        OccupationalDiseaseExposureHistory = SHFSObject["Risk_Working_State"]?.ToString() ?? string.Empty;
                    }

                    if (SHFSObject.ContainsKey("Risk_Dust"))
                    {
                        Risk_Dust = SHFSObject["Risk_Dust"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Dust_Protect"))
                    {
                        var risk_Dust_Protect = SHFSObject["Risk_Dust_Protect"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(risk_Dust_Protect))
                        {
                            var risk_Dust_ProtectObj = JObject.Parse(risk_Dust_Protect);
                            if (risk_Dust_ProtectObj != null)
                            {
                                Risk_Dust_Protect = risk_Dust_ProtectObj["label"].ToString() ?? string.Empty;
                                Risk_Dust_Is_Quit = risk_Dust_ProtectObj["value"]["value"].ToString() ?? string.Empty;
                            }
                        }
                    }
                    if (SHFSObject.ContainsKey("Risk_Radiate"))
                    {
                        Risk_Radiate = SHFSObject["Risk_Radiate"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Radiate_Protect"))
                    {
                        var risk_Radiate_Protect = SHFSObject["Risk_Radiate_Protect"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(risk_Radiate_Protect))
                        {
                            var risk_Radiate_ProtectObj = JObject.Parse(risk_Radiate_Protect);
                            if (risk_Radiate_ProtectObj != null)
                            {
                                Risk_Radiate_Protect = risk_Radiate_ProtectObj["label"].ToString() ?? string.Empty;
                                Risk_Radiate_Is_Quit = risk_Radiate_ProtectObj["value"]["value"].ToString() ?? string.Empty;
                            }
                        }
                    }

                    if (SHFSObject.ContainsKey("Risk_Physics"))
                    {
                        Risk_Physics = SHFSObject["Risk_Physics"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Physics_Protect"))
                    {
                        var risk_Physics_Protect = SHFSObject["Risk_Physics_Protect"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(risk_Physics_Protect))
                        {
                            var risk_Physics_ProtectObj = JObject.Parse(risk_Physics_Protect);
                            if (risk_Physics_ProtectObj != null)
                            {
                                Risk_Physics_Protect = risk_Physics_ProtectObj["label"].ToString() ?? string.Empty;
                                Risk_Physics_Is_Quit = risk_Physics_ProtectObj["value"]["value"].ToString() ?? string.Empty;
                            }
                        }
                    }

                    if (SHFSObject.ContainsKey("Risk_Chemical"))
                    {
                        Risk_Chemical = SHFSObject["Risk_Chemical"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Chemical_Protect"))
                    {
                        var risk_Chemical_Protect = SHFSObject["Risk_Chemical_Protect"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(risk_Chemical_Protect))
                        {
                            var risk_Chemical_ProtectObj = JObject.Parse(risk_Chemical_Protect);
                            if (risk_Chemical_ProtectObj != null)
                            {
                                Risk_Chemical_Protect = risk_Chemical_ProtectObj["label"].ToString() ?? string.Empty;
                                Risk_Chemical_Is_Quit = risk_Chemical_ProtectObj["value"]["value"].ToString() ?? string.Empty;
                            }
                        }
                    }

                    if (SHFSObject.ContainsKey("Risk_Other"))
                    {
                        Risk_Other = SHFSObject["Risk_Other"]?.ToString() ?? string.Empty;
                    }
                    if (SHFSObject.ContainsKey("Risk_Other_Protect"))
                    {
                        var risk_Other_Protect = SHFSObject["Risk_Other_Protect"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(risk_Other_Protect))
                        {
                            var risk_Other_ProtectObj = JObject.Parse(risk_Other_Protect);
                            if (risk_Other_ProtectObj != null)
                            {
                                Risk_Other_Protect = risk_Other_ProtectObj["label"].ToString() ?? string.Empty;
                                Risk_Other_Is_Quit = risk_Other_ProtectObj["value"]["value"].ToString() ?? string.Empty;
                            }
                        }
                    }
                }
            }

            var result = htmlContent;
            result = result.Replace("{{PatientName}}", PatientName);
            result = result.Replace("{{PhysicalExamNumberTitle}}", PhysicalExamNumberTitle);
            result = result.Replace("{{PhysicalExamNumber}}", PhysicalExamNumberHtml);
            result = result.Replace("{{ExamTime}}", ExamTime);
            result = result.Replace("{{PrincipalName}}", PrincipalName);
            result = result.Replace("{{Symptom1}}", Symptom1);
            result = result.Replace("{{Symptom2}}", Symptom2);
            result = result.Replace("{{Symptom3}}", Symptom3);
            result = result.Replace("{{Symptom4}}", Symptom4);
            result = result.Replace("{{Symptom5}}", Symptom5);
            result = result.Replace("{{Symptom6}}", Symptom6);
            result = result.Replace("{{Symptom7}}", Symptom7);
            result = result.Replace("{{Symptom8}}", Symptom8);
            result = result.Replace("{{Symptom9}}", Symptom9);
            result = result.Replace("{{Symptom10}}", Symptom10);
            result = result.Replace("{{Temperature}}", Temperature);
            result = result.Replace("{{Pulse_Frequency}}", Pulse_Frequency);
            result = result.Replace("{{Breathe}}", Breathe);
            result = result.Replace("{{Sbp_Left}}", Sbp_Left);
            result = result.Replace("{{Dbp_Left}}", Dbp_Left);
            result = result.Replace("{{Sbp_Right}}", Sbp_Right);
            result = result.Replace("{{Dbp_Right}}", Dbp_Right);
            result = result.Replace("{{Height}}", Height);
            result = result.Replace("{{Weight}}", Weight);
            result = result.Replace("{{Waist}}", Waist);
            result = result.Replace("{{Bmi}}", Bmi);
            result = result.Replace("{{Older_Health_State}}", Older_Health_State);
            result = result.Replace("{{Older_Living_Ability}}", Older_Living_Ability);
            result = result.Replace("{{Older_Intelligence}}", Older_Intelligence);
            result = result.Replace("{{Older_Cognitive}}", Older_Cognitive);
            result = result.Replace("{{Older_Depression}}", Older_Depression);
            result = result.Replace("{{Older_Emotional}}", Older_Emotional);
            result = result.Replace("{{EatingHabits1}}", EatingHabits1);
            result = result.Replace("{{EatingHabits2}}", EatingHabits2);
            result = result.Replace("{{EatingHabits3}}", EatingHabits3);
            result = result.Replace("{{Exercise_Frequency}}", Exercise_Frequency);
            result = result.Replace("{{Exercise_Daily_Time}}", Exercise_Daily_Time);
            result = result.Replace("{{Exercise_Time}}", Exercise_Time);
            result = result.Replace("{{Exercise_Way}}", Exercise_Way);
            result = result.Replace("{{Smoking_State}}", Smoking_State);
            result = result.Replace("{{Smoking_Daily}}", Smoking_Daily);
            result = result.Replace("{{Smoking_Start_Age}}", Smoking_Start_Age);
            result = result.Replace("{{Smoking_Quit_Age}}", Smoking_Quit_Age);
            result = result.Replace("{{Drinking_Frequency}}", Drinking_Frequency);
            result = result.Replace("{{Drinking_Daily}}", Drinking_Daily);
            result = result.Replace("{{Drinking_Quit_Age}}", Drinking_Quit_Age);
            result = result.Replace("{{Drinking_Is_Quit}}", Drinking_Is_Quit);
            result = result.Replace("{{Drinking_Start_Age}}", Drinking_Start_Age);
            result = result.Replace("{{Drinking_Is_Drunk}}", Drinking_Is_Drunk);
            result = result.Replace("{{Drinking_Type1}}", Drinking_Type1);
            result = result.Replace("{{Drinking_Type2}}", Drinking_Type2);
            result = result.Replace("{{Drinking_Type3}}", Drinking_Type3);
            result = result.Replace("{{Drinking_Type4}}", Drinking_Type4);
            result = result.Replace("{{Risk_Working_Type}}", Risk_Working_Type);
            result = result.Replace("{{Risk_Working_Time}}", Risk_Working_Time);
            result = result.Replace("{{OccupationalDiseaseExposureHistory}}", OccupationalDiseaseExposureHistory);
            result = result.Replace("{{Risk_Dust}}", Risk_Dust);
            result = result.Replace("{{Risk_Dust_Protect}}", Risk_Dust_Protect);
            result = result.Replace("{{Risk_Dust_Is_Quit}}", Risk_Dust_Is_Quit);
            result = result.Replace("{{Risk_Radiate}}", Risk_Radiate);
            result = result.Replace("{{Risk_Radiate_Protect}}", Risk_Radiate_Protect);
            result = result.Replace("{{Risk_Radiate_Is_Quit}}", Risk_Radiate_Is_Quit);
            result = result.Replace("{{Risk_Physics}}", Risk_Physics);
            result = result.Replace("{{Risk_Physics_Protect}}", Risk_Physics_Protect);
            result = result.Replace("{{Risk_Physics_Is_Quit}}", Risk_Physics_Is_Quit);
            result = result.Replace("{{Risk_Chemical}}", Risk_Chemical);
            result = result.Replace("{{Risk_Chemical_Protect}}", Risk_Chemical_Protect);
            result = result.Replace("{{Risk_Chemical_Is_Quit}}", Risk_Chemical_Is_Quit);
            result = result.Replace("{{Risk_Other}}", Risk_Other);
            result = result.Replace("{{Risk_Other_Protect}}", Risk_Other_Protect);
            result = result.Replace("{{Risk_Other_Is_Quit}}", Risk_Other_Is_Quit);
            return result;
        }

        /// <summary>
        /// 绑定健康体检html模板2
        /// </summary>
        private string BindHealthExam2InfoHtml(string htmlContent, PatientDTO patientInfo, List<ExamRecordDTO> examRecords)
        {
            var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();
            var OralCavity = "";
            var Dentition1 = "";
            var Dentition2 = "";
            var Dentition3 = "";
            var MissingToothLT = "";
            var MissingToothRT = "";
            var MissingToothLB = "";
            var MissingToothRB = "";

            var DentalCariesLT = "";
            var DentalCariesRT = "";
            var DentalCariesLB = "";
            var DentalCariesRB = "";

            var DentureLT = "";
            var DentureRT = "";
            var DentureLB = "";
            var DentureRB = "";

            var Pharynx = "";
            var Risk_Other_Left = "";
            var Risk_Other_Right = "";
            var Correct_Risk_Other_Left = "";
            var Correct_Risk_Other_Right = "";
            var Hearing = "";
            var Athletic = "";
            var Eye_Ground = "";
            var Skin = "";
            var Sclera = "";
            var Lymph_Node = "";
            var Barrel_Chest = "";
            var Breath_Sounds = "";
            var Rale = "";
            var Heart_Rhythm = "";
            var Heart_Murmur = "";
            var Abdominal_Tenderness = "";
            var Abdominal_Mass = "";
            var Hepatomegaly = "";
            var Splenomegalia = "";
            var Shifting_Dullness = "";
            var Lower_Limb_Edema = "";
            var Pedal_Artery = "";
            var Rectal_Touch = "";
            var Mammary1 = "";
            var Mammary2 = "";
            var Mammary3 = "";
            var Vulvar = "";
            var Vagina = "";
            var Cervical = "";
            var Uteri_Body = "";
            var Uteri_Adnexa = "";
            var Examination_Other = "";
            var Heart_Rate = "";

            var Blood_Hgb = "";
            var Blood_Wbc = "";
            var Blood_Rlt = "";
            var Blood_Routine_Other = "";
            var PRO = "";
            var GLU = "";
            var KET = "";
            var BLD = "";
            var Urinary_Routine_Other = "";
            var FastingBloodGlucose = "";

            var ZQCTExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZQCT") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(ZQCTExamRecordData?.ExamData))
            {
                var ZQCTObject = JObject.Parse(ZQCTExamRecordData.ExamData);
                if (ZQCTObject != null)
                {
                    var positionDic = new Dictionary<int, int>() {
                        { 1, 8 },{ 2, 7 },{ 3, 6 },{ 4, 5 },{ 5, 4 },{ 6, 3 },{ 7, 2 } ,{ 8, 1 },
                        { 9, 1 },{ 10, 2 },{ 11, 3 },{ 12, 4 },{ 13, 5 },{ 14, 6 },{ 15, 7 } ,{ 16, 8 },
                        { 17, 8 },{ 18, 7 },{ 19, 6 },{ 20, 5 },{ 21, 4 },{ 22, 3 },{ 23, 2 } ,{ 24, 1 },
                        { 25, 1 },{ 26, 2 },{ 27, 3 },{ 28, 4 },{ 29, 5 },{ 30, 6 },{ 31, 7 } ,{ 32, 8 },
                    };
                    if (ZQCTObject.ContainsKey("Lip"))
                    {
                        OralCavity = ZQCTObject["Lip"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Dentition"))
                    {
                        var denntition = ZQCTObject["Dentition"]?.ToString() ?? string.Empty;
                        var denntitionObj = JObject.Parse(denntition);
                        if (denntitionObj != null)
                        {
                            if (denntitionObj.ContainsKey("status"))
                            {
                                var statusList = (JArray)denntitionObj["status"];
                                if (statusList?.Count > 0)
                                {
                                    for (var i = 1; i <= statusList.Count; i++)
                                    {
                                        var val = statusList[(i - 1)].ToString();
                                        if (val == "1")
                                        {
                                            Dentition1 = (Convert.ToInt32(val) + 1).ToString();
                                            var detail_aList = (JArray)denntitionObj["detail_a"];
                                            if (detail_aList?.Count > 0)
                                            {
                                                var MissingToothLTList = new List<int>();
                                                var MissingToothRTList = new List<int>();
                                                var MissingToothLBList = new List<int>();
                                                var MissingToothRBList = new List<int>();
                                                for (var j = 1; j <= detail_aList.Count; j++)
                                                {
                                                    var value = Convert.ToInt32((detail_aList[(j - 1)]?.ToString() ?? string.Empty));
                                                    var tempVal = positionDic[value];
                                                    if (value <= 8)
                                                    {
                                                        MissingToothLTList.Add(tempVal);
                                                    }
                                                    else if (value <= 16)
                                                    {
                                                        MissingToothRTList.Add(tempVal);
                                                    }
                                                    else if (value <= 24)
                                                    {
                                                        MissingToothLBList.Add(tempVal);
                                                    }
                                                    else if (value <= 32)
                                                    {
                                                        MissingToothRBList.Add(tempVal);
                                                    }
                                                }
                                                MissingToothLT = String.Join("", MissingToothLTList.OrderByDescending(c => c).ToList());
                                                MissingToothRT = String.Join("", MissingToothRTList.OrderBy(c => c).ToList());
                                                MissingToothLB = String.Join("", MissingToothLBList.OrderByDescending(c => c).ToList());
                                                MissingToothRB = String.Join("", MissingToothRBList.OrderBy(c => c).ToList());
                                                MissingToothRB = string.IsNullOrEmpty(MissingToothRB) ? "&nbsp;" : MissingToothRB;
                                                MissingToothLB = string.IsNullOrEmpty(MissingToothLB) ? "&nbsp;" : MissingToothLB;
                                                MissingToothRT = string.IsNullOrEmpty(MissingToothRT) ? "&nbsp;" : MissingToothRT;
                                                MissingToothLT = string.IsNullOrEmpty(MissingToothLT) ? "&nbsp;" : MissingToothLT;
                                            }
                                        }
                                        else if (val == "2")
                                        {
                                            Dentition2 = (Convert.ToInt32(val) + 1).ToString();
                                            var detail_bList = (JArray)denntitionObj["detail_b"];
                                            var MissingToothLTList = new List<int>();
                                            var MissingToothRTList = new List<int>();
                                            var MissingToothLBList = new List<int>();
                                            var MissingToothRBList = new List<int>();
                                            for (var j = 1; j <= detail_bList.Count; j++)
                                            {
                                                var value = Convert.ToInt32((detail_bList[(j - 1)]?.ToString() ?? string.Empty));
                                                var tempVal = positionDic[value];
                                                if (value <= 8)
                                                {
                                                    MissingToothLTList.Add(tempVal);
                                                }
                                                else if (value <= 16)
                                                {
                                                    value = value - 8;
                                                    MissingToothRTList.Add(tempVal);
                                                }
                                                else if (value <= 24)
                                                {
                                                    value = value - 16;
                                                    MissingToothLBList.Add(tempVal);
                                                }
                                                else if (j <= 32)
                                                {
                                                    value = value - 24;
                                                    MissingToothRBList.Add(tempVal);
                                                }
                                            }
                                            DentalCariesLT = String.Join("", MissingToothLTList.OrderByDescending(c => c).ToList());
                                            DentalCariesRT = String.Join("", MissingToothRTList.OrderBy(c => c).ToList());
                                            DentalCariesLB = String.Join("", MissingToothLBList.OrderByDescending(c => c).ToList());
                                            DentalCariesRB = String.Join("", MissingToothRBList.OrderBy(c => c).ToList());
                                            DentalCariesLT = string.IsNullOrEmpty(DentalCariesLT) ? "&nbsp;" : DentalCariesLT;
                                            DentalCariesRT = string.IsNullOrEmpty(DentalCariesRT) ? "&nbsp;" : DentalCariesRT;
                                            DentalCariesLB = string.IsNullOrEmpty(DentalCariesLB) ? "&nbsp;" : DentalCariesLB;
                                            DentalCariesRB = string.IsNullOrEmpty(DentalCariesRB) ? "&nbsp;" : DentalCariesRB;
                                        }
                                        else if (val == "3")
                                        {
                                            Dentition3 = (Convert.ToInt32(val) + 1).ToString();
                                            var detail_cList = (JArray)denntitionObj["detail_c"];
                                            var MissingToothLTList = new List<int>();
                                            var MissingToothRTList = new List<int>();
                                            var MissingToothLBList = new List<int>();
                                            var MissingToothRBList = new List<int>();
                                            for (var j = 1; j <= detail_cList.Count; j++)
                                            {
                                                var value = Convert.ToInt32((detail_cList[(j - 1)]?.ToString() ?? string.Empty));
                                                var tempVal = positionDic[value];
                                                if (value <= 8)
                                                {
                                                    MissingToothLTList.Add(tempVal);
                                                }
                                                else if (value <= 16)
                                                {
                                                    value = value - 8;
                                                    MissingToothRTList.Add(tempVal);
                                                }
                                                else if (value <= 24)
                                                {
                                                    value = value - 16;
                                                    MissingToothLBList.Add(tempVal);
                                                }
                                                else if (value <= 32)
                                                {
                                                    value = value - 24;
                                                    MissingToothRBList.Add(tempVal);
                                                }
                                            }
                                            DentureLT = String.Join("", MissingToothLTList.OrderByDescending(c => c).ToList());
                                            DentureRT = String.Join("", MissingToothRTList.OrderBy(c => c).ToList());
                                            DentureLB = String.Join("", MissingToothLBList.OrderByDescending(c => c).ToList());
                                            DentureRB = String.Join("", MissingToothRBList.OrderBy(c => c).ToList());
                                            DentureLT = string.IsNullOrEmpty(DentureLT) ? "&nbsp;" : DentureLT;
                                            DentureRT = string.IsNullOrEmpty(DentureRT) ? "&nbsp;" : DentureRT;
                                            DentureLB = string.IsNullOrEmpty(DentureLB) ? "&nbsp;" : DentureLB;
                                            DentureRB = string.IsNullOrEmpty(DentureRB) ? "&nbsp;" : DentureRB;
                                        }
                                    }
                                    if (string.IsNullOrEmpty(Dentition1) && string.IsNullOrEmpty(Dentition2) && string.IsNullOrEmpty(Dentition3))
                                    {
                                        Dentition1 = "1";
                                    }
                                }
                            }
                        }
                    }
                    if (ZQCTObject.ContainsKey("Pharynx"))
                    {
                        Pharynx = ZQCTObject["Pharynx"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Heart_Rate"))
                    {
                        Heart_Rate = ZQCTObject["Heart_Rate"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Vision_Left"))
                    {
                        Risk_Other_Left = ZQCTObject["Vision_Left"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Vision_Right"))
                    {
                        Risk_Other_Right = ZQCTObject["Vision_Right"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Vision_Corrective_Left"))
                    {
                        Correct_Risk_Other_Left = ZQCTObject["Vision_Corrective_Left"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Vision_Corrective_Right"))
                    {
                        Correct_Risk_Other_Right = ZQCTObject["Vision_Corrective_Right"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Hearing"))
                    {
                        Hearing = ZQCTObject["Hearing"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Athletic"))
                    {
                        Athletic = ZQCTObject["Athletic"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Eye_Ground"))
                    {
                        Eye_Ground = ZQCTObject["Eye_Ground"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Skin"))
                    {
                        Skin = ZQCTObject["Skin"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Sclera"))
                    {
                        Sclera = ZQCTObject["Sclera"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Lymph_Node"))
                    {
                        Lymph_Node = ZQCTObject["Lymph_Node"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Barrel_Chest"))
                    {
                        Barrel_Chest = ZQCTObject["Barrel_Chest"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Breath_Sounds"))
                    {
                        Breath_Sounds = ZQCTObject["Breath_Sounds"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Rale"))
                    {
                        Rale = ZQCTObject["Rale"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Heart_Rhythm"))
                    {
                        Heart_Rhythm = ZQCTObject["Heart_Rhythm"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Heart_Murmur"))
                    {
                        Heart_Murmur = ZQCTObject["Heart_Murmur"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Abdominal_Tenderness"))
                    {
                        Abdominal_Tenderness = ZQCTObject["Abdominal_Tenderness"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Abdominal_Mass"))
                    {
                        Abdominal_Mass = ZQCTObject["Abdominal_Mass"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Hepatomegaly"))
                    {
                        Hepatomegaly = ZQCTObject["Hepatomegaly"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Splenomegalia"))
                    {
                        Splenomegalia = ZQCTObject["Splenomegalia"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Shifting_Dullness"))
                    {
                        Shifting_Dullness = ZQCTObject["Shifting_Dullness"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Lower_Limb_Edema"))
                    {
                        Lower_Limb_Edema = ZQCTObject["Lower_Limb_Edema"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Pedal_Artery"))
                    {
                        Pedal_Artery = ZQCTObject["Pedal_Artery"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Rectal_Touch"))
                    {
                        Rectal_Touch = ZQCTObject["Rectal_Touch"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Mammary"))
                    {
                        // Mammary = ZQCTObject["Mammary"]?.ToString() ?? string.Empty;
                        var mammary = (JArray)ZQCTObject["Mammary"];
                        if (mammary?.Count > 0)
                        {
                            var mammaryList = mammary.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= mammaryList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    Mammary1 = mammaryList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    Mammary2 = mammaryList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    Mammary3 = mammaryList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (ZQCTObject.ContainsKey("Vulvar"))
                    {
                        Vulvar = ZQCTObject["Vulvar"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Vagina"))
                    {
                        Vagina = ZQCTObject["Vagina"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Cervical"))
                    {
                        Cervical = ZQCTObject["Cervical"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Uteri_Body"))
                    {
                        Uteri_Body = ZQCTObject["Uteri_Body"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Uteri_Adnexa"))
                    {
                        Uteri_Adnexa = ZQCTObject["Uteri_Adnexa"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Examination_Other"))
                    {
                        Examination_Other = ZQCTObject["Examination_Other"]?.ToString() ?? string.Empty;
                    }
                    if (ZQCTObject.ContainsKey("Pharynx"))
                    {
                        Pharynx = ZQCTObject["Pharynx"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var FZJCExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "FZJC") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(FZJCExamRecordData?.ExamData))
            {
                var FZJCObject = JObject.Parse(FZJCExamRecordData.ExamData);
                if (FZJCObject != null)
                {
                    if (FZJCObject.ContainsKey("Blood_Hgb"))
                    {
                        Blood_Hgb = FZJCObject["Blood_Hgb"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Blood_Wbc"))
                    {
                        Blood_Wbc = FZJCObject["Blood_Wbc"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Blood_Rlt"))
                    {
                        Blood_Rlt = FZJCObject["Blood_Rlt"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Blood_Routine_Other"))
                    {
                        Blood_Routine_Other = FZJCObject["Blood_Routine_Other"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Urinary_Pro"))
                    {
                        var urinary_Pro = FZJCObject["Urinary_Pro"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(urinary_Pro))
                        {
                            var urinary_ProObj = JObject.Parse(urinary_Pro);
                            if (urinary_ProObj != null)
                            {
                                if (urinary_ProObj.ContainsKey("PRO"))
                                {
                                    PRO = urinary_ProObj["PRO"]?.ToString() ?? string.Empty;
                                }
                                if (urinary_ProObj.ContainsKey("GLU"))
                                {
                                    GLU = urinary_ProObj["GLU"]?.ToString() ?? string.Empty;
                                }
                                if (urinary_ProObj.ContainsKey("KET"))
                                {
                                    KET = urinary_ProObj["KET"]?.ToString() ?? string.Empty;
                                }
                                if (urinary_ProObj.ContainsKey("BLD"))
                                {
                                    BLD = urinary_ProObj["BLD"]?.ToString() ?? string.Empty;
                                }
                                if (urinary_ProObj.ContainsKey("LEU"))
                                {
                                    Urinary_Routine_Other += "白细胞" + (urinary_ProObj["LEU"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("NIT"))
                                {
                                    Urinary_Routine_Other += "亚硝酸盐" + (urinary_ProObj["NIT"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("UBG"))
                                {
                                    Urinary_Routine_Other += "尿胆原" + (urinary_ProObj["UBG"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("BIL"))
                                {
                                    Urinary_Routine_Other += "胆红素" + (urinary_ProObj["BIL"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("PH"))
                                {
                                    Urinary_Routine_Other += "PH值" + (urinary_ProObj["PH"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("SG"))
                                {
                                    Urinary_Routine_Other += "比重" + (urinary_ProObj["SG"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                                if (urinary_ProObj.ContainsKey("VC"))
                                {
                                    Urinary_Routine_Other += "维生素C" + (urinary_ProObj["VC"]?.ToString() ?? string.Empty) + ";&nbsp;";
                                }
                            }
                        }
                    }
                }
            }

            var curExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZZYBZK") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(curExamRecordData?.ExamData))
            {
                var ZZYBZKObject = JObject.Parse(curExamRecordData.ExamData);
                if (ZZYBZKObject != null)
                {
                    if (ZZYBZKObject.ContainsKey("Fasting_Glucose_Mmol"))
                    {
                        FastingBloodGlucose = ZZYBZKObject["Fasting_Glucose_Mmol"]?.ToString() ?? string.Empty;
                    }
                }
            }
            var result = htmlContent;
            result = result.Replace("{{OralCavity}}", OralCavity);
            result = result.Replace("{{Dentition1}}", Dentition1);
            result = result.Replace("{{Dentition2}}", Dentition2);
            result = result.Replace("{{Dentition3}}", Dentition3);

            result = result.Replace("{{MissingToothLT}}", MissingToothLT);
            result = result.Replace("{{MissingToothRT}}", MissingToothRT);
            result = result.Replace("{{MissingToothLB}}", MissingToothLB);
            result = result.Replace("{{MissingToothRB}}", MissingToothRB);
            result = result.Replace("{{DentalCariesLT}}", DentalCariesLT);
            result = result.Replace("{{DentalCariesRT}}", DentalCariesRT);
            result = result.Replace("{{DentalCariesLB}}", DentalCariesLB);
            result = result.Replace("{{DentalCariesRB}}", DentalCariesRB);
            result = result.Replace("{{DentureLT}}", DentureLT);
            result = result.Replace("{{DentureRT}}", DentureRT);
            result = result.Replace("{{DentureLB}}", DentureLB);
            result = result.Replace("{{DentureRB}}", DentureRB);

            result = result.Replace("{{Heart_Rate}}", Heart_Rate);
            result = result.Replace("{{Pharynx}}", Pharynx);
            result = result.Replace("{{Risk_Other_Left}}", Risk_Other_Left);
            result = result.Replace("{{Risk_Other_Right}}", Risk_Other_Right);
            result = result.Replace("{{Correct_Risk_Other_Left}}", Correct_Risk_Other_Left);
            result = result.Replace("{{Correct_Risk_Other_Right}}", Correct_Risk_Other_Right);
            result = result.Replace("{{Hearing}}", Hearing);
            result = result.Replace("{{Athletic}}", Athletic);
            result = result.Replace("{{Eye_Ground}}", Eye_Ground);
            result = result.Replace("{{Skin}}", Skin);
            result = result.Replace("{{Sclera}}", Sclera);
            result = result.Replace("{{Lymph_Node}}", Lymph_Node);
            result = result.Replace("{{Barrel_Chest}}", Barrel_Chest);
            result = result.Replace("{{Breath_Sounds}}", Breath_Sounds);
            result = result.Replace("{{Rale}}", Rale);
            result = result.Replace("{{Heart_Rhythm}}", Heart_Rhythm);
            result = result.Replace("{{Heart_Murmur}}", Heart_Murmur);
            result = result.Replace("{{Abdominal_Tenderness}}", Abdominal_Tenderness);
            result = result.Replace("{{Abdominal_Mass}}", Abdominal_Mass);
            result = result.Replace("{{Hepatomegaly}}", Hepatomegaly);
            result = result.Replace("{{Splenomegalia}}", Splenomegalia);
            result = result.Replace("{{Shifting_Dullness}}", Shifting_Dullness);
            result = result.Replace("{{Lower_Limb_Edema}}", Lower_Limb_Edema);
            result = result.Replace("{{Pedal_Artery}}", Pedal_Artery);
            result = result.Replace("{{Rectal_Touch}}", Rectal_Touch);
            result = result.Replace("{{Mammary1}}", Mammary1);
            result = result.Replace("{{Mammary2}}", Mammary2);
            result = result.Replace("{{Mammary3}}", Mammary3);
            result = result.Replace("{{Vulvar}}", Vulvar);
            result = result.Replace("{{Vagina}}", Vagina);
            result = result.Replace("{{Cervical}}", Cervical);
            result = result.Replace("{{Uteri_Body}}", Uteri_Body);
            result = result.Replace("{{Uteri_Adnexa}}", Uteri_Adnexa);
            result = result.Replace("{{Examination_Other}}", Examination_Other);
            result = result.Replace("{{Blood_Hgb}}", Blood_Hgb);
            result = result.Replace("{{Blood_Wbc}}", Blood_Wbc);
            result = result.Replace("{{Blood_Rlt}}", Blood_Rlt);
            result = result.Replace("{{Blood_Routine_Other}}", Blood_Routine_Other);
            result = result.Replace("{{PRO}}", PRO);
            result = result.Replace("{{GLU}}", GLU);
            result = result.Replace("{{KET}}", KET);
            result = result.Replace("{{BLD}}", BLD);
            result = result.Replace("{{Urinary_Routine_Other}}", Urinary_Routine_Other);
            result = result.Replace("{{FastingBloodGlucose}}", FastingBloodGlucose);
            return result;
        }

        /// <summary>
        /// 绑定健康体检html模板3
        /// </summary>
        private string BindHealthExam3InfoHtml(string htmlContent, PatientDTO patientInfo, List<ExamRecordDTO> examRecords)
        {
            var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();

            var Cardiogram = "";
            var Microalbuminuria = "";
            var Fecal_Bld = "";
            var Hbalc = "";
            var Hbsag = "";
            var Liver_Alt = "";
            var Liver_Ast = "";
            var Liver_Alb = "";
            var Liver_Tbil = "";
            var Liver_Dbil = "";
            var Renal_Cr = "";
            var Renal_Bun = "";
            var Renal_Spc = "";
            var Renal_Ssc = "";
            var Lipid_Cho = "";
            var Lipid_Tg = "";
            var Lipid_Ldl = "";
            var Lipid_Hdl = "";
            var Chest_X_Ray = "";
            var B_Ultrasonic = "";
            var B_Us_Other = "";
            var Cervical_Smear = "";
            var B_Us_Others = "";
            var TableNumber1 = "";
            var TableNumber2 = "";
            var TableNumber3 = "";
            var TableNumber4 = "";
            var TableNumber5 = "";
            var Health_Renal1 = "";
            var Health_Renal2 = "";
            var Health_Renal3 = "";
            var Health_Renal4 = "";
            var Health_Renal5 = "";


            var Health_Heart1 = "";
            var Health_Heart2 = "";
            var Health_Heart3 = "";
            var Health_Heart4 = "";
            var Health_Heart5 = "";
            var Health_Heart6 = "";
            var BloodVessel1 = "";
            var BloodVessel2 = "";
            var BloodVessel3 = "";
            var EyeVessel1 = "";
            var EyeVessel2 = "";
            var EyeVessel3 = "";
            var Health_Nervous = "";
            var Health_Other = "";
            var HospitalizationInOut1 = "/";
            var HospitalizationReason1 = "";
            var HospitalizationName1 = "";
            var HospitalizationNum1 = "";
            var HospitalizationInOut2 = "/";
            var HospitalizationReason2 = "";
            var HospitalizationName2 = "";
            var HospitalizationNum2 = "";
            var HomeInOut1 = "/";
            var HomeReason1 = "";
            var HomeName1 = "";
            var HomeNum1 = "";
            var HomeInOut2 = "/";
            var HomeReason2 = "";
            var HomeName2 = "";
            var HomeNum2 = "";

            var FZJCExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "FZJC") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(FZJCExamRecordData?.ExamData))
            {
                var FZJCObject = JObject.Parse(FZJCExamRecordData.ExamData);
                if (FZJCObject != null)
                {
                    if (FZJCObject.ContainsKey("Fecal_Bld"))
                    {
                        Fecal_Bld = FZJCObject["Fecal_Bld"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Hbalc"))
                    {
                        Hbalc = FZJCObject["Hbalc"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Hbsag"))
                    {
                        Hbsag = FZJCObject["Hbsag"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Liver_Alt"))
                    {
                        Liver_Alt = FZJCObject["Liver_Alt"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Urinary_Pro"))
                    {
                        var urinary_Pro = FZJCObject["Urinary_Pro"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(urinary_Pro))
                        {
                            var urinary_ProObj = JObject.Parse(urinary_Pro);
                            if (urinary_ProObj != null)
                            {
                                if (urinary_ProObj.ContainsKey("PRO"))
                                {
                                    Microalbuminuria = urinary_ProObj["PRO"]?.ToString() ?? string.Empty;
                                }
                            }
                        }
                    }

                    if (FZJCObject.ContainsKey("Ecg_Img"))
                    {
                        Cardiogram = FZJCObject["Ecg_Img"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Liver_Ast"))
                    {
                        Liver_Ast = FZJCObject["Liver_Ast"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Liver_Alb"))
                    {
                        Liver_Alb = FZJCObject["Liver_Alb"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Liver_Tbil"))
                    {
                        Liver_Tbil = FZJCObject["Liver_Tbil"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Liver_Dbil"))
                    {
                        Liver_Dbil = FZJCObject["Liver_Dbil"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Renal_Cr"))
                    {
                        Renal_Cr = FZJCObject["Renal_Cr"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Renal_Bun"))
                    {
                        Renal_Bun = FZJCObject["Renal_Bun"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Renal_Spc"))
                    {
                        Renal_Spc = FZJCObject["Renal_Spc"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Renal_Ssc"))
                    {
                        Renal_Ssc = FZJCObject["Renal_Ssc"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Lipid_Cho"))
                    {
                        Lipid_Cho = FZJCObject["Lipid_Cho"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Lipid_Tg"))
                    {
                        Lipid_Tg = FZJCObject["Lipid_Tg"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Lipid_Ldl"))
                    {
                        Lipid_Ldl = FZJCObject["Lipid_Ldl"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Lipid_Hdl"))
                    {
                        Lipid_Hdl = FZJCObject["Lipid_Hdl"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Chest_X_Ray"))
                    {
                        Chest_X_Ray = FZJCObject["Chest_X_Ray"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("B_Ultrasonic"))
                    {
                        B_Ultrasonic = FZJCObject["B_Ultrasonic"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("B_Us_Other"))
                    {
                        B_Us_Other = FZJCObject["B_Us_Other"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("Cervical_Smear"))
                    {
                        Cervical_Smear = FZJCObject["Cervical_Smear"]?.ToString() ?? string.Empty;
                    }
                    if (FZJCObject.ContainsKey("B_Us_Others"))
                    {
                        B_Us_Others = FZJCObject["B_Us_Others"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var XCZYWTExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "XCZYWT") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(XCZYWTExamRecordData?.ExamData))
            {
                var XCZYWTObject = JObject.Parse(XCZYWTExamRecordData.ExamData);
                if (XCZYWTObject != null)
                {
                    if (XCZYWTObject.ContainsKey("Health_Cranial"))
                    {
                        var health_Cranial = (JArray)XCZYWTObject["Health_Cranial"];
                        if (health_Cranial?.Count > 0)
                        {
                            var health_CranialList = health_Cranial.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= health_CranialList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    TableNumber1 = health_CranialList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    TableNumber2 = health_CranialList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    TableNumber3 = health_CranialList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    TableNumber4 = health_CranialList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    TableNumber5 = health_CranialList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (XCZYWTObject.ContainsKey("Health_Renal"))
                    {
                        var health_Renal = (JArray)XCZYWTObject["Health_Renal"];
                        if (health_Renal?.Count > 0)
                        {
                            var health_RenalList = health_Renal.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= health_RenalList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    Health_Renal1 = health_RenalList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    Health_Renal2 = health_RenalList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    Health_Renal3 = health_RenalList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    Health_Renal4 = health_RenalList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    Health_Renal5 = health_RenalList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (XCZYWTObject.ContainsKey("Health_Cardiac"))
                    {
                        var health_Cardiac = (JArray)XCZYWTObject["Health_Cardiac"];
                        if (health_Cardiac?.Count > 0)
                        {
                            var health_CardiacList = health_Cardiac.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= health_CardiacList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    Health_Heart1 = health_CardiacList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    Health_Heart2 = health_CardiacList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    Health_Heart3 = health_CardiacList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    Health_Heart4 = health_CardiacList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    Health_Heart5 = health_CardiacList[(i - 1)].ToString();
                                }
                                else if (i == 6)
                                {
                                    Health_Heart6 = health_CardiacList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (XCZYWTObject.ContainsKey("Health_Vascular"))
                    {
                        var health_Vascular = (JArray)XCZYWTObject["Health_Vascular"];
                        if (health_Vascular?.Count > 0)
                        {
                            var health_VascularList = health_Vascular.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= health_VascularList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    BloodVessel1 = health_VascularList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    BloodVessel2 = health_VascularList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    BloodVessel3 = health_VascularList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (XCZYWTObject.ContainsKey("Health_Eye"))
                    {
                        var health_Eye = (JArray)XCZYWTObject["Health_Eye"];
                        if (health_Eye?.Count > 0)
                        {
                            var health_EyeList = health_Eye.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= health_EyeList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    EyeVessel1 = health_EyeList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    EyeVessel2 = health_EyeList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    EyeVessel3 = health_EyeList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (XCZYWTObject.ContainsKey("Health_Nervous"))
                    {
                        Health_Nervous = XCZYWTObject["Health_Nervous"]?.ToString() ?? string.Empty;
                    }
                    if (XCZYWTObject.ContainsKey("Health_Other"))
                    {
                        Health_Other = XCZYWTObject["Health_Other"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var ZYYYFMYGHYFJZSExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZYYYFMYGHYFJZS") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(ZYYYFMYGHYFJZSExamRecordData?.ExamData))
            {
                var ZYYYFMYGHYFJZSObject = JObject.Parse(ZYYYFMYGHYFJZSExamRecordData.ExamData);
                if (ZYYYFMYGHYFJZSObject != null)
                {
                    if (ZYYYFMYGHYFJZSObject.ContainsKey("Hospitalization_History"))
                    {
                        var hospitalization_History = (JArray)ZYYYFMYGHYFJZSObject["Hospitalization_History"];
                        if (hospitalization_History?.Count > 0)
                        {
                            for (var i = 1; i <= hospitalization_History.Count; i++)
                            {
                                var item = hospitalization_History[(i - 1)];
                                if (string.IsNullOrEmpty(item["id"]?.ToString()))
                                {
                                    continue;
                                }
                                if (i == 1)
                                {
                                    HospitalizationInOut1 = (item["Admission_Date"]?.ToString() ?? string.Empty) + "/" + (item["Discharge_Date"]?.ToString() ?? string.Empty);
                                    HospitalizationReason1 = item["Reason"]?.ToString() ?? string.Empty;
                                    HospitalizationName1 = item["Name_Of_Medical_Institution"]?.ToString() ?? string.Empty;
                                    HospitalizationNum1 = item["Patient_Number"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 2)
                                {
                                    HospitalizationInOut2 = (item["Admission_Date"]?.ToString() ?? string.Empty) + "/" + (item["Discharge_Date"]?.ToString() ?? string.Empty);
                                    HospitalizationReason2 = item["Reason"]?.ToString() ?? string.Empty;
                                    HospitalizationName2 = item["Name_Of_Medical_Institution"]?.ToString() ?? string.Empty;
                                    HospitalizationNum2 = item["Patient_Number"]?.ToString() ?? string.Empty;
                                }
                            }
                        }
                    }

                    if (ZYYYFMYGHYFJZSObject.ContainsKey("Hospital_At_Home"))
                    {
                        var hospital_At_Home = (JArray)ZYYYFMYGHYFJZSObject["Hospital_At_Home"];
                        if (hospital_At_Home?.Count > 0)
                        {
                            for (var i = 1; i <= hospital_At_Home.Count; i++)
                            {
                                var item = hospital_At_Home[(i - 1)];
                                if (string.IsNullOrEmpty(item["id"]?.ToString()))
                                {
                                    continue;
                                }
                                if (i == 1)
                                {
                                    HomeInOut1 = (item["Bed_Construction_Date"]?.ToString() ?? string.Empty) + "/" + (item["Discharge_Date"]?.ToString() ?? string.Empty);
                                    HomeReason1 = item["Reason"]?.ToString() ?? string.Empty;
                                    HomeName1 = item["Name_Of_Medical_Institution"]?.ToString() ?? string.Empty;
                                    HomeNum1 = item["Patient_Number"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 2)
                                {
                                    HomeInOut2 = (item["Bed_Construction_Date"]?.ToString() ?? string.Empty) + "/" + (item["Discharge_Date"]?.ToString() ?? string.Empty);
                                    HomeReason2 = item["Reason"]?.ToString() ?? string.Empty;
                                    HomeName2 = item["Name_Of_Medical_Institution"]?.ToString() ?? string.Empty;
                                    HomeNum2 = item["Patient_Number"]?.ToString() ?? string.Empty;
                                }
                            }
                        }
                    }
                }
            }

            var result = htmlContent;
            result = result.Replace("{{Microalbuminuria}}", Microalbuminuria);
            result = result.Replace("{{Fecal_Bld}}", Fecal_Bld);
            result = result.Replace("{{Hbalc}}", Hbalc);
            result = result.Replace("{{Hbsag}}", Hbsag);
            result = result.Replace("{{Liver_Alt}}", Liver_Alt);
            result = result.Replace("{{Liver_Ast}}", Liver_Ast);
            result = result.Replace("{{Liver_Alb}}", Liver_Alb);
            result = result.Replace("{{Liver_Tbil}}", Liver_Tbil);
            result = result.Replace("{{Liver_Dbil}}", Liver_Dbil);
            result = result.Replace("{{Renal_Cr}}", Renal_Cr);
            result = result.Replace("{{Renal_Bun}}", Renal_Bun);
            result = result.Replace("{{Renal_Spc}}", Renal_Spc);
            result = result.Replace("{{Renal_Ssc}}", Renal_Ssc);
            result = result.Replace("{{Lipid_Cho}}", Lipid_Cho);
            result = result.Replace("{{Lipid_Tg}}", Lipid_Tg);
            result = result.Replace("{{Lipid_Ldl}}", Lipid_Ldl);
            result = result.Replace("{{Lipid_Hdl}}", Lipid_Hdl);
            result = result.Replace("{{Chest_X_Ray}}", Chest_X_Ray);
            result = result.Replace("{{B_Ultrasonic}}", B_Ultrasonic);
            result = result.Replace("{{B_Us_Other}}", B_Us_Other);
            result = result.Replace("{{Cervical_Smear}}", Cervical_Smear);
            result = result.Replace("{{B_Us_Others}}", B_Us_Others);
            result = result.Replace("{{TableNumber1}}", TableNumber1);
            result = result.Replace("{{TableNumber2}}", TableNumber2);
            result = result.Replace("{{TableNumber3}}", TableNumber3);
            result = result.Replace("{{TableNumber4}}", TableNumber4);
            result = result.Replace("{{TableNumber5}}", TableNumber5);
            result = result.Replace("{{Health_Renal1}}", Health_Renal1);
            result = result.Replace("{{Health_Renal2}}", Health_Renal2);
            result = result.Replace("{{Health_Renal3}}", Health_Renal3);
            result = result.Replace("{{Health_Renal4}}", Health_Renal4);
            result = result.Replace("{{Health_Renal5}}", Health_Renal5);
            result = result.Replace("{{Health_Heart1}}", Health_Heart1);
            result = result.Replace("{{Health_Heart2}}", Health_Heart2);
            result = result.Replace("{{Health_Heart3}}", Health_Heart3);
            result = result.Replace("{{Health_Heart4}}", Health_Heart4);
            result = result.Replace("{{Health_Heart5}}", Health_Heart5);
            result = result.Replace("{{Health_Heart6}}", Health_Heart6);
            result = result.Replace("{{BloodVessel1}}", BloodVessel1);
            result = result.Replace("{{BloodVessel2}}", BloodVessel2);
            result = result.Replace("{{BloodVessel3}}", BloodVessel3);
            result = result.Replace("{{EyeVessel1}}", EyeVessel1);
            result = result.Replace("{{EyeVessel2}}", EyeVessel2);
            result = result.Replace("{{EyeVessel3}}", EyeVessel3);
            result = result.Replace("{{Health_Nervous}}", Health_Nervous);
            result = result.Replace("{{Health_Other}}", Health_Other);
            result = result.Replace("{{HospitalizationInOut1}}", HospitalizationInOut1);
            result = result.Replace("{{HospitalizationReason1}}", HospitalizationReason1);
            result = result.Replace("{{HospitalizationName1}}", HospitalizationName1);
            result = result.Replace("{{HospitalizationNum1}}", HospitalizationNum1);
            result = result.Replace("{{HospitalizationInOut2}}", HospitalizationInOut2);
            result = result.Replace("{{HospitalizationReason2}}", HospitalizationReason2);
            result = result.Replace("{{HospitalizationName2}}", HospitalizationName2);
            result = result.Replace("{{HospitalizationNum2}}", HospitalizationNum2);
            result = result.Replace("{{HomeInOut1}}", HomeInOut1);
            result = result.Replace("{{HomeReason1}}", HomeReason1);
            result = result.Replace("{{HomeName1}}", HomeName1);
            result = result.Replace("{{HomeNum1}}", HomeNum1);
            result = result.Replace("{{HomeInOut2}}", HomeInOut2);
            result = result.Replace("{{HomeReason2}}", HomeReason2);
            result = result.Replace("{{HomeName2}}", HomeName2);
            result = result.Replace("{{HomeNum2}}", HomeNum2);
            result = result.Replace("{{Cardiogram}}", Cardiogram);
            return result;
        }

        /// <summary>
        /// 绑定健康体检html模板4
        /// </summary>
        private string BindHealthExam4InfoHtml(string htmlContent, PatientDTO patientInfo, List<ExamRecordDTO> examRecords)
        {
            var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            curExamRecord.ExamRecordDatas = curExamRecord.ExamRecordDatas ?? new List<ExamRecordDataDTO>();

            var MedicineName1 = "-";
            var MedicineUsage1 = "";
            var MedicineDosage1 = "";
            var MedicineUseTime1 = "";
            var MedicationCompliance1 = "";
            var MedicineName2 = "-";
            var MedicineUsage2 = "";
            var MedicineDosage2 = "";
            var MedicineUseTime2 = "";
            var MedicationCompliance2 = "";
            var MedicineName3 = "-";
            var MedicineUsage3 = "";
            var MedicineDosage3 = "";
            var MedicineUseTime3 = "";
            var MedicationCompliance3 = "";
            var MedicineName4 = "-";
            var MedicineUsage4 = "";
            var MedicineDosage4 = "";
            var MedicineUseTime4 = "";
            var MedicationCompliance4 = "";
            var MedicineName5 = "-";
            var MedicineUsage5 = "";
            var MedicineDosage5 = "";
            var MedicineUseTime5 = "";
            var MedicationCompliance5 = "";
            var MedicineName6 = "-";
            var MedicineUsage6 = "";
            var MedicineDosage6 = "";
            var MedicineUseTime6 = "";
            var MedicationCompliance6 = "";
            var InoculateName1 = "-";
            var InoculateTime1 = "";
            var InoculateOrgName1 = "";
            var InoculateName2 = "-";
            var InoculateTime2 = "";
            var InoculateOrgName2 = "";
            var InoculateName3 = "-";
            var InoculateTime3 = "";
            var InoculateOrgName3 = "";
            var HealthEvaluationResult = "";
            var HealthEvaluationAbnormal = "";
            var ChronicDisease = "";
            var ReExamination = "";
            var Referral = "";
            var RiskFactorControl1 = "";
            var RiskFactorControl2 = "";
            var RiskFactorControl3 = "";
            var RiskFactorControl4 = "";
            var RiskFactorControl5 = "";
            var RiskFactorControl6 = "";
            var WeightLoss = "";
            var Vaccination = "";
            var OtherRiskFactor = "";

            var ZYYYFMYGHYFJZSExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZYYYFMYGHYFJZS") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(ZYYYFMYGHYFJZSExamRecordData?.ExamData))
            {
                var ZYYYFMYGHYFJZSObject = JObject.Parse(ZYYYFMYGHYFJZSExamRecordData.ExamData);
                if (ZYYYFMYGHYFJZSObject != null)
                {
                    if (ZYYYFMYGHYFJZSObject.ContainsKey("Inoculate_History"))
                    {
                        var inoculate_History = (JArray)ZYYYFMYGHYFJZSObject["Inoculate_History"];
                        if (inoculate_History?.Count > 0)
                        {
                            for (var i = 1; i <= inoculate_History.Count; i++)
                            {
                                var item = inoculate_History[(i - 1)];
                                if (string.IsNullOrEmpty(item["id"]?.ToString()))
                                {
                                    continue;
                                }
                                if (i == 1)
                                {
                                    InoculateName1 = item["Vaccine_Name"]?.ToString() ?? string.Empty;
                                    InoculateTime1 = item["Vaccination_Date"]?.ToString() ?? string.Empty;
                                    InoculateOrgName1 = item["Vaccination_Facility"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 2)
                                {
                                    InoculateName2 = item["Vaccine_Name"]?.ToString() ?? string.Empty;
                                    InoculateTime2 = item["Vaccination_Date"]?.ToString() ?? string.Empty;
                                    InoculateOrgName2 = item["Vaccination_Facility"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 3)
                                {
                                    InoculateName3 = item["Vaccine_Name"]?.ToString() ?? string.Empty;
                                    InoculateTime3 = item["Vaccination_Date"]?.ToString() ?? string.Empty;
                                    InoculateOrgName3 = item["Vaccination_Facility"]?.ToString() ?? string.Empty;
                                }
                            }
                        }
                    }

                    if (ZYYYFMYGHYFJZSObject.ContainsKey("Drug_Usage"))
                    {
                        var drug_Usage = (JArray)ZYYYFMYGHYFJZSObject["Drug_Usage"];
                        if (drug_Usage?.Count > 0)
                        {
                            for (var i = 1; i <= drug_Usage.Count; i++)
                            {
                                var item = drug_Usage[(i - 1)];
                                if (string.IsNullOrEmpty(item["id"]?.ToString()))
                                {
                                    continue;
                                }
                                if (i == 1)
                                {
                                    MedicineName1 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage1 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage1 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime1 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance1 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 2)
                                {
                                    MedicineName2 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage2 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage2 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime2 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance2 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 3)
                                {
                                    MedicineName3 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage3 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage3 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime3 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance3 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 4)
                                {
                                    MedicineName4 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage4 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage4 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime4 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance4 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 5)
                                {
                                    MedicineName5 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage5 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage5 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime5 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance5 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                                else if (i == 6)
                                {
                                    MedicineName6 = item["Medicine_Name"]?.ToString() ?? string.Empty;
                                    MedicineUsage6 = item["Medicine_Usage"]?.ToString() ?? string.Empty;
                                    MedicineDosage6 = item["Medicine_Dosage"]?.ToString() ?? string.Empty;
                                    MedicineUseTime6 = item["Medicine_Time"]?.ToString() ?? string.Empty;
                                    MedicationCompliance6 = item["Medicine_Compliance"]?.ToString() ?? string.Empty;
                                }
                            }
                        }
                    }

                }
            }

            var JKPJJZDExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "JKPJJZD") ?? new ExamRecordDataDTO();
            if (!string.IsNullOrEmpty(JKPJJZDExamRecordData?.ExamData))
            {
                var JKPJJZDObject = JObject.Parse(JKPJJZDExamRecordData.ExamData);
                if (JKPJJZDObject != null)
                {
                    if (JKPJJZDObject.ContainsKey("Appraisal_Flag"))
                    {
                        HealthEvaluationResult = JKPJJZDObject["Appraisal_Flag"]?.ToString() ?? string.Empty;
                    }
                    if (JKPJJZDObject.ContainsKey("Appraisal"))
                    {
                        // HealthEvaluationAbnormal = JKPJJZDObject["Appraisal"]?.ToString() ?? string.Empty;
                        var appraisal = (JArray)JKPJJZDObject["Appraisal"];
                        if (appraisal?.Count > 0)
                        {
                            for (var i = 1; i <= appraisal.Count; i++)
                            {
                                var item = appraisal[(i - 1)]?.ToString() ?? string.Empty;
                                HealthEvaluationAbnormal += $"<p><span>异常" + i + $"</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span style='border-bottom: 1px solid #000;min-height: 30px;'>{item}</span></p> <br />";
                            }
                        }
                    }

                    if (JKPJJZDObject.ContainsKey("Risk_Control"))
                    {
                        var risk_ControlStr = JKPJJZDObject["Risk_Control"]?.ToString() ?? string.Empty;
                        var risk_ControlObject = JObject.Parse(risk_ControlStr);
                        if (risk_ControlObject != null)
                        {
                            var risk_Control = (JArray)risk_ControlObject["values"];
                            if (risk_Control?.Count > 0)
                            {
                                var risk_ControlList = risk_Control.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                                for (var i = 1; i <= risk_ControlList.Count; i++)
                                {
                                    var item = risk_ControlList[(i - 1)]?.ToString() ?? string.Empty;
                                    if (i == 1)
                                    {
                                        RiskFactorControl1 = item;
                                    }
                                    else if (i == 2)
                                    {
                                        RiskFactorControl2 = item;
                                    }
                                    else if (i == 3)
                                    {
                                        RiskFactorControl3 = item;
                                    }
                                    else if (i == 4)
                                    {
                                        RiskFactorControl4 = item;
                                    }
                                    else if (i == 5)
                                    {
                                        RiskFactorControl5 = item;
                                    }
                                    else if (i == 6)
                                    {
                                        RiskFactorControl6 = item;
                                    }
                                }
                            }

                            if (risk_ControlObject.ContainsKey("weightLoss"))
                            {
                                WeightLoss = risk_ControlObject["weightLoss"]?.ToString() ?? string.Empty;
                            }
                            if (risk_ControlObject.ContainsKey("vaccinations"))
                            {
                                Vaccination = risk_ControlObject["vaccinations"]?.ToString() ?? string.Empty;
                            }
                            if (risk_ControlObject.ContainsKey("other"))
                            {
                                OtherRiskFactor = risk_ControlObject["other"]?.ToString() ?? string.Empty;
                            }
                        }
                    }
                    if (JKPJJZDObject.ContainsKey("Guidance"))
                    {
                        var guidance = (JArray)JKPJJZDObject["Guidance"];
                        if (guidance?.Count > 0)
                        {
                            var guidanceList = guidance.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= guidanceList.Count; i++)
                            {
                                var item = guidanceList[(i - 1)]?.ToString() ?? string.Empty;
                                if (i == 1)
                                {
                                    ChronicDisease = item;
                                }
                                else if (i == 2)
                                {
                                    ReExamination = item;
                                }
                                else if (i == 3)
                                {
                                    Referral = item;
                                }
                            }
                        }
                    }
                }
            }

            var result = htmlContent;
            result = result.Replace("{{MedicineName1}}", MedicineName1);
            result = result.Replace("{{MedicineUsage1}}", MedicineUsage1);
            result = result.Replace("{{MedicineDosage1}}", MedicineDosage1);
            result = result.Replace("{{MedicineUseTime1}}", MedicineUseTime1);
            result = result.Replace("{{MedicationCompliance1}}", MedicationCompliance1);
            result = result.Replace("{{MedicineName2}}", MedicineName2);
            result = result.Replace("{{MedicineUsage2}}", MedicineUsage2);
            result = result.Replace("{{MedicineDosage2}}", MedicineDosage2);
            result = result.Replace("{{MedicineUseTime2}}", MedicineUseTime2);
            result = result.Replace("{{MedicationCompliance2}}", MedicationCompliance2);
            result = result.Replace("{{MedicineName3}}", MedicineName3);
            result = result.Replace("{{MedicineUsage3}}", MedicineUsage3);
            result = result.Replace("{{MedicineDosage3}}", MedicineDosage3);
            result = result.Replace("{{MedicineUseTime3}}", MedicineUseTime3);
            result = result.Replace("{{MedicationCompliance3}}", MedicationCompliance3);
            result = result.Replace("{{MedicineName4}}", MedicineName4);
            result = result.Replace("{{MedicineUsage4}}", MedicineUsage4);
            result = result.Replace("{{MedicineDosage4}}", MedicineDosage4);
            result = result.Replace("{{MedicineUseTime4}}", MedicineUseTime4);
            result = result.Replace("{{MedicationCompliance4}}", MedicationCompliance4);
            result = result.Replace("{{MedicineName5}}", MedicineName5);
            result = result.Replace("{{MedicineUsage5}}", MedicineUsage5);
            result = result.Replace("{{MedicineDosage5}}", MedicineDosage5);
            result = result.Replace("{{MedicineUseTime5}}", MedicineUseTime5);
            result = result.Replace("{{MedicationCompliance5}}", MedicationCompliance5);
            result = result.Replace("{{MedicineName6}}", MedicineName6);
            result = result.Replace("{{MedicineUsage6}}", MedicineUsage6);
            result = result.Replace("{{MedicineDosage6}}", MedicineDosage6);
            result = result.Replace("{{MedicineUseTime6}}", MedicineUseTime6);
            result = result.Replace("{{MedicationCompliance6}}", MedicationCompliance6);
            result = result.Replace("{{InoculateName1}}", InoculateName1);
            result = result.Replace("{{InoculateTime1}}", InoculateTime1);
            result = result.Replace("{{InoculateOrgName1}}", InoculateOrgName1);
            result = result.Replace("{{InoculateName2}}", InoculateName2);
            result = result.Replace("{{InoculateTime2}}", InoculateTime2);
            result = result.Replace("{{InoculateOrgName2}}", InoculateOrgName2);
            result = result.Replace("{{InoculateName3}}", InoculateName3);
            result = result.Replace("{{InoculateTime3}}", InoculateTime3);
            result = result.Replace("{{InoculateOrgName3}}", InoculateOrgName3);
            result = result.Replace("{{HealthEvaluationResult}}", HealthEvaluationResult);
            result = result.Replace("{{HealthEvaluationAbnormal}}", HealthEvaluationAbnormal);
            result = result.Replace("{{ChronicDisease}}", ChronicDisease);
            result = result.Replace("{{ReExamination}}", ReExamination);
            result = result.Replace("{{Referral}}", Referral);
            result = result.Replace("{{RiskFactorControl1}}", RiskFactorControl1);
            result = result.Replace("{{RiskFactorControl2}}", RiskFactorControl2);
            result = result.Replace("{{RiskFactorControl3}}", RiskFactorControl3);
            result = result.Replace("{{RiskFactorControl4}}", RiskFactorControl4);
            result = result.Replace("{{RiskFactorControl5}}", RiskFactorControl5);
            result = result.Replace("{{RiskFactorControl6}}", RiskFactorControl6);
            result = result.Replace("{{WeightLoss}}", WeightLoss);
            result = result.Replace("{{Vaccination}}", Vaccination);
            result = result.Replace("{{OtherRiskFactor}}", OtherRiskFactor);
            return result;
        }

        /// <summary>
        /// utc转bj时间
        /// </summary>
        private DateTime UTCToBeijing(DateTime utcTime)
        {
            // 获取中国标准时间 (CST) 的时区信息
            TimeZoneInfo cstZone = TimeZoneInfo.FindSystemTimeZoneById("China Standard Time");
            // 将UTC时间转换为北京时间
            DateTime beijingTime = TimeZoneInfo.ConvertTimeFromUtc(utcTime, cstZone);
            return beijingTime;
        }

        /// <summary>
        /// 绑定个人信息html模板
        /// </summary>
        private string BindPersonInfoHtml(string htmlContent, PatientDTO patientInfo, PatientExtensionDTO patientExtensionInfo)
        {
            var PatientName = patientInfo.PatientName;
            var PhysicalExamNumber = patientInfo.RecordNo;
            var PhysicalExamNumberHtml = "";
            var PhysicalExamNumberTitle = "";
            if (!string.IsNullOrEmpty(PhysicalExamNumber))
            {
                char[] chars = PhysicalExamNumber.ToCharArray();
                foreach (char c in chars)
                {
                    PhysicalExamNumberHtml += "<div class='box'><span>" + c + "</span></div>";
                }
                PhysicalExamNumberTitle = "<span style='font-weight: bold;'>编号：</span>";
            }
            var SexType = "";
            if (patientInfo.PatientGender == GenderEnum.Unspecified)
            {
                SexType = "9";
            }
            else if (patientInfo.PatientGender == GenderEnum.Unknown)
            {
                SexType = "0";
            }
            else if (patientInfo.PatientGender == GenderEnum.Female)
            {
                SexType = "2";
            }
            else if (patientInfo.PatientGender == GenderEnum.Male)
            {
                SexType = "1";
            }
            var WorkUnit = "";
            var ContactName = patientInfo.EmergencyName;
            var ContactPhone = patientInfo.EmergencyPhone;
            var BloodType = "";
            var PermanentlyResideType = "";
            var RH = "";
            var Career = "";
            var ProviderPaymentsOther = "";
            var PaymentsMethod1 = "";
            var PaymentsMethod2 = "";
            var PaymentsMethod3 = "";
            var EducationLevel = "";
            var MaritalStatus = "";
            var HistoryDrugAllergiesOther = "";
            var DrugAllergiesMethod1 = "";
            var DrugAllergiesMethod2 = "";
            var DrugAllergiesMethod3 = "";
            var DrugAllergiesMethod4 = "";
            var ExposeMethod1 = "";
            var ExposeMethod2 = "";
            var ExposeMethod3 = "";
            var HistoryPreviousDiseaseMalignancy = "";
            var HistoryPreviousDiseaseOccupationalDisease = "";
            var HistoryPreviousDiseaseOther = "";
            var DiseaseDuration1 = "";
            var DiseaseDurationTime1 = "";
            var DiseaseDuration2 = "";
            var DiseaseDurationTime2 = "";
            var DiseaseDuration3 = "";
            var DiseaseDurationTime3 = "";
            var DiseaseDuration4 = "";
            var DiseaseDurationTime4 = "";
            var DiseaseDuration5 = "";
            var DiseaseDurationTime5 = "";
            var DiseaseDuration6 = "";
            var DiseaseDurationTime6 = "";
            var OperationOneName = "";
            var OperationOneTime = "";
            var OperationTwoName = "";
            var OperationTwoTime = "";
            var ExposeMethodOperation = "";
            var TraumaOneName = "";
            var TraumaOneTime = "";
            var TraumaTwoName = "";
            var TraumaTwoTime = "";
            var ExposeMethodOtherProvinces = "";
            var TransfusionOneName = "";
            var TransfusionOneTime = "";
            var TransfusionTwoName = "";
            var TransfusionTwoTime = "";
            var ExposeMethodBloodTransfusion = "";
            var FatherSicken1 = "";
            var FatherSicken2 = "";
            var FatherSicken3 = "";
            var FatherSicken4 = "";
            var FatherSicken5 = "";
            var MotherSicken1 = "";
            var MotherSicken2 = "";
            var MotherSicken3 = "";
            var MotherSicken4 = "";
            var MotherSicken5 = "";
            var BrotherSicken1 = "";
            var BrotherSicken2 = "";
            var BrotherSicken3 = "";
            var BrotherSicken4 = "";
            var BrotherSicken5 = "";
            var ChildSicken1 = "";
            var ChildSicken2 = "";
            var ChildSicken3 = "";
            var ChildSicken4 = "";
            var ChildSicken5 = "";
            var HistoryGeneticName = "";
            var HistoryGeneticOther = "";
            var DisabilitySituationOther = "";
            var DisabilitySituationOther1 = "";
            var DisabilitySituationOther2 = "";
            var DisabilitySituationOther3 = "";
            var DisabilitySituationOther4 = "";
            var DisabilitySituationOther5 = "";
            var KitchenExhaustFacilities = "";
            var FuelOther = "";
            var Fuel = "";
            var DrinkingWaterOther = "";
            var DrinkingWater = "";
            var Toilet = "";
            var AnimalPen = "";


            var NationalityBox = "";
            var Nationality = "";
            var Birthday = "";
            if (patientInfo.Birthday > DateTime.MinValue)
            {
                Birthday = UTCToBeijing(patientInfo.Birthday).ToString("yyy-MM-dd");
            }
            var IDCardNo = patientInfo.CardNo ?? string.Empty;
            var Phone = patientInfo.Phone ?? string.Empty;
            if (patientInfo.Nationality == "汉族" || patientInfo.Nationality == "汉")
            {
                NationalityBox = "01";
            }
            else if (string.IsNullOrEmpty(patientInfo.Nationality))
            {
                NationalityBox = "";
            }
            else
            {
                NationalityBox = "99";
                Nationality = patientInfo.Nationality;
            }
            var HistorySicken = "";

            if (!string.IsNullOrEmpty(patientExtensionInfo.ExtensionData))
            {
                var jObject = JObject.Parse(patientExtensionInfo.ExtensionData);
                if (jObject != null)
                {
                    if (jObject.ContainsKey("workUnit"))
                    {
                        WorkUnit = jObject["workUnit"]?.ToString() ?? string.Empty;
                    }
                    // if (jObject.ContainsKey("contactName"))
                    // {
                    //     ContactName = jObject["contactName"]?.ToString() ?? string.Empty;
                    // }
                    // if (jObject.ContainsKey("contactPhone"))
                    // {
                    //     ContactPhone = jObject["contactPhone"]?.ToString() ?? string.Empty;
                    // }
                    if (jObject.ContainsKey("permanentlyResideType"))
                    {
                        PermanentlyResideType = jObject["permanentlyResideType"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("bloodType"))
                    {
                        BloodType = jObject["bloodType"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("rh"))
                    {
                        RH = jObject["rh"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("career"))
                    {
                        Career = jObject["career"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("providerPaymentsOther"))
                    {
                        ProviderPaymentsOther = jObject["providerPaymentsOther"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("providerPayments"))
                    {
                        PaymentsMethod1 = jObject["providerPayments"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("educationLevel"))
                    {
                        EducationLevel = jObject["educationLevel"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("maritalStatus"))
                    {
                        MaritalStatus = jObject["maritalStatus"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyDrugAllergiesOther"))
                    {
                        HistoryDrugAllergiesOther = jObject["historyDrugAllergiesOther"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyDrugAllergies"))
                    {
                        var historyDrugAllergies = (JArray)jObject["historyDrugAllergies"];
                        if (historyDrugAllergies?.Count > 0)
                        {
                            var historyDrugAllergiesList = historyDrugAllergies.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= historyDrugAllergiesList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    DrugAllergiesMethod1 = historyDrugAllergiesList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    DrugAllergiesMethod2 = historyDrugAllergiesList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    DrugAllergiesMethod3 = historyDrugAllergiesList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    DrugAllergiesMethod4 = historyDrugAllergiesList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("historyExposure"))
                    {
                        var historyExposure = (JArray)jObject["historyExposure"];
                        if (historyExposure?.Count > 0)
                        {
                            var historyExposureList = historyExposure.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= historyExposureList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    ExposeMethod1 = historyExposureList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    ExposeMethod2 = historyExposureList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    ExposeMethod3 = historyExposureList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("historyPreviousDiseaseMalignancy"))
                    {
                        HistoryPreviousDiseaseMalignancy = jObject["historyPreviousDiseaseMalignancy"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyPreviousDiseaseOccupationalDisease"))
                    {
                        HistoryPreviousDiseaseOccupationalDisease = jObject["historyPreviousDiseaseOccupationalDisease"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyPreviousDiseaseOther"))
                    {
                        HistoryPreviousDiseaseOther = jObject["historyPreviousDiseaseOther"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyPreviousDiseaseOther"))
                    {
                        HistoryPreviousDiseaseOther = jObject["historyPreviousDiseaseOther"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyPreviousDisease"))
                    {
                        var historyPreviousDisease = (JArray)jObject["historyPreviousDisease"];
                        var diseaseDiagnosisTime = JObject.Parse(jObject["diseaseDiagnosisTime"].ToString());
                        if (historyPreviousDisease?.Count > 0)
                        {
                            for (var i = 1; i <= historyPreviousDisease.Count; i++)
                            {
                                var diseaseStr = diseaseDiagnosisTime[historyPreviousDisease[(i - 1)].ToString()]?.ToString() ?? string.Empty;
                                if (string.IsNullOrEmpty(diseaseStr))
                                {
                                    continue;
                                }
                                var curTimeTemp = Convert.ToDateTime(diseaseStr);
                                var year = curTimeTemp.Year.ToString();
                                var month = curTimeTemp.Month.ToString();
                                if (month.Length == 1)
                                {
                                    month = "0" + month;
                                }
                                var spanStr = "&nbsp;&nbsp;";
                                if (i == 1)
                                {
                                    DiseaseDuration1 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration1.Length == 1)
                                    {
                                        DiseaseDuration1 = spanStr + DiseaseDuration1;
                                    }
                                    DiseaseDurationTime1 = year + "年" + month + "月";
                                }
                                else if (i == 2)
                                {
                                    DiseaseDuration2 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration2.Length == 1)
                                    {
                                        DiseaseDuration2 = spanStr + DiseaseDuration2;
                                    }
                                    DiseaseDurationTime2 = year + "年" + month + "月";
                                }
                                else if (i == 3)
                                {
                                    DiseaseDuration3 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration3.Length == 1)
                                    {
                                        DiseaseDuration3 = spanStr + DiseaseDuration3;
                                    }
                                    DiseaseDurationTime3 = year + "年" + month + "月";
                                }
                                else if (i == 4)
                                {
                                    DiseaseDuration4 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration4.Length == 1)
                                    {
                                        DiseaseDuration4 = spanStr + DiseaseDuration4;
                                    }
                                    DiseaseDurationTime4 = year + "年" + month + "月";
                                }
                                else if (i == 5)
                                {
                                    DiseaseDuration5 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration5.Length == 1)
                                    {
                                        DiseaseDuration5 = spanStr + DiseaseDuration5;
                                    }
                                    DiseaseDurationTime5 = year + "年" + month + "月";
                                }
                                else if (i == 6)
                                {
                                    DiseaseDuration6 = historyPreviousDisease[(i - 1)].ToString();
                                    if (DiseaseDuration6.Length == 1)
                                    {
                                        DiseaseDuration6 = spanStr + DiseaseDuration6;
                                    }
                                    DiseaseDurationTime6 = year + "年" + month + "月";
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("operation"))
                    {
                        ExposeMethodOperation = jObject["operation"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("operationOne"))
                    {
                        var operationOne = jObject["operationOne"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(operationOne))
                        {
                            var operationOneObject = JObject.Parse(operationOne);
                            if (operationOneObject != null)
                            {
                                if (operationOneObject.ContainsKey("name"))
                                {
                                    OperationOneName = operationOneObject["name"].ToString();
                                }

                                if (operationOneObject.ContainsKey("time"))
                                {
                                    OperationOneTime = Convert.ToDateTime(operationOneObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("operationTwo"))
                    {
                        var operationTwo = jObject["operationTwo"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(operationTwo))
                        {
                            var operationTwoObject = JObject.Parse(operationTwo);
                            if (operationTwoObject != null)
                            {
                                if (operationTwoObject.ContainsKey("name"))
                                {
                                    OperationTwoName = operationTwoObject["name"].ToString();
                                }
                                if (operationTwoObject.ContainsKey("time"))
                                {
                                    OperationTwoTime = Convert.ToDateTime(operationTwoObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }

                    if (jObject.ContainsKey("trauma"))
                    {
                        ExposeMethodOtherProvinces = jObject["trauma"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("traumaOne"))
                    {
                        var traumaOne = jObject["traumaOne"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(traumaOne))
                        {
                            var traumaOneObject = JObject.Parse(traumaOne);
                            if (traumaOneObject != null)
                            {
                                if (traumaOneObject.ContainsKey("name"))
                                {
                                    TraumaOneName = traumaOneObject["name"].ToString();
                                }
                                if (traumaOneObject.ContainsKey("time"))
                                {
                                    TraumaOneTime = Convert.ToDateTime(traumaOneObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("traumaTwo"))
                    {
                        var traumaTwo = jObject["traumaTwo"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(traumaTwo))
                        {
                            var traumaTwoObject = JObject.Parse(traumaTwo);
                            if (traumaTwoObject != null)
                            {
                                if (traumaTwoObject.ContainsKey("name"))
                                {
                                    TraumaTwoName = traumaTwoObject["name"].ToString();
                                }
                                if (traumaTwoObject.ContainsKey("time"))
                                {
                                    TraumaTwoTime = Convert.ToDateTime(traumaTwoObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("transfusion"))
                    {
                        ExposeMethodBloodTransfusion = jObject["transfusion"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("transfusionOne"))
                    {
                        var transfusionOne = jObject["transfusionOne"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(transfusionOne))
                        {
                            var transfusionOneObject = JObject.Parse(transfusionOne);
                            if (transfusionOneObject != null)
                            {
                                if (transfusionOneObject.ContainsKey("name"))
                                {
                                    TransfusionOneName = transfusionOneObject["name"].ToString();
                                }
                                if (transfusionOneObject.ContainsKey("time"))
                                {
                                    TransfusionOneTime = Convert.ToDateTime(transfusionOneObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("transfusionTwo"))
                    {
                        var transfusionTwo = jObject["transfusionTwo"]?.ToString() ?? string.Empty;
                        if (!string.IsNullOrEmpty(transfusionTwo))
                        {
                            var transfusionTwoObject = JObject.Parse(transfusionTwo);
                            if (transfusionTwoObject != null)
                            {
                                if (transfusionTwoObject.ContainsKey("name"))
                                {
                                    TransfusionTwoName = transfusionTwoObject["name"].ToString();
                                }
                                if (transfusionTwoObject.ContainsKey("time"))
                                {
                                    TransfusionTwoTime = Convert.ToDateTime(transfusionTwoObject["time"].ToString()).ToString("yyyy-MM-dd");
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("father"))
                    {
                        var father = (JArray)jObject["father"];
                        if (father?.Count > 0)
                        {
                            var fatherList = father.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= fatherList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    FatherSicken1 = fatherList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    FatherSicken2 = fatherList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    FatherSicken3 = fatherList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    FatherSicken4 = fatherList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    FatherSicken5 = fatherList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("mother"))
                    {
                        var mother = (JArray)jObject["mother"];
                        if (mother?.Count > 0)
                        {
                            var motherList = mother.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= motherList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    MotherSicken1 = motherList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    MotherSicken2 = motherList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    MotherSicken3 = motherList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    MotherSicken4 = motherList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    MotherSicken5 = motherList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("sibling"))
                    {
                        var sibling = (JArray)jObject["sibling"];
                        if (sibling?.Count > 0)
                        {
                            var siblingList = sibling.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= siblingList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    BrotherSicken1 = siblingList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    BrotherSicken2 = siblingList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    BrotherSicken3 = siblingList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    BrotherSicken4 = siblingList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    BrotherSicken5 = siblingList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("children"))
                    {
                        var children = (JArray)jObject["children"];
                        if (children?.Count > 0)
                        {
                            var childrenList = children.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= childrenList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    ChildSicken1 = childrenList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    ChildSicken2 = childrenList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    ChildSicken3 = childrenList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    ChildSicken4 = childrenList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    ChildSicken5 = childrenList[(i - 1)].ToString();
                                }
                            }
                        }
                    }

                    if (jObject.ContainsKey("historyGenetic"))
                    {
                        HistoryGeneticOther = jObject["historyGenetic"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("historyGeneticName"))
                    {
                        HistoryGeneticName = jObject["historyGeneticName"]?.ToString() ?? string.Empty;
                    }

                    if (jObject.ContainsKey("disabilitySituation"))
                    {
                        var disabilitySituation = (JArray)jObject["disabilitySituation"];
                        if (disabilitySituation?.Count > 0)
                        {
                            var disabilitySituationList = disabilitySituation.OrderBy(c => Convert.ToInt32(c.ToString()))?.ToList() ?? new List<JToken>();
                            for (var i = 1; i <= disabilitySituationList.Count; i++)
                            {
                                if (i == 1)
                                {
                                    DisabilitySituationOther1 = disabilitySituationList[(i - 1)].ToString();
                                }
                                else if (i == 2)
                                {
                                    DisabilitySituationOther2 = disabilitySituationList[(i - 1)].ToString();
                                }
                                else if (i == 3)
                                {
                                    DisabilitySituationOther3 = disabilitySituationList[(i - 1)].ToString();
                                }
                                else if (i == 4)
                                {
                                    DisabilitySituationOther4 = disabilitySituationList[(i - 1)].ToString();
                                }
                                else if (i == 5)
                                {
                                    DisabilitySituationOther5 = disabilitySituationList[(i - 1)].ToString();
                                }
                            }
                        }
                    }
                    if (jObject.ContainsKey("disabilitySituationOther"))
                    {
                        DisabilitySituationOther = jObject["disabilitySituationOther"]?.ToString() ?? string.Empty;
                    }

                    if (jObject.ContainsKey("kitchenExhaustFacilities"))
                    {
                        KitchenExhaustFacilities = jObject["kitchenExhaustFacilities"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("fuel"))
                    {
                        Fuel = jObject["fuel"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("drinkingWater"))
                    {
                        DrinkingWater = jObject["drinkingWater"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("toilet"))
                    {
                        Toilet = jObject["toilet"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("animalPen"))
                    {
                        AnimalPen = jObject["animalPen"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("drinkingWaterOther"))
                    {
                        DrinkingWaterOther = jObject["drinkingWaterOther"]?.ToString() ?? string.Empty;
                    }
                    if (jObject.ContainsKey("fuelOther"))
                    {
                        FuelOther = jObject["fuelOther"]?.ToString() ?? string.Empty;
                    }
                }
            }

            var result = htmlContent;
            result = result.Replace("{{PatientName}}", PatientName);
            result = result.Replace("{{PhysicalExamNumberTitle}}", PhysicalExamNumberTitle);
            result = result.Replace("{{PhysicalExamNumber}}", PhysicalExamNumberHtml);
            result = result.Replace("{{EducationLevel}}", EducationLevel);
            result = result.Replace("{{SexType}}", SexType);
            result = result.Replace("{{Birthday}}", Birthday);
            result = result.Replace("{{IDCardNo}}", IDCardNo);
            result = result.Replace("{{WorkUnit}}", WorkUnit);
            result = result.Replace("{{Phone}}", Phone);
            result = result.Replace("{{ContactName}}", ContactName);
            result = result.Replace("{{ContactPhone}}", ContactPhone);
            result = result.Replace("{{PermanentlyResideType}}", PermanentlyResideType);
            result = result.Replace("{{NationalityBox}}", NationalityBox);
            result = result.Replace("{{Nationality}}", Nationality);
            result = result.Replace("{{BloodType}}", BloodType);
            result = result.Replace("{{RH}}", RH);
            result = result.Replace("{{Career}}", Career);
            result = result.Replace("{{ProviderPaymentsOther}}", ProviderPaymentsOther);
            result = result.Replace("{{MaritalStatus}}", MaritalStatus);
            result = result.Replace("{{PaymentsMethod1}}", PaymentsMethod1);
            result = result.Replace("{{PaymentsMethod2}}", PaymentsMethod2);
            result = result.Replace("{{PaymentsMethod3}}", PaymentsMethod3);
            result = result.Replace("{{HistoryDrugAllergiesOther}}", HistoryDrugAllergiesOther);
            result = result.Replace("{{DrugAllergiesMethod1}}", DrugAllergiesMethod1);
            result = result.Replace("{{DrugAllergiesMethod2}}", DrugAllergiesMethod2);
            result = result.Replace("{{DrugAllergiesMethod3}}", DrugAllergiesMethod3);
            result = result.Replace("{{DrugAllergiesMethod4}}", DrugAllergiesMethod4);
            result = result.Replace("{{ExposeMethod1}}", ExposeMethod1);
            result = result.Replace("{{ExposeMethod2}}", ExposeMethod2);
            result = result.Replace("{{ExposeMethod3}}", ExposeMethod3);
            result = result.Replace("{{HistoryPreviousDiseaseMalignancy}}", HistoryPreviousDiseaseMalignancy);
            result = result.Replace("{{HistoryPreviousDiseaseOccupationalDisease}}", HistoryPreviousDiseaseOccupationalDisease);
            result = result.Replace("{{HistoryPreviousDiseaseOther}}", HistoryPreviousDiseaseOther);
            result = result.Replace("{{DiseaseDuration1}}", DiseaseDuration1);
            result = result.Replace("{{DiseaseDurationTime1}}", DiseaseDurationTime1);
            result = result.Replace("{{DiseaseDuration2}}", DiseaseDuration2);
            result = result.Replace("{{DiseaseDurationTime2}}", DiseaseDurationTime2);
            result = result.Replace("{{DiseaseDuration3}}", DiseaseDuration3);
            result = result.Replace("{{DiseaseDurationTime3}}", DiseaseDurationTime3);
            result = result.Replace("{{DiseaseDuration4}}", DiseaseDuration4);
            result = result.Replace("{{DiseaseDurationTime4}}", DiseaseDurationTime4);
            result = result.Replace("{{DiseaseDuration5}}", DiseaseDuration5);
            result = result.Replace("{{DiseaseDurationTime5}}", DiseaseDurationTime5);
            result = result.Replace("{{DiseaseDuration6}}", DiseaseDuration6);
            result = result.Replace("{{DiseaseDurationTime6}}", DiseaseDurationTime6);
            result = result.Replace("{{OperationOneName}}", OperationOneName);
            result = result.Replace("{{OperationOneTime}}", OperationOneTime);
            result = result.Replace("{{OperationTwoName}}", OperationTwoName);
            result = result.Replace("{{OperationTwoTime}}", OperationTwoTime);
            result = result.Replace("{{ExposeMethodOperation}}", ExposeMethodOperation);
            result = result.Replace("{{TraumaOneName}}", TraumaOneName);
            result = result.Replace("{{TraumaOneTime}}", TraumaOneTime);
            result = result.Replace("{{TraumaTwoName}}", TraumaTwoName);
            result = result.Replace("{{TraumaTwoTime}}", TraumaTwoTime);
            result = result.Replace("{{ExposeMethodOtherProvinces}}", ExposeMethodOtherProvinces);
            result = result.Replace("{{TransfusionOneName}}", TransfusionOneName);
            result = result.Replace("{{TransfusionOneTime}}", TransfusionOneTime);
            result = result.Replace("{{TransfusionTwoName}}", TransfusionTwoName);
            result = result.Replace("{{TransfusionTwoTime}}", TransfusionTwoTime);
            result = result.Replace("{{ExposeMethodBloodTransfusion}}", ExposeMethodBloodTransfusion);
            result = result.Replace("{{FatherSicken1}}", FatherSicken1);
            result = result.Replace("{{FatherSicken2}}", FatherSicken2);
            result = result.Replace("{{FatherSicken3}}", FatherSicken3);
            result = result.Replace("{{FatherSicken4}}", FatherSicken4);
            result = result.Replace("{{FatherSicken5}}", FatherSicken5);
            result = result.Replace("{{MotherSicken1}}", MotherSicken1);
            result = result.Replace("{{MotherSicken2}}", MotherSicken2);
            result = result.Replace("{{MotherSicken3}}", MotherSicken3);
            result = result.Replace("{{MotherSicken4}}", MotherSicken4);
            result = result.Replace("{{MotherSicken5}}", MotherSicken5);
            result = result.Replace("{{BrotherSicken1}}", BrotherSicken1);
            result = result.Replace("{{BrotherSicken2}}", BrotherSicken2);
            result = result.Replace("{{BrotherSicken3}}", BrotherSicken3);
            result = result.Replace("{{BrotherSicken4}}", BrotherSicken4);
            result = result.Replace("{{BrotherSicken5}}", BrotherSicken5);
            result = result.Replace("{{ChildSicken1}}", ChildSicken1);
            result = result.Replace("{{ChildSicken2}}", ChildSicken2);
            result = result.Replace("{{ChildSicken3}}", ChildSicken3);
            result = result.Replace("{{ChildSicken4}}", ChildSicken4);
            result = result.Replace("{{ChildSicken5}}", ChildSicken5);
            result = result.Replace("{{HistorySicken}}", HistorySicken);
            result = result.Replace("{{HistoryGeneticName}}", HistoryGeneticName);
            result = result.Replace("{{HistoryGeneticOther}}", HistoryGeneticOther);
            result = result.Replace("{{DisabilitySituationOther}}", DisabilitySituationOther);
            result = result.Replace("{{DisabilitySituationOther1}}", DisabilitySituationOther1);
            result = result.Replace("{{DisabilitySituationOther2}}", DisabilitySituationOther2);
            result = result.Replace("{{DisabilitySituationOther3}}", DisabilitySituationOther3);
            result = result.Replace("{{DisabilitySituationOther4}}", DisabilitySituationOther4);
            result = result.Replace("{{DisabilitySituationOther5}}", DisabilitySituationOther5);
            result = result.Replace("{{KitchenExhaustFacilities}}", KitchenExhaustFacilities);
            result = result.Replace("{{FuelOther}}", FuelOther);
            result = result.Replace("{{Fuel}}", Fuel);
            result = result.Replace("{{DrinkingWaterOther}}", DrinkingWaterOther);
            result = result.Replace("{{DrinkingWater}}", DrinkingWater);
            result = result.Replace("{{Toilet}}", Toilet);
            result = result.Replace("{{AnimalPen}}", AnimalPen);
            return result;
        }

        // /// <summary>
        /// 轮序读取体检数据
        /// </summary>
        private void GetInspectedRegisterInfoList()
        {
            if (string.IsNullOrEmpty(_cloudServerUrl))
            {
                var registerInfoList = _registerInfoDBService.GetInspectedRegisterInfoListDBAsync(null, null, true).Result;
                if (registerInfoList?.Count > 0)
                {
                    List<OrganizationDTO> orgDataList = null;
                    var orgCodes = registerInfoList.Where(c => !string.IsNullOrEmpty(c.OrganizationCode)).Select(c => c.OrganizationCode).Distinct().ToList();
                    if (orgCodes?.Count > 0)
                    {
                        var orgReq = new GetOrganizationListDBRequest()
                        {
                            Codes = orgCodes
                        };
                        orgDataList = _vitalOrganizationDBService.GetOrganizationListAsync(orgReq).Result;
                    }
                    foreach (var registerInfo in registerInfoList)
                    {
                        //保存报告内容到报告表
                        var reportQueryReq = new GetReportConditionRequest()
                        {
                            PhysicalExamNumber = registerInfo.PhysicalExamNumber
                        };
                        var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
                        if (reportInfos?.Count > 0)
                        {
                            var reportInfoList = reportInfos.ToList();
                            //验证 个人基本信息是否存在，或者是否需要更新
                            CheckIsNeedUpdatePersonReport(registerInfo, reportInfoList);
                            var ability_Code = "";
                            //验证 个人健康体检是否存在，或者是否需要更新
                            CheckIsNeedUpdateHealthExamReport(registerInfo, reportInfoList, ref ability_Code);
                            //验证老年人生活自理能力评估表
                            CheckIsNeedUpdateOldPersonSelfCareReport(registerInfo, reportInfoList, ability_Code);
                            //验证老年人中医药健康管理服务记录表
                            CheckIsNeedUpdateOldPersonTCMCDetailRecordReport(registerInfo, reportInfoList);

                            //验证部分和其他的所有报告更新时间, 只要有一个部分报告的更新时间大于 part的更新时间，则更新part报告                            
                            var partReportInfo = reportInfoList.FirstOrDefault(c => c.Key == "Part") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                            var otherReportList = reportInfoList.Where(c => c.Key != "Part")?.ToList() ?? new List<WingInterfaceLibrary.DTO.Vital.ReportDTO>();
                            bool isMergePartReport = false;
                            if (string.IsNullOrEmpty(partReportInfo?.Code))
                            {
                                //没有生成那么则自动生成一份
                                isMergePartReport = true;
                            }
                            else
                            {
                                //已经生成，则比对生成
                                if (otherReportList?.Count > 0)
                                {
                                    foreach (var item in otherReportList)
                                    {
                                        if (item.UpdateTime > partReportInfo.UpdateTime)
                                        {
                                            isMergePartReport = true;
                                            break;
                                        }
                                    }
                                }
                            }
                            if (isMergePartReport)
                            {
                                var joinQueueReq = new VitalReportConvertDetailRequest()
                                {
                                    PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                                    ExamType = 0,
                                    ReportCode = "",
                                    Type = 1,
                                    IsCover = true
                                };
                                ReadyDataMergePartReport(joinQueueReq);
                            }


                            if (orgDataList?.Count > 0)
                            {
                                var orgInfo = orgDataList.FirstOrDefault(c => c.Code == registerInfo.OrganizationCode) ?? new WingInterfaceLibrary.DTO.Vital.OrganizationDTO();
                                DealWithAllExamItems(registerInfo, orgInfo, reportInfoList);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 验证老年人中医药健康管理服务记录表
        /// </summary>
        private bool CheckIsNeedUpdateOldPersonTCMCDetailRecordReport(RegisterInfoDTO registerInfo, List<WingInterfaceLibrary.DTO.Vital.ReportDTO> reportInfoList)
        {
            //看看有没有创建健康体检检查
            var examReq = new GetExamRecordListDBRequest()
            {
                PatientCode = registerInfo.IDCardNo,
                Keys = new List<string>() { "HEITCMC" },
                IsValidOperationDoctor = false,
                PhysicalExamNumber = registerInfo.PhysicalExamNumber
            };
            var examRecords = _vitalExamDBService.GetExamRecordListAsync(examReq).Result;
            if (examRecords == null || examRecords.Count <= 0)
            {
                Logger.WriteLineWarn($"CheckIsNeedUpdateOldPersonTCMCDetailRecordReport examRecords is null, request:{JsonConvert.SerializeObject(examReq)}");
                return true;
            }
            var isExistReport = examRecords.Exists(c => c.ExamState == ExamStateEnum.Reported);
            if (!isExistReport)
            {
                Logger.WriteLineWarn($"CheckIsNeedUpdateOldPersonTCMCDetailRecordReport TCMC report is not exist, request:{JsonConvert.SerializeObject(examReq)}");
                return true;
            }
            var isExistHEITCMCDetailRecord = reportInfoList.Exists(c => c.Key == "HEITCMCDetailRecord");
            var isNeedUpdateHEITCMCDetailRecord = false;
            if (!isExistHEITCMCDetailRecord)
            {
                isNeedUpdateHEITCMCDetailRecord = true;
            }
            else
            {
                //存在，则遍历检查的更新时间和报告的更新时间
                var findHealthExamReport = reportInfoList.FirstOrDefault(c => c.Key == "HEITCMCDetailRecord") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                foreach (var examInfo in examRecords)
                {
                    if (examInfo.UpdateTime > findHealthExamReport.UpdateTime)
                    {
                        isNeedUpdateHEITCMCDetailRecord = true;
                        Logger.WriteLineWarn($"CheckIsNeedUpdateOldPersonTCMCDetailRecordReport HEILNRZLNLPG isNeedUpdateHEITCMCDetailRecord:{isNeedUpdateHEITCMCDetailRecord} examInfo.UpdateTime:{examInfo.UpdateTime}, findHealthExamReport.UpdateTime:{findHealthExamReport.UpdateTime}");
                        break;
                    }
                }
            }
            if (isNeedUpdateHEITCMCDetailRecord)
            {
                //同步创建个人基本信息
                var reportReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                    ExamCode = "",
                    IDCardNo = registerInfo.IDCardNo,
                    ExamType = 94,
                    Type = 1,
                    IsCover = true
                };
                var personReport = SavePersonOrHealthExamReportAsync(reportReq).Result;
            }
            return true;
        }

        /// <summary>
        /// 云端处理验证老年人生活自理能力评估表
        /// </summary>
        private bool CheckIsNeedUpdateOldPersonSelfCareReport(RegisterInfoDTO registerInfo, List<WingInterfaceLibrary.DTO.Vital.ReportDTO> reportInfoList, string ability_Code)
        {
            //看看有没有创建健康体检检查
            var examReq = new GetPatientLastExamDBRequest()
            {
                PatientCode = registerInfo.IDCardNo,
                IsValidOperationDoctor = false,
                Key = "LNRZLNLPG"
            };
            var examRecord = _vitalExamDBService.GetPatientLastExamAsync(examReq).Result;
            if (string.IsNullOrEmpty(examRecord?.Code))
            {
                Logger.WriteLineWarn($"CheckIsNeedUpdateOldPersonSelfCareReport examRecords is null, request:{JsonConvert.SerializeObject(examReq)}");
                return true;
            }

            var isExistHEILNRZLNLPG = reportInfoList.Exists(c => c.Key == "HEILNRZLNLPG");
            var isNeedUpdateLNRZLNLPG = false;
            if (!isExistHEILNRZLNLPG)
            {
                isNeedUpdateLNRZLNLPG = true;
            }
            else
            {
                //存在，则遍历检查的更新时间和报告的更新时间
                var findHealthExamReport = reportInfoList.FirstOrDefault(c => c.Key == "HEILNRZLNLPG") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();

                if (examRecord.UpdateTime > findHealthExamReport.UpdateTime)
                {
                    isNeedUpdateLNRZLNLPG = true;
                    Logger.WriteLineWarn($"CheckIsNeedUpdateOldPersonSelfCareReport HEILNRZLNLPG isNeedUpdateLNRZLNLPG:{isNeedUpdateLNRZLNLPG} examInfo.UpdateTime:{examRecord.UpdateTime}, findHealthExamReport.UpdateTime:{findHealthExamReport.UpdateTime}");
                }

            }
            if (isNeedUpdateLNRZLNLPG)
            {
                //同步创建个人基本信息
                var reportReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                    ExamCode = "",
                    IDCardNo = registerInfo.IDCardNo,
                    ExamType = 95,
                    Type = 1,
                    IsCover = true
                };
                var personReport = SavePersonOrHealthExamReportAsync(reportReq).Result;
            }
            return true;

        }


        /// <summary>
        /// 云端轮询处理健康体检数据
        /// </summary>
        private bool CheckIsNeedUpdateHealthExamReport(RegisterInfoDTO registerInfo, List<WingInterfaceLibrary.DTO.Vital.ReportDTO> reportInfoList, ref string ability_Code)
        {
            //看看有没有创建健康体检检查
            var examReq = new GetExamRecordListDBRequest()
            {
                PatientCode = registerInfo.IDCardNo,
                Keys = new List<string>() { "ZZYBZK", "SHFS", "ZQCT", "FZJC", "XCZYWT", "ZYYYFMYGHYFJZS", "JKPJJZD" },
                IsValidOperationDoctor = false,
                PhysicalExamNumber = registerInfo.PhysicalExamNumber
            };
            var examRecords = _vitalExamDBService.GetExamRecordListAsync(examReq).Result;
            if (examRecords == null || examRecords.Count <= 0)
            {
                Logger.WriteLineWarn($"GetInspectedRegisterInfoList examRecords is null, request:{JsonConvert.SerializeObject(examReq)}");
                return true;
            }
            // 
            var curExamRecord = examRecords?.Where(c => c.ExamState == ExamStateEnum.Inspected)?.OrderByDescending(c => c.ExamTime)?.FirstOrDefault() ?? new ExamRecordDTO();
            if (curExamRecord?.ExamRecordDatas?.Count > 0)
            {
                var curExamRecordData = curExamRecord.ExamRecordDatas?.FirstOrDefault(c => c.Key == "ZZYBZK") ?? new ExamRecordDataDTO();
                if (!string.IsNullOrEmpty(curExamRecordData?.ExamData))
                {
                    var ZZYBZKObject = JObject.Parse(curExamRecordData.ExamData);
                    if (ZZYBZKObject != null)
                    {
                        if (ZZYBZKObject.ContainsKey("Ability_Code"))
                        {
                            ability_Code = ZZYBZKObject["Ability_Code"]?.ToString() ?? string.Empty;
                        }
                    }
                }
            }
            var isExistHealthExam = reportInfoList.Exists(c => c.Key == "HEIHealthExamination");
            bool isNeedUpdateHealthExam = false;
            if (!isExistHealthExam)
            {
                //不存在健康检查报告，则需要更新或者创建
                isNeedUpdateHealthExam = true;
            }
            else
            {
                //存在，则遍历检查的更新时间和报告的更新时间
                var findHealthExamReport = reportInfoList.FirstOrDefault(c => c.Key == "HEIHealthExamination") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                foreach (var examInfo in examRecords)
                {
                    Logger.WriteLineWarn($"GetInspectedRegisterInfoList HEIHealthExamination examInfo.CardNo:{examInfo.CardNo}, examInfo.UpdateTime:{examInfo.UpdateTime}, findHealthExamReport.UpdateTime:{findHealthExamReport.UpdateTime}");

                    if (examInfo.UpdateTime > findHealthExamReport.UpdateTime)
                    {
                        isNeedUpdateHealthExam = true;
                        Logger.WriteLineWarn($"GetInspectedRegisterInfoList HEIHealthExamination isNeedUpdateHealthExam:{isNeedUpdateHealthExam} examInfo.UpdateTime:{examInfo.UpdateTime}, findHealthExamReport.UpdateTime:{findHealthExamReport.UpdateTime}");
                        break;
                    }
                }
            }
            if (isNeedUpdateHealthExam)
            {
                var reportReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                    ExamCode = "",
                    IDCardNo = registerInfo.IDCardNo,
                    ExamType = 97,
                    Type = 1,
                    IsCover = true
                };
                var personReport = SavePersonOrHealthExamReportAsync(reportReq).Result;
            }
            return true;
        }

        /// <summary>
        /// 云端轮询处理个人基本信息表数据
        /// </summary>
        private bool CheckIsNeedUpdatePersonReport(RegisterInfoDTO registerInfo, List<WingInterfaceLibrary.DTO.Vital.ReportDTO> reportInfoList)
        {
            var isExistCover = reportInfoList.Exists(c => c.Key == "HEICover");
            if (isExistCover)
            {
                var isExistPerson = reportInfoList.Exists(c => c.Key == "HEIPersonInfo");
                var isNeedUpdatePerson = false;
                if (!isExistPerson)
                {
                    Logger.WriteLineWarn($"GetInspectedRegisterInfoList isExistPerson false");
                    isNeedUpdatePerson = true;
                }
                else
                {
                    //根据报告创建时间和居民信息更新时间比对
                    var personReportInfo = reportInfoList.FirstOrDefault(c => c.Key == "HEIPersonInfo") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                    var dbReq = new GetPatientByCodeDBRequest()
                    {
                        Code = registerInfo.IDCardNo
                    };
                    var patientInfo = _vitalPatientDBService.GetPatientByCodeAsync(dbReq).Result;
                    if (string.IsNullOrEmpty(patientInfo?.Code))
                    {
                        Logger.WriteLineWarn($"GetInspectedRegisterInfoList patientInfo is null, request:{JsonConvert.SerializeObject(dbReq)}");
                        return true;
                    }
                    var extensionDetailReq = new GetPatientExtensionByCodeAndKeyDBRequest()
                    {
                        PatientCode = registerInfo.IDCardNo,
                        Key = "PatientHealthInfo",
                        IsValidOperationDoctor = false
                    };
                    var patientExtensionInfo = _vitalPatientExtensionDBService.GetPatientExtensionDetailByCodeAndKeyAsync(extensionDetailReq).Result;
                    if (string.IsNullOrEmpty(patientExtensionInfo?.ExtensionData))
                    {
                        Logger.WriteLineWarn($"GetInspectedRegisterInfoList patientExtensionInfo is null, request:{JsonConvert.SerializeObject(extensionDetailReq)}");
                        patientExtensionInfo = new PatientExtensionDTO()
                        {
                            UpdateTime = DateTime.MinValue
                        };
                    }
                    if (patientInfo.UpdateTime > personReportInfo.UpdateTime || patientExtensionInfo.UpdateTime > personReportInfo.UpdateTime)
                    {
                        isNeedUpdatePerson = true;
                    }
                }
                if (isNeedUpdatePerson)
                {
                    //同步创建个人基本信息
                    var reportReq = new VitalReportConvertDetailRequest()
                    {
                        PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                        ExamCode = "",
                        IDCardNo = registerInfo.IDCardNo,
                        ExamType = 96,
                        Type = 1,
                        IsCover = true
                    };
                    Logger.WriteLineWarn($"GetInspectedRegisterInfoList Person Report Start, reportReq:{JsonConvert.SerializeObject(reportReq)}");
                    var personReport = SavePersonOrHealthExamReportAsync(reportReq).Result;
                }
                return true;
            }
            return true;
        }

        /// <summary>
        /// 轮询触发处理数据
        /// </summary>
        private void DealWithAllExamItems(RegisterInfoDTO registerInfo, WingInterfaceLibrary.DTO.Vital.OrganizationDTO orgInfo, List<WingInterfaceLibrary.DTO.Vital.ReportDTO> reportInfoList)
        {
            if (orgInfo.ExamItemCodes == null || orgInfo.ExamItemCodes.Count <= 0)
            {
                orgInfo.ExamItemCodes = new List<string>()
                {
                    "HEIBasic", "HEIUrinalysis", "HEIUltrasonic", "HEIBiochemical", "HEIBloodRoutine", "HEIECG", "HEITCMC"
                };
            }
            //验证检查是否都已经完成
            bool isCompleted = true;
            var reportDBReq = new GetReportConditionRequest()
            {
                PhysicalExamNumber = registerInfo.PhysicalExamNumber
            };
            var reportList = _vitalReportDBService.GetVitalReportListDBAsync(reportDBReq).Result;
            var reportRecords = new List<WingInterfaceLibrary.DTO.Vital.ReportDTO>();
            if (reportList?.Count > 0)
            {
                foreach (var item in orgInfo.ExamItemCodes)
                {
                    var reportInfo = reportList.FirstOrDefault(c => c.Key == item && c.ReportState == ReportStateEnum.Reported);
                    if (string.IsNullOrEmpty(reportInfo?.Code))
                    {
                        isCompleted = isCompleted & false;
                        break;
                    }
                    else
                    {
                        reportRecords.Add(reportInfo);
                    }
                }
            }
            if (isCompleted)
            {
                var isExistHEIConclusionDetailRecord = reportInfoList.Exists(c => c.Key == "HEIConclusion");
                var isNeedUpdateHEIConclusionRecord = false;
                if (!isExistHEIConclusionDetailRecord)
                {
                    isNeedUpdateHEIConclusionRecord = true;
                }
                else
                {
                    //存在，则遍历检查的更新时间和报告的更新时间
                    var findConclusionReport = reportInfoList.FirstOrDefault(c => c.Key == "HEIConclusion") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                    foreach (var reportInfo in reportRecords)
                    {
                        if (reportInfo.UpdateTime > findConclusionReport.UpdateTime)
                        {
                            isNeedUpdateHEIConclusionRecord = true;
                            Logger.WriteLineWarn($"HEIConclusion HEIConclusion isNeedUpdateHEIConclusionRecord:{isNeedUpdateHEIConclusionRecord} reportInfo.UpdateTime:{reportInfo.UpdateTime}, findConclusionReport.UpdateTime:{findConclusionReport.UpdateTime}");
                            break;
                        }
                    }
                }
                if (isNeedUpdateHEIConclusionRecord)
                {
                    //获取检查信息
                    var res2 = CreateReportHEIConclusion("HEIConclusion", registerInfo.PhysicalExamNumber);
                }
            }
        }

        /// <summary>
        /// 计算中医体质
        /// </summary>
        /// <param name="data"></param>
        /// <remarks></remarks>
        private string CaculateHEITCMCResultSimple(string data)
        {
            var result = "";
            if (!string.IsNullOrEmpty(data))
            {
                JObject json = JObject.Parse(data);
                if (json == null)
                {
                    return "";
                }

                var Ping_Score_Result = "";
                if (json.ContainsKey("Ping_Score"))
                {
                    var childJson = JObject.Parse(json["Ping_Score"].ToString());
                    if (childJson.ContainsKey("Result"))
                    {
                        Ping_Score_Result = childJson["Result"]?.ToString() ?? string.Empty;
                    }
                }
                var exceptionKey = "";
                if (json.ContainsKey("PhysicalConclusion"))
                {
                    exceptionKey = json["PhysicalConclusion"]?.ToString() ?? string.Empty;
                }
                if (Ping_Score_Result != "否")
                {
                    result = $"{GetHEITCMCKeyDesc("Ping_Score")}（{Ping_Score_Result}）    ";
                }
                if (!string.IsNullOrEmpty(exceptionKey))
                {
                    var physicalConclusion = "";
                    if (json.ContainsKey(exceptionKey))
                    {
                        var childJson = JObject.Parse(json[exceptionKey].ToString());
                        if (childJson.ContainsKey("Result"))
                        {
                            physicalConclusion = childJson["Result"]?.ToString() ?? string.Empty;
                        }
                    }
                    result += GetHEITCMCKeyDesc(exceptionKey) + $"（{physicalConclusion}）\n";
                }
                //如果是平和质且为否的情况下，不展示
                if (Ping_Score_Result != "否")
                {
                    var Ping_ScoreDesc = "";
                    if (_zhongYiTiZhiDescConfigDic?.Count > 0 && _zhongYiTiZhiDescConfigDic.Keys.Contains("Ping_Score"))
                    {
                        Ping_ScoreDesc = _zhongYiTiZhiDescConfigDic["Ping_Score"];
                    }
                    result += $"{Ping_ScoreDesc}\n";
                }
                // result += $"{GetHEITCMCKeyDesc(exceptionKey)}\n";
                var exceptionDesc = "";
                if (_zhongYiTiZhiDescConfigDic?.Count > 0 && _zhongYiTiZhiDescConfigDic.Keys.Contains(exceptionKey))
                {
                    exceptionDesc = _zhongYiTiZhiDescConfigDic[exceptionKey];
                }
                result += $"{exceptionDesc}\n";
            }
            return result;
        }


        /// <summary>
        /// 计算中医体质
        /// </summary>
        /// <param name="data"></param>
        /// <remarks></remarks>
        private string CaculateHEITCMCResult(string data)
        {
            var result = "";
            if (!string.IsNullOrEmpty(data))
            {
                var dic = new Dictionary<string, int>();
                var dicResult = new Dictionary<string, string>();
                JObject json = JObject.Parse(data);
                if (json == null)
                {
                    return "";
                }

                var Qi_Score = -1;
                if (json.ContainsKey("Qi_Score"))
                {
                    var temp = json["Qi_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Qi_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Qi_Score", Qi_Score);

                var Yang_Score = -1;
                if (json.ContainsKey("Yang_Score"))
                {
                    var temp = json["Yang_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Yang_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Yang_Score", Yang_Score);

                var Yin_Score = -1;
                if (json.ContainsKey("Yin_Score"))
                {
                    var temp = json["Yin_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Yin_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Yin_Score", Yin_Score);

                var Tan_Score = -1;
                if (json.ContainsKey("Tan_Score"))
                {
                    var temp = json["Tan_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Tan_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Tan_Score", Tan_Score);

                var Shi_Score = -1;
                if (json.ContainsKey("Shi_Score"))
                {
                    var temp = json["Shi_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Shi_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Shi_Score", Shi_Score);

                var Xue_Score = -1;
                if (json.ContainsKey("Xue_Score"))
                {
                    var temp = json["Xue_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Xue_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Xue_Score", Xue_Score);

                var Qiyu_Score = -1;
                if (json.ContainsKey("Qiyu_Score"))
                {
                    var temp = json["Qiyu_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Qiyu_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Qiyu_Score", Qiyu_Score);

                var Te_Score = -1;
                if (json.ContainsKey("Te_Score"))
                {
                    var temp = json["Te_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Te_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Te_Score", Te_Score);

                var Ping_Score = -1;
                if (json.ContainsKey("Ping_Score"))
                {
                    var temp = json["Ping_Score"]?.ToString() ?? string.Empty;
                    if (!string.IsNullOrEmpty(temp))
                    {
                        Ping_Score = Convert.ToInt32(temp);
                    }
                }
                dic.Add("Ping_Score", Ping_Score);
                foreach (var key in dic.Keys)
                {
                    if (dic[key] > -1)
                    {
                        var caculateResult = GetHEITCMCConclusion(key, dic[key], dic);
                        if (!string.IsNullOrEmpty(caculateResult))
                        {
                            dicResult.Add(key, caculateResult);
                        }
                    }
                }
                var exceptionKey = "";
                if (dicResult?.Count > 0)
                {
                    if (dicResult.Keys.Contains("Ping_Score"))
                    {
                        result = $"{GetHEITCMCKeyDesc("Ping_Score")}（{dicResult["Ping_Score"]}）    ";
                    }
                    foreach (var item in dicResult.Keys)
                    {
                        if (item != "Ping_Score" && dicResult[item] != "否")
                        {
                            result += GetHEITCMCKeyDesc(item) + $"（{dicResult[item]}）\n";
                            exceptionKey = item;
                        }
                    }
                }
                // result += $"{GetHEITCMCKeyDesc("Ping_Score")}\n";
                var Ping_ScoreDesc = "";
                if (_zhongYiTiZhiDescConfigDic?.Count > 0 && _zhongYiTiZhiDescConfigDic.Keys.Contains("Ping_Score"))
                {
                    Ping_ScoreDesc = _zhongYiTiZhiDescConfigDic["Ping_Score"];
                }
                result += $"{Ping_ScoreDesc}\n";
                // result += $"{GetHEITCMCKeyDesc(exceptionKey)}\n";
                var exceptionDesc = "";
                if (_zhongYiTiZhiDescConfigDic?.Count > 0 && _zhongYiTiZhiDescConfigDic.Keys.Contains(exceptionKey))
                {
                    exceptionDesc = _zhongYiTiZhiDescConfigDic[exceptionKey];
                }
                result += $"{exceptionDesc}\n";
            }
            return result;
        }

        /// <summary>
        /// Unicode转中文
        /// </summary>
        /// <param name="unicodeString"></param>
        /// <returns></returns>
        private string ConvertToChinese(string unicodeString)
        {
            byte[] utf8Bytes = Encoding.UTF8.GetBytes(unicodeString);
            var chineseWord = Encoding.UTF8.GetString(utf8Bytes);
            return System.Net.WebUtility.UrlDecode(chineseWord);
        }

        /// <summary>
        /// 获取中医体质描述
        /// </summary>
        /// <param name="data"></param>
        /// <remarks></remarks>
        private string GetHEITCMCKeyDesc(string key)
        {
            var res = "";
            switch (key)
            {
                case "Ping_Score":
                    res = "平和质";
                    break;
                case "Qi_Score":
                    res = "气虚质";
                    break;
                case "Yin_Score":
                    res = "阴虚质";
                    break;
                case "Tan_Score":
                    res = "痰湿质";
                    break;
                case "Shi_Score":
                    res = "湿热质";
                    break;
                case "Yang_Score":
                    res = "阳虚质";
                    break;
                case "Te_Score":
                    res = "特禀质";
                    break;
                case "Qiyu_Score":
                    res = "气郁质";
                    break;
                case "Xue_Score":
                    res = "血瘀质";
                    break;
                default:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 获取中医体质结论
        /// </summary>
        /// <param name="data"></param>
        /// <remarks></remarks>
        private string GetHEITCMCConclusion(string key, int score, Dictionary<string, int> dic)
        {
            if (key == "Ping_Score")
            {
                if (score >= 17)
                {
                    if (
                        dic["Qi_Score"] > 8 || dic["Yang_Score"] > 8 || dic["Yin_Score"] > 8 || dic["Tan_Score"] > 8 || dic["Shi_Score"] > 8 ||
                        dic["Xue_Score"] > 8 || dic["Qiyu_Score"] > 8 || dic["Te_Score"] > 8 || dic["Ping_Score"] > 8)
                    {
                        if (dic["Qi_Score"] > 10 || dic["Yang_Score"] > 10 || dic["Yin_Score"] > 10 || dic["Tan_Score"] > 10 || dic["Shi_Score"] > 10 ||
                            dic["Xue_Score"] > 10 || dic["Qiyu_Score"] > 10 || dic["Te_Score"] > 10 || dic["Ping_Score"] > 10)
                        {
                            return "否";
                        }
                        else
                        {
                            return "基本是";
                        }
                    }
                    else
                    {
                        return "是";
                    }
                }
                else
                {
                    return "否";
                }
            }
            else
            {
                if (score >= 11)
                {
                    return "是";
                }
                else if (score == 9 && score == 10)
                {
                    return "倾向是";
                }
                else
                {
                    return "否";
                }
            }
        }

        /// <summary>
        /// 获取报告code和报告模板json
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public Tuple<string, string> GetVitalReportTemplateJson(string key, string examData = "")
        {
            var vitalTemplateCode = "";
            var vitalTemplateJson = "";
            if (!string.IsNullOrEmpty(key))
            {
                if (key == "HEIUltrasonic" || key == "HEIAll")
                { }
                else
                {
                    var reportTemplateInfo = _vitalTemplateDBService.GetTemplateInfoByConditionDBAsync("", key, TemplateTypeEnum.Report).Result ?? new TemplateDTO();
                    vitalTemplateCode = reportTemplateInfo.Code;
                    vitalTemplateJson = reportTemplateInfo.TemplateContent;
                }
                if (key == "HEIECG" && !string.IsNullOrWhiteSpace(examData))
                {
                    try
                    {
                        JObject json = JObject.Parse(examData);
                        if (json != null && json.ContainsKey("Analyse12"))
                        {
                            var Analyse12 = json["Analyse12"].ToString();
                            if (!string.IsNullOrWhiteSpace(Analyse12))
                            {
                                JObject jsonAnalyse12 = JObject.Parse(Analyse12);
                                var DiagnosticEleSignature = jsonAnalyse12 != null && jsonAnalyse12.ContainsKey("DiagnosticEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["DiagnosticEleSignature"].ToString());
                                var ReviewEleSignature = jsonAnalyse12 != null && jsonAnalyse12.ContainsKey("ReviewEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["ReviewEleSignature"].ToString());
                                if (DiagnosticEleSignature && ReviewEleSignature)
                                {
                                    var templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "MiShanTemplate", "HEIECGAuditAndExam.json");
                                    if (File.Exists(templatePath))
                                    {
                                        vitalTemplateJson = File.ReadAllText(templatePath);
                                        vitalTemplateCode = string.Empty;
                                    }
                                }
                                else if (DiagnosticEleSignature)
                                {
                                    var templatePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "MiShanTemplate", "HEIECGOnlySign.json");
                                    if (File.Exists(templatePath))
                                    {
                                        vitalTemplateJson = File.ReadAllText(templatePath);
                                        vitalTemplateCode = string.Empty;
                                    }
                                }
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.WriteLineWarn($"ReportService GetVitalReportTemplateJson analysis ecg json failed, ex:{ex}");
                    }
                }
                if (string.IsNullOrEmpty(vitalTemplateJson))
                {
                    if (key == "HEIPersonInfo" || key == "HEIHealthExamination1" || key == "HEIHealthExamination2" || key == "HEIHealthExamination3" || key == "HEIHealthExamination4"
                        || key == "HEILNRZLNLPG" || key == "HEITCMCDetailRecord1" || key == "HEITCMCDetailRecord2")
                    {
                        if (key == "HEIPersonInfo")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "personalInfoBasicTable.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEIHealthExamination1")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable1.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEIHealthExamination2")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable2.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEIHealthExamination3")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable3.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEIHealthExamination4")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "examTable4.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEILNRZLNLPG")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "oldPersonAssess.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEITCMCDetailRecord1")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "traditionalChineseMedicineHealthTable1.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                        else if (key == "HEITCMCDetailRecord2")
                        {
                            var templateContent = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Vital", "PersonArchivesTemplate", "traditionalChineseMedicineHealthTable2.html");
                            vitalTemplateJson = File.ReadAllText(templateContent);
                        }
                    }
                }
            }
            return Tuple.Create(vitalTemplateCode, vitalTemplateJson);
        }


        /// <summary>
        /// 获取年龄
        /// </summary>
        /// <returns></returns>
        private int CalculateAge(DateTime birthDate)
        {
            DateTime currentDate = DateTime.Now; // 获取当前日期时间            
            int years = currentDate.Year - birthDate.Year; // 计算年份之差            
            if (currentDate < birthDate.AddYears(years)) // 如果还未过完指定年份
                return --years; // 则将年份减去1            
            return years; // 否则返回正常的年份
        }

        /// <summary>
        /// 将vital数据转换为报告所需的dataJson
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public string GetVitalReportDataJson(string key, string examData, RegisterInfoDTO registerInfo, WingInterfaceLibrary.DTO.Vital.OrganizationDTO orgInfo, ExamDTO examInfo, out string doctorSignStr, out string doctorNameStr)
        {
            doctorSignStr = "";
            doctorNameStr = "";
            var vitalReportDataJson = "";
            if (string.IsNullOrEmpty(key))
            {
                return vitalReportDataJson;
            }

            var getPatientsRequest = new WingInterfaceLibrary.Request.DBVitalRequest.GetPatientListDBRequest();
            getPatientsRequest.Codes = new List<string>() { registerInfo.IDCardNo };
            getPatientsRequest.IsValidOperationDoctor = false;
            var doctorSign = "";
            var examDoctor = "";
            if (examInfo != null)
            {
                doctorSign = examInfo.ExamSign;
                examDoctor = examInfo.ExamDoctor;
                //超声和尿常规在工作站没有账号, 也可以有账号，但是可以做检查，云端也没有
                //生化和血常规在云端有账号，工作站也没有账号
                if (key == "HEIUrinalysis")
                {
                    if (string.IsNullOrEmpty(doctorSign))
                    {
                        doctorSign = "";
                        examDoctor = "";
                        //尿常规
                        if (!string.IsNullOrEmpty(orgInfo?.UrinalysisSignature))
                        {
                            doctorSign = orgInfo.UrinalysisSignature;
                        }
                    }
                }
            }
            else
            {
                var patients = _vitalPatientDBService.GetPatientListAsync(getPatientsRequest).Result;
                var patientInfo = patients.FirstOrDefault(x => x.CardNo == registerInfo.IDCardNo);
                examDoctor = patientInfo?.CreatedDoctorName ?? string.Empty;
                var userInfo = _vitalUserDBService.GetUserDetailAsync(new GetUserDBRequest
                {
                    Code = patientInfo?.CreatedDoctor ?? string.Empty,
                }).Result;
                if (!string.IsNullOrEmpty(userInfo?.Signature))
                {
                    doctorSign = userInfo.Signature;
                }
            }
            if (key == "HEIUrinalysis")
            {
                if (string.IsNullOrEmpty(doctorSign))
                {
                    var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "UrinalysisSign.png"); ;
                    var fileToken = UploadFileAsync(path, Path.GetFileName(path)).Result;
                    if (!string.IsNullOrWhiteSpace(fileToken))
                    {
                        doctorSign = fileToken;
                    }
                }
            }
            // else if (key == "HEIBloodRoutine")
            // {
            //     if (string.IsNullOrEmpty(doctorSign))
            //     {
            //         var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "BloodSign.png"); ;
            //         var fileToken = UploadFileAsync(path, Path.GetFileName(path)).Result;
            //         if (!string.IsNullOrWhiteSpace(fileToken))
            //         {
            //             doctorSign = fileToken;
            //         }
            //     }
            // }
            // else if (key == "HEIBiochemical")
            // {
            //     if (string.IsNullOrEmpty(doctorSign))
            //     {
            //         var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "BiochemicalSign.png"); ;
            //         var fileToken = UploadFileAsync(path, Path.GetFileName(path)).Result;
            //         if (!string.IsNullOrWhiteSpace(fileToken))
            //         {
            //             doctorSign = fileToken;
            //         }
            //     }
            // }
            // else if (key == "HEIBasic")
            // {
            //     if (string.IsNullOrEmpty(doctorSign))
            //     {
            //         var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "DefaultReportTemplate", "BasicSign.png"); ;
            //         var fileToken = UploadFileAsync(path, Path.GetFileName(path)).Result;
            //         if (!string.IsNullOrWhiteSpace(fileToken))
            //         {
            //             doctorSign = fileToken;
            //         }
            //     }
            // }

            doctorSignStr = doctorSign;
            doctorNameStr = examDoctor;
            var orgName = orgInfo?.OrganizationName ?? string.Empty;
            var initiatorName = examDoctor;
            var healthCreateTime = registerInfo?.CreateTime.ToString("yyyy-MM-dd") ?? "";

            var Name = registerInfo?.Name ?? string.Empty;
            var PhysicalExamNumber = registerInfo?.PhysicalExamNumber ?? string.Empty;
            var IDCardNo = registerInfo?.IDCardNo ?? string.Empty;
            var Sex = registerInfo?.Sex ?? string.Empty;
            var Age = "";
            if (registerInfo?.Birthday > DateTime.MinValue)
            {
                Age = CalculateAge(registerInfo.Birthday).ToString() ?? string.Empty;
            }
            else
            {
                Age = registerInfo?.Age ?? string.Empty;
            }
            var Birthday = "";
            if (registerInfo?.Birthday > Convert.ToDateTime("1900-1-1"))
            {
                Birthday = registerInfo?.Birthday.ToUniversalTime().ToString("yyyy-MM-dd") ?? "";
            }
            var Phone = registerInfo?.Phone ?? string.Empty;
            var Adress = registerInfo?.Adress ?? string.Empty;

            var item = "zh-CN";
            var reportDatas = new List<ReportElementDTO>();
            if (!string.IsNullOrEmpty(examData))
            {
                JObject json = JObject.Parse(examData);
                if (json == null)
                {
                    return vitalReportDataJson;
                }

                var temperature = "";
                if (json.ContainsKey("Temperature"))
                {
                    temperature = json["Temperature"].ToString();
                }
                var fasting_Glucose_Mmol = "";
                if (json.ContainsKey("Fasting_Glucose_Mmol"))
                {
                    fasting_Glucose_Mmol = json["Fasting_Glucose_Mmol"].ToString();
                }
                var Blood_Hgb = "";
                if (json.ContainsKey("Blood_Hgb"))
                {
                    Blood_Hgb = json["Blood_Hgb"].ToString();
                }
                var Blood_Wbc = "";
                if (json.ContainsKey("Blood_Wbc"))
                {
                    Blood_Wbc = json["Blood_Wbc"].ToString();
                }
                var Blood_Rlt = "";
                if (json.ContainsKey("Blood_Rlt"))
                {
                    Blood_Rlt = json["Blood_Rlt"].ToString();
                }
                var Lipid_Hdl = "";
                if (json.ContainsKey("Lipid_Hdl"))
                {
                    Lipid_Hdl = json["Lipid_Hdl"].ToString();
                }
                var Lipid_Tg = "";
                if (json.ContainsKey("Lipid_Tg"))
                {
                    Lipid_Tg = json["Lipid_Tg"].ToString();
                }
                var Lipid_Ldl = "";
                if (json.ContainsKey("Lipid_Ldl"))
                {
                    Lipid_Ldl = json["Lipid_Ldl"].ToString();
                }
                var Lipid_Cho = "";
                if (json.ContainsKey("Lipid_Cho"))
                {
                    Lipid_Cho = json["Lipid_Cho"].ToString();
                }
                var Height = "";
                if (json.ContainsKey("Height"))
                {
                    Height = json["Height"].ToString();
                }
                var Weight = "";
                if (json.ContainsKey("Weight"))
                {
                    Weight = json["Weight"].ToString();
                }
                var Bmi = "";
                if (json.ContainsKey("Bmi"))
                {
                    Bmi = json["Bmi"].ToString();
                }
                var WAISTLINE = "";
                if (json.ContainsKey("WAISTLINE"))
                {
                    WAISTLINE = json["WAISTLINE"].ToString();
                }
                var Breathe = "";
                if (json.ContainsKey("Breathe"))
                {
                    Breathe = json["Breathe"].ToString();
                }
                var Dbp_Right = "";
                if (json.ContainsKey("Dbp_Right"))
                {
                    Dbp_Right = json["Dbp_Right"].ToString();//血压右侧舒张压
                }
                var Sbp_Right = "";
                if (json.ContainsKey("Sbp_Right"))
                {
                    Sbp_Right = json["Sbp_Right"].ToString();//血压右侧收缩压
                }
                var Sbp_Left = "";
                if (json.ContainsKey("Sbp_Left"))
                {
                    Sbp_Left = json["Sbp_Left"].ToString();//血压左侧收缩压
                }
                var Dbp_Left = "";
                if (json.ContainsKey("Dbp_Left"))
                {
                    Dbp_Left = json["Dbp_Left"].ToString();//血压左侧舒张压
                }
                var Dbp = "";
                if (json.ContainsKey("Dbp"))
                {
                    Dbp = json["Dbp"].ToString();//舒张压
                }
                var Sbp = "";
                if (json.ContainsKey("Sbp"))
                {
                    Sbp = json["Sbp"].ToString();//收缩压
                }

                if (string.IsNullOrEmpty(Dbp_Right) && string.IsNullOrEmpty(Sbp_Right) && string.IsNullOrEmpty(Sbp_Left) && string.IsNullOrEmpty(Dbp_Left))
                {
                    Dbp_Right = Dbp_Left = Dbp;
                    Sbp_Right = Sbp_Left = Sbp;
                }
                var leftBloodPressure = "";
                var rightBloodPressure = "";
                if (!string.IsNullOrEmpty(Dbp_Right) || !string.IsNullOrEmpty(Sbp_Right))
                {
                    rightBloodPressure = Sbp_Right + "/" + Dbp_Right;
                }
                if (!string.IsNullOrEmpty(Sbp_Left) || !string.IsNullOrEmpty(Dbp_Left))
                {
                    leftBloodPressure = Sbp_Left + "/" + Dbp_Left;
                }
                var Pulse_Beat = "";
                if (json.ContainsKey("Pulse_Beat"))
                {
                    Pulse_Beat = json["Pulse_Beat"].ToString();
                }
                var Pulse_Frequency = "";
                if (json.ContainsKey("Pulse_Frequency"))
                {
                    Pulse_Frequency = json["Pulse_Frequency"]?.ToString() ?? string.Empty;
                }
                var PRO = "";
                if (json.ContainsKey("PRO"))
                {
                    PRO = json["PRO"].ToString();
                }
                var Urinary_Glu = "";
                if (json.ContainsKey("Urinary_Glu"))
                {
                    Urinary_Glu = json["Urinary_Glu"].ToString();
                }
                var GLU = "";
                if (json.ContainsKey("GLU"))
                {
                    GLU = json["GLU"].ToString();
                }
                var Urinary_Ket = "";
                if (json.ContainsKey("Urinary_Ket"))
                {
                    Urinary_Ket = json["Urinary_Ket"].ToString();
                }
                var KET = "";
                if (json.ContainsKey("KET"))
                {
                    KET = json["KET"].ToString();
                }
                var Urinary_Bld = "";
                if (json.ContainsKey("Urinary_Bld"))
                {
                    Urinary_Bld = json["Urinary_Bld"].ToString();
                }
                var UBG = "";
                if (json.ContainsKey("UBG"))
                {
                    UBG = json["UBG"].ToString();
                }
                var NIT = "";
                if (json.ContainsKey("NIT"))
                {
                    NIT = json["NIT"].ToString();
                }
                var PH = "";
                if (json.ContainsKey("PH"))
                {
                    PH = json["PH"].ToString();
                }
                var SG = "";
                if (json.ContainsKey("SG"))
                {
                    SG = json["SG"].ToString();
                }
                var VC = "";
                if (json.ContainsKey("VC"))
                {
                    VC = json["VC"].ToString();
                }
                var BIL = "";
                if (json.ContainsKey("BIL"))
                {
                    BIL = json["BIL"].ToString();
                }
                var LEU = "";
                if (json.ContainsKey("LEU"))
                {
                    LEU = json["LEU"].ToString();
                }
                var Blood_Sugar = "";
                if (json.ContainsKey("Blood_Sugar"))
                {
                    Blood_Sugar = json["Blood_Sugar"].ToString();
                }
                var BLD = "";
                if (json.ContainsKey("BLD"))
                {
                    BLD = json["BLD"].ToString();
                }
                var RBC = "";
                if (json.ContainsKey("RBC"))
                {
                    RBC = json["RBC"].ToString();
                }
                var BCH_Glu = "";
                if (json.ContainsKey("BCH_Glu"))
                {
                    BCH_Glu = json["BCH_Glu"].ToString();
                }
                var Liver_Alt = "";
                if (json.ContainsKey("Liver_Alt"))
                {
                    Liver_Alt = json["Liver_Alt"].ToString();
                }
                var Liver_Ast = "";
                if (json.ContainsKey("Liver_Ast"))
                {
                    Liver_Ast = json["Liver_Ast"].ToString();
                }
                var Liver_Tbil = "";
                if (json.ContainsKey("Liver_Tbil"))
                {
                    Liver_Tbil = json["Liver_Tbil"].ToString();
                }
                var Liver_Alb = "";
                if (json.ContainsKey("Liver_Alb"))
                {
                    Liver_Alb = json["Liver_Alb"].ToString();
                }
                var Liver_Dbil = "";
                if (json.ContainsKey("Liver_Dbil"))
                {
                    Liver_Dbil = json["Liver_Dbil"].ToString();
                }
                var Renal_Cr = "";
                if (json.ContainsKey("Renal_Cr"))
                {
                    Renal_Cr = json["Renal_Cr"].ToString();
                }
                var Renal_Bun = "";
                if (json.ContainsKey("Renal_Bun"))
                {
                    Renal_Bun = json["Renal_Bun"].ToString();
                }
                var Renal_Spc = "";
                if (json.ContainsKey("Renal_Spc"))
                {
                    Renal_Spc = json["Renal_Spc"].ToString();
                }
                var Renal_Ssc = "";
                if (json.ContainsKey("Renal_Ssc"))
                {
                    Renal_Ssc = json["Renal_Ssc"].ToString();
                }
                var HEART12 = "";
                if (json.ContainsKey("HEART12"))
                {
                    HEART12 = json["HEART12"].ToString();
                }
                var Analyse12 = "";
                if (json.ContainsKey("Analyse12"))
                {
                    Analyse12 = json["Analyse12"].ToString();
                }
                var ASSESS12 = "";
                if (json.ContainsKey("ASSESS12"))
                {
                    ASSESS12 = json["ASSESS12"].ToString();
                }
                var ECG12 = "";
                if (json.ContainsKey("ECG12"))
                {
                    ECG12 = json["ECG12"].ToString();
                }



                var Blood_Lymphocyte = "";
                if (json.ContainsKey("Blood_Lymphocyte"))
                {
                    Blood_Lymphocyte = json["Blood_Lymphocyte"].ToString();
                }
                var Blood_Neutrophils = "";
                if (json.ContainsKey("Blood_Neutrophils"))
                {
                    Blood_Neutrophils = json["Blood_Neutrophils"].ToString();
                }
                var Blood_Monocyte = "";
                if (json.ContainsKey("Blood_Monocyte"))
                {
                    Blood_Monocyte = json["Blood_Monocyte"].ToString();
                }
                var Blood_LymphocyteRatio = "";
                if (json.ContainsKey("Blood_LymphocyteRatio"))
                {
                    Blood_LymphocyteRatio = json["Blood_LymphocyteRatio"].ToString();
                }
                var Blood_NeutrophilRatio = "";
                if (json.ContainsKey("Blood_NeutrophilRatio"))
                {
                    Blood_NeutrophilRatio = json["Blood_NeutrophilRatio"].ToString();
                }

                var Blood_MonocyteRatio = "";
                if (json.ContainsKey("Blood_MonocyteRatio"))
                {
                    Blood_MonocyteRatio = json["Blood_MonocyteRatio"].ToString();
                }
                var Blood_Mch = "";
                if (json.ContainsKey("Blood_Mch"))
                {
                    Blood_Mch = json["Blood_Mch"].ToString();
                }
                var Blood_McV = "";
                if (json.ContainsKey("Blood_McV"))
                {
                    Blood_McV = json["Blood_McV"].ToString();
                }
                var Blood_Mchc = "";
                if (json.ContainsKey("Blood_Mchc"))
                {
                    Blood_Mchc = json["Blood_Mchc"].ToString();
                }
                var Blood_ErythrocyteDW = "";
                if (json.ContainsKey("Blood_ErythrocyteDW"))
                {
                    Blood_ErythrocyteDW = json["Blood_ErythrocyteDW"].ToString();
                }
                var Blood_Hematocrit = "";
                if (json.ContainsKey("Blood_Hematocrit"))
                {
                    Blood_Hematocrit = json["Blood_Hematocrit"].ToString();
                }
                var Blood_MeanPlateletVolume = "";
                if (json.ContainsKey("Blood_MeanPlateletVolume"))
                {
                    Blood_MeanPlateletVolume = json["Blood_MeanPlateletVolume"].ToString();
                }
                var Blood_PlateletDW = "";
                if (json.ContainsKey("Blood_PlateletDW"))
                {
                    Blood_PlateletDW = json["Blood_PlateletDW"].ToString();
                }
                var Blood_PlateletPct = "";
                if (json.ContainsKey("Blood_PlateletPct"))
                {
                    Blood_PlateletPct = json["Blood_PlateletPct"].ToString();
                }

                // var HealthGuidance = "";
                // if (json.ContainsKey("HealthGuidance"))
                // {
                //     HealthGuidance = json["HealthGuidance"].ToString();
                // }

                // var ExceptionalResults = "";
                // if (json.ContainsKey("ExceptionalResults"))
                // {
                //     ExceptionalResults = json["ExceptionalResults"].ToString();
                // }
                var dictionary = new Dictionary<string, string>();
                var dicJson = "";
                // var imageList = new List<string>();
                // var descriptionList = new List<string>();
                // var summaryList = new List<string>();
                var dataItemList = new List<DataItemDTO>();
                if (registerInfo.DiagnosisExceptionResult?.Count > 0)
                {
                    dataItemList = registerInfo.DiagnosisExceptionResult;
                }
                switch (key)
                {
                    case "HEIBasic":////1-基础检查; 

                        // if (!string.IsNullOrEmpty(Pulse_Frequency))
                        // {
                        //     var val = Convert.ToInt32(Pulse_Frequency) / 4;
                        //     Breathe = val.ToString();
                        // }

                        if (!string.IsNullOrEmpty(Pulse_Beat))
                        {
                            if (int.TryParse(Pulse_Beat, out int xinlv) && xinlv > 0)
                            {
                                Breathe = (xinlv / 4).ToString();
                            }
                        }
                        if (string.IsNullOrWhiteSpace(Breathe) && !string.IsNullOrEmpty(registerInfo.PhysicalExamNumber))
                        {
                            var ecgExamInfo = _vitalExamDBService.GetExamInfoByConditionDBAsync(registerInfo.PhysicalExamNumber, "HEIECG").Result;
                            if (!string.IsNullOrEmpty(ecgExamInfo?.ExamData))
                            {
                                JObject ecgDataObj = JObject.Parse(ecgExamInfo.ExamData);
                                if (ecgDataObj != null)
                                {
                                    if (ecgDataObj.ContainsKey("HEART12"))
                                    {
                                        if (int.TryParse(ecgDataObj["HEART12"].ToString(), out int xinlv) && xinlv > 0)
                                        {
                                            Breathe = (xinlv / 4).ToString();
                                        }
                                    }
                                }
                            }
                        }
                        if (string.IsNullOrWhiteSpace(Breathe) && !string.IsNullOrEmpty(Pulse_Frequency))
                        {
                            if (int.TryParse(Pulse_Frequency, out int xinlv) && xinlv > 0)
                            {
                                Breathe = (xinlv / 4).ToString();
                            }
                        }
                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPhysicalExamNumberKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362w", "PhysicalExamNumber", PhysicalExamNumber,item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientWeightKey").Value, "0003B8C3-397C-4688-985D-55B757E3855D", "PatientWeight", Weight, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientHeightKey").Value, "9556EA12-CFB3-4AD5-8717-1057D1D5A3EE", "PatientHeight", Height, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientRightBloodPressureKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3617", "PatientRightBloodPressure", rightBloodPressure, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLeftBloodPressureKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3616", "PatientLeftBloodPressure", leftBloodPressure, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientWaistKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3615", "PatientWaist", WAISTLINE, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientWeightIndexKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3614", "PatientWeightIndex", Bmi, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientRespiratoryRateKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3613", "PatientRespiratoryRate", Breathe, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPulseRateKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3612", "PatientPulseRate", Pulse_Frequency, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientTemperatureKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3611", "PatientTemperature", temperature, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBloodSugarKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362y", "PatientBloodSugar", fasting_Glucose_Mmol, item),

                        });
                        break;
                    case "HEIUrinalysis"://2-尿常规检查; 
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item));

                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientUrinaryProteinKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361p", "PatientUrinaryProtein", PRO, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGlucoseKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361q", "PatientGlucose", GLU, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientUrinaryKetoneBodyKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361r", "PatientUrinaryKetoneBody", KET, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBldKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361s", "PatientBld", BLD, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientUrobilinogenKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361t", "PatientUrobilinogen", UBG, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNitriteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361u", "PatientNitrite", NIT, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPHValueKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361v", "PatientPHValue", PH, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientProportionKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361w", "PatientProportion", SG, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientVitaminCKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361x", "PatientVitaminC", VC, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBilirubinKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361y", "PatientBilirubin", BIL, item));
                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLeukocyteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3619", "PatientLeukocyte", LEU, item));

                        dictionary.Add("PatientUrinaryProtein", GetResultPromptByElementKey("HEIUrinalysis", "PatientUrinaryProtein", PRO));
                        dictionary.Add("PatientGlucose", GetResultPromptByElementKey("HEIUrinalysis", "PatientGlucose", GLU));
                        dictionary.Add("PatientUrinaryKetoneBody", GetResultPromptByElementKey("HEIUrinalysis", "PatientUrinaryKetoneBody", KET));
                        dictionary.Add("PatientBld", GetResultPromptByElementKey("HEIUrinalysis", "PatientBld", BLD));
                        dictionary.Add("PatientUrobilinogen", GetResultPromptByElementKey("HEIUrinalysis", "PatientUrobilinogen", UBG));
                        dictionary.Add("PatientNitrite", GetResultPromptByElementKey("HEIUrinalysis", "PatientNitrite", NIT));
                        dictionary.Add("PatientPHValue", GetResultPromptByElementKey("HEIUrinalysis", "PatientPHValue", PH));
                        dictionary.Add("PatientProportion", GetResultPromptByElementKey("HEIUrinalysis", "PatientProportion", SG));
                        dictionary.Add("PatientVitaminC", GetResultPromptByElementKey("HEIUrinalysis", "PatientVitaminC", VC));
                        dictionary.Add("PatientBilirubin", GetResultPromptByElementKey("HEIUrinalysis", "PatientBilirubin", BIL));
                        dictionary.Add("PatientLeukocyte", GetResultPromptByElementKey("HEIUrinalysis", "PatientLeukocyte", LEU));
                        dicJson = JsonConvert.SerializeObject(dictionary);

                        //根据key获取所有的key
                        var keyReq = new GetDictionaryNameAndUnitByKeysDBRequest()
                        {
                            Keys = new List<string>() { "PRO", "GLU", "KET", "BLD", "UBG", "NIT", "PH", "SG", "VC", "BIL", "LEU" }
                        };
                        var dics = _dictionaryDBService.GetDictionaryNameAndUnitByKeysAsync(keyReq).Result ?? new List<DictionaryWithUnitDTO>();
                        var PRO_Unit = "";
                        var GLU_Unit = "";
                        var KET_Unit = "";
                        var BLD_Unit = "";
                        var UBG_Unit = "";
                        var NIT_Unit = "";
                        var PH_Unit = "";
                        var SG_Unit = "";
                        var VC_Unit = "";
                        var BIL_Unit = "";
                        var LEU_Unit = "";
                        if (dics?.Count > 0)
                        {
                            PRO_Unit = (dics.FirstOrDefault(c => c.Key.Contains("PRO")) ?? new DictionaryWithUnitDTO()).Unit;
                            GLU_Unit = (dics.FirstOrDefault(c => c.Key.Contains("GLU")) ?? new DictionaryWithUnitDTO()).Unit;
                            KET_Unit = (dics.FirstOrDefault(c => c.Key.Contains("KET")) ?? new DictionaryWithUnitDTO()).Unit;
                            LEU_Unit = (dics.FirstOrDefault(c => c.Key.Contains("LEU")) ?? new DictionaryWithUnitDTO()).Unit;
                            BLD_Unit = (dics.FirstOrDefault(c => c.Key.Contains("BLD")) ?? new DictionaryWithUnitDTO()).Unit;
                            UBG_Unit = (dics.FirstOrDefault(c => c.Key.Contains("UBG")) ?? new DictionaryWithUnitDTO()).Unit;
                            BIL_Unit = (dics.FirstOrDefault(c => c.Key.Contains("BIL")) ?? new DictionaryWithUnitDTO()).Unit;
                            PH_Unit = (dics.FirstOrDefault(c => c.Key.Contains("PH")) ?? new DictionaryWithUnitDTO()).Unit;
                            SG_Unit = (dics.FirstOrDefault(c => c.Key.Contains("SG")) ?? new DictionaryWithUnitDTO()).Unit;
                            VC_Unit = (dics.FirstOrDefault(c => c.Key.Contains("VC")) ?? new DictionaryWithUnitDTO()).Unit;
                            NIT_Unit = (dics.FirstOrDefault(c => c.Key.Contains("NIT")) ?? new DictionaryWithUnitDTO()).Unit;
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_PRO"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_PRO").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", PRO, PRO_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_PRO",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", PRO, PRO_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_GLU"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_GLU").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", GLU, GLU_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_GLU",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientGlucose", GLU, GLU_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_KET"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_KET").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", KET, KET_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_KET",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryKetoneBody", KET, KET_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_BLD"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_BLD").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", BLD, BLD_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_BLD",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientBld", BLD, BLD_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_UBG"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_UBG").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", UBG, UBG_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_UBG",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrobilinogen", UBG, UBG_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_NIT"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_NIT").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", NIT, NIT_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_NIT",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientNitrite", NIT, NIT_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_PH"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_PH").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", PH, PH_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_PH",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientPHValue", PH, PH_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_SG"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_SG").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", SG, SG_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_SG",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientProportion", SG, SG_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_VC"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_VC").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", VC, VC_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_VC",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientVitaminC", VC, VC_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_BIL"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_BIL").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", BIL, BIL_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_BIL",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientBilirubin", BIL, BIL_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIUrinalysis_LEU"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIUrinalysis_LEU").Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientUrinaryProtein", LEU, LEU_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIUrinalysis_LEU",
                                Value = GetResultPromptMsgByKey("HEIUrinalysis", "PatientLeukocyte", LEU, LEU_Unit)
                            });
                        }
                        var res = _registerInfoDBService.UpdateResultsAndSuggestionsAsync(registerInfo.Code, "", "", dataItemList).Result;

                        reportDatas.Add(new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementResultPromptKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362x", "ResultPrompt", dicJson, item));

                        break;
                    case "HEIBiochemical":// 4-生化检查;

                        dictionary.Add("PatientSerumAlanineAminotransferase", GetResultPromptByElementKey("HEIBiochemical", "PatientSerumAlanineAminotransferase", Liver_Alt));
                        dictionary.Add("PatientSGOT", GetResultPromptByElementKey("HEIBiochemical", "PatientSGOT", Liver_Ast));
                        dictionary.Add("PatientTotalBilirubin", GetResultPromptByElementKey("HEIBiochemical", "PatientTotalBilirubin", Liver_Tbil));
                        dictionary.Add("PatientAlbumin", GetResultPromptByElementKey("HEIBiochemical", "PatientAlbumin", Liver_Alb));
                        dictionary.Add("PatientConjugatedBilirubin", GetResultPromptByElementKey("HEIBiochemical", "PatientConjugatedBilirubin", Liver_Dbil));
                        dictionary.Add("PatientCreatinine", GetResultPromptByElementKey("HEIBiochemical", "PatientCreatinine", Renal_Cr));
                        dictionary.Add("PatientUreaNitrogen", GetResultPromptByElementKey("HEIBiochemical", "PatientUreaNitrogen", Renal_Bun));
                        dictionary.Add("PatientBloodPotassiumConcentration", GetResultPromptByElementKey("HEIBiochemical", "PatientBloodPotassiumConcentration", Renal_Spc));
                        dictionary.Add("PatientBloodSodiumConcentration", GetResultPromptByElementKey("HEIBiochemical", "PatientBloodSodiumConcentration", Renal_Ssc));
                        dictionary.Add("PatientTotalCholesterol", GetResultPromptByElementKey("HEIBiochemical", "PatientTotalCholesterol", Lipid_Cho));
                        dictionary.Add("PatientTriglyceride", GetResultPromptByElementKey("HEIBiochemical", "PatientTriglyceride", Lipid_Tg));
                        dictionary.Add("PatientLowDensityCholesterol", GetResultPromptByElementKey("HEIBiochemical", "PatientLowDensityCholesterol", Lipid_Ldl));
                        dictionary.Add("PatientHighDensityCholesterol", GetResultPromptByElementKey("HEIBiochemical", "PatientHighDensityCholesterol", Lipid_Hdl));
                        dictionary.Add("PatientDextrose", GetResultPromptByElementKey("HEIBiochemical", "PatientDextrose", BCH_Glu));
                        FillResultPrompt(key, dictionary, json);
                        dicJson = JsonConvert.SerializeObject(dictionary);

                        var keyReq1 = new GetDictionaryNameAndUnitByKeysDBRequest()
                        {
                            Keys = new List<string>() { "Liver_Alt", "Liver_Ast", "Liver_Tbil", "Liver_Alb", "Liver_Dbil", "Renal_Cr", "Renal_Bun", "Renal_Spc", "Renal_Ssc",
                            "Lipid_Cho", "Lipid_Tg", "Lipid_Ldl", "Lipid_Hdl", "BCH_Glu"}
                        };
                        FillExamDataKey(key, keyReq1);
                        var dics1 = _dictionaryDBService.GetDictionaryNameAndUnitByKeysAsync(keyReq1).Result ?? new List<DictionaryWithUnitDTO>();
                        var Liver_Alt_Unit = "";
                        var Liver_Ast_Unit = "";
                        var Liver_Tbil_Unit = "";
                        var Liver_Alb_Unit = "";
                        var Liver_Dbil_Unit = "";
                        var Renal_Cr_Unit = "";
                        var Renal_Bun_Unit = "";
                        var Renal_Spc_Unit = "";
                        var Renal_Ssc_Unit = "";
                        var Lipid_Cho_Unit = "";
                        var Lipid_Tg_Unit = "";
                        var Lipid_Ldl_Unit = "";
                        var Lipid_Hdl_Unit = "";
                        var BCH_Glu_Unit = "";
                        if (dics1?.Count > 0)
                        {
                            Liver_Alt_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Liver_Alt")) ?? new DictionaryWithUnitDTO()).Unit;
                            Liver_Ast_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Liver_Ast")) ?? new DictionaryWithUnitDTO()).Unit;
                            Liver_Tbil_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Liver_Tbil")) ?? new DictionaryWithUnitDTO()).Unit;
                            Liver_Alb_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Liver_Alb")) ?? new DictionaryWithUnitDTO()).Unit;
                            Liver_Dbil_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Liver_Dbil")) ?? new DictionaryWithUnitDTO()).Unit;
                            Renal_Cr_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Renal_Cr")) ?? new DictionaryWithUnitDTO()).Unit;
                            Renal_Bun_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Renal_Bun")) ?? new DictionaryWithUnitDTO()).Unit;
                            Renal_Spc_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Renal_Spc")) ?? new DictionaryWithUnitDTO()).Unit;
                            Renal_Ssc_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Renal_Ssc")) ?? new DictionaryWithUnitDTO()).Unit;
                            Lipid_Cho_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Lipid_Cho")) ?? new DictionaryWithUnitDTO()).Unit;
                            Lipid_Tg_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Lipid_Tg")) ?? new DictionaryWithUnitDTO()).Unit;
                            Lipid_Ldl_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Lipid_Ldl")) ?? new DictionaryWithUnitDTO()).Unit;
                            Lipid_Hdl_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("Lipid_Hdl")) ?? new DictionaryWithUnitDTO()).Unit;
                            BCH_Glu_Unit = (dics1.FirstOrDefault(c => c.Key.Contains("BCH_Glu")) ?? new DictionaryWithUnitDTO()).Unit;
                        }
                        RefreshDateItems(key, dataItemList, json, dics1);

                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_GLU"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_GLU").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientDextrose", BCH_Glu, BCH_Glu_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_GLU",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientDextrose", BCH_Glu, BCH_Glu_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Lipid_Hdl"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Lipid_Hdl").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientHighDensityCholesterol", Lipid_Hdl, Lipid_Hdl_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Lipid_Hdl",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientHighDensityCholesterol", Lipid_Hdl, Lipid_Hdl_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Lipid_Ldl"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Lipid_Ldl").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientLowDensityCholesterol", Lipid_Ldl, Lipid_Ldl_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Lipid_Ldl",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientLowDensityCholesterol", Lipid_Ldl, Lipid_Ldl_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Lipid_Tg"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Lipid_Tg").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTriglyceride", Lipid_Tg, Lipid_Tg_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Lipid_Tg",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTriglyceride", Lipid_Tg, Lipid_Tg_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Lipid_Cho"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Lipid_Cho").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTotalCholesterol", Lipid_Cho, Lipid_Cho_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Lipid_Cho",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTotalCholesterol", Lipid_Cho, Lipid_Cho_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Renal_Ssc"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Renal_Ssc").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientBloodSodiumConcentration", Renal_Ssc, Renal_Ssc_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Renal_Ssc",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientBloodSodiumConcentration", Renal_Ssc, Renal_Ssc_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Renal_Spc"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Renal_Spc").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientBloodPotassiumConcentration", Renal_Spc, Renal_Spc_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Renal_Spc",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientBloodPotassiumConcentration", Renal_Spc, Renal_Spc_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Renal_Bun"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Renal_Bun").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientUreaNitrogen", Renal_Bun, Renal_Bun_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Renal_Bun",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientUreaNitrogen", Renal_Bun, Renal_Bun_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Renal_Cr"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Renal_Cr").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientCreatinine", Renal_Cr, Renal_Cr_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Renal_Cr",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientCreatinine", Renal_Cr, Renal_Cr_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Liver_Dbil"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Liver_Dbil").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientConjugatedBilirubin", Liver_Dbil, Liver_Dbil_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Liver_Dbil",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientConjugatedBilirubin", Liver_Dbil, Liver_Dbil_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Liver_Alb"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Liver_Alb").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientAlbumin", Liver_Alb, Liver_Alb_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Liver_Alb",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientAlbumin", Liver_Alb, Liver_Alb_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Liver_Tbil"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Liver_Tbil").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTotalBilirubin", Liver_Tbil, Liver_Tbil_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Liver_Tbil",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientTotalBilirubin", Liver_Tbil, Liver_Tbil_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Liver_Ast"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Liver_Ast").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientSGOT", Liver_Ast, Liver_Ast_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Liver_Ast",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientSGOT", Liver_Ast, Liver_Ast_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBiochemical_Liver_Alt"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBiochemical_Liver_Alt").Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientSerumAlanineAminotransferase", Liver_Alt, Liver_Alt_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBiochemical_Liver_Alt",
                                Value = GetResultPromptMsgByKey("HEIBiochemical", "PatientSerumAlanineAminotransferase", Liver_Alt, Liver_Alt_Unit)
                            });
                        }
                        var res1 = _registerInfoDBService.UpdateResultsAndSuggestionsAsync(registerInfo.Code, "", "", dataItemList).Result;

                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientSerumAlanineAminotransferaseKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3621", "PatientSerumAlanineAminotransferase", Liver_Alt, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientSGOTKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3622", "PatientSGOT", Liver_Ast, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientTotalBilirubinKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3623", "PatientTotalBilirubin", Liver_Tbil, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAlbuminKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3624", "PatientAlbumin", Liver_Alb, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientConjugatedBilirubinKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3625", "PatientConjugatedBilirubin", Liver_Dbil, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientCreatinineKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3626", "PatientCreatinine", Renal_Cr, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientUreaNitrogenKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3627", "PatientUreaNitrogen", Renal_Bun, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBloodPotassiumConcentrationKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3628", "PatientBloodPotassiumConcentration", Renal_Spc, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBloodSodiumConcentrationKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3629", "PatientBloodSodiumConcentration", Renal_Ssc, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientTotalCholesterolKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3620", "PatientTotalCholesterol", Lipid_Cho, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientTriglycerideKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362a", "PatientTriglyceride", Lipid_Tg, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLowDensityCholesterolKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362b", "PatientLowDensityCholesterol", Lipid_Ldl, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientHighDensityCholesterolKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362c", "PatientHighDensityCholesterol", Lipid_Hdl, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientDextroseKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362d", "PatientDextrose", BCH_Glu, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementResultPromptKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362x", "ResultPrompt", dicJson, item)
                        });
                        FillReportData(key, reportDatas, json, item);
                        break;
                    case "HEIBloodRoutine":// 5-血常规检查; 

                        dictionary.Add("PatientHemoglobin", GetResultPromptByElementKey("HEIBloodRoutine", "PatientHemoglobin", Blood_Hgb));
                        dictionary.Add("PatientLeukocyte", GetResultPromptByElementKey("HEIBloodRoutine", "PatientLeukocyte", Blood_Wbc));
                        dictionary.Add("PatientPlatelet", GetResultPromptByElementKey("HEIBloodRoutine", "PatientPlatelet", Blood_Rlt));
                        dictionary.Add("PatientLymphocyte", GetResultPromptByElementKey("HEIBloodRoutine", "PatientLymphocyte", Blood_Lymphocyte));
                        dictionary.Add("PatientNeutrophils", GetResultPromptByElementKey("HEIBloodRoutine", "PatientNeutrophils", Blood_Neutrophils));
                        dictionary.Add("PatientMonocyte", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMonocyte", Blood_Monocyte));
                        dictionary.Add("PatientLymphocyteRatio", GetResultPromptByElementKey("HEIBloodRoutine", "PatientLymphocyteRatio", Blood_LymphocyteRatio));
                        dictionary.Add("PatientNeutrophilRatio", GetResultPromptByElementKey("HEIBloodRoutine", "PatientNeutrophilRatio", Blood_NeutrophilRatio));
                        dictionary.Add("PatientMonocyteRatio", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMonocyteRatio", Blood_MonocyteRatio));
                        dictionary.Add("PatientErythrocyte", GetResultPromptByElementKey("HEIBloodRoutine", "PatientErythrocyte", RBC));
                        dictionary.Add("PatientMch", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMch", Blood_Mch));
                        dictionary.Add("PatientMcV", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMcV", Blood_McV));
                        dictionary.Add("PatientMchc", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMchc", Blood_Mchc));
                        dictionary.Add("PatientErythrocyteDistributionWidth", GetResultPromptByElementKey("HEIBloodRoutine", "PatientErythrocyteDistributionWidth", Blood_ErythrocyteDW));
                        dictionary.Add("PatientHematocrit", GetResultPromptByElementKey("HEIBloodRoutine", "PatientHematocrit", Blood_Hematocrit));
                        dictionary.Add("PatientMeanPlateletVolume", GetResultPromptByElementKey("HEIBloodRoutine", "PatientMeanPlateletVolume", Blood_MeanPlateletVolume));
                        dictionary.Add("PatientPlateletDistributionWidth", GetResultPromptByElementKey("HEIBloodRoutine", "PatientPlateletDistributionWidth", Blood_PlateletDW));
                        dictionary.Add("PatientPlateletPct", GetResultPromptByElementKey("HEIBloodRoutine", "PatientPlateletPct", Blood_PlateletPct));

                        FillResultPrompt(key, dictionary, json);
                        dicJson = JsonConvert.SerializeObject(dictionary);

                        var keyReq2 = new GetDictionaryNameAndUnitByKeysDBRequest()
                        {
                            Keys = new List<string>() { "Blood_Hgb", "Blood_Wbc", "Blood_Rlt", "Blood_Lymphocyte", "Blood_Neutrophils", "Blood_Monocyte",
                            "Blood_LymphocyteRatio", "Blood_NeutrophilRatio", "Blood_MonocyteRatio", "RBC", "Blood_Mch", "Blood_McV", "Blood_Mchc",
                            "Blood_ErythrocyteDW", "Blood_Hematocrit", "Blood_MeanPlateletVolume", "Blood_PlateletDW", "Blood_PlateletPct"}
                        };
                        FillExamDataKey(key, keyReq2);
                        var dics2 = _dictionaryDBService.GetDictionaryNameAndUnitByKeysAsync(keyReq2).Result ?? new List<DictionaryWithUnitDTO>();
                        var Blood_Hgb_Unit = "";
                        var Blood_Wbc_Unit = "";
                        var Blood_Rlt_Unit = "";
                        var Blood_Lymphocyte_Unit = "";
                        var Blood_Neutrophils_Unit = "";
                        var Blood_Monocyte_Unit = "";
                        var Blood_LymphocyteRatio_Unit = "";
                        var Blood_NeutrophilRatio_Unit = "";
                        var Blood_MonocyteRatio_Unit = "";
                        var RBC_Unit = "";
                        var Blood_Mch_Unit = "";
                        var Blood_McV_Unit = "";
                        var Blood_Mchc_Unit = "";
                        var Blood_ErythrocyteDW_Unit = "";
                        var Blood_Hematocrit_Unit = "";
                        var Blood_MeanPlateletVolume_Unit = "";
                        var Blood_PlateletDW_Unit = "";
                        var Blood_PlateletPct_Unit = "";
                        if (dics2?.Count > 0)
                        {
                            Blood_Hgb_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Hgb")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Wbc_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Wbc")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Rlt_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Rlt")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Lymphocyte_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Lymphocyte")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Neutrophils_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Neutrophils")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Monocyte_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Monocyte")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_LymphocyteRatio_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_LymphocyteRatio")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_NeutrophilRatio_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_NeutrophilRatio")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_MonocyteRatio_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_MonocyteRatio")) ?? new DictionaryWithUnitDTO()).Unit;
                            RBC_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("RBC")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Mch_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Mch")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_McV_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_McV")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Mchc_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Mchc")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_ErythrocyteDW_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_ErythrocyteDW")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_Hematocrit_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_Hematocrit")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_MeanPlateletVolume_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_MeanPlateletVolume")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_PlateletDW_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_PlateletDW")) ?? new DictionaryWithUnitDTO()).Unit;
                            Blood_PlateletPct_Unit = (dics2.FirstOrDefault(c => c.Key.Contains("Blood_PlateletPct")) ?? new DictionaryWithUnitDTO()).Unit;
                        }

                        RefreshDateItems(key, dataItemList, json, dics2);
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_PlateletPct"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_PlateletPct").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlateletPct", Blood_PlateletPct, Blood_PlateletPct_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_PlateletPct",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlateletPct", Blood_PlateletPct, Blood_PlateletPct_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_PlateletDW"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_PlateletDW").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlateletDistributionWidth", Blood_PlateletDW, Blood_PlateletDW_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_PlateletDW",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlateletDistributionWidth", Blood_PlateletDW, Blood_PlateletDW_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_MeanPlateletVolume"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_MeanPlateletVolume").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMeanPlateletVolume", Blood_MeanPlateletVolume, Blood_MeanPlateletVolume_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_MeanPlateletVolume",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMeanPlateletVolume", Blood_MeanPlateletVolume, Blood_MeanPlateletVolume_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Hematocrit"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Hematocrit").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientHematocrit", Blood_Hematocrit, Blood_Hematocrit_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Hematocrit",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientHematocrit", Blood_Hematocrit, Blood_Hematocrit_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_ErythrocyteDW"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_ErythrocyteDW").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientErythrocyteDistributionWidth", Blood_ErythrocyteDW, Blood_ErythrocyteDW_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_ErythrocyteDW",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientErythrocyteDistributionWidth", Blood_ErythrocyteDW, Blood_ErythrocyteDW_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Mchc"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Mchc").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMchc", Blood_Mchc, Blood_Mchc_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Mchc",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMchc", Blood_Mchc, Blood_Mchc_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_McV"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_McV").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMcV", Blood_McV, Blood_McV_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_McV",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMcV", Blood_McV, Blood_McV_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Mch"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Mch").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMch", Blood_Mch, Blood_Mch_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Mch",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMch", Blood_Mch, Blood_Mch_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_BLD"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_BLD").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientErythrocyte", RBC, RBC_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_BLD",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientErythrocyte", RBC, RBC_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_MonocyteRatio"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_MonocyteRatio").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMonocyteRatio", Blood_MonocyteRatio, Blood_MonocyteRatio_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_MonocyteRatio",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMonocyteRatio", Blood_MonocyteRatio, Blood_MonocyteRatio_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_NeutrophilRatio"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_NeutrophilRatio").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientNeutrophilRatio", Blood_NeutrophilRatio, Blood_NeutrophilRatio_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_NeutrophilRatio",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientNeutrophilRatio", Blood_NeutrophilRatio, Blood_NeutrophilRatio_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_LymphocyteRatio"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_LymphocyteRatio").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLymphocyteRatio", Blood_LymphocyteRatio, Blood_LymphocyteRatio_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_LymphocyteRatio",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLymphocyteRatio", Blood_LymphocyteRatio, Blood_LymphocyteRatio_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Monocyte"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Monocyte").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMonocyte", Blood_Monocyte, Blood_Monocyte_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Monocyte",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientMonocyte", Blood_Monocyte, Blood_Monocyte_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Neutrophils"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Neutrophils").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientNeutrophils", Blood_Neutrophils, Blood_Neutrophils_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Neutrophils",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientNeutrophils", Blood_Neutrophils, Blood_Neutrophils_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Lymphocyte"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Lymphocyte").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLymphocyte", Blood_Lymphocyte, Blood_Lymphocyte_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Lymphocyte",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLymphocyte", Blood_Lymphocyte, Blood_Lymphocyte_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Rlt"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Rlt").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlatelet", Blood_Rlt, Blood_Rlt_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Rlt",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientPlatelet", Blood_Rlt, Blood_Rlt_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Wbc"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Wbc").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLeukocyte", Blood_Wbc, Blood_Wbc_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Wbc",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientLeukocyte", Blood_Wbc, Blood_Wbc_Unit)
                            });
                        }
                        if (dataItemList.Exists(c => c.Key == "HEIBloodRoutine_Blood_Hgb"))
                        {
                            dataItemList.FirstOrDefault(c => c.Key == "HEIBloodRoutine_Blood_Hgb").Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientHemoglobin", Blood_Hgb, Blood_Hgb_Unit);
                        }
                        else
                        {
                            dataItemList.Add(new DataItemDTO()
                            {
                                Key = "HEIBloodRoutine_Blood_Hgb",
                                Value = GetResultPromptMsgByKey("HEIBloodRoutine", "PatientHemoglobin", Blood_Hgb, Blood_Hgb_Unit)
                            });
                        }
                        var res2 = _registerInfoDBService.UpdateResultsAndSuggestionsAsync(registerInfo.Code, "", "", dataItemList).Result;

                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientHemoglobinKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3618", "PatientHemoglobin", Blood_Hgb, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLeukocyteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3619", "PatientLeukocyte", Blood_Wbc, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPlateletKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A3610", "PatientPlatelet", Blood_Rlt, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLymphocyteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361a", "PatientLymphocyte", Blood_Lymphocyte, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNeutrophilsKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361b", "PatientNeutrophils", Blood_Neutrophils, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMonocyteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361c", "PatientMonocyte", Blood_Monocyte, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientLymphocyteRatioKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361d", "PatientLymphocyteRatio", Blood_LymphocyteRatio, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNeutrophilRatioKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361e", "PatientNeutrophilRatio", Blood_NeutrophilRatio, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMonocyteRatioKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361f", "PatientMonocyteRatio", Blood_MonocyteRatio, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientErythrocyteKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361g", "PatientErythrocyte", RBC, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMchKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361h", "PatientMch", Blood_Mch, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMcVKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361i", "PatientMcV", Blood_McV, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMchcKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361j", "PatientMchc", Blood_Mchc, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientErythrocyteDistributionWidthKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361k", "PatientErythrocyteDistributionWidth", Blood_ErythrocyteDW, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientHematocritKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361l", "PatientHematocrit", Blood_Hematocrit, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientMeanPlateletVolumeKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361m", "PatientMeanPlateletVolume", Blood_MeanPlateletVolume, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPlateletDistributionWidthKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361n", "PatientPlateletDistributionWidth", Blood_PlateletDW, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPlateletPctKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361o", "PatientPlateletPct", Blood_PlateletPct, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementResultPromptKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362x", "ResultPrompt", dicJson, item)

                        });
                        FillReportData(key, reportDatas, json, item);
                        break;
                    case "HEIECG"://6-心电检查;
                        var index = "";
                        var HR = "";
                        var PR = "";
                        var PDur = "";
                        var QRSDur = "";
                        var TDur = "";
                        var QTDur = "";
                        var QTCDur = "";
                        var QT_QTC_Dur = "";
                        var PAxis = "";
                        var QRSAxis = "";
                        var TAxis = "";
                        var P_QRS_T_Axis = "";
                        var Rv5_Sv1_1 = "";
                        var Rv5_Sv1_2 = "";
                        if (!string.IsNullOrEmpty(Analyse12))
                        {
                            JObject jsonAnalyse12 = JObject.Parse(Analyse12);
                            if (jsonAnalyse12 != null)
                            {
                                if (string.IsNullOrWhiteSpace(doctorSign))
                                {
                                    if (jsonAnalyse12.ContainsKey("DiagnosticEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["DiagnosticEleSignature"].ToString()))
                                    {
                                        doctorSign = jsonAnalyse12["DiagnosticEleSignature"].ToString();
                                    }
                                }
                                if (string.IsNullOrWhiteSpace(examDoctor))
                                {
                                    if (jsonAnalyse12.ContainsKey("ReviewEleSignature") && !string.IsNullOrWhiteSpace(jsonAnalyse12["ReviewEleSignature"].ToString()))
                                    {
                                        examDoctor = jsonAnalyse12["ReviewEleSignature"].ToString();
                                    }
                                }
                                if (jsonAnalyse12.ContainsKey("Rv5_Sv1_1"))
                                {
                                    Rv5_Sv1_1 = jsonAnalyse12["Rv5_Sv1_1"].ToString() + " mv";
                                }
                                if (jsonAnalyse12.ContainsKey("Rv5_Sv1_2"))
                                {
                                    Rv5_Sv1_2 = jsonAnalyse12["Rv5_Sv1_2"].ToString().Trim() + " mv";
                                }
                                if (jsonAnalyse12.ContainsKey("QRSAxis"))
                                {
                                    QRSAxis = jsonAnalyse12["QRSAxis"].ToString() + " deg";
                                }
                                if (jsonAnalyse12.ContainsKey("TAxis"))
                                {
                                    TAxis = jsonAnalyse12["TAxis"].ToString() + " deg";
                                }
                                if (jsonAnalyse12.ContainsKey("P_QRS_T_Axis"))
                                {
                                    P_QRS_T_Axis = jsonAnalyse12["P_QRS_T_Axis"].ToString() + " deg";
                                }
                                if (jsonAnalyse12.ContainsKey("QTDur"))
                                {
                                    QTDur = jsonAnalyse12["QTDur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("QTCDur"))
                                {
                                    QTCDur = jsonAnalyse12["QTCDur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("QT_QTC_Dur"))
                                {
                                    QT_QTC_Dur = jsonAnalyse12["QT_QTC_Dur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("PAxis"))
                                {
                                    PAxis = jsonAnalyse12["PAxis"].ToString() + " deg";
                                }
                                if (jsonAnalyse12.ContainsKey("QRSDur"))
                                {
                                    QRSDur = jsonAnalyse12["QRSDur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("TDur"))
                                {
                                    TDur = jsonAnalyse12["TDur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("HR"))
                                {
                                    HR = jsonAnalyse12["HR"].ToString() + " bpm";
                                }
                                if (jsonAnalyse12.ContainsKey("PR"))
                                {
                                    PR = jsonAnalyse12["PR"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("PDur"))
                                {
                                    PDur = jsonAnalyse12["PDur"].ToString() + " ms";
                                }
                                if (jsonAnalyse12.ContainsKey("index"))
                                {
                                    index = jsonAnalyse12["index"].ToString();
                                }
                            }
                        }
                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientExamDateKey").Value, "95bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientExamDate", UTCToBeijing(examInfo.ExamTime).ToString("yyyy-MM-dd HH:mm:ss.fff"), item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementHospitalNameKey").Value, "7abbaa61-a91d-4399-a3f4-df29117d2d59", "HospitalName", orgInfo.OrganizationName, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPhysicalExamNumberKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362w", "PhysicalExamNumber", PhysicalExamNumber,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientEcgDiagnosticResultKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362e", "PatientEcgDiagnosticResult", ASSESS12, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPAxisKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362f", "PatientPAxis", PAxis, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientQRSAxisKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362g", "PatientQRSAxis", QRSAxis, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientTAxisKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362h", "PatientTAxis", TAxis, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPWidthKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362i", "PatientPWidth", "", item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPRIntervalKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362j", "PatientPRInterval", PR, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientQRSDurationKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362k", "PatientQRSDuration", QRSDur, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientQTQTCIntervalKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362l", "PatientQTQTCInterval", QT_QTC_Dur, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientRV5SVKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362m", "PatientRV5SV", Rv5_Sv1_1, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientRV5SV1Key").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362n", "PatientRV5SV1", Rv5_Sv1_2, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementHRKey").Value, "33D46E3F-A089-4B0A-8D1D-F4C7556BD62D", "HR", HR, item),

                            new ReportElementArrayDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientECGImageKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A361z", "PatientECGImage", new List<string>() { ECG12 }, item),

                        });
                        break;
                    case "HEITCMC"://7-中医体质;
                        var ChineseMedicineResults = CaculateHEITCMCResultSimple(examData);
                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElemenPatientChineseMedicineResultsKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientChineseMedicineResults", ChineseMedicineResults, item),
                        });
                        break;
                    default:
                        break;
                }
            }
            else
            {
                switch (key)
                {
                    case "HEICover"://98-封面；                        
                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientDomicileAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362p1", "PatientDomicileAddress", "",item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientStressNameKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362q", "PatientStressName", "",item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientVillageKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362r", "PatientVillage", "",item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientFilingUnitKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362s", "PatientFilingUnit", orgName,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElemenPatientArchivistKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362t", "PatientArchivist", initiatorName,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResponsibleDoctorKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362u", "PatientResponsibleDoctor", initiatorName,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientCreatedDateKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362v", "PatientCreatedDate", healthCreateTime,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPhysicalExamNumberKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362w", "PhysicalExamNumber", PhysicalExamNumber,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementHospitalNameKey").Value, "7abbaa61-a91d-4399-a3f4-df29117d2d59", "HospitalName", orgInfo.OrganizationName, item),

                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),
                        });
                        break;
                    case "HEIConclusion":// 99-结论
                        var tupleResult = CaculateReportConClusion(registerInfo);
                        reportDatas.AddRange(new List<ReportElementDTO>()
                        {
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientNameKey").Value, "91bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientName", Name,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientAgeKey").Value, "92bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientAge", Age,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientGenderKey").Value, "93bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientGender", Sex,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientPhoneNumberKey").Value, "94bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientPhoneNumber", Phone,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientIdKey").Value, "90bbaa61-a91d-4399-a3f4-df29117d2d59", "PatientId", IDCardNo, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientBirthdayKey").Value, "9aabaa61-a91d-4399-a3f4-df29117d2d59", "PatientBirthday", Birthday,item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientResidentialAddressKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362o", "PatientResidentialAddress", Adress, item),


                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientExceptionalResultsKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362p", "PatientExceptionalResults", tupleResult.Item1, item),
                            new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementPatientHealthGuidanceKey").Value, "A1843C2E-B60A-483F-A5F0-B26F475A362z", "PatientHealthGuidance", tupleResult.Item2, item),
                        });
                        break;
                    default:
                        break;
                }
            }

            if (doctorSign == "-1")
            {
                doctorSign = "";
            }
            reportDatas.AddRange(new List<ReportElementDTO>()
            {
                new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementReportTimeKey").Value, "4abbaa61-a91d-4399-a3f4-df29117d2d59", "ReportTime", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff"), item),
                new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementInspectionPhysicianKey").Value, "6abbaa61-a91d-4399-a3f4-df29117d2d59", "InspectionPhysician", examDoctor, item),
                new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementReportPhysicianKey").Value, "5abbaa61-a91d-4399-a3f4-df29117d2d59", "ReportPhysician", examDoctor, item),
                new ReportElementStringDTO(LanguageSetting.GetParammeter<StringParameter>(item, "Language", "ElementDigitalSignatureKey").Value, "53545af1-59b7-41a8-9bd5-bdba1aaf9891", "DigitalSignature", doctorSign, item)
            });

            if (reportDatas?.Count > 0)
            {
                vitalReportDataJson = Newtonsoft.Json.JsonConvert.SerializeObject(reportDatas);
            }
            return vitalReportDataJson;
        }

        /// <summary>
        /// 生成诊断结果
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        private Tuple<string, string> CaculateReportConClusion(RegisterInfoDTO registerInfo)
        {
            bool isNeedUpdate = false;
            var conclusionMsg = "";
            var suggestMsg = "";
            if (!string.IsNullOrEmpty(registerInfo?.ResultsAndSuggestions))
            {
                conclusionMsg = registerInfo.ResultsAndSuggestions;
            }
            else
            {
                if (registerInfo?.DiagnosisExceptionResult?.Count > 0)
                {
                    var exceptionMsg = "";
                    foreach (var item in registerInfo.DiagnosisExceptionResult)
                    {
                        if (!string.IsNullOrEmpty(item.Value) && !string.IsNullOrEmpty(item.Key))
                        {
                            var tempStr = item.Value.Split('：');
                            if (tempStr.Length > 0)
                            {
                                if (_exceptionDescConfigDic.Keys.Contains(item.Key))
                                {
                                    conclusionMsg += tempStr[0] + _exceptionDescConfigDic[item.Key] + "\n";
                                }
                            }
                            exceptionMsg += item.Value + "\n";
                        }
                    }
                    conclusionMsg += exceptionMsg;
                }
                isNeedUpdate = true;
            }
            if (!string.IsNullOrEmpty(registerInfo?.HealthGuidance))
            {
                suggestMsg = registerInfo.HealthGuidance;
            }
            else
            {
                suggestMsg = $"【B超指导】\n建议定期按时进行B超筛查\n【心电指导】\n建议定期按时进行心电筛查";
                isNeedUpdate = true;
            }
            if (isNeedUpdate)
            {
                //更新db
                var res = _registerInfoDBService.UpdateResultsAndSuggestionsAsync(registerInfo.Code, conclusionMsg, suggestMsg, null).Result;
            }
            return Tuple.Create<string, string>(conclusionMsg, suggestMsg);
        }

        /// <summary>
        /// 异常提示值
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        private string GetResultPromptMsgByKey(string category, string elementTagName, string value, string unit)
        {
            var returnValue = "";
            try
            {
                var key = category + "_" + elementTagName;
                if (_reportConfigDic.ContainsKey(key))
                {
                    var configValue = _reportConfigDic[key];
                    if (string.IsNullOrEmpty(configValue.MinValue) || string.IsNullOrEmpty(configValue.MaxValue) || string.IsNullOrEmpty(value))
                    {
                        return returnValue;
                    }
                    if (value.Contains("+"))
                    {
                        returnValue = configValue.Description + "异常";
                        return returnValue;
                    }
                    double minValue = Convert.ToDouble(configValue.MinValue);
                    double maxValue = Convert.ToDouble(configValue.MaxValue);
                    double curValue = Convert.ToDouble(value);
                    if (curValue < minValue)
                    {
                        returnValue = configValue.Description + "（偏低）：" + curValue + "(" + unit + ")";
                    }
                    else if (curValue > maxValue)
                    {
                        returnValue = configValue.Description + "（偏高）：" + curValue + "(" + unit + ")";
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"GetResultPromptMsgByKey Error, elementTagName:{elementTagName}, category:{category}, value:{value}");
                return "";
            }
            return returnValue;
        }

        /// <summary>
        /// 计算是否正常，并返回提示值
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        private string GetResultPromptByElementKey(string category, string elementTagName, string value)
        {
            var returnValue = "";
            try
            {
                var key = category + "_" + elementTagName;
                if (_reportConfigDic.ContainsKey(key))
                {
                    var configValue = _reportConfigDic[key];
                    if (string.IsNullOrEmpty(configValue.MinValue) || string.IsNullOrEmpty(configValue.MaxValue) || string.IsNullOrEmpty(value))
                    {
                        return returnValue;
                    }
                    if (value.Contains("+"))
                    {
                        returnValue = "异常";
                        return returnValue;
                    }
                    double minValue = Convert.ToDouble(configValue.MinValue);
                    double maxValue = Convert.ToDouble(configValue.MaxValue);
                    double curValue = Convert.ToDouble(value);
                    if (curValue < minValue)
                    {
                        returnValue = "↓";
                    }
                    else if (curValue > maxValue)
                    {
                        returnValue = "↑";
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"GetResultPromptByElementKey Error, elementTagName:{elementTagName}, category:{category}, value:{value}");
                return "";
            }
            return returnValue;
        }

        /// <summary>
        /// 获取报告处理类型根据key
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        private int GetReportTemplateType(string key)
        {
            var type = -1;
            //1-基础检查; 2-尿常规检查; 3-超声检查; 4-生化检查; 5-血常规检查; 6-心电检查; 7-中医体质； 95-老年人；95-中医药；96-个人；97-健康；98-封面；99-结论
            switch (key)
            {
                case "HEIBasic":
                    type = 1;
                    break;
                case "HEIUrinalysis":
                    type = 2;
                    break;
                case "HEIUltrasonic":
                    type = 3;
                    break;
                case "HEIBiochemical":
                    type = 4;
                    break;
                case "HEIBloodRoutine":
                    type = 5;
                    break;
                case "HEIECG":
                    type = 6;
                    break;
                case "HEITCMC":
                    type = 7;
                    break;
                case "HEITCMCDetailRecord":
                    type = 94;
                    break;
                case "HEIHealthExamination":
                    type = 95;
                    break;
                case "HEIPersonInfo":
                    type = 96;
                    break;
                case "HEILNRZLNLPG":
                    type = 97;
                    break;
                case "HEICover":
                    type = 98;
                    break;
                case "HEIConclusion":
                    type = 99;
                    break;
                default:
                    break;
            }
            return type;
        }

        /// <summary>
        /// 加入队列对外接口
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public async Task<bool> JoinVitalReportQueueAsync(VitalReportConvertDetailRequest request)
        {
            var res = true;
            try
            {
                if (request.ExamType == 199)
                {
                    var joinQueueReq = new VitalReportConvertDetailRequest()
                    {
                        PhysicalExamNumber = request.PhysicalExamNumber,
                        ExamType = request.ExamType,
                        ReportCode = request.ReportCode,
                        Type = 1,
                        IsCover = true
                    };
                    JoinDealWithReportQueue(joinQueueReq);
                    return true;
                }

                if (!string.IsNullOrEmpty(request.PhysicalExamNumber))
                {
                    //98-封面；99-结论
                    if (request.ExamType == 98 || request.ExamType == 99)
                    {
                        if (request.ExamType == 98)
                        {
                            var res1 = CreateReportCovert("HEICover", request.PhysicalExamNumber);
                        }
                        else if (request.ExamType == 99)
                        {
                            var res2 = CreateReportHEIConclusion("HEIConclusion", request.PhysicalExamNumber);
                        }
                        return true;
                    }
                    //不为空，表示是工作站，生化和血常规不做；
                    if (_isUbuntu)
                    {
                        if (request.ExamType == 4 || request.ExamType == 5)
                        {
                            Logger.WriteLineWarn($"JoinVitalReportQueueAsync not excute in _cloudServerUrl: {_cloudServerUrl}, request:{JsonConvert.SerializeObject(request)}");
                            return true;
                        }
                    }
                    if (!string.IsNullOrEmpty(request.ExamCode))
                    {
                        var dbReq = new GetExamDBRequest()
                        {
                            Code = request.ExamCode
                        };
                        var examItem = await _vitalExamDBService.GetExamDetailAsync(dbReq);

                        if (!string.IsNullOrEmpty(examItem?.Code))
                        {
                            var regDBReq = new GetRegisterPersonRequest()
                            {
                                PhysicalExamNumber = request.PhysicalExamNumber
                            };
                            var registerInfo = await _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq);
                            if (string.IsNullOrEmpty(registerInfo?.Code))
                            {
                                Logger.WriteLineWarn($"GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                                return true;
                            }
                            var orgReq = new GetOrganizationDBRequest()
                            {
                                Code = registerInfo.OrganizationCode
                            };
                            var orgInfo = _vitalOrganizationDBService.GetOrganizationDetailAsync(orgReq).Result;
                            if (string.IsNullOrEmpty(orgInfo?.Code))
                            {
                                Logger.WriteLineWarn($"GetOrganizationDetailAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                                return true;
                            }

                            //添加封面报告和结论报告；                    
                            if (examItem.ExamState != ExamStateEnum.Reported)
                            {
                                var patientCode = examItem.PatientCode;
                                var examType = GetReportTemplateType(examItem.Key);
                                var reportDataJson = GetVitalReportDataJson(examItem.Key, examItem.ExamData, registerInfo, orgInfo, examItem, out string doctorSign, out string doctorName);
                                if (string.IsNullOrEmpty(reportDataJson))
                                {
                                    Logger.WriteLineWarn($"GetVitalReportDataJson Error, PhysicalExamNumber:{request.PhysicalExamNumber}, ExamData:{examItem.ExamData}");
                                    return true;
                                }
                                var tupleInfo = GetVitalReportTemplateJson(examItem.Key, examItem.ExamData);
                                if (tupleInfo == null)
                                {
                                    Logger.WriteLineWarn($"GetVitalReportTemplateJson Error, PhysicalExamNumber:{request.PhysicalExamNumber}, examType:{examType}");
                                    return true;
                                }
                                var reportCode = "";
                                var reportInfo = _vitalReportDBService.GetVitalReportInfoByReportCodeDBAsync("", examItem.Code).Result;
                                if (reportInfo == null || string.IsNullOrWhiteSpace(reportInfo.Code))
                                {
                                    //2.找到并生成其他的检验报告；
                                    var dbRequest = new CreateReportDBRequest
                                    {
                                        Key = examItem.Key,
                                        ContractedDoctor = examItem.ContractedDoctor,
                                        ReportState = ReportStateEnum.Unchecked,
                                        TemplateCode = examItem.TemplateCode,//模板code
                                        PatientCode = examItem.PatientCode,
                                        ReportDataJson = reportDataJson,
                                        ReportTemplateJson = tupleInfo.Item2,
                                        PhysicalExamNumber = request.PhysicalExamNumber,
                                        ExamCode = examItem.Code,
                                        OperationDoctor = examItem.ExamDoctor,
                                        DoctorSign = TransferImgToBase64(doctorSign),
                                        DoctorName = doctorName,
                                        //OperationDoctor = !string.IsNullOrEmpty(first.ExamDoctor) ? first.ExamDoctor : "8404B000976740D4CC9FAF99FF7698F1"
                                    };
                                    reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
                                }
                                else
                                {
                                    var updateReq = new UpdateReportDBRequest()
                                    {
                                        Code = reportInfo.Code,
                                        ReportDataJson = reportDataJson,
                                        ReportTemplateJson = tupleInfo.Item2,
                                        TemplateCode = tupleInfo.Item1,//模板code
                                        DoctorSign = TransferImgToBase64(doctorSign),
                                        DoctorName = doctorName,
                                    };
                                    var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                                    reportCode = reportInfo.Code;
                                }
                                if (!string.IsNullOrWhiteSpace(reportCode))
                                {
                                    /// 0- 全部；1-基础检查; 2-尿常规检查; 3-超声检查; 4-生化检查; 5-血常规检查; 6-心电检查
                                    var joinQueueReq = new VitalReportConvertDetailRequest()
                                    {
                                        PhysicalExamNumber = request.PhysicalExamNumber,
                                        ExamType = examType,
                                        ReportCode = reportCode,
                                        Type = 1,
                                        IsCover = true
                                    };
                                    JoinDealWithReportQueue(joinQueueReq);
                                }
                            }

                            if (registerInfo.State == ExamStateEnum.Unchecked)
                            {
                                registerInfo.State = ExamStateEnum.Inspected;
                                var rw = await _registerInfoDBService.UpdateRegisterInfoAsync(registerInfo);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"JoinVitalReportQueueAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}, ex:{ex}");
                return true;
            }
            return res;
        }

        /// <summary>
        /// 转换base64
        /// </summary>
        /// <param name="fileUrl"></param>
        /// <remarks></remarks>
        private string TransferImgToBase64(string fileUrl)
        {
            if (!string.IsNullOrEmpty(fileUrl))
            {
                using (HttpClient client = new HttpClient())
                {
                    using (HttpResponseMessage response = client.GetAsync(fileUrl).Result)
                    {
                        var bytes = response.Content.ReadAsByteArrayAsync().Result;
                        var base64String = "data:image/jpg;base64," + Convert.ToBase64String(bytes);
                        return base64String;
                    }
                }
            }
            return "";
        }

        /// <summary>
        /// 上传Base64图像
        /// </summary>
        /// <returns></returns>
        public async Task<string> UploadBaseb4ToImgAsync(UploadBaseb4ToImgRequest request)
        {
            var base64String = request.Base64String;
            if (!string.IsNullOrWhiteSpace(base64String))
            {
                try
                {
                    return await TransferBase64ToImg(base64String);
                }
                catch (Exception ex)
                {
                    Logger.WriteLineWarn($"ReportService UploadBaseb4ToImgAsync failed, ex:{ex}");
                }
            }
            return string.Empty;
        }

        /// <summary>
        /// base64转图片
        /// </summary>
        /// <param name="fileUrl"></param>
        /// <remarks></remarks>
        private async Task<string> TransferBase64ToImg(string base64)
        {
            var imgUrl = "";
            if (!string.IsNullOrEmpty(base64))
            {
                var base64String = base64;
                byte[] imageBytes = Convert.FromBase64String(base64String);
                using (var bitmap = SkiaSharp.SKBitmap.Decode(imageBytes))
                {
                    var imagePath = ReportPreviewHelper.GetSaveFilePath() + ".jpg";
                    using (var fileStream = new FileStream(imagePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                    {
                        bitmap.Encode(fileStream, SkiaSharp.SKEncodedImageFormat.Jpeg, 100);
                    }
                    if (File.Exists(imagePath))
                    {
                        var fileToken = UploadFileAsync(imagePath, Path.GetFileName(imagePath)).Result;
                        if (!string.IsNullOrWhiteSpace(fileToken))
                        {
                            imgUrl = fileToken;
                        }
                        File.Delete(imagePath);
                    }
                }

                // using (var ms = new MemoryStream(imageBytes, 0, imageBytes.Length))
                // {
                //     // 使用Bitmap从MemoryStream创建图片
                //     using (Bitmap image = new Bitmap(ms))
                //     {
                //         var imgFile = ReportPreviewHelper.GetSaveFilePath() + ".jpg";
                //         // 保存图片到文件
                //         image.Save(imgFile);
                //         if (File.Exists(imgFile))
                //         {
                //             var fileToken = UploadFileAsync(imgFile, Path.GetFileName(imgFile)).Result;
                //             if (!string.IsNullOrWhiteSpace(fileToken))
                //             {
                //                 imgUrl = fileToken;
                //             }
                //             File.Delete(imgFile);
                //         }
                //     }
                // }
            }
            return imgUrl;
        }

        /// <summary>
        /// 创建报告封面
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <remarks>true</remarks>
        private bool CreateReportCovert(string key = "HEICover", string physicalExamNumber = "")
        {
            var examType = GetReportTemplateType(key);
            var tupleInfo = GetVitalReportTemplateJson(key);
            if (tupleInfo == null)
            {
                Logger.WriteLineWarn($"GetVitalReportTemplateJson Error, PhysicalExamNumber:{physicalExamNumber}, examType:{examType}");
                return true;
            }

            //单独生成部分合并报告
            var regDBReq = new GetRegisterPersonRequest()
            {
                PhysicalExamNumber = physicalExamNumber
            };
            var registerInfo = _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq).Result;
            if (string.IsNullOrEmpty(registerInfo?.Code))
            {
                Logger.WriteLineWarn($"GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }

            var patientCode = registerInfo.IDCardNo;
            var orgReq = new GetOrganizationDBRequest()
            {
                Code = registerInfo.OrganizationCode
            };
            var orgInfo = _vitalOrganizationDBService.GetOrganizationDetailAsync(orgReq).Result;
            if (string.IsNullOrEmpty(orgInfo?.Code))
            {
                Logger.WriteLineWarn($"GetOrganizationDetailAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }

            var reportDataJson = GetVitalReportDataJson(key, "", registerInfo, orgInfo, null, out _, out _);
            if (string.IsNullOrEmpty(reportDataJson))
            {
                Logger.WriteLineWarn($"GetVitalReportDataJson Covert Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }

            //看看检查是否已经创建
            var reportDBReq = new GetReportConditionRequest()
            {
                PhysicalExamNumber = physicalExamNumber,
                Key = key
            };
            var reportInfo = new WingInterfaceLibrary.DTO.Vital.ReportDTO()
            {
                Key = key,
                PhysicalExamNumber = physicalExamNumber
            };
            var reportList = _vitalReportDBService.GetVitalReportListDBAsync(reportDBReq).Result;
            if (reportList?.Count > 0)
            {
                reportInfo = reportList.FirstOrDefault();
            }
            var reportCode = "";
            if (string.IsNullOrEmpty(reportInfo?.Code))
            {
                //找到并生成其他的检验报告；
                var dbRequest = new CreateReportDBRequest
                {
                    Key = key,
                    ContractedDoctor = "",
                    ReportState = ReportStateEnum.Unchecked,
                    TemplateCode = tupleInfo.Item1,//模板code
                    PatientCode = patientCode,
                    ReportDataJson = reportDataJson,
                    ReportTemplateJson = tupleInfo.Item2,
                    PhysicalExamNumber = physicalExamNumber,
                    ExamCode = "",
                    OperationDoctor = ""
                };
                reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
            }
            else
            {
                var updateReq = new UpdateReportDBRequest()
                {
                    Code = reportInfo.Code,
                    ReportDataJson = reportDataJson,
                    ReportTemplateJson = tupleInfo.Item2,
                    TemplateCode = tupleInfo.Item1,//模板code
                };
                var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                reportCode = reportInfo.Code;
            }
            Logger.WriteLineWarn($"Cover Report Create Completed, reportCode:{reportCode}");
            if (!string.IsNullOrWhiteSpace(reportCode))
            {
                /// 0- 全部；1-基础检查; 2-尿常规检查; 3-超声检查; 4-生化检查; 5-血常规检查; 6-心电检查
                var joinQueueReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = physicalExamNumber,
                    ExamType = examType,
                    ReportCode = reportCode,
                    Type = 1,
                    IsCover = true
                };
                JoinDealWithReportQueue(joinQueueReq);
            }
            return true;
        }

        /// <summary>
        /// 创建远程心电报告
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public async Task<bool> CreateRemoteHEIEcgReport(VitalReportConvertDetailRequest request)
        {
            var res = false;
            string physicalExamNumber = "";
            string key = "HEIECG";
            var ecgRecord = request.RecordCode;
            var type = request.Type;
            try
            {
                //同步数据到vital report
                //修改 vital report和exam 状态
                //加入到合并队列
                var record = await _vitalElectrocardiogramDBService.GetExamElectrocardiogramDBAsync(ecgRecord);
                if (!string.IsNullOrEmpty(record?.Code))
                {
                    physicalExamNumber = record.PhysicalExamNumber;
                }
                if (string.IsNullOrEmpty(physicalExamNumber))
                {
                    Logger.WriteLineWarn($"CreateRemoteHEIEcgReport GetExamElectrocardiogramDBAsync Error, ecgRecord:{ecgRecord}");
                    return true;
                }
                var patientCode = "";
                var regDBReq = new GetRegisterPersonRequest()
                {
                    PhysicalExamNumber = physicalExamNumber
                };
                var registerInfo = await _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq);
                if (string.IsNullOrEmpty(registerInfo?.Code))
                {
                    Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                    return true;
                }
                else
                {
                    patientCode = registerInfo.IDCardNo;
                }

                var examCode = "";
                var examInfo = await _vitalExamDBService.GetExamInfoByConditionDBAsync(physicalExamNumber, key);
                if (!string.IsNullOrEmpty(examInfo?.Code))
                {
                    examCode = examInfo.Code;
                }
                else
                {
                    var dbRequest = new CreateExamDBRequest
                    {
                        Key = key,
                        BatchNumber = "",
                        PatientCode = patientCode,
                        ContractedDoctor = "",
                        TemplateCode = ecgRecord,
                        ExamDoctor = "",
                        ExamData = new Dictionary<string, string>(),
                        PhysicalExamNumber = physicalExamNumber
                    };
                    examCode = await _vitalExamDBService.CreateExamAsync(dbRequest);
                }
                var reportDBReq = new GetReportConditionRequest()
                {
                    PhysicalExamNumber = physicalExamNumber,
                    Key = key
                };
                var ecgPDFUrl = "";
                var operationDoctor = "";
                //杏聆荟超声机
                var findReportInfo = new WingInterfaceLibrary.DB.Request.GetReportByRecordCodeDBRequest()
                {
                    RecordCode = ecgRecord
                };
                var vinnoReportList = await _reportDBService.GetReportByRecordCodeAsync(findReportInfo);
                if (vinnoReportList?.Count > 0)
                {
                    var vinnoReportInfo = vinnoReportList.OrderByDescending(c => c.CreateTime).FirstOrDefault() ?? new WingInterfaceLibrary.DTO.Report.ReportDTO();
                    if (!string.IsNullOrEmpty(vinnoReportInfo?.ReportCode) & vinnoReportInfo?.ReportPreviewList?.Count > 0)
                    {
                        ecgPDFUrl = vinnoReportInfo.ReportPreviewList.FirstOrDefault(c => c.FileType == UploadFileTypeEnum.PDF)?.FileToken ?? string.Empty;
                    }
                    operationDoctor = vinnoReportInfo.ReportUserCode;
                }
                var imgList = new List<string>();
                if (!string.IsNullOrEmpty(ecgPDFUrl) && _isNeedImageReport)
                {
                    var tempImgList = ReportPreviewHelper.GetImageReportPreviewByPDFFile(ecgPDFUrl, false);
                    if (tempImgList?.Count > 0)
                    {
                        for (var j = 0; j < tempImgList.Count; j++)
                        {
                            var localPathImg = tempImgList[j];
                            var imgUrl = UploadFileAsync(localPathImg, Path.GetFileName(localPathImg)).Result;
                            if (!string.IsNullOrWhiteSpace(imgUrl))
                            {
                                imgList.Add(imgUrl);
                            }
                        }
                    }
                }
                //看看检查是否已经创建
                var reportInfo = new WingInterfaceLibrary.DTO.Vital.ReportDTO()
                {
                    Key = key,
                    PhysicalExamNumber = physicalExamNumber
                };
                var reportList = await _vitalReportDBService.GetVitalReportListDBAsync(reportDBReq);
                if (reportList?.Count > 0)
                {
                    reportInfo = reportList.FirstOrDefault();
                }
                var reportCode = "";
                if (string.IsNullOrEmpty(reportInfo?.Code))
                {
                    //找到并生成其他的检验报告；
                    var dbRequest = new CreateReportDBRequest
                    {
                        Key = key,
                        ContractedDoctor = "",
                        ReportState = ReportStateEnum.Reported,
                        TemplateCode = "",//杏聆荟超声报告id
                        PatientCode = patientCode,
                        ReportDataJson = "",
                        ReportTemplateJson = "",
                        PhysicalExamNumber = physicalExamNumber,
                        ExamCode = examCode,
                        OperationDoctor = operationDoctor,
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = ecgPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        }
                        //OperationDoctor = !string.IsNullOrEmpty(first.ExamDoctor) ? first.ExamDoctor : "8404B000976740D4CC9FAF99FF7698F1"
                    };
                    if (imgList?.Count > 0)
                    {
                        dbRequest.PreviewList = new List<PreviewDTO>() {
                                new PreviewDTO()
                                {
                                    FileToken = string.Join(";", imgList),
                                    ThumbnailToken = "",
                                    Language = "zh-CN",
                                    FileType = (int)UploadFileTypeEnum.JPG
                                }
                            };
                    }
                    reportCode = await _vitalReportDBService.CreateReportAsync(dbRequest);
                }
                else
                {
                    var updateReq = new UpdateReportDBRequest()
                    {
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = ecgPDFUrl,
                            ThumbnailToken = "",
                            Language = "zh-CN",
                            FileType = 16
                        },
                        Code = reportInfo.Code,
                        Key = reportInfo.Key,
                        PatientCode = reportInfo.PatientCode,
                        ContractedDoctor = reportInfo.ContractedDoctor,
                        ReportDataJson = reportInfo.ReportDataJson,
                        ReportTemplateJson = reportInfo.ReportTemplateJson,
                        ReportState = ReportStateEnum.Reported,
                        TemplateCode = reportInfo.TemplateCode,
                        PhysicalExamNumber = reportInfo.PhysicalExamNumber,
                        ExamCode = examCode,
                        PreviewList = reportInfo.PreviewList
                    };
                    if (imgList?.Count > 0)
                    {
                        if (updateReq.PreviewList?.Count > 0)
                        {
                            foreach (var previewInfo in updateReq.PreviewList)
                            {
                                if (previewInfo.FileType == (int)UploadFileTypeEnum.JPG && previewInfo.Language == "zh-CN")
                                {
                                    previewInfo.FileToken = string.Join(";", imgList);
                                }
                            }
                        }
                        else
                        {
                            updateReq.PreviewList = new List<PreviewDTO>();
                            updateReq.PreviewList.Add(new PreviewDTO()
                            {
                                FileToken = string.Join(";", imgList),
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = (int)UploadFileTypeEnum.JPG
                            });
                        }
                    }
                    var updateRes = await _vitalReportDBService.UpdateReportAsync(updateReq);
                    reportCode = reportInfo.Code;
                }
                //需要判断是否更新状态，如果是自动生成的，则不更新，否则更新
                var examRequst = new UpdateExamDBRequest()
                {
                    Code = examCode,
                    ExamState = ExamStateEnum.Reported,
                    ExamDoctor = operationDoctor
                };
                var updateRes1 = await _vitalExamDBService.UpdateExamStateAsync(examRequst);

                var joinQueueReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = physicalExamNumber,
                    ExamType = 3,
                    ReportCode = reportCode,
                    Type = 1,
                    IsCover = true
                };
                ReadyDataMergePartReport(joinQueueReq);
                res = true;


                if (registerInfo.State == ExamStateEnum.Unchecked)
                {
                    registerInfo.State = ExamStateEnum.Inspected;
                    var rw = await _registerInfoDBService.UpdateRegisterInfoAsync(registerInfo);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport Error, PhysicalExamNumber:{physicalExamNumber}, error:{ex}");
                return true;
            }
            return res;
        }

        /// <summary>
        /// 创建超声影像报告
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        public async Task<bool> CreateHEIUltrasonicExamAndReport(CreateHEIUltrasonicExamAndReportRequest request)
        {
            var res = true;
            string physicalExamNumber = "";
            string key = "HEIUltrasonic";
            var ultrasonicCode = request.UltrasonicCode;
            var type = request.Type;
            try
            {
                var vinnoRecordInfo = await _recordsInfoDBService.FindRecordInfoByCodeAsync(ultrasonicCode);
                if (!string.IsNullOrEmpty(vinnoRecordInfo?.RecordCode))
                {
                    physicalExamNumber = vinnoRecordInfo.DevicePatientID;
                }
                // Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport Start, request:{JsonConvert.SerializeObject(request)}");
                if (string.IsNullOrEmpty(physicalExamNumber))
                {
                    Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport FindRecordInfoByCodeAsync Error, VinnoRecordCode:{ultrasonicCode}");
                    return true;
                }
                var patientCode = "";
                var regDBReq = new GetRegisterPersonRequest()
                {
                    PhysicalExamNumber = physicalExamNumber
                };
                var registerInfo = await _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq);
                if (string.IsNullOrEmpty(registerInfo?.Code))
                {
                    Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                    return true;
                }
                else
                {
                    patientCode = registerInfo.IDCardNo;
                }
                // Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport Log, vinnoRecordInfo:{JsonConvert.SerializeObject(vinnoRecordInfo)}, registerInfo:{JsonConvert.SerializeObject(registerInfo)}");
                if (type == 0)
                {
                    var dbRequest = new CreateExamDBRequest
                    {
                        Key = key,
                        BatchNumber = "",
                        PatientCode = patientCode,
                        ContractedDoctor = "",
                        TemplateCode = ultrasonicCode,
                        ExamDoctor = "",
                        ExamData = new Dictionary<string, string>(),
                        PhysicalExamNumber = physicalExamNumber
                    };
                    var result = await _vitalExamDBService.CreateExamAsync(dbRequest);
                    if (!string.IsNullOrEmpty(result))
                    {
                        res = true;
                    }
                }
                else
                {
                    var examCode = "";
                    var examInfo = await _vitalExamDBService.GetExamInfoByConditionDBAsync(physicalExamNumber, key);
                    if (!string.IsNullOrEmpty(examInfo?.Code))
                    {
                        examCode = examInfo.Code;
                    }
                    else
                    {
                        var dbRequest = new CreateExamDBRequest
                        {
                            Key = key,
                            BatchNumber = "",
                            PatientCode = patientCode,
                            ContractedDoctor = "",
                            TemplateCode = ultrasonicCode,
                            ExamDoctor = "",
                            ExamData = new Dictionary<string, string>(),
                            PhysicalExamNumber = physicalExamNumber
                        };
                        examCode = await _vitalExamDBService.CreateExamAsync(dbRequest);
                    }
                    var reportDBReq = new GetReportConditionRequest()
                    {
                        PhysicalExamNumber = physicalExamNumber,
                        Key = key
                    };
                    var ultrasonicPdfUrl = "";
                    var operationDoctor = "";
                    //杏聆荟超声机
                    var findReportInfo = new WingInterfaceLibrary.DB.Request.GetReportByRecordCodeDBRequest()
                    {
                        RecordCode = ultrasonicCode
                    };
                    var vinnoReportList = await _reportDBService.GetReportByRecordCodeAsync(findReportInfo);
                    if (vinnoReportList?.Count > 0)
                    {
                        var vinnoReportInfo = vinnoReportList.OrderByDescending(c => c.CreateTime).FirstOrDefault() ?? new WingInterfaceLibrary.DTO.Report.ReportDTO();
                        if (!string.IsNullOrEmpty(vinnoReportInfo?.ReportCode) & vinnoReportInfo?.ReportPreviewList?.Count > 0)
                        {
                            ultrasonicPdfUrl = vinnoReportInfo.ReportPreviewList.FirstOrDefault(c => c.FileType == UploadFileTypeEnum.PDF)?.FileToken ?? string.Empty;
                        }
                        operationDoctor = vinnoReportInfo.ReportUserCode;
                    }
                    var imgList = new List<string>();
                    if (!string.IsNullOrEmpty(ultrasonicPdfUrl) && _isNeedImageReport)
                    {
                        var tempImgList = ReportPreviewHelper.GetImageReportPreviewByPDFFile(ultrasonicPdfUrl, false);
                        if (tempImgList?.Count > 0)
                        {
                            for (var j = 0; j < tempImgList.Count; j++)
                            {
                                var localPathImg = tempImgList[j];
                                var imgUrl = UploadFileAsync(localPathImg, Path.GetFileName(localPathImg)).Result;
                                if (!string.IsNullOrWhiteSpace(imgUrl))
                                {
                                    imgList.Add(imgUrl);
                                }
                            }
                        }
                    }
                    //看看检查是否已经创建
                    var reportInfo = new WingInterfaceLibrary.DTO.Vital.ReportDTO()
                    {
                        Key = key,
                        PhysicalExamNumber = physicalExamNumber
                    };
                    var reportList = await _vitalReportDBService.GetVitalReportListDBAsync(reportDBReq);
                    if (reportList?.Count > 0)
                    {
                        reportInfo = reportList.FirstOrDefault();
                    }
                    var reportCode = "";
                    if (string.IsNullOrEmpty(reportInfo?.Code))
                    {
                        //找到并生成其他的检验报告；
                        var dbRequest = new CreateReportDBRequest
                        {
                            Key = key,
                            ContractedDoctor = "",
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = "",//杏聆荟超声报告id
                            PatientCode = patientCode,
                            ReportDataJson = "",
                            ReportTemplateJson = "",
                            PhysicalExamNumber = physicalExamNumber,
                            ExamCode = examCode,
                            OperationDoctor = operationDoctor,
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = ultrasonicPdfUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            }
                            //OperationDoctor = !string.IsNullOrEmpty(first.ExamDoctor) ? first.ExamDoctor : "8404B000976740D4CC9FAF99FF7698F1"
                        };
                        if (imgList?.Count > 0)
                        {
                            dbRequest.PreviewList = new List<PreviewDTO>() {
                                new PreviewDTO()
                                {
                                    FileToken = string.Join(";", imgList),
                                    ThumbnailToken = "",
                                    Language = "zh-CN",
                                    FileType = (int)UploadFileTypeEnum.JPG
                                }
                            };
                        }
                        reportCode = await _vitalReportDBService.CreateReportAsync(dbRequest);
                    }
                    else
                    {
                        var updateReq = new UpdateReportDBRequest()
                        {
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = ultrasonicPdfUrl,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = 16
                            },
                            Code = reportInfo.Code,
                            Key = reportInfo.Key,
                            PatientCode = reportInfo.PatientCode,
                            ContractedDoctor = reportInfo.ContractedDoctor,
                            ReportDataJson = reportInfo.ReportDataJson,
                            ReportTemplateJson = reportInfo.ReportTemplateJson,
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = reportInfo.TemplateCode,
                            PhysicalExamNumber = reportInfo.PhysicalExamNumber,
                            ExamCode = examCode,
                            PreviewList = reportInfo.PreviewList
                        };
                        if (imgList?.Count > 0)
                        {
                            if (updateReq.PreviewList?.Count > 0)
                            {
                                foreach (var previewInfo in updateReq.PreviewList)
                                {
                                    if (previewInfo.FileType == (int)UploadFileTypeEnum.JPG && previewInfo.Language == "zh-CN")
                                    {
                                        previewInfo.FileToken = string.Join(";", imgList);
                                    }
                                }
                            }
                            else
                            {
                                updateReq.PreviewList = new List<PreviewDTO>();
                                updateReq.PreviewList.Add(new PreviewDTO()
                                {
                                    FileToken = string.Join(";", imgList),
                                    ThumbnailToken = "",
                                    Language = "zh-CN",
                                    FileType = (int)UploadFileTypeEnum.JPG
                                });
                            }
                        }
                        var updateRes = await _vitalReportDBService.UpdateReportAsync(updateReq);
                        reportCode = reportInfo.Code;
                    }
                    var examRequst = new UpdateExamDBRequest()
                    {
                        Code = examCode,
                        ExamState = ExamStateEnum.Reported,
                        ExamDoctor = operationDoctor
                    };
                    var updateRes1 = await _vitalExamDBService.UpdateExamStateAsync(examRequst);

                    var joinQueueReq = new VitalReportConvertDetailRequest()
                    {
                        PhysicalExamNumber = physicalExamNumber,
                        ExamType = 3,
                        ReportCode = reportCode,
                        Type = 1,
                        IsCover = true
                    };
                    ReadyDataMergePartReport(joinQueueReq);
                    res = true;
                }

                if (registerInfo.State == ExamStateEnum.Unchecked)
                {
                    registerInfo.State = ExamStateEnum.Inspected;
                    var rw = await _registerInfoDBService.UpdateRegisterInfoAsync(registerInfo);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"CreateHEIUltrasonicExamAndReport Error, PhysicalExamNumber:{physicalExamNumber}, error:{ex}");
                return true;
            }
            return res;
        }

        /// <summary>
        /// 创建报告结论
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <remarks>true</remarks>
        private bool CreateReportHEIConclusion(string key = "HEIConclusion", string physicalExamNumber = "")
        {
            var examType = GetReportTemplateType(key);
            var tupleInfo = GetVitalReportTemplateJson(key);
            if (tupleInfo == null)
            {
                Logger.WriteLineWarn($"GetVitalReportTemplateJson Error, PhysicalExamNumber:{physicalExamNumber}, examType:{examType}");
                return true;
            }

            //单独生成部分合并报告
            var regDBReq = new GetRegisterPersonRequest()
            {
                PhysicalExamNumber = physicalExamNumber
            };
            var registerInfo = _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq).Result;
            if (string.IsNullOrEmpty(registerInfo?.Code))
            {
                Logger.WriteLineWarn($"GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }
            var patientCode = registerInfo.IDCardNo;
            var orgReq = new GetOrganizationDBRequest()
            {
                Code = registerInfo.OrganizationCode
            };
            var orgInfo = _vitalOrganizationDBService.GetOrganizationDetailAsync(orgReq).Result;
            if (string.IsNullOrEmpty(orgInfo?.Code))
            {
                Logger.WriteLineWarn($"GetOrganizationDetailAsync Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }

            var reportDataJson = GetVitalReportDataJson(key, "", registerInfo, orgInfo, null, out _, out _);
            if (string.IsNullOrEmpty(reportDataJson))
            {
                Logger.WriteLineWarn($"GetVitalReportDataJson HEIConclusion Error, PhysicalExamNumber:{physicalExamNumber}");
                return true;
            }
            var reportCode = "";
            var reportQueryReq = new GetReportConditionRequest()
            {
                PhysicalExamNumber = physicalExamNumber
            };
            var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportQueryReq).Result;
            if (reportInfos?.Count > 0 && reportInfos.Any(c => c.Key == "HEIConclusion"))
            {
                var reportInfo = reportInfos.FirstOrDefault(c => c.Key == "HEIConclusion") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                //找到
                var updateReq = new UpdateReportDBRequest()
                {
                    TemplateCode = tupleInfo.Item1,//模板code
                    Code = reportInfo.Code,
                    ReportDataJson = reportDataJson,
                    ReportTemplateJson = tupleInfo.Item2,
                };
                var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                reportCode = reportInfo.Code;
            }
            else
            {
                //找到并生成其他的检验报告；
                var dbRequest = new CreateReportDBRequest
                {
                    Key = key,
                    ContractedDoctor = "",
                    ReportState = ReportStateEnum.Unchecked,
                    TemplateCode = tupleInfo.Item1,//模板code
                    PatientCode = patientCode,
                    ReportDataJson = reportDataJson,
                    ReportTemplateJson = tupleInfo.Item2,
                    PhysicalExamNumber = physicalExamNumber,
                    ExamCode = "",
                    OperationDoctor = ""
                    //OperationDoctor = !string.IsNullOrEmpty(first.ExamDoctor) ? first.ExamDoctor : "8404B000976740D4CC9FAF99FF7698F1"
                };
                reportCode = _vitalReportDBService.CreateReportAsync(dbRequest).Result;
            }

            if (!string.IsNullOrWhiteSpace(reportCode))
            {
                /// 0- 全部；1-基础检查; 2-尿常规检查; 3-超声检查; 4-生化检查; 5-血常规检查; 6-心电检查
                var joinQueueReq = new VitalReportConvertDetailRequest()
                {
                    PhysicalExamNumber = physicalExamNumber,
                    ExamType = examType,
                    ReportCode = reportCode,
                    Type = 1,
                    IsCover = true
                };
                JoinDealWithReportQueue(joinQueueReq);
            }
            return true;
        }

        /// <summary>
        /// 部分已出报告，合并报告方法
        /// </summary>
        private bool DealWithVitalPartMergePDFReport(MergePDFFileInfo mergeInfo)
        {
            var key = mergeInfo.RegisterInfo.PhysicalExamNumber + "_" + mergeInfo.ExamType;
            var isCompleted = true;
            var registerInfo = mergeInfo.RegisterInfo;
            var orgInfo = mergeInfo.OrganizationInfo;

            //验证是否都已完成报告，包括封面和结论
            string patientCode = "";
            if (orgInfo.ExamItemCodes == null || orgInfo.ExamItemCodes.Count <= 0)
            {
                orgInfo.ExamItemCodes = new List<string>()
                {
                    "HEICover", "HEIPersonInfo", "HEIHealthExamination", "HEILNRZLNLPG", "HEITCMCDetailRecord", "HEICover", "HEIBasic", "HEIUrinalysis", "HEIUltrasonic", "HEIBiochemical", "HEIBloodRoutine", "HEIECG", "HEITCMC", "HEIConclusion"
                };
            }
            else
            {
                orgInfo.ExamItemCodes.Insert(0, "HEICover");
                orgInfo.ExamItemCodes.Insert(1, "HEIPersonInfo");
                orgInfo.ExamItemCodes.Insert(2, "HEIHealthExamination");
                orgInfo.ExamItemCodes.Insert(3, "HEILNRZLNLPG");
                orgInfo.ExamItemCodes.Insert(4, "HEITCMCDetailRecord");
                orgInfo.ExamItemCodes.Insert(5, "HEICover");
                orgInfo.ExamItemCodes.Add("HEITCMC");
                orgInfo.ExamItemCodes.Add("HEIConclusion");
            }
            orgInfo.ExamItemCodes = orgInfo.ExamItemCodes.Distinct().ToList();
            var reportDBReq = new GetReportConditionRequest()
            {
                PhysicalExamNumber = registerInfo.PhysicalExamNumber
            };
            var reportInfos = _vitalReportDBService.GetVitalReportListDBAsync(reportDBReq).Result;
            if (reportInfos?.Count > 0)
            {
                foreach (var item in orgInfo.ExamItemCodes)
                {
                    if (item == "HEIHealthExamination" || item == "HEILNRZLNLPG" || item == "HEIPersonInfo")
                    {
                        continue;
                    }
                    var reportInfo = reportInfos.FirstOrDefault(c => c.Key == item && c.ReportState == ReportStateEnum.Reported);
                    if (string.IsNullOrEmpty(reportInfo?.Code))
                    {
                        isCompleted = isCompleted & false;
                        break;
                    }
                }
            }

            //查询所有的随访报告
            var reqDB = new PrescriptionDBPageRequest()
            {
                PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                PageIndex = 1,
                PageSize = 200
            };
            var prescriptionPDFList = new List<string>();
            var prescriptionList = _prescriptionDBService.GetPrescriptionPageAsync(reqDB).Result;
            if (prescriptionList?.PageData?.Count > 0)
            {
                prescriptionPDFList = prescriptionList.PageData.Where(c => !string.IsNullOrEmpty(c.PreviewUrl))?.Select(c => c.PreviewUrl)?.Distinct()?.ToList() ?? new List<string>();
            }

            if (reportInfos?.Count > 0)
            {
                var partReportInfo = reportInfos.FirstOrDefault(c => c.Key == "Part") ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                var otherReportList = reportInfos.Where(c => c.Key != "Part")?.ToList() ?? new List<WingInterfaceLibrary.DTO.Vital.ReportDTO>();
                bool isMergePartReport = false;
                if (string.IsNullOrEmpty(partReportInfo?.Code))
                {
                    //没有生成那么则自动生成一份
                    isMergePartReport = true;
                }
                else
                {
                    //已经生成，则比对生成
                    if (otherReportList?.Count > 0)
                    {
                        foreach (var item in otherReportList)
                        {
                            if (item.UpdateTime > partReportInfo.UpdateTime)
                            {
                                isMergePartReport = true;
                                break;
                            }
                        }
                    }
                }
                if (!isMergePartReport)
                {
                    return true;
                }
                var partReportData = reportInfos.FirstOrDefault(c => c.Key == "Part" && c.ReportState == ReportStateEnum.Reported) ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                //部分报告，则是修改
                var otherReportDatas = reportInfos.Where(c => c.Key != "Part" && c.ReportState == ReportStateEnum.Reported)?.ToList() ?? new List<WingInterfaceLibrary.DTO.Vital.ReportDTO>();
                if (otherReportDatas?.Count > 0)
                {
                    var allPdf = "";
                    var streamList = new List<Stream>();
                    var filePathList = new List<string>();
                    using (var finalDocument = new PdfDocument())
                    {
                        foreach (var reportKey in orgInfo.ExamItemCodes)
                        {
                            var otherReportItem = otherReportDatas.FirstOrDefault(c => c.Key == reportKey) ?? new WingInterfaceLibrary.DTO.Vital.ReportDTO();
                            try
                            {
                                if (string.IsNullOrEmpty(otherReportItem?.Code))
                                {
                                    continue;
                                }
                                var fileName = Path.GetFileName(otherReportItem.PreviewInfo.FileToken);
                                var fileList = new List<string>() { otherReportItem.PreviewInfo.FileToken };
                                var _fileTransfer = new FileTransferService();
                                var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", registerInfo.PhysicalExamNumber);
                                if (!Directory.Exists(_tempExmaFolder))
                                {
                                    Directory.CreateDirectory(_tempExmaFolder);
                                }
                                var filePath = Path.Combine(_tempExmaFolder, fileName);
                                if (File.Exists(filePath))
                                {
                                    File.Delete(filePath);
                                }
                                var downloadRes = _fileTransfer.DownloadFileAsync(fileList, _tempExmaFolder, fileName).Result;
                                if (!string.IsNullOrEmpty(downloadRes))
                                {
                                    //Get the stream from an existing PDF document 
                                    var firstStream = new FileStream(filePath, FileMode.Open, FileAccess.Read);
                                    streamList.Add(firstStream);
                                }


                                filePathList.Add(filePath);
                            }
                            catch (Exception dEx)
                            {
                                Logger.WriteLineWarn($"Part Downdload File Error, reportKey:{reportKey}, ex: {dEx}");
                            }
                        }

                        if (prescriptionPDFList?.Count > 0)
                        {
                            foreach (var prescriptionPDF in prescriptionPDFList)
                            {
                                try
                                {
                                    var fileName = Path.GetFileName(prescriptionPDF);
                                    var fileList = new List<string>() { prescriptionPDF };
                                    var _fileTransfer = new FileTransferService();
                                    var _tempExmaFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "VitalRepertPDFTemp", registerInfo.PhysicalExamNumber);
                                    if (!Directory.Exists(_tempExmaFolder))
                                    {
                                        Directory.CreateDirectory(_tempExmaFolder);
                                    }
                                    var filePath = Path.Combine(_tempExmaFolder, fileName);
                                    if (File.Exists(filePath))
                                    {
                                        File.Delete(filePath);
                                    }
                                    var downloadRes = _fileTransfer.DownloadFileAsync(fileList, _tempExmaFolder, fileName).Result;
                                    if (!string.IsNullOrEmpty(downloadRes))
                                    {
                                        //Get the stream from an existing PDF document 
                                        var firstStream = new FileStream(filePath, FileMode.Open, FileAccess.Read);
                                        streamList.Add(firstStream);
                                    }

                                    filePathList.Add(filePath);
                                }
                                catch (Exception dEx)
                                {
                                    Logger.WriteLineWarn($"Part Downdload prescriptionPDF File Error, prescriptionPDF:{prescriptionPDF}, ex: {dEx}");
                                }
                            }
                        }

                        if (streamList?.Count > 0)
                        {
                            //Merge PDF documents 
                            try
                            {
                                Stream[] streams = streamList.ToArray();
                                PdfDocumentBase.Merge(finalDocument, streams);
                            }
                            catch (Exception ex)
                            {
                                Logger.WriteLineWarn($"Part PdfDocumentBase.Merge Error, PhysicalExamNumber:{registerInfo.PhysicalExamNumber}, ex: {ex}");
                            }
                            //Save the document into a stream 
                            using (MemoryStream outputStream = new MemoryStream())
                            {
                                //确认保存路径
                                finalDocument.Save(outputStream);
                                var bytes = outputStream.GetBuffer();
                                if (bytes != null)
                                {
                                    var pdfFile = ReportPreviewHelper.GetSaveFilePath() + ".pdf";
                                    File.WriteAllBytes(pdfFile, bytes);
                                    if (File.Exists(pdfFile))
                                    {
                                        var fileToken = UploadFileAsync(pdfFile, Path.GetFileName(pdfFile)).Result;
                                        if (!string.IsNullOrWhiteSpace(fileToken))
                                        {
                                            allPdf = fileToken;
                                        }
                                        File.Delete(pdfFile);
                                    }
                                }
                            }
                            foreach (var streamItem in streamList)
                            {
                                streamItem.Close();
                                streamItem.Dispose();
                            }
                            foreach (var filePathItem in filePathList)
                            {
                                if (File.Exists(filePathItem))
                                {
                                    File.Delete(filePathItem);
                                }
                            }
                        }
                        else
                        {
                            Logger.WriteLineWarn($"Part Stream List Error, PhysicalExamNumber:{registerInfo.PhysicalExamNumber}, streamList Count : 0");
                            if (_mergeReportSequence.Keys.Contains(key))
                            {
                                _mergeReportSequence.Remove(key, out _);
                            }
                            return true;
                        }
                    }
                    Logger.WriteLineWarn($"Part Merge Success , allPdf: {allPdf}");
                    //4.创建合并报告数据；
                    var tupleInfo = GetVitalReportTemplateJson("HEIAll");
                    if (tupleInfo == null)
                    {
                        if (_mergeReportSequence.Keys.Contains(key))
                        {
                            _mergeReportSequence.Remove(key, out _);
                        }
                        return true;
                    }
                    var reportDataJson = "";
                    //将所有的datajson都拼接起来
                    if (!string.IsNullOrEmpty(partReportData?.Code))
                    {
                        var updateReq = new UpdateReportDBRequest()
                        {
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = allPdf,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = (int)UploadFileTypeEnum.PDF
                            },
                            Code = partReportData.Code,
                            Key = partReportData.Key,
                            PatientCode = partReportData.PatientCode,
                            ContractedDoctor = partReportData.ContractedDoctor,
                            ReportDataJson = reportDataJson,
                            ReportTemplateJson = tupleInfo.Item2,
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = tupleInfo.Item1,//模板code
                            PhysicalExamNumber = partReportData.PhysicalExamNumber,
                            ExamCode = partReportData.ExamCode,
                        };
                        var createAllReportResult = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                    }
                    else
                    {
                        var dbReq = new CreateReportDBRequest
                        {
                            Key = "Part",
                            ContractedDoctor = "",
                            ReportState = ReportStateEnum.Reported,
                            TemplateCode = tupleInfo.Item1,//模板code
                            PatientCode = patientCode,
                            ReportDataJson = reportDataJson,
                            ReportTemplateJson = tupleInfo.Item2,
                            PhysicalExamNumber = registerInfo.PhysicalExamNumber,
                            ExamCode = "",
                            OperationDoctor = "8404B000976740D4CC9FAF99FF7698F1",
                            PreviewInfo = new PreviewDTO()
                            {
                                FileToken = allPdf,
                                ThumbnailToken = "",
                                Language = "zh-CN",
                                FileType = (int)UploadFileTypeEnum.PDF
                            },
                        };
                        var createAllReportResult = _vitalReportDBService.CreateReportAsync(dbReq).Result;
                    }

                    if (isCompleted)
                    {
                        //更新注册项目的状态
                        registerInfo.State = ExamStateEnum.Reported;
                        var res = _registerInfoDBService.UpdateRegisterInfoAsync(registerInfo).Result;
                    }
                }
            }
            if (_mergeReportSequence.Keys.Contains(key))
            {
                _mergeReportSequence.Remove(key, out _);
            }
            return true;
        }

        /// <summary>
        /// 本地加入队列
        /// </summary>
        /// <param name="request"></param>
        /// <remarks>true</remarks>
        private void JoinDealWithReportQueue(VitalReportConvertDetailRequest request)
        {
            _vitalReportConverterExecutor.Add(DoVitalReportConverter, request);
        }

        /// <summary>
        /// 报告分类生成
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <remarks>true</remarks>
        private bool DoVitalReportConverter(VitalReportConvertDetailRequest reportInfo)
        {
            var type = reportInfo.Type;
            if (type == 0 || type == 2)
            {
                ExcuteVitalReportConverter(reportInfo, UploadFileTypeEnum.JPG);
            }
            if (type == 0 || type == 1)
            {
                ExcuteVitalReportConverter(reportInfo, UploadFileTypeEnum.PDF);
            }
            if (reportInfo.ExamType != 199)
            {
                //部分报告合并
                ReadyDataMergePartReport(reportInfo);
            }
            else
            {
                if (!string.IsNullOrEmpty(reportInfo.PhysicalExamNumber))
                {
                    //体检的需要合并报告
                    ReadyDataMergePartReport(reportInfo);
                }
            }
            return true;
        }

        /// <summary>
        /// 部分报告数据准备
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <remarks>true</remarks>
        private void ReadyDataMergePartReport(VitalReportConvertDetailRequest request)
        {
            var mergeInfo = new MergePDFFileInfo();
            //单独生成部分合并报告
            var regDBReq = new GetRegisterPersonRequest()
            {
                PhysicalExamNumber = request.PhysicalExamNumber
            };
            var registerInfo = _registerInfoDBService.GetRegisterPersonInfoByPhysicalExamNumberDBAsync(regDBReq).Result;
            if (string.IsNullOrEmpty(registerInfo?.Code))
            {
                Logger.WriteLineWarn($"GetRegisterPersonInfoByPhysicalExamNumberDBAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                return;
            }
            mergeInfo.RegisterInfo = registerInfo;
            var orgReq = new GetOrganizationDBRequest()
            {
                Code = registerInfo.OrganizationCode
            };
            var orgInfo = _vitalOrganizationDBService.GetOrganizationDetailAsync(orgReq).Result;
            if (string.IsNullOrEmpty(orgInfo?.Code))
            {
                Logger.WriteLineWarn($"GetOrganizationDetailAsync Error, PhysicalExamNumber:{request.PhysicalExamNumber}");
                return;
            }
            mergeInfo.OrganizationInfo = orgInfo;
            mergeInfo.ExamType = request.ExamType;


            var key = mergeInfo.RegisterInfo.PhysicalExamNumber + "_" + mergeInfo.ExamType;
            if (!_mergeReportSequence.Keys.Contains(key))
            {
                _mergeReportSequence.TryAdd(key, mergeInfo.RegisterInfo.PhysicalExamNumber);
            }
            else
            {
                Logger.WriteLineWarn($"DealWithVitalPartMergePDFReport is exist queue, PhysicalExamNumber:{mergeInfo.RegisterInfo.PhysicalExamNumber}");
                return;
            }
            _vitalPartReportMergeExecutor.Add(DealWithVitalPartMergePDFReport, mergeInfo);
        }

        /// <summary>
        /// 执行报告生成
        /// </summary>
        /// <param name="reportReq"></param>
        /// <param name="fileType"></param>
        /// <remarks>true</remarks>
        private bool DoReportConverterPrescription(VitalReportConvertDetailRequest reportReq, UploadFileTypeEnum fileType)
        {
            var dbRequest = new GetPrescriptionDBRequest()
            {
                Code = reportReq.ReportCode
            };
            var prescriptionInfo = _prescriptionDBService.GetPrescriptionDetailAsync(dbRequest).Result;
            if (prescriptionInfo == null || string.IsNullOrWhiteSpace(prescriptionInfo.Code))
            {
                return true;
            }
            //如果不覆盖
            if (!reportReq.IsCover)
            {
                if (fileType == UploadFileTypeEnum.PDF)
                {
                    if (prescriptionInfo.PreviewUrl != null)
                    {
                        return true;
                    }
                }
                else if (fileType == UploadFileTypeEnum.JPG)
                {
                    if (prescriptionInfo.PreviewUrl != null)
                    {
                        return true;
                    }
                }
            }
            Logger.WriteLineInfo($"vital prescription convert to {fileType.ToString()} start, ReportCode:{reportReq.ReportCode}");

            var firstTemplate = _vitalTemplateDBService.GetTemplateDetailByKeyAsync(new GetTemplateByKeyDBRequest { Key = "Key", Value = "CF_" + prescriptionInfo.PrescriptionTemplateKey }).Result;
            if (firstTemplate != null)
            {
                var templateTagIds = GetAllTagIdJson(firstTemplate.TemplateContent);
                var prescriptionData = GetReportDataJson(prescriptionInfo.PrescriptionData, templateTagIds);

                var commonReportInfo = new WingInterfaceLibrary.DTO.Report.ReportDTO()
                {
                    ReportCode = prescriptionInfo.Code,
                    ReportTemplateJson = firstTemplate.TemplateContent,
                    ReportDatasJson = prescriptionData
                };
                var pathList = ReportPreviewHelper.GetReportPreviewPath(commonReportInfo, fileType);
                if (pathList.Any())
                {
                    var thumbnailToken = string.Empty;
                    var imageUrlList = new List<string>();
                    for (var i = 0; i < pathList.Count; i++)
                    {
                        var localPath = pathList[i];
                        var imageUrl = UploadFileAsync(localPath, Path.GetFileName(localPath)).Result;
                        if (!string.IsNullOrWhiteSpace(imageUrl))
                        {
                            imageUrlList.Add(imageUrl);
                        }
                        if (i == 0 && fileType == UploadFileTypeEnum.JPG)
                        {
                            thumbnailToken = CreateThumbnailFile(localPath, 100, 30).Result;
                        }
                    }
                    var updateReq = new SetPrescriptionPreviewInfoDBRequest
                    {
                        Code = prescriptionInfo.Code,
                        PreviewUrl = string.Join(";", imageUrlList)
                    };
                    var result1 = _prescriptionDBService.SetPrescriptionPreviewInfoAsync(updateReq).Result;
                }
            }
            return true;
        }

        /// <summary>
        /// 提取tag
        /// </summary>
        /// <param name="jsonString"></param>
        /// <returns></returns>
        private Dictionary<string, string> GetAllTagIdJson(string jsonString)
        {
            var results = new Dictionary<string, string>();
            var matches = Regex.Matches(jsonString, "\\\"Tag\\\"[\\s]?:[\\s]?(\\{[^}]*\\})");
            foreach (Match match in matches)
            {
                if (match.Groups.Count > 1)
                {
                    dynamic json = JObject.Parse(match.Groups[1].Value);
                    if (json != null)
                    {
                        var name = (string)json.Name;
                        var id = (string)json.Id;
                        if (!results.ContainsKey(name))
                        {
                            results.Add(name, id);
                        }
                    }
                }
            }
            return results;
        }

        /// <summary>
        /// 数据处理
        /// </summary>
        private string GetReportDataJson(string jsonString, Dictionary<string, string> dic)
        {
            var arrayTypeKeys = new List<string> { "PatientInspectionItems", "History", "Conclusion", "PatientExaminationWay" };
            var document = JObject.Parse(jsonString);

            var list = new List<ReportDataItem>();
            foreach (var property in document)
            {
                var tagId = dic.ContainsKey(property.Key) ? dic[property.Key] : string.Empty;
                var reportDataItem = new ReportDataItem();
                reportDataItem.Key = tagId;
                reportDataItem.TagId = tagId;
                reportDataItem.Name = property.Key;
                if (arrayTypeKeys.Contains(property.Key))
                {
                    reportDataItem.Value = property.Value.ToObject<List<string>>();
                }
                else
                {
                    if (property.Key == "PatientGender")
                    {
                        if (property.Value?.ToString() == GenderEnum.Male.ToString("D"))
                        {
                            reportDataItem.Value = "男";
                        }
                        else
                        {
                            reportDataItem.Value = "女";
                        }
                    }
                    else if (property.Key == "ReportDescription")
                    {
                        if (string.IsNullOrEmpty(property.Value?.ToString()))
                        {
                            reportDataItem.Value = "无";
                        }
                        else
                        {
                            reportDataItem.Value = property.Value.ToString();
                        }
                    }
                    else
                    {
                        reportDataItem.Value = property.Value.ToString();
                    }
                }
                list.Add(reportDataItem);
            }
            if (!list.Exists(c => c.Key == "ReportDescription"))
            {
                list.Add(new ReportDataItem()
                {
                    Key = dic.ContainsKey("ReportDescription") ? dic["ReportDescription"] : string.Empty,
                    TagId = dic.ContainsKey("ReportDescription") ? dic["ReportDescription"] : string.Empty,
                    Name = "ReportDescription",
                    Value = "无"
                });
            }
            var json = System.Text.Json.JsonSerializer.Serialize(list);
            return json;
        }

        /// <summary>
        /// 数据转换实体类
        /// </summary>
        public class ReportDataItem
        {
            public string Key { get; set; }

            public string TagId { get; set; }

            public string Name { get; set; }

            public dynamic Value { get; set; }

        }
        /// <summary>
        /// 执行报告生成
        /// </summary>
        /// <param name="reportInfo"></param>
        /// <param name="fileType"></param>
        /// <remarks>true</remarks>
        private bool ExcuteVitalReportConverter(VitalReportConvertDetailRequest reportReq, UploadFileTypeEnum fileType)
        {
            try
            {
                /// 0- 全部；1-基础检查; 2-尿常规检查; 3-超声检查; 4-生化检查; 5-血常规检查; 6-心电检查
                //获取报告信息
                if (reportReq.ExamType == 3)
                {
                    DoReportConverter(reportReq.ReportCode, fileType, true);
                    return true;
                }

                if (reportReq.ExamType == 199)
                {
                    DoReportConverterPrescription(reportReq, fileType);
                    return true;
                }

                var reportInfo = _vitalReportDBService.GetVitalReportInfoByReportCodeDBAsync(reportReq.ReportCode, "").Result;
                if (reportInfo == null || string.IsNullOrWhiteSpace(reportInfo.Code))
                {
                    return true;
                }
                //如果不覆盖
                if (!reportReq.IsCover)
                {
                    if (fileType == UploadFileTypeEnum.PDF)
                    {
                        if (reportInfo.PreviewInfo != null && reportInfo.PreviewInfo.FileType == (int)UploadFileTypeEnum.PDF)
                        {
                            return true;
                        }
                    }
                    else if (fileType == UploadFileTypeEnum.JPG)
                    {
                        if (reportInfo.PreviewInfo != null && reportInfo.PreviewInfo.FileType == (int)UploadFileTypeEnum.JPG)
                        {
                            return true;
                        }
                    }
                }
                Logger.WriteLineInfo($"vital report convert to {fileType.ToString()} start, patientCode:{reportInfo.PatientCode}, examCode:{reportInfo.ExamCode}, reportCode:{reportInfo.Code}");
                var commonReportInfo = new WingInterfaceLibrary.DTO.Report.ReportDTO()
                {
                    RecordCode = reportInfo.ExamCode,
                    ReportCode = reportInfo.Code,
                    ReportTemplateJson = reportInfo.ReportTemplateJson,
                    ReportDatasJson = reportInfo.ReportDataJson
                };
                var pathList = ReportPreviewHelper.GetReportPreviewPath(commonReportInfo, fileType);
                if (pathList.Any())
                {
                    var thumbnailToken = string.Empty;
                    var imageUrlList = new List<string>();
                    var imgList = new List<string>();
                    for (var i = 0; i < pathList.Count; i++)
                    {
                        var localPath = pathList[i];
                        var imageUrl = UploadFileAsync(localPath, Path.GetFileName(localPath)).Result;
                        if (!string.IsNullOrWhiteSpace(imageUrl))
                        {
                            imageUrlList.Add(imageUrl);
                        }
                        if (i == 0 && fileType == UploadFileTypeEnum.JPG)
                        {
                            thumbnailToken = CreateThumbnailFile(localPath, 100, 30).Result;
                        }
                        if (fileType == UploadFileTypeEnum.PDF && _isNeedImageReport)
                        {
                            var tempImgList = ReportPreviewHelper.GetImageReportPreviewByPDFFile(localPath);
                            if (tempImgList?.Count > 0)
                            {
                                for (var j = 0; j < tempImgList.Count; j++)
                                {
                                    var localPathImg = tempImgList[j];
                                    var imgUrl = UploadFileAsync(localPathImg, Path.GetFileName(localPathImg)).Result;
                                    if (!string.IsNullOrWhiteSpace(imgUrl))
                                    {
                                        imgList.Add(imgUrl);
                                    }
                                }
                            }
                        }
                    }
                    var tempPreviewList = new List<PreviewDTO>();
                    if (reportInfo.PreviewList?.Count > 0)
                    {
                        foreach (var previewInfo in reportInfo.PreviewList)
                        {
                            if (previewInfo.FileType == (int)UploadFileTypeEnum.JPG && previewInfo.Language == "zh-CN")
                            {
                                previewInfo.FileToken = string.Join(";", imgList);
                            }
                            tempPreviewList.Add(new PreviewDTO()
                            {
                                FileToken = previewInfo.FileToken,
                                ThumbnailToken = previewInfo.ThumbnailToken,
                                Language = previewInfo.Language,
                                FileType = previewInfo.FileType,
                            });
                        }
                    }
                    else
                    {
                        tempPreviewList.Add(new PreviewDTO()
                        {
                            FileToken = string.Join(";", imgList),
                            ThumbnailToken = thumbnailToken,
                            Language = "zh-CN",
                            FileType = (int)UploadFileTypeEnum.JPG
                        });
                    }
                    var updateReq = new UpdateReportDBRequest()
                    {
                        PreviewInfo = new PreviewDTO()
                        {
                            FileToken = string.Join(";", imageUrlList),
                            ThumbnailToken = thumbnailToken,
                            Language = "zh-CN",
                            FileType = (int)fileType
                        },
                        Code = reportInfo.Code,
                        Key = reportInfo.Key,
                        PatientCode = reportInfo.PatientCode,
                        ContractedDoctor = reportInfo.ContractedDoctor,
                        ReportDataJson = reportInfo.ReportDataJson,
                        ReportTemplateJson = reportInfo.ReportTemplateJson,
                        ReportState = ReportStateEnum.Reported,
                        TemplateCode = reportInfo.TemplateCode,
                        PhysicalExamNumber = reportInfo.PhysicalExamNumber,
                        ExamCode = reportInfo.ExamCode,
                        PreviewList = tempPreviewList
                    };
                    var updateRes = _vitalReportDBService.UpdateReportAsync(updateReq).Result;
                    var examRequst = new UpdateExamDBRequest()
                    {
                        Code = reportInfo.ExamCode,
                        ExamState = ExamStateEnum.Reported
                    };
                    updateRes = _vitalExamDBService.UpdateExamStateAsync(examRequst).Result;
                    //心电
                    if (reportInfo.Key=="HEIECG")
                    {
                        var record =   _vitalElectrocardiogramDBService.GetElectrocardiogramDBByPhysicalExamNumberDBAsync(reportInfo.PhysicalExamNumber).Result;
                        record.ExamState = ExamStateEnum.Reported;
                        _vitalElectrocardiogramDBService.UpdateExamElectrocardiogramDBAsync(record);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ExcuteVitalReportConverter err {ex}");
            }
            return true;
        }

    }

    /// <summary>
    /// 合并报告请求实体
    /// </summary>
    internal class MergePDFFileInfo
    {
        /// <summary>
        /// 注册信息列表
        /// </summary>
        public RegisterInfoDTO RegisterInfo { get; set; }

        /// <summary>
        /// 检查性信息表
        /// </summary>
        public WingInterfaceLibrary.DTO.Vital.OrganizationDTO OrganizationInfo { get; set; }

        /// <summary>
        /// 诊断类型
        /// </summary>
        public int ExamType { get; set; }
    }



    /// <summary>
    /// TCMC数据展示类型
    /// </summary>
    internal class TCMCScoreInfo
    {
        /// <summary>
        /// TCMC的Key
        /// </summary>
        public string Key { get; set; }
        /// <summary>
        /// 得分
        /// </summary>
        public string Score { get; set; }
        /// <summary>
        /// 是
        /// </summary>
        public string Result1 { get; set; }
        /// <summary>
        /// 倾向是
        /// </summary>
        public string Result2 { get; set; }
        /// <summary>
        /// 健康指导
        /// </summary>
        public Dictionary<string, string> HealthGuidance { get; set; } = new Dictionary<string, string>();
    }

    /// <summary>
    /// 报告配置
    /// </summary>
    internal class VitalReportConfigInfo
    {
        /// <summary>
        /// Vital字典key
        /// </summary>
        public string DictionaryKey { get; set; }
        /// <summary>
        /// 报告模板Tag Name
        /// </summary>
        public string ElementTagKey { get; set; }
        /// <summary>
        /// 描述
        /// </summary>
        public string Description { get; set; }
        /// <summary>
        /// 单位
        /// </summary>
        public string Unit { get; set; }
        /// <summary>
        /// 最小值
        /// </summary>
        public string MinValue { get; set; }
        /// <summary>
        /// 最大值
        /// </summary>
        public string MaxValue { get; set; }
    }
}